@@ -16,10 +16,11 @@ class SoundSet(Struct):
 			self.sounds.Add(Sound(filename))
 
 class Image(Struct):
-	def __init__(self, name="", filename=""):
+	def __init__(self, name="", filename="", linear_mapping=0):
 		Struct.__init__(self, "CDataImage")
 		self.name = String(name)
 		self.filename = String(filename)
+		self.flag = Int(linear_mapping)
 		self.id = TextureHandle()
 
 class SpriteSet(Struct):
@@ -244,8 +245,7 @@ container.images.Add(image_particles)
 container.images.Add(Image("cursor", "gui_cursor.png"))
 container.images.Add(Image("banner", "gui_logo.png"))
 container.images.Add(image_emoticons)
-container.images.Add(Image("console_bg", "console.png"))
-container.images.Add(Image("console_bar", "console_bar.png"))
+container.images.Add(Image("background_noise", "background_noise.png"))
 container.images.Add(image_speedup_arrow)
 container.images.Add(image_guibuttons)
 container.images.Add(image_guiicons)
@@ -254,6 +254,7 @@ container.images.Add(image_audio_source)
 container.images.Add(image_strongweak)
 container.images.Add(image_hud)
 container.images.Add(image_extras)
+container.images.Add(Image("raceflag", "race_flag.png"))
 
 container.pickups.Add(Pickup("health"))
 container.pickups.Add(Pickup("armor"))
@@ -272,7 +273,7 @@ set_speedup_arrow = SpriteSet("speedup_arrow", image_speedup_arrow, 1, 1)
 set_guibuttons = SpriteSet("guibuttons", image_guibuttons, 12, 4)
 set_guiicons = SpriteSet("guiicons", image_guiicons, 12, 2)
 set_audio_source = SpriteSet("audio_source", image_audio_source, 1, 1)
-set_strongweak = SpriteSet("strongweak", image_strongweak, 2, 1)
+set_strongweak = SpriteSet("strongweak", image_strongweak, 3, 1)
 set_hud = SpriteSet("hud", image_hud, 16, 16)
 set_extras = SpriteSet("extras", image_extras, 16, 16)
 
@@ -416,6 +417,7 @@ container.sprites.Add(Sprite("audio_source", set_audio_source, 0,0,1,1))
 
 container.sprites.Add(Sprite("hook_strong", set_strongweak, 0,0,1,1))
 container.sprites.Add(Sprite("hook_weak", set_strongweak, 1,0,1,1))
+container.sprites.Add(Sprite("hook_icon", set_strongweak, 2,0,1,1))
 
 container.sprites.Add(Sprite("hud_airjump", set_hud, 0,0,2,2))
 container.sprites.Add(Sprite("hud_airjump_empty", set_hud, 2,0,2,2))
@@ -446,8 +448,11 @@ container.sprites.Add(Sprite("hud_teleport_laser", set_hud, 8,4,2,2))
 container.sprites.Add(Sprite("hud_practice_mode", set_hud, 4,6,2,2))
 container.sprites.Add(Sprite("hud_dummy_hammer", set_hud, 6,6,2,2))
 container.sprites.Add(Sprite("hud_dummy_copy", set_hud, 8,6,2,2))
+container.sprites.Add(Sprite("hud_lock_mode", set_hud, 10,6,2,2))
+container.sprites.Add(Sprite("hud_team0_mode", set_hud, 12,6,2,2))
 
 container.sprites.Add(Sprite("part_snowflake", set_extras, 0,0,2,2))
+container.sprites.Add(Sprite("part_sparkle", set_extras, 2,0,2,2))
 
 
 anim = Animation("base")
@@ -501,50 +506,52 @@ anim.front_foot.frames.Add(AnimKeyframe(0.8, 8, 0, 0))
 anim.front_foot.frames.Add(AnimKeyframe(1.0,-10,-4, 0.2))
 container.animations.Add(anim)
 
+# the run_left animation is taken directly from run_right, only the x and rotate values are flipped,
+# and each string is run backwards, to account for how it's played in game.
 anim = Animation("run_left")
-anim.body.frames.Add(AnimKeyframe(0.0, 0, -2, 0))
+anim.body.frames.Add(AnimKeyframe(0.0, 0, -1, 0))
 anim.body.frames.Add(AnimKeyframe(0.2, 0, 0, 0))
-anim.body.frames.Add(AnimKeyframe(0.4, 0, 0, 0))
-anim.body.frames.Add(AnimKeyframe(0.6, 0, -2, 0))
+anim.body.frames.Add(AnimKeyframe(0.4, 0, -1, 0))
+anim.body.frames.Add(AnimKeyframe(0.6, 0, 0, 0))
 anim.body.frames.Add(AnimKeyframe(0.8, 0, 0, 0))
-anim.body.frames.Add(AnimKeyframe(1.0, 0, -2, 0))
+anim.body.frames.Add(AnimKeyframe(1.0, 0, -1, 0))
 
-anim.back_foot.frames.Add(AnimKeyframe(0.0, 18, -11, -0.27))
+anim.back_foot.frames.Add(AnimKeyframe(0.0, 18, -8, -0.27))
 anim.back_foot.frames.Add(AnimKeyframe(0.2, 6, 0, 0))
 anim.back_foot.frames.Add(AnimKeyframe(0.4, -7, 0, 0))
 anim.back_foot.frames.Add(AnimKeyframe(0.6, -13, -4.5, 0.05))
-anim.back_foot.frames.Add(AnimKeyframe(0.8, 0, -11, -0.2))
-anim.back_foot.frames.Add(AnimKeyframe(1.0, 18, -11, -0.27))
+anim.back_foot.frames.Add(AnimKeyframe(0.8, 0, -8, -0.2))
+anim.back_foot.frames.Add(AnimKeyframe(1.0, 18, -8, -0.27))
 
-anim.front_foot.frames.Add(AnimKeyframe(0.0, -13, -4.5, 0.05))
-anim.front_foot.frames.Add(AnimKeyframe(0.2, -14, -7, 0.1))
-anim.front_foot.frames.Add(AnimKeyframe(0.4, 11, -13, -0.3))
-anim.front_foot.frames.Add(AnimKeyframe(0.6, 18, -11, -0.27))
-anim.front_foot.frames.Add(AnimKeyframe(0.8, 3, 0, -0.02))
-anim.front_foot.frames.Add(AnimKeyframe(1.0, -13, -4.5, 0.05))
+anim.front_foot.frames.Add(AnimKeyframe(0.0, -11, -2.5, 0.05))
+anim.front_foot.frames.Add(AnimKeyframe(0.2, -14, -5, 0.1))
+anim.front_foot.frames.Add(AnimKeyframe(0.4, 11, -8, -0.3))
+anim.front_foot.frames.Add(AnimKeyframe(0.6, 18, -8, -0.27))
+anim.front_foot.frames.Add(AnimKeyframe(0.8, 3, 0, 0))
+anim.front_foot.frames.Add(AnimKeyframe(1.0, -11, -2.5, 0.05))
 container.animations.Add(anim)
 
 anim = Animation("run_right")
-anim.body.frames.Add(AnimKeyframe(0.0, 0, -2, 0))
+anim.body.frames.Add(AnimKeyframe(0.0, 0, -1, 0))
 anim.body.frames.Add(AnimKeyframe(0.2, 0, 0, 0))
-anim.body.frames.Add(AnimKeyframe(0.4, 0, -2, 0))
-anim.body.frames.Add(AnimKeyframe(0.6, 0, 0, 0))
+anim.body.frames.Add(AnimKeyframe(0.4, 0, 0, 0))
+anim.body.frames.Add(AnimKeyframe(0.6, 0, -1, 0))
 anim.body.frames.Add(AnimKeyframe(0.8, 0, 0, 0))
-anim.body.frames.Add(AnimKeyframe(1.0, 0, -2, 0))
+anim.body.frames.Add(AnimKeyframe(1.0, 0, -1, 0))
 
-anim.back_foot.frames.Add(AnimKeyframe(0.0, -18, -11, 0.27))
-anim.back_foot.frames.Add(AnimKeyframe(0.2, 0, -11, 0.2))
+anim.back_foot.frames.Add(AnimKeyframe(0.0, -18, -8, 0.27))
+anim.back_foot.frames.Add(AnimKeyframe(0.2, 0, -8, 0.2))
 anim.back_foot.frames.Add(AnimKeyframe(0.4, 13, -4.5, -0.05))
 anim.back_foot.frames.Add(AnimKeyframe(0.6, 7, 0, 0))
 anim.back_foot.frames.Add(AnimKeyframe(0.8, -6, 0, 0))
-anim.back_foot.frames.Add(AnimKeyframe(1.0, -18, -11, 0.27))
+anim.back_foot.frames.Add(AnimKeyframe(1.0, -18, -8, 0.27))
 
-anim.front_foot.frames.Add(AnimKeyframe(0.0, 13, -4.5, -0.05))
-anim.front_foot.frames.Add(AnimKeyframe(0.2, -3, 0, 0.02))
-anim.front_foot.frames.Add(AnimKeyframe(0.4, -18, -11, 0.27))
-anim.front_foot.frames.Add(AnimKeyframe(0.6, -11, -13, 0.3))
-anim.front_foot.frames.Add(AnimKeyframe(0.8, 14, -7, -0.1))
-anim.front_foot.frames.Add(AnimKeyframe(1.0, 13, -4.5, -0.05))
+anim.front_foot.frames.Add(AnimKeyframe(0.0, 11, -2.5, -0.05))
+anim.front_foot.frames.Add(AnimKeyframe(0.2, -3, 0, 0))
+anim.front_foot.frames.Add(AnimKeyframe(0.4, -18, -8, 0.27))
+anim.front_foot.frames.Add(AnimKeyframe(0.6, -11, -8, 0.3))
+anim.front_foot.frames.Add(AnimKeyframe(0.8, 14, -5, -0.1))
+anim.front_foot.frames.Add(AnimKeyframe(1.0, 11, -2.5, -0.05))
 container.animations.Add(anim)
 
 anim = Animation("hammer_swing")
@@ -580,7 +587,7 @@ weapon.visual_size.Set(64)
 weapon.offsetx.Set(32)
 weapon.offsety.Set(4)
 # the number after the plus sign is the sprite scale, which is calculated for all sprites ( w / sqrt(w² * h²) ) of the additionally added x offset, which is added now,
-# since the muzzle image is 32 pixels bigger, devided by 2, because a sprite's position is always at the center of the sprite image itself
+# since the muzzle image is 32 pixels bigger, divided by 2, because a sprite's position is always at the center of the sprite image itself
 # => the offset added, bcs the sprite is bigger now, but should not be shifted to the left
 # => 96 / sqrt(64×64+96×96)  (the original sprite scale)
 # => 64 × original sprite scale (the actual size of the sprite ingame see weapon.visual_size above)
