@@ -7,4 +7,5 @@ os.chdir(os.path.dirname(__file__) + "/../..")
 
 for lang in twlang.languages():
 	content = copy_fix(lang, delete_unused=True, append_missing=True, delete_empty=False)
-	open(lang, "w").write(content)
+	with open(lang, "w", encoding="utf-8") as f:
+		f.write(content)
