@@ -9,7 +9,6 @@ static int compul(const void *a, const void *b)
 	return ul_a->upper - ul_b->upper;
 }
 
-extern "C" {
 int str_utf8_tolower(int code)
 {
 	struct UPPER_LOWER key;
@@ -17,11 +16,10 @@ int str_utf8_tolower(int code)
 	key.upper = code;
 	res = (UPPER_LOWER *)bsearch(&key, tolowermap, NUM_TOLOWER, sizeof(struct UPPER_LOWER), compul);
 
-	if(res == NULL)
+	if(res == nullptr)
 		return code;
 	return res->lower;
 }
-}
 
 #define TOLOWER_DATA
 #include "tolower_data.h"
