@@ -1,17 +1,8 @@
-#include <stdint.h>
+#ifndef TOLOWER_DATA
+#error "This file must only be included in `tolower.cpp`"
+#endif
 
-struct UPPER_LOWER
-{
-	int32_t upper;
-	int32_t lower;
-};
-
-enum
-{
-	NUM_TOLOWER = 1390,
-};
-
-static const struct UPPER_LOWER tolower[] = {
+const struct UPPER_LOWER tolowermap[] = {
 	{65, 97},
 	{66, 98},
 	{67, 99},
@@ -967,6 +958,7 @@ static const struct UPPER_LOWER tolower[] = {
 	{11308, 11356},
 	{11309, 11357},
 	{11310, 11358},
+	{11311, 11359},
 	{11360, 11361},
 	{11362, 619},
 	{11363, 7549},
@@ -1147,10 +1139,17 @@ static const struct UPPER_LOWER tolower[] = {
 	{42938, 42939},
 	{42940, 42941},
 	{42942, 42943},
+	{42944, 42945},
 	{42946, 42947},
 	{42948, 42900},
 	{42949, 642},
 	{42950, 7566},
+	{42951, 42952},
+	{42953, 42954},
+	{42960, 42961},
+	{42966, 42967},
+	{42968, 42969},
+	{42997, 42998},
 	{65313, 65345},
 	{65314, 65346},
 	{65315, 65347},
@@ -1253,6 +1252,41 @@ static const struct UPPER_LOWER tolower[] = {
 	{66769, 66809},
 	{66770, 66810},
 	{66771, 66811},
+	{66928, 66967},
+	{66929, 66968},
+	{66930, 66969},
+	{66931, 66970},
+	{66932, 66971},
+	{66933, 66972},
+	{66934, 66973},
+	{66935, 66974},
+	{66936, 66975},
+	{66937, 66976},
+	{66938, 66977},
+	{66940, 66979},
+	{66941, 66980},
+	{66942, 66981},
+	{66943, 66982},
+	{66944, 66983},
+	{66945, 66984},
+	{66946, 66985},
+	{66947, 66986},
+	{66948, 66987},
+	{66949, 66988},
+	{66950, 66989},
+	{66951, 66990},
+	{66952, 66991},
+	{66953, 66992},
+	{66954, 66993},
+	{66956, 66995},
+	{66957, 66996},
+	{66958, 66997},
+	{66959, 66998},
+	{66960, 66999},
+	{66961, 67000},
+	{66962, 67001},
+	{66964, 67003},
+	{66965, 67004},
 	{68736, 68800},
 	{68737, 68801},
 	{68738, 68802},
