@@ -8,10 +8,15 @@
 #include "message.h"
 #include <base/hash.h>
 
-#include <game/generated/protocol.h>
-
+#include <engine/client/enums.h>
 #include <engine/friends.h>
+#include <engine/shared/translation_context.h>
+
+#include <game/generated/protocol.h>
+#include <game/generated/protocol7.h>
+
 #include <functional>
+#include <optional>
 
 struct SWarning;
 
@@ -22,8 +27,6 @@ enum
 	RECORDER_RACE = 2,
 	RECORDER_REPLAYS = 3,
 	RECORDER_MAX = 4,
-
-	NUM_DUMMIES = 2,
 };
 
 typedef bool (*CLIENTFUNC_FILTER)(const void *pData, int DataSize, void *pUser);
@@ -31,7 +34,7 @@ struct CChecksumData;
 
 class IClient : public IInterface
 {
-	MACRO_INTERFACE("client", 0)
+	MACRO_INTERFACE("client")
 public:
 	/* Constants: Client States
 		STATE_OFFLINE - The client is offline.
@@ -61,48 +64,53 @@ public:
 	{
 		LOADING_STATE_DETAIL_INITIAL,
 		LOADING_STATE_DETAIL_LOADING_MAP,
+		LOADING_STATE_DETAIL_LOADING_DEMO,
 		LOADING_STATE_DETAIL_SENDING_READY,
 		LOADING_STATE_DETAIL_GETTING_READY,
 	};
 
-	typedef std::function<void()> TMapLoadingCallbackFunc;
+	enum ELoadingCallbackDetail
+	{
+		LOADING_CALLBACK_DETAIL_MAP,
+		LOADING_CALLBACK_DETAIL_DEMO,
+	};
+	typedef std::function<void(ELoadingCallbackDetail Detail)> TLoadingCallback;
+	CTranslationContext m_TranslationContext;
 
 protected:
 	// quick access to state of the client
-	EClientState m_State;
-	ELoadingStateDetail m_LoadingStateDetail;
+	EClientState m_State = IClient::STATE_OFFLINE;
+	ELoadingStateDetail m_LoadingStateDetail = LOADING_STATE_DETAIL_INITIAL;
 	int64_t m_StateStartTime;
 
 	// quick access to time variables
-	int m_aPrevGameTick[NUM_DUMMIES];
-	int m_aCurGameTick[NUM_DUMMIES];
-	float m_aGameIntraTick[NUM_DUMMIES];
-	float m_aGameTickTime[NUM_DUMMIES];
-	float m_aGameIntraTickSincePrev[NUM_DUMMIES];
+	int m_aPrevGameTick[NUM_DUMMIES] = {0, 0};
+	int m_aCurGameTick[NUM_DUMMIES] = {0, 0};
+	float m_aGameIntraTick[NUM_DUMMIES] = {0.0f, 0.0f};
+	float m_aGameTickTime[NUM_DUMMIES] = {0.0f, 0.0f};
+	float m_aGameIntraTickSincePrev[NUM_DUMMIES] = {0.0f, 0.0f};
 
-	int m_aPredTick[NUM_DUMMIES];
-	float m_aPredIntraTick[NUM_DUMMIES];
+	int m_aPredTick[NUM_DUMMIES] = {0, 0};
+	float m_aPredIntraTick[NUM_DUMMIES] = {0.0f, 0.0f};
 
-	float m_LocalTime;
-	float m_RenderFrameTime;
+	float m_LocalTime = 0.0f;
+	float m_GlobalTime = 0.0f;
+	float m_RenderFrameTime = 0.0001f;
+	float m_FrameTimeAvg = 0.0001f;
 
-	int m_GameTickSpeed;
+	TLoadingCallback m_LoadingCallback = nullptr;
 
-	float m_FrameTimeAvg;
-
-	TMapLoadingCallbackFunc m_MapLoadingCBFunc;
+	char m_aNews[3000] = "";
+	int m_Points = -1;
+	int64_t m_ReconnectTime = 0;
 
 public:
-	char m_aNews[3000];
-	char m_aMapDownloadUrl[256];
-	int m_Points;
-	int64_t m_ReconnectTime;
-
 	class CSnapItem
 	{
 	public:
 		int m_Type;
-		int m_ID;
+		int m_Id;
+		const void *m_pData;
 		int m_DataSize;
 	};
 
@@ -131,7 +139,7 @@ public:
 	inline int64_t StateStartTime() const { return m_StateStartTime; }
 	void SetLoadingStateDetail(ELoadingStateDetail LoadingStateDetail) { m_LoadingStateDetail = LoadingStateDetail; }
 
-	void SetMapLoadingCBFunc(TMapLoadingCallbackFunc &&Func) { m_MapLoadingCBFunc = std::move(Func); }
+	void SetLoadingCallback(TLoadingCallback &&Func) { m_LoadingCallback = std::move(Func); }
 
 	// tick time access
 	inline int PrevGameTick(int Conn) const { return m_aPrevGameTick[Conn]; }
@@ -141,11 +149,12 @@ public:
 	inline float PredIntraGameTick(int Conn) const { return m_aPredIntraTick[Conn]; }
 	inline float IntraGameTickSincePrev(int Conn) const { return m_aGameIntraTickSincePrev[Conn]; }
 	inline float GameTickTime(int Conn) const { return m_aGameTickTime[Conn]; }
-	inline int GameTickSpeed() const { return m_GameTickSpeed; }
+	inline int GameTickSpeed() const { return SERVER_TICK_SPEED; }
 
 	// other time access
 	inline float RenderFrameTime() const { return m_RenderFrameTime; }
 	inline float LocalTime() const { return m_LocalTime; }
+	inline float GlobalTime() const { return m_GlobalTime; }
 	inline float FrameTimeAvg() const { return m_FrameTimeAvg; }
 
 	// actions
@@ -155,19 +164,20 @@ public:
 	// dummy
 	virtual void DummyDisconnect(const char *pReason) = 0;
 	virtual void DummyConnect() = 0;
-	virtual bool DummyConnected() = 0;
-	virtual bool DummyConnecting() = 0;
-	virtual bool DummyAllowed() = 0;
+	virtual bool DummyConnected() const = 0;
+	virtual bool DummyConnecting() const = 0;
+	virtual bool DummyConnectingDelayed() const = 0;
+	virtual bool DummyAllowed() const = 0;
 
 	virtual void Restart() = 0;
 	virtual void Quit() = 0;
 	virtual const char *DemoPlayer_Play(const char *pFilename, int StorageType) = 0;
 #if defined(CONF_VIDEORECORDER)
-	virtual const char *DemoPlayer_Render(const char *pFilename, int StorageType, const char *pVideoName, int SpeedIndex) = 0;
+	virtual const char *DemoPlayer_Render(const char *pFilename, int StorageType, const char *pVideoName, int SpeedIndex, bool StartPaused = false) = 0;
 #endif
-	virtual void DemoRecorder_Start(const char *pFilename, bool WithTimestamp, int Recorder) = 0;
+	virtual void DemoRecorder_Start(const char *pFilename, bool WithTimestamp, int Recorder, bool Verbose = false) = 0;
 	virtual void DemoRecorder_HandleAutoStart() = 0;
-	virtual void DemoRecorder_Stop(int Recorder, bool RemoveFile = false) = 0;
+	virtual void DemoRecorder_UpdateReplayRecorder() = 0;
 	virtual class IDemoRecorder *DemoRecorder(int Recorder) = 0;
 	virtual void AutoScreenshot_Start() = 0;
 	virtual void AutoStatScreenshot_Start() = 0;
@@ -176,10 +186,10 @@ public:
 
 	// gfx
 	virtual void SwitchWindowScreen(int Index) = 0;
-	virtual void SetWindowParams(int FullscreenMode, bool IsBorderless, bool AllowResizing) = 0;
+	virtual void SetWindowParams(int FullscreenMode, bool IsBorderless) = 0;
 	virtual void ToggleWindowVSync() = 0;
-	virtual void LoadFont() = 0;
 	virtual void Notify(const char *pTitle, const char *pMessage) = 0;
+	virtual void OnWindowResize() = 0;
 
 	virtual void UpdateAndSwap() = 0;
 
@@ -198,15 +208,22 @@ public:
 	virtual int *GetInput(int Tick, int IsDummy = 0) const = 0;
 
 	// remote console
-	virtual void RconAuth(const char *pUsername, const char *pPassword) = 0;
+	virtual void RconAuth(const char *pUsername, const char *pPassword, bool Dummy) = 0;
 	virtual bool RconAuthed() const = 0;
 	virtual bool UseTempRconCommands() const = 0;
 	virtual void Rcon(const char *pLine) = 0;
+	virtual bool ReceivingRconCommands() const = 0;
+	virtual float GotRconCommandsPercentage() const = 0;
+	virtual bool ReceivingMaplist() const = 0;
+	virtual float GotMaplistPercentage() const = 0;
+	virtual const std::vector<std::string> &MaplistEntries() const = 0;
 
 	// server info
 	virtual void GetServerInfo(class CServerInfo *pServerInfo) const = 0;
+	virtual bool ServerCapAnyPlayerFlag() const = 0;
 
 	virtual int GetPredictionTime() = 0;
+	virtual int GetPredictionTick() = 0;
 
 	// snapshot interface
 
@@ -218,35 +235,41 @@ public:
 	};
 
 	// TODO: Refactor: should redo this a bit i think, too many virtual calls
-	virtual int SnapNumItems(int SnapID) const = 0;
-	virtual void *SnapFindItem(int SnapID, int Type, int ID) const = 0;
-	virtual void *SnapGetItem(int SnapID, int Index, CSnapItem *pItem) const = 0;
-	virtual int SnapItemSize(int SnapID, int Index) const = 0;
-	virtual void SnapInvalidateItem(int SnapID, int Index) = 0;
+	virtual int SnapNumItems(int SnapId) const = 0;
+	virtual const void *SnapFindItem(int SnapId, int Type, int Id) const = 0;
+	virtual CSnapItem SnapGetItem(int SnapId, int Index) const = 0;
 
 	virtual void SnapSetStaticsize(int ItemType, int Size) = 0;
+	virtual void SnapSetStaticsize7(int ItemType, int Size) = 0;
 
 	virtual int SendMsg(int Conn, CMsgPacker *pMsg, int Flags) = 0;
 	virtual int SendMsgActive(CMsgPacker *pMsg, int Flags) = 0;
 
 	template<class T>
-	int SendPackMsgActive(T *pMsg, int Flags)
+	int SendPackMsgActive(T *pMsg, int Flags, bool NoTranslate = false)
 	{
-		CMsgPacker Packer(pMsg->MsgID(), false);
+		CMsgPacker Packer(T::ms_MsgId, false, NoTranslate);
 		if(pMsg->Pack(&Packer))
 			return -1;
 		return SendMsgActive(&Packer, Flags);
 	}
 
+	template<class T>
+	int SendPackMsg(int Conn, T *pMsg, int Flags, bool NoTranslate = false)
+	{
+		CMsgPacker Packer(T::ms_MsgId, false, NoTranslate);
+		if(pMsg->Pack(&Packer))
+			return -1;
+		return SendMsg(Conn, &Packer, Flags);
+	}
+
 	//
 	virtual const char *PlayerName() const = 0;
-	virtual const char *DummyName() const = 0;
+	virtual const char *DummyName() = 0;
 	virtual const char *ErrorString() const = 0;
 	virtual const char *LatestVersion() const = 0;
 	virtual bool ConnectionProblems() const = 0;
 
-	virtual bool SoundInitFailed() const = 0;
-
 	virtual IGraphics::CTextureHandle GetDebugFont() const = 0; // TODO: remove this function
 
 	//DDRace
@@ -256,7 +279,12 @@ public:
 	virtual SHA256_DIGEST GetCurrentMapSha256() const = 0;
 	virtual unsigned GetCurrentMapCrc() const = 0;
 
-	virtual int GetCurrentRaceTime() = 0;
+	const char *News() const { return m_aNews; }
+	int Points() const { return m_Points; }
+	int64_t ReconnectTime() const { return m_ReconnectTime; }
+	void SetReconnectTime(int64_t ReconnectTime) { m_ReconnectTime = ReconnectTime; }
+
+	virtual bool IsSixup() const = 0;
 
 	virtual void RaceRecord_Start(const char *pFilename) = 0;
 	virtual void RaceRecord_Stop() = 0;
@@ -275,15 +303,51 @@ public:
 
 	virtual void GetSmoothTick(int *pSmoothTick, float *pSmoothIntraTick, float MixAmount) = 0;
 
-	virtual SWarning *GetCurWarning() = 0;
+	virtual void AddWarning(const SWarning &Warning) = 0;
+	virtual std::optional<SWarning> CurrentWarning() = 0;
+
 	virtual CChecksumData *ChecksumData() = 0;
-	virtual bool InfoTaskRunning() = 0;
 	virtual int UdpConnectivity(int NetType) = 0;
+
+	/**
+	 * Opens a link in the browser.
+	 *
+	 * @param pLink The link to open in a browser.
+	 *
+	 * @return `true` on success, `false` on failure.
+	 *
+	 * @remark This may not be called with untrusted input or it'll result in arbitrary code execution, especially on Windows.
+	 */
+	virtual bool ViewLink(const char *pLink) = 0;
+	/**
+	 * Opens a file or directory with the default program.
+	 *
+	 * @param pFilename The file or folder to open with the default program.
+	 *
+	 * @return `true` on success, `false` on failure.
+	 *
+	 * @remark This may not be called with untrusted input or it'll result in arbitrary code execution, especially on Windows.
+	 */
+	virtual bool ViewFile(const char *pFilename) = 0;
+
+#if defined(CONF_FAMILY_WINDOWS)
+	virtual void ShellRegister() = 0;
+	virtual void ShellUnregister() = 0;
+#endif
+
+	enum EMessageBoxType
+	{
+		MESSAGE_BOX_TYPE_ERROR,
+		MESSAGE_BOX_TYPE_WARNING,
+		MESSAGE_BOX_TYPE_INFO,
+	};
+	virtual void ShowMessageBox(const char *pTitle, const char *pMessage, EMessageBoxType Type = MESSAGE_BOX_TYPE_ERROR) = 0;
+	virtual void GetGpuInfoString(char (&aGpuInfo)[256]) = 0;
 };
 
 class IGameClient : public IInterface
 {
-	MACRO_INTERFACE("gameclient", 0)
+	MACRO_INTERFACE("gameclient")
 protected:
 public:
 	virtual void OnConsoleInit() = 0;
@@ -299,31 +363,42 @@ public:
 	virtual void OnUpdate() = 0;
 	virtual void OnStateChange(int NewState, int OldState) = 0;
 	virtual void OnConnected() = 0;
-	virtual void OnMessage(int MsgID, CUnpacker *pUnpacker, int Conn, bool Dummy) = 0;
+	virtual void OnMessage(int MsgId, CUnpacker *pUnpacker, int Conn, bool Dummy) = 0;
 	virtual void OnPredict() = 0;
 	virtual void OnActivateEditor() = 0;
+	virtual void OnWindowResize() = 0;
 
 	virtual int OnSnapInput(int *pData, bool Dummy, bool Force) = 0;
 	virtual void OnDummySwap() = 0;
 	virtual void SendDummyInfo(bool Start) = 0;
-	virtual int GetLastRaceTick() = 0;
 
 	virtual const char *GetItemName(int Type) const = 0;
 	virtual const char *Version() const = 0;
 	virtual const char *NetVersion() const = 0;
+	virtual const char *NetVersion7() const = 0;
 	virtual int DDNetVersion() const = 0;
 	virtual const char *DDNetVersionStr() const = 0;
 
 	virtual void OnDummyDisconnect() = 0;
 	virtual void DummyResetInput() = 0;
 	virtual void Echo(const char *pString) = 0;
-	virtual bool CanDisplayWarning() = 0;
-	virtual bool IsDisplayingWarning() = 0;
+
+	virtual bool CanDisplayWarning() const = 0;
+	virtual void RenderShutdownMessage() = 0;
 
 	virtual CNetObjHandler *GetNetObjHandler() = 0;
+	virtual protocol7::CNetObjHandler *GetNetObjHandler7() = 0;
+
+	virtual int ClientVersion7() const = 0;
+
+	virtual void ApplySkin7InfoFromSnapObj(const protocol7::CNetObj_De_ClientInfo *pObj, int ClientId) = 0;
+	virtual int OnDemoRecSnap7(class CSnapshot *pFrom, class CSnapshot *pTo, int Conn) = 0;
+	virtual int TranslateSnap(class CSnapshot *pSnapDstSix, class CSnapshot *pSnapSrcSeven, int Conn, bool Dummy) = 0;
+
+	virtual void InitializeLanguage() = 0;
 };
 
-void SnapshotRemoveExtraProjectileInfo(unsigned char *pData);
+void SnapshotRemoveExtraProjectileInfo(class CSnapshot *pSnap);
 
 extern IGameClient *CreateGameClient();
 #endif
