@@ -23,10 +23,10 @@ bool CGLSL::LoadShader(CGLSLCompiler *pCompiler, IStorage *pStorage, const char
 {
 	if(m_IsLoaded)
 		return true;
-	IOHANDLE f = pStorage->OpenFile(pFile, IOFLAG_READ | IOFLAG_SKIP_BOM, IStorage::TYPE_ALL);
 
+	CLineReader LineReader;
 	std::vector<std::string> vLines;
-	if(f)
+	if(LineReader.OpenFile(pStorage->OpenFile(pFile, IOFLAG_READ, IStorage::TYPE_ALL)))
 	{
 		EBackendType BackendType = pCompiler->m_IsOpenGLES ? BACKEND_TYPE_OPENGL_ES : BACKEND_TYPE_OPENGL;
 		bool IsNewOpenGL = (BackendType == BACKEND_TYPE_OPENGL ? (pCompiler->m_OpenGLVersionMajor >= 4 || (pCompiler->m_OpenGLVersionMajor == 3 && pCompiler->m_OpenGLVersionMinor == 3)) : pCompiler->m_OpenGLVersionMajor >= 3);
@@ -71,7 +71,7 @@ bool CGLSL::LoadShader(CGLSLCompiler *pCompiler, IStorage *pStorage, const char
 			}
 		}
 
-		for(CGLSLCompiler::SGLSLCompilerDefine &Define : pCompiler->m_vDefines)
+		for(const CGLSLCompiler::SGLSLCompilerDefine &Define : pCompiler->m_vDefines)
 		{
 			vLines.push_back(std::string("#define ") + Define.m_DefineName + std::string(" ") + Define.m_DefineValue + std::string("\r\n"));
 		}
@@ -81,17 +81,13 @@ bool CGLSL::LoadShader(CGLSLCompiler *pCompiler, IStorage *pStorage, const char
 			vLines.emplace_back("#extension GL_EXT_texture_array : enable\r\n");
 		}
 
-		CLineReader LineReader;
-		LineReader.Init(f);
-		char *pReadLine = NULL;
-		while((pReadLine = LineReader.Get()))
+		while(const char *pReadLine = LineReader.Get())
 		{
 			std::string Line;
 			pCompiler->ParseLine(Line, pReadLine, Type == GL_FRAGMENT_SHADER ? GLSL_SHADER_COMPILER_TYPE_FRAGMENT : GLSL_SHADER_COMPILER_TYPE_VERTEX);
 			Line.append("\r\n");
 			vLines.push_back(Line);
 		}
-		io_close(f);
 
 		const char **ShaderCode = new const char *[vLines.size()];
 
@@ -126,7 +122,7 @@ bool CGLSL::LoadShader(CGLSLCompiler *pCompiler, IStorage *pStorage, const char
 		m_Type = Type;
 		m_IsLoaded = true;
 
-		m_ShaderID = shader;
+		m_ShaderId = shader;
 
 		return true;
 	}
@@ -139,17 +135,17 @@ void CGLSL::DeleteShader()
 	if(!IsLoaded())
 		return;
 	m_IsLoaded = false;
-	glDeleteShader(m_ShaderID);
+	glDeleteShader(m_ShaderId);
 }
 
-bool CGLSL::IsLoaded()
+bool CGLSL::IsLoaded() const
 {
 	return m_IsLoaded;
 }
 
-TWGLuint CGLSL::GetShaderID()
+TWGLuint CGLSL::GetShaderId() const
 {
-	return m_ShaderID;
+	return m_ShaderId;
 }
 
 CGLSL::CGLSL()
