@@ -14,7 +14,7 @@
 
 void CGLSLProgram::CreateProgram()
 {
-	m_ProgramID = glCreateProgram();
+	m_ProgramId = glCreateProgram();
 }
 
 void CGLSLProgram::DeleteProgram()
@@ -22,44 +22,44 @@ void CGLSLProgram::DeleteProgram()
 	if(!m_IsLinked)
 		return;
 	m_IsLinked = false;
-	glDeleteProgram(m_ProgramID);
+	glDeleteProgram(m_ProgramId);
 }
 
-bool CGLSLProgram::AddShader(CGLSL *pShader)
+bool CGLSLProgram::AddShader(CGLSL *pShader) const
 {
 	if(pShader->IsLoaded())
 	{
-		glAttachShader(m_ProgramID, pShader->GetShaderID());
+		glAttachShader(m_ProgramId, pShader->GetShaderId());
 		return true;
 	}
 	return false;
 }
 
-void CGLSLProgram::DetachShader(CGLSL *pShader)
+void CGLSLProgram::DetachShader(CGLSL *pShader) const
 {
 	if(pShader->IsLoaded())
 	{
-		DetachShaderByID(pShader->GetShaderID());
+		DetachShaderById(pShader->GetShaderId());
 	}
 }
 
-void CGLSLProgram::DetachShaderByID(TWGLuint ShaderID)
+void CGLSLProgram::DetachShaderById(TWGLuint ShaderId) const
 {
-	glDetachShader(m_ProgramID, ShaderID);
+	glDetachShader(m_ProgramId, ShaderId);
 }
 
 void CGLSLProgram::LinkProgram()
 {
-	glLinkProgram(m_ProgramID);
+	glLinkProgram(m_ProgramId);
 	int LinkStatus;
-	glGetProgramiv(m_ProgramID, GL_LINK_STATUS, &LinkStatus);
+	glGetProgramiv(m_ProgramId, GL_LINK_STATUS, &LinkStatus);
 	m_IsLinked = LinkStatus == GL_TRUE;
 	if(!m_IsLinked)
 	{
 		char aInfoLog[1024];
 		char aFinalMessage[1536];
-		int iLogLength;
-		glGetProgramInfoLog(m_ProgramID, 1024, &iLogLength, aInfoLog);
+		int LogLength;
+		glGetProgramInfoLog(m_ProgramId, 1024, &LogLength, aInfoLog);
 		str_format(aFinalMessage, sizeof(aFinalMessage), "Error! Shader program wasn't linked! The linker returned:\n\n%s", aInfoLog);
 		dbg_msg("glslprogram", "%s", aFinalMessage);
 	}
@@ -68,19 +68,19 @@ void CGLSLProgram::LinkProgram()
 	DetachAllShaders();
 }
 
-void CGLSLProgram::DetachAllShaders()
+void CGLSLProgram::DetachAllShaders() const
 {
 	TWGLuint aShaders[100];
 	GLsizei ReturnedCount = 0;
 	while(true)
 	{
-		glGetAttachedShaders(m_ProgramID, 100, &ReturnedCount, aShaders);
+		glGetAttachedShaders(m_ProgramId, 100, &ReturnedCount, aShaders);
 
 		if(ReturnedCount > 0)
 		{
 			for(GLsizei i = 0; i < ReturnedCount; ++i)
 			{
-				DetachShaderByID(aShaders[i]);
+				DetachShaderById(aShaders[i]);
 			}
 		}
 
@@ -119,20 +119,20 @@ void CGLSLProgram::SetUniform(int Loc, const bool Value)
 	glUniform1i(Loc, (int)Value);
 }
 
-int CGLSLProgram::GetUniformLoc(const char *pName)
+int CGLSLProgram::GetUniformLoc(const char *pName) const
 {
-	return glGetUniformLocation(m_ProgramID, pName);
+	return glGetUniformLocation(m_ProgramId, pName);
 }
 
-void CGLSLProgram::UseProgram()
+void CGLSLProgram::UseProgram() const
 {
 	if(m_IsLinked)
-		glUseProgram(m_ProgramID);
+		glUseProgram(m_ProgramId);
 }
 
-TWGLuint CGLSLProgram::GetProgramID()
+TWGLuint CGLSLProgram::GetProgramId() const
 {
-	return m_ProgramID;
+	return m_ProgramId;
 }
 
 CGLSLProgram::CGLSLProgram()
