@@ -5,11 +5,14 @@
 #endif
 
 #include <SDL.h>
+#include <SDL_messagebox.h>
+#include <SDL_vulkan.h>
 
 #include <base/math.h>
 #include <cstdlib>
 
 #include <engine/shared/config.h>
+#include <engine/shared/localization.h>
 
 #include <base/tl/threading.h>
 
@@ -72,7 +75,8 @@ void CGraphicsBackend_Threaded::ThreadFunc(void *pUser)
 	}
 }
 
-CGraphicsBackend_Threaded::CGraphicsBackend_Threaded()
+CGraphicsBackend_Threaded::CGraphicsBackend_Threaded(TTranslateFunc &&TranslateFunc) :
+	m_TranslateFunc(std::move(TranslateFunc))
 {
 	m_pBuffer = nullptr;
 	m_pProcessor = nullptr;
@@ -97,6 +101,7 @@ void CGraphicsBackend_Threaded::StopProcessor()
 	m_Shutdown = true;
 	{
 		std::unique_lock<std::mutex> Lock(m_BufferSwapMutex);
+		m_Warning = m_pProcessor->GetWarning();
 		m_BufferSwapCond.notify_all();
 	}
 	thread_wait(m_pThread);
@@ -104,16 +109,33 @@ void CGraphicsBackend_Threaded::StopProcessor()
 
 void CGraphicsBackend_Threaded::RunBuffer(CCommandBuffer *pBuffer)
 {
+	SGfxErrorContainer Error;
 #ifdef CONF_WEBASM
 	// run everything single threaded for now, context binding in a thread seems to not work as of now
-	RunBufferSingleThreadedUnsafe(pBuffer);
+	Error = m_pProcessor->GetError();
+	if(Error.m_ErrorType == GFX_ERROR_TYPE_NONE)
+	{
+		RunBufferSingleThreadedUnsafe(pBuffer);
+	}
 #else
 	WaitForIdle();
-	std::unique_lock<std::mutex> Lock(m_BufferSwapMutex);
-	m_pBuffer = pBuffer;
-	m_BufferInProcess.store(true, std::memory_order_relaxed);
-	m_BufferSwapCond.notify_all();
+	{
+		std::unique_lock<std::mutex> Lock(m_BufferSwapMutex);
+		Error = m_pProcessor->GetError();
+		if(Error.m_ErrorType == GFX_ERROR_TYPE_NONE)
+		{
+			m_pBuffer = pBuffer;
+			m_BufferInProcess.store(true, std::memory_order_relaxed);
+			m_BufferSwapCond.notify_all();
+		}
+	}
 #endif
+
+	// Process error after lock is released to prevent deadlock
+	if(Error.m_ErrorType != GFX_ERROR_TYPE_NONE)
+	{
+		ProcessError(Error);
+	}
 }
 
 void CGraphicsBackend_Threaded::RunBufferSingleThreadedUnsafe(CCommandBuffer *pBuffer)
@@ -132,6 +154,31 @@ void CGraphicsBackend_Threaded::WaitForIdle()
 	m_BufferSwapCond.wait(Lock, [this]() { return m_pBuffer == nullptr; });
 }
 
+void CGraphicsBackend_Threaded::ProcessError(const SGfxErrorContainer &Error)
+{
+	std::string VerboseStr = "Graphics Assertion:";
+	for(const auto &ErrStr : Error.m_vErrors)
+	{
+		VerboseStr.append("\n");
+		if(ErrStr.m_RequiresTranslation)
+			VerboseStr.append(m_TranslateFunc(ErrStr.m_Err.c_str(), ""));
+		else
+			VerboseStr.append(ErrStr.m_Err);
+	}
+	dbg_assert(false, VerboseStr.c_str());
+}
+
+bool CGraphicsBackend_Threaded::GetWarning(std::vector<std::string> &WarningStrings)
+{
+	if(m_Warning.m_WarningType != GFX_WARNING_TYPE_NONE)
+	{
+		m_Warning.m_WarningType = GFX_WARNING_TYPE_NONE;
+		WarningStrings = m_Warning.m_vWarnings;
+		return true;
+	}
+	return false;
+}
+
 // ------------ CCommandProcessorFragment_General
 
 void CCommandProcessorFragment_General::Cmd_Signal(const CCommandBuffer::SCommand_Signal *pCommand)
@@ -163,7 +210,7 @@ void CCommandProcessorFragment_SDL::Cmd_Init(const SCommand_Init *pCommand)
 void CCommandProcessorFragment_SDL::Cmd_Shutdown(const SCommand_Shutdown *pCommand)
 {
 	if(m_GLContext)
-		SDL_GL_MakeCurrent(NULL, NULL);
+		SDL_GL_MakeCurrent(nullptr, nullptr);
 }
 
 void CCommandProcessorFragment_SDL::Cmd_Swap(const CCommandBuffer::SCommand_Swap *pCommand)
@@ -180,7 +227,7 @@ void CCommandProcessorFragment_SDL::Cmd_VSync(const CCommandBuffer::SCommand_VSy
 
 void CCommandProcessorFragment_SDL::Cmd_WindowCreateNtf(const CCommandBuffer::SCommand_WindowCreateNtf *pCommand)
 {
-	m_pWindow = SDL_GetWindowFromID(pCommand->m_WindowID);
+	m_pWindow = SDL_GetWindowFromID(pCommand->m_WindowId);
 	// Android destroys windows when they are not visible, so we get the new one and work with that
 	// The graphic context does not need to be recreated, just unbound see @see SCommand_WindowDestroyNtf
 #ifdef CONF_PLATFORM_ANDROID
@@ -223,14 +270,85 @@ bool CCommandProcessorFragment_SDL::RunCommand(const CCommandBuffer::SCommand *p
 
 // ------------ CCommandProcessor_SDL_GL
 
+void CCommandProcessor_SDL_GL::HandleError()
+{
+	switch(m_Error.m_ErrorType)
+	{
+	case GFX_ERROR_TYPE_INIT:
+		m_Error.m_vErrors.emplace_back(SGfxErrorContainer::SError{true, Localizable("Failed during initialization. Try to change gfx_backend to OpenGL or Vulkan in settings_ddnet.cfg in the config directory and try again.", "Graphics error")});
+		break;
+	case GFX_ERROR_TYPE_OUT_OF_MEMORY_IMAGE:
+		[[fallthrough]];
+	case GFX_ERROR_TYPE_OUT_OF_MEMORY_BUFFER:
+		[[fallthrough]];
+	case GFX_ERROR_TYPE_OUT_OF_MEMORY_STAGING:
+		m_Error.m_vErrors.emplace_back(SGfxErrorContainer::SError{true, Localizable("Out of VRAM. Try removing custom assets (skins, entities, etc.), especially those with high resolution.", "Graphics error")});
+		break;
+	case GFX_ERROR_TYPE_RENDER_RECORDING:
+		m_Error.m_vErrors.emplace_back(SGfxErrorContainer::SError{true, Localizable("An error during command recording occurred. Try to update your GPU drivers.", "Graphics error")});
+		break;
+	case GFX_ERROR_TYPE_RENDER_CMD_FAILED:
+		m_Error.m_vErrors.emplace_back(SGfxErrorContainer::SError{true, Localizable("A render command failed. Try to update your GPU drivers.", "Graphics error")});
+		break;
+	case GFX_ERROR_TYPE_RENDER_SUBMIT_FAILED:
+		m_Error.m_vErrors.emplace_back(SGfxErrorContainer::SError{true, Localizable("Submitting the render commands failed. Try to update your GPU drivers.", "Graphics error")});
+		break;
+	case GFX_ERROR_TYPE_SWAP_FAILED:
+		m_Error.m_vErrors.emplace_back(SGfxErrorContainer::SError{true, Localizable("Failed to swap framebuffers. Try to update your GPU drivers.", "Graphics error")});
+		break;
+	case GFX_ERROR_TYPE_UNKNOWN:
+		[[fallthrough]];
+	default:
+		m_Error.m_vErrors.emplace_back(SGfxErrorContainer::SError{true, Localizable("Unknown error. Try to change gfx_backend to OpenGL or Vulkan in settings_ddnet.cfg in the config directory and try again.", "Graphics error")});
+		break;
+	}
+}
+
+void CCommandProcessor_SDL_GL::HandleWarning()
+{
+	switch(m_Warning.m_WarningType)
+	{
+	case GFX_WARNING_TYPE_INIT_FAILED:
+		m_Warning.m_vWarnings.emplace_back(Localizable("Could not initialize the given graphics backend, reverting to the default backend now.", "Graphics error"));
+		break;
+	case GFX_WARNING_TYPE_INIT_FAILED_MISSING_INTEGRATED_GPU_DRIVER:
+		m_Warning.m_vWarnings.emplace_back(Localizable("Could not initialize the given graphics backend, this is probably because you didn't install the driver of the integrated graphics card.", "Graphics error"));
+		break;
+	case GFX_WARNING_MISSING_EXTENSION:
+		// ignore this warning for now
+		return;
+	case GFX_WARNING_LOW_ON_MEMORY:
+		// ignore this warning for now
+		return;
+	default:
+		dbg_msg("gfx", "unhandled warning %d", (int)m_Warning.m_WarningType);
+		break;
+	}
+}
+
 void CCommandProcessor_SDL_GL::RunBuffer(CCommandBuffer *pBuffer)
 {
 	m_pGLBackend->StartCommands(pBuffer->m_CommandCount, pBuffer->m_RenderCallCount);
 
 	for(CCommandBuffer::SCommand *pCommand = pBuffer->Head(); pCommand; pCommand = pCommand->m_pNext)
 	{
-		if(m_pGLBackend->RunCommand(pCommand))
+		auto Res = m_pGLBackend->RunCommand(pCommand);
+		if(Res == ERunCommandReturnTypes::RUN_COMMAND_COMMAND_HANDLED)
+		{
 			continue;
+		}
+		else if(Res == ERunCommandReturnTypes::RUN_COMMAND_COMMAND_ERROR)
+		{
+			m_Error = m_pGLBackend->GetError();
+			HandleError();
+			return;
+		}
+		else if(Res == ERunCommandReturnTypes::RUN_COMMAND_COMMAND_WARNING)
+		{
+			m_Warning = m_pGLBackend->GetWarning();
+			HandleWarning();
+			return;
+		}
 
 		if(m_SDL.RunCommand(pCommand))
 			continue;
@@ -299,6 +417,21 @@ CCommandProcessor_SDL_GL::~CCommandProcessor_SDL_GL()
 	delete m_pGLBackend;
 }
 
+const SGfxErrorContainer &CCommandProcessor_SDL_GL::GetError() const
+{
+	return m_Error;
+}
+
+void CCommandProcessor_SDL_GL::ErroneousCleanup()
+{
+	m_pGLBackend->ErroneousCleanup();
+}
+
+const SGfxWarningContainer &CCommandProcessor_SDL_GL::GetWarning() const
+{
+	return m_Warning;
+}
+
 // ------------ CGraphicsBackend_SDL_GL
 
 static bool BackendInitGlew(EBackendType BackendType, int &GlewMajor, int &GlewMinor, int &GlewPatch)
@@ -311,7 +444,9 @@ static bool BackendInitGlew(EBackendType BackendType, int &GlewMajor, int &GlewM
 #ifdef CONF_GLEW_HAS_CONTEXT_INIT
 		if(GLEW_OK != glewContextInit())
 #else
-		if(GLEW_OK != glewInit())
+		GLenum InitResult = glewInit();
+		const char *pVideoDriver = SDL_GetCurrentVideoDriver();
+		if(GLEW_OK != InitResult && pVideoDriver && !str_comp(pVideoDriver, "wayland") && GLEW_ERROR_NO_GLX_DISPLAY != InitResult)
 #endif
 			return false;
 
@@ -333,7 +468,7 @@ static bool BackendInitGlew(EBackendType BackendType, int &GlewMajor, int &GlewM
 			return true;
 		}
 #endif
-// Don't allow GL 3.3, if the driver doesn't support atleast OpenGL 4.5
+// Don't allow GL 3.3, if the driver doesn't support at least OpenGL 4.5
 #ifndef CONF_FAMILY_WINDOWS
 		if(GLEW_VERSION_4_4)
 		{
@@ -622,7 +757,7 @@ void CGraphicsBackend_SDL_GL::ClampDriverVersion(EBackendType BackendType)
 			g_Config.m_GfxGLMinor = 0;
 			g_Config.m_GfxGLPatch = 0;
 
-			// GLES also doesnt know GL_QUAD
+			// GLES also doesn't know GL_QUAD
 			g_Config.m_GfxQuadAsTriangle = 1;
 		}
 #else
@@ -641,6 +776,23 @@ void CGraphicsBackend_SDL_GL::ClampDriverVersion(EBackendType BackendType)
 	}
 }
 
+bool CGraphicsBackend_SDL_GL::ShowMessageBox(unsigned Type, const char *pTitle, const char *pMsg)
+{
+	if(m_pProcessor != nullptr)
+	{
+		m_pProcessor->ErroneousCleanup();
+	}
+	// TODO: Remove this workaround when https://github.com/libsdl-org/SDL/issues/3750 is
+	// fixed and pass the window to SDL_ShowSimpleMessageBox to make the popup modal instead
+	// of destroying the window before opening the popup.
+	if(m_pWindow != nullptr)
+	{
+		SDL_DestroyWindow(m_pWindow);
+		m_pWindow = nullptr;
+	}
+	return SDL_ShowSimpleMessageBox(Type, pTitle, pMsg, nullptr) == 0;
+}
+
 bool CGraphicsBackend_SDL_GL::IsModernAPI(EBackendType BackendType)
 {
 	if(BackendType == BACKEND_TYPE_OPENGL)
@@ -732,7 +884,13 @@ bool CGraphicsBackend_SDL_GL::GetDriverVersion(EGraphicsDriverAgeType DriverAgeT
 	return false;
 }
 
-static void DisplayToVideoMode(CVideoMode *pVMode, SDL_DisplayMode *pMode, int HiDPIScale, int RefreshRate)
+const char *CGraphicsBackend_SDL_GL::GetScreenName(int Screen) const
+{
+	const char *pName = SDL_GetDisplayName(Screen);
+	return pName == nullptr ? "unknown/error" : pName;
+}
+
+static void DisplayToVideoMode(CVideoMode *pVMode, SDL_DisplayMode *pMode, float HiDPIScale, int RefreshRate)
 {
 	pVMode->m_CanvasWidth = pMode->w * HiDPIScale;
 	pVMode->m_CanvasHeight = pMode->h * HiDPIScale;
@@ -745,18 +903,17 @@ static void DisplayToVideoMode(CVideoMode *pVMode, SDL_DisplayMode *pMode, int H
 	pVMode->m_Format = pMode->format;
 }
 
-void CGraphicsBackend_SDL_GL::GetVideoModes(CVideoMode *pModes, int MaxModes, int *pNumModes, int HiDPIScale, int MaxWindowWidth, int MaxWindowHeight, int ScreenID)
+void CGraphicsBackend_SDL_GL::GetVideoModes(CVideoMode *pModes, int MaxModes, int *pNumModes, float HiDPIScale, int MaxWindowWidth, int MaxWindowHeight, int ScreenId)
 {
 	SDL_DisplayMode DesktopMode;
-	int maxModes = SDL_GetNumDisplayModes(ScreenID);
-	int numModes = 0;
+	int MaxModesAvailable = SDL_GetNumDisplayModes(ScreenId);
 
 	// Only collect fullscreen modes when requested, that makes sure in windowed mode no refresh rates are shown that aren't supported without
 	// fullscreen anyway(except fullscreen desktop)
-	bool IsFullscreenDestkop = m_pWindow != NULL && (((SDL_GetWindowFlags(m_pWindow) & SDL_WINDOW_FULLSCREEN_DESKTOP) == SDL_WINDOW_FULLSCREEN_DESKTOP) || g_Config.m_GfxFullscreen == 3);
-	bool CollectFullscreenModes = m_pWindow == NULL || ((SDL_GetWindowFlags(m_pWindow) & SDL_WINDOW_FULLSCREEN) != 0 && !IsFullscreenDestkop);
+	bool IsFullscreenDestkop = m_pWindow != nullptr && (((SDL_GetWindowFlags(m_pWindow) & SDL_WINDOW_FULLSCREEN_DESKTOP) == SDL_WINDOW_FULLSCREEN_DESKTOP) || g_Config.m_GfxFullscreen == 3);
+	bool CollectFullscreenModes = m_pWindow == nullptr || ((SDL_GetWindowFlags(m_pWindow) & SDL_WINDOW_FULLSCREEN) != 0 && !IsFullscreenDestkop);
 
-	if(SDL_GetDesktopDisplayMode(ScreenID, &DesktopMode) < 0)
+	if(SDL_GetDesktopDisplayMode(ScreenId, &DesktopMode) < 0)
 	{
 		dbg_msg("gfx", "unable to get display mode: %s", SDL_GetError());
 	}
@@ -764,65 +921,66 @@ void CGraphicsBackend_SDL_GL::GetVideoModes(CVideoMode *pModes, int MaxModes, in
 	constexpr int ModeCount = 256;
 	SDL_DisplayMode aModes[ModeCount];
 	int NumModes = 0;
-	for(int i = 0; i < maxModes && NumModes < ModeCount; i++)
+	for(int i = 0; i < MaxModesAvailable && NumModes < ModeCount; i++)
 	{
-		SDL_DisplayMode mode;
-		if(SDL_GetDisplayMode(ScreenID, i, &mode) < 0)
+		SDL_DisplayMode Mode;
+		if(SDL_GetDisplayMode(ScreenId, i, &Mode) < 0)
 		{
 			dbg_msg("gfx", "unable to get display mode: %s", SDL_GetError());
 			continue;
 		}
 
-		aModes[NumModes] = mode;
+		aModes[NumModes] = Mode;
 		++NumModes;
 	}
 
-	auto &&ModeInsert = [&](SDL_DisplayMode &mode) {
-		if(numModes < MaxModes)
+	int NumModesInserted = 0;
+	auto &&ModeInsert = [&](SDL_DisplayMode &Mode) {
+		if(NumModesInserted < MaxModes)
 		{
 			// if last mode was equal, ignore this one --- in fullscreen this can really only happen if the screen
 			// supports different color modes
 			// in non fullscren these are the modes that show different refresh rate, but are basically the same
-			if(numModes > 0 && pModes[numModes - 1].m_WindowWidth == mode.w && pModes[numModes - 1].m_WindowHeight == mode.h && (pModes[numModes - 1].m_RefreshRate == mode.refresh_rate || (mode.refresh_rate != DesktopMode.refresh_rate && !CollectFullscreenModes)))
+			if(NumModesInserted > 0 && pModes[NumModesInserted - 1].m_WindowWidth == Mode.w && pModes[NumModesInserted - 1].m_WindowHeight == Mode.h && (pModes[NumModesInserted - 1].m_RefreshRate == Mode.refresh_rate || (Mode.refresh_rate != DesktopMode.refresh_rate && !CollectFullscreenModes)))
 				return;
 
-			DisplayToVideoMode(&pModes[numModes], &mode, HiDPIScale, !CollectFullscreenModes ? DesktopMode.refresh_rate : mode.refresh_rate);
-			numModes++;
+			DisplayToVideoMode(&pModes[NumModesInserted], &Mode, HiDPIScale, !CollectFullscreenModes ? DesktopMode.refresh_rate : Mode.refresh_rate);
+			NumModesInserted++;
 		}
 	};
 
 	for(int i = 0; i < NumModes; i++)
 	{
-		SDL_DisplayMode &mode = aModes[i];
+		SDL_DisplayMode &Mode = aModes[i];
 
-		if(mode.w > MaxWindowWidth || mode.h > MaxWindowHeight)
+		if(Mode.w > MaxWindowWidth || Mode.h > MaxWindowHeight)
 			continue;
 
-		ModeInsert(mode);
+		ModeInsert(Mode);
 
 		if(IsFullscreenDestkop)
 			break;
 
-		if(numModes >= MaxModes)
+		if(NumModesInserted >= MaxModes)
 			break;
 	}
-	*pNumModes = numModes;
+	*pNumModes = NumModesInserted;
 }
 
-void CGraphicsBackend_SDL_GL::GetCurrentVideoMode(CVideoMode &CurMode, int HiDPIScale, int MaxWindowWidth, int MaxWindowHeight, int ScreenID)
+void CGraphicsBackend_SDL_GL::GetCurrentVideoMode(CVideoMode &CurMode, float HiDPIScale, int MaxWindowWidth, int MaxWindowHeight, int ScreenId)
 {
-	SDL_DisplayMode DPMode;
+	SDL_DisplayMode DpMode;
 	// if "real" fullscreen, obtain the video mode for that
 	if((SDL_GetWindowFlags(m_pWindow) & SDL_WINDOW_FULLSCREEN_DESKTOP) == SDL_WINDOW_FULLSCREEN)
 	{
-		if(SDL_GetCurrentDisplayMode(ScreenID, &DPMode))
+		if(SDL_GetCurrentDisplayMode(ScreenId, &DpMode))
 		{
 			dbg_msg("gfx", "unable to get display mode: %s", SDL_GetError());
 		}
 	}
 	else
 	{
-		if(SDL_GetDesktopDisplayMode(ScreenID, &DPMode) < 0)
+		if(SDL_GetDesktopDisplayMode(ScreenId, &DpMode) < 0)
 		{
 			dbg_msg("gfx", "unable to get display mode: %s", SDL_GetError());
 		}
@@ -830,15 +988,20 @@ void CGraphicsBackend_SDL_GL::GetCurrentVideoMode(CVideoMode &CurMode, int HiDPI
 		{
 			int Width = 0;
 			int Height = 0;
-			SDL_GL_GetDrawableSize(m_pWindow, &Width, &Height);
-			DPMode.w = Width;
-			DPMode.h = Height;
+			if(m_BackendType != EBackendType::BACKEND_TYPE_VULKAN)
+				SDL_GL_GetDrawableSize(m_pWindow, &Width, &Height);
+			else
+				SDL_Vulkan_GetDrawableSize(m_pWindow, &Width, &Height);
+			// SDL video modes are in screen space which are logical pixels
+			DpMode.w = Width / HiDPIScale;
+			DpMode.h = Height / HiDPIScale;
 		}
 	}
-	DisplayToVideoMode(&CurMode, &DPMode, HiDPIScale, DPMode.refresh_rate);
+	DisplayToVideoMode(&CurMode, &DpMode, HiDPIScale, DpMode.refresh_rate);
 }
 
-CGraphicsBackend_SDL_GL::CGraphicsBackend_SDL_GL()
+CGraphicsBackend_SDL_GL::CGraphicsBackend_SDL_GL(TTranslateFunc &&TranslateFunc) :
+	CGraphicsBackend_Threaded(std::move(TranslateFunc))
 {
 	mem_zero(m_aErrorString, std::size(m_aErrorString));
 }
@@ -863,6 +1026,13 @@ int CGraphicsBackend_SDL_GL::Init(const char *pName, int *pScreen, int *pWidth,
 		SDL_GetVersion(&Linked);
 		dbg_msg("sdl", "SDL version %d.%d.%d (compiled = %d.%d.%d)", Linked.major, Linked.minor, Linked.patch,
 			Compiled.major, Compiled.minor, Compiled.patch);
+
+#if CONF_PLATFORM_LINUX && SDL_VERSION_ATLEAST(2, 0, 22)
+		// needed to workaround SDL from forcing exclusively X11 if linking against the GLX flavour of GLEW instead of the EGL one
+		// w/o this on Wayland systems (no XWayland support) SDL's Video subsystem will fail to load (starting from SDL2.30+)
+		if(Linked.major == 2 && Linked.minor >= 30)
+			SDL_SetHint(SDL_HINT_VIDEODRIVER, "x11,wayland");
+#endif
 	}
 
 	if(!SDL_WasInit(SDL_INIT_VIDEO))
@@ -980,10 +1150,8 @@ int CGraphicsBackend_SDL_GL::Init(const char *pName, int *pScreen, int *pWidth,
 	}
 
 	// set flags
-	int SdlFlags = SDL_WINDOW_INPUT_GRABBED | SDL_WINDOW_INPUT_FOCUS | SDL_WINDOW_MOUSE_FOCUS;
+	int SdlFlags = SDL_WINDOW_INPUT_GRABBED | SDL_WINDOW_INPUT_FOCUS | SDL_WINDOW_MOUSE_FOCUS | SDL_WINDOW_ALLOW_HIGHDPI;
 	SdlFlags |= (IsOpenGLFamilyBackend) ? SDL_WINDOW_OPENGL : SDL_WINDOW_VULKAN;
-	if(Flags & IGraphicsBackend::INITFLAG_HIGHDPI)
-		SdlFlags |= SDL_WINDOW_ALLOW_HIGHDPI;
 	if(Flags & IGraphicsBackend::INITFLAG_RESIZABLE)
 		SdlFlags |= SDL_WINDOW_RESIZABLE;
 	if(Flags & IGraphicsBackend::INITFLAG_BORDERLESS)
@@ -1029,9 +1197,6 @@ int CGraphicsBackend_SDL_GL::Init(const char *pName, int *pScreen, int *pWidth,
 		}
 	}
 
-	if(g_Config.m_InpMouseOld)
-		SDL_SetHint(SDL_HINT_MOUSE_RELATIVE_MODE_WARP, "1");
-
 	m_pWindow = SDL_CreateWindow(
 		pName,
 		SDL_WINDOWPOS_CENTERED_DISPLAY(*pScreen),
@@ -1041,7 +1206,7 @@ int CGraphicsBackend_SDL_GL::Init(const char *pName, int *pScreen, int *pWidth,
 		SdlFlags);
 
 	// set caption
-	if(m_pWindow == NULL)
+	if(m_pWindow == nullptr)
 	{
 		dbg_msg("gfx", "unable to create window: %s", SDL_GetError());
 		if(m_BackendType == BACKEND_TYPE_VULKAN)
@@ -1058,9 +1223,10 @@ int CGraphicsBackend_SDL_GL::Init(const char *pName, int *pScreen, int *pWidth,
 	{
 		m_GLContext = SDL_GL_CreateContext(m_pWindow);
 
-		if(m_GLContext == NULL)
+		if(m_GLContext == nullptr)
 		{
 			SDL_DestroyWindow(m_pWindow);
+			m_pWindow = nullptr;
 			dbg_msg("gfx", "unable to create graphic context: %s", SDL_GetError());
 			return EGraphicsBackendErrorCodes::GRAPHICS_BACKEND_ERROR_CODE_GL_CONTEXT_FAILED;
 		}
@@ -1069,25 +1235,32 @@ int CGraphicsBackend_SDL_GL::Init(const char *pName, int *pScreen, int *pWidth,
 		{
 			SDL_GL_DeleteContext(m_GLContext);
 			SDL_DestroyWindow(m_pWindow);
+			m_pWindow = nullptr;
 			return EGraphicsBackendErrorCodes::GRAPHICS_BACKEND_ERROR_CODE_UNKNOWN;
 		}
 	}
 
 	int InitError = 0;
-	const char *pErrorStr = NULL;
+	const char *pErrorStr = nullptr;
 
 	InitError = IsVersionSupportedGlew(m_BackendType, g_Config.m_GfxGLMajor, g_Config.m_GfxGLMinor, g_Config.m_GfxGLPatch, GlewMajor, GlewMinor, GlewPatch);
 
 	// SDL_GL_GetDrawableSize reports HiDPI resolution even with SDL_WINDOW_ALLOW_HIGHDPI not set, which is wrong
-	if(SdlFlags & SDL_WINDOW_ALLOW_HIGHDPI && IsOpenGLFamilyBackend)
-		SDL_GL_GetDrawableSize(m_pWindow, pCurrentWidth, pCurrentHeight);
+	if(SdlFlags & SDL_WINDOW_ALLOW_HIGHDPI)
+	{
+		if(IsOpenGLFamilyBackend)
+			SDL_GL_GetDrawableSize(m_pWindow, pCurrentWidth, pCurrentHeight);
+		else
+			SDL_Vulkan_GetDrawableSize(m_pWindow, pCurrentWidth, pCurrentHeight);
+	}
 	else
 		SDL_GetWindowSize(m_pWindow, pCurrentWidth, pCurrentHeight);
+	SDL_GetWindowSize(m_pWindow, pWidth, pHeight);
 
 	if(IsOpenGLFamilyBackend)
 	{
 		SDL_GL_SetSwapInterval(Flags & IGraphicsBackend::INITFLAG_VSYNC ? 1 : 0);
-		SDL_GL_MakeCurrent(NULL, NULL);
+		SDL_GL_MakeCurrent(nullptr, nullptr);
 	}
 
 	if(InitError != 0)
@@ -1095,6 +1268,7 @@ int CGraphicsBackend_SDL_GL::Init(const char *pName, int *pScreen, int *pWidth,
 		if(m_GLContext)
 			SDL_GL_DeleteContext(m_GLContext);
 		SDL_DestroyWindow(m_pWindow);
+		m_pWindow = nullptr;
 
 		// try setting to glew supported version
 		g_Config.m_GfxGLMajor = GlewMajor;
@@ -1119,7 +1293,7 @@ int CGraphicsBackend_SDL_GL::Init(const char *pName, int *pScreen, int *pWidth,
 	CmdPre.m_pVendorString = m_aVendorString;
 	CmdPre.m_pVersionString = m_aVersionString;
 	CmdPre.m_pRendererString = m_aRendererString;
-	CmdPre.m_pGPUList = &m_GPUList;
+	CmdPre.m_pGpuList = &m_GpuList;
 	CmdBuffer.AddCommandUnsafe(CmdPre);
 	RunBufferSingleThreadedUnsafe(&CmdBuffer);
 	CmdBuffer.Reset();
@@ -1143,7 +1317,7 @@ int CGraphicsBackend_SDL_GL::Init(const char *pName, int *pScreen, int *pWidth,
 		CmdGL.m_pBufferMemoryUsage = &m_BufferMemoryUsage;
 		CmdGL.m_pStreamMemoryUsage = &m_StreamMemoryUsage;
 		CmdGL.m_pStagingMemoryUsage = &m_StagingMemoryUsage;
-		CmdGL.m_pGPUList = &m_GPUList;
+		CmdGL.m_pGpuList = &m_GpuList;
 		CmdGL.m_pReadPresentedImageDataFunc = &m_ReadPresentedImageDataFunc;
 		CmdGL.m_pStorage = pStorage;
 		CmdGL.m_pCapabilities = &m_Capabilites;
@@ -1197,6 +1371,7 @@ int CGraphicsBackend_SDL_GL::Init(const char *pName, int *pScreen, int *pWidth,
 		if(m_GLContext)
 			SDL_GL_DeleteContext(m_GLContext);
 		SDL_DestroyWindow(m_pWindow);
+		m_pWindow = nullptr;
 
 		// try setting to version string's supported version
 		if(InitError == -2)
@@ -1206,7 +1381,7 @@ int CGraphicsBackend_SDL_GL::Init(const char *pName, int *pScreen, int *pWidth,
 			g_Config.m_GfxGLPatch = m_Capabilites.m_ContextPatch;
 		}
 
-		if(pErrorStr != NULL)
+		if(pErrorStr != nullptr)
 		{
 			str_copy(m_aErrorString, pErrorStr);
 		}
@@ -1258,8 +1433,10 @@ int CGraphicsBackend_SDL_GL::Shutdown()
 	delete m_pProcessor;
 	m_pProcessor = nullptr;
 
-	SDL_GL_DeleteContext(m_GLContext);
+	if(m_GLContext != nullptr)
+		SDL_GL_DeleteContext(m_GLContext);
 	SDL_DestroyWindow(m_pWindow);
+	m_pWindow = nullptr;
 
 	SDL_QuitSubSystem(SDL_INIT_VIDEO);
 	return 0;
@@ -1285,9 +1462,9 @@ uint64_t CGraphicsBackend_SDL_GL::StagingMemoryUsage() const
 	return m_StagingMemoryUsage;
 }
 
-const TTWGraphicsGPUList &CGraphicsBackend_SDL_GL::GetGPUs() const
+const TTwGraphicsGpuList &CGraphicsBackend_SDL_GL::GetGpus() const
 {
-	return m_GPUList;
+	return m_GpuList;
 }
 
 void CGraphicsBackend_SDL_GL::Minimize()
@@ -1300,7 +1477,7 @@ void CGraphicsBackend_SDL_GL::Maximize()
 	// TODO: SDL
 }
 
-void CGraphicsBackend_SDL_GL::SetWindowParams(int FullscreenMode, bool IsBorderless, bool AllowResizing)
+void CGraphicsBackend_SDL_GL::SetWindowParams(int FullscreenMode, bool IsBorderless)
 {
 	if(FullscreenMode > 0)
 	{
@@ -1329,14 +1506,14 @@ void CGraphicsBackend_SDL_GL::SetWindowParams(int FullscreenMode, bool IsBorderl
 			SDL_SetWindowFullscreen(m_pWindow, 0);
 			SDL_SetWindowBordered(m_pWindow, SDL_TRUE);
 			SDL_SetWindowResizable(m_pWindow, SDL_FALSE);
-			SDL_DisplayMode DPMode;
-			if(SDL_GetDesktopDisplayMode(g_Config.m_GfxScreen, &DPMode) < 0)
+			SDL_DisplayMode DpMode;
+			if(SDL_GetDesktopDisplayMode(g_Config.m_GfxScreen, &DpMode) < 0)
 			{
 				dbg_msg("gfx", "unable to get display mode: %s", SDL_GetError());
 			}
 			else
 			{
-				ResizeWindow(DPMode.w, DPMode.h, DPMode.refresh_rate);
+				ResizeWindow(DpMode.w, DpMode.h, DpMode.refresh_rate);
 				SDL_SetWindowPosition(m_pWindow, SDL_WINDOWPOS_CENTERED_DISPLAY(g_Config.m_GfxScreen), SDL_WINDOWPOS_CENTERED_DISPLAY(g_Config.m_GfxScreen));
 			}
 		}
@@ -1361,6 +1538,8 @@ bool CGraphicsBackend_SDL_GL::SetWindowScreen(int Index)
 	{
 		return false;
 	}
+	// Todo SDL: remove this when fixed (changing screen when in fullscreen is bugged)
+	SDL_SetWindowBordered(m_pWindow, SDL_TRUE); //fixing primary monitor goes black when switch screen (borderless OpenGL)
 
 	SDL_SetWindowPosition(m_pWindow,
 		SDL_WINDOWPOS_CENTERED_DISPLAY(Index),
@@ -1378,6 +1557,7 @@ bool CGraphicsBackend_SDL_GL::UpdateDisplayMode(int Index)
 		return false;
 	}
 
+	g_Config.m_GfxScreen = Index;
 	g_Config.m_GfxDesktopWidth = DisplayMode.w;
 	g_Config.m_GfxDesktopHeight = DisplayMode.h;
 
@@ -1415,7 +1595,7 @@ bool CGraphicsBackend_SDL_GL::ResizeWindow(int w, int h, int RefreshRate)
 	{
 #ifdef CONF_FAMILY_WINDOWS
 		// in windows make the window windowed mode first, this prevents strange window glitches (other games probably do something similar)
-		SetWindowParams(0, true, true);
+		SetWindowParams(0, true);
 #endif
 		SDL_DisplayMode SetMode = {};
 		SDL_DisplayMode ClosestMode = {};
@@ -1425,9 +1605,9 @@ bool CGraphicsBackend_SDL_GL::ResizeWindow(int w, int h, int RefreshRate)
 		SetMode.refresh_rate = RefreshRate;
 		SDL_SetWindowDisplayMode(m_pWindow, SDL_GetClosestDisplayMode(g_Config.m_GfxScreen, &SetMode, &ClosestMode));
 #ifdef CONF_FAMILY_WINDOWS
-		// now change it back to fullscreen, this will restore the above set state, bcs SDL saves fullscreen modes appart from other video modes (as of SDL 2.0.16)
+		// now change it back to fullscreen, this will restore the above set state, bcs SDL saves fullscreen modes apart from other video modes (as of SDL 2.0.16)
 		// see implementation of SDL_SetWindowDisplayMode
-		SetWindowParams(1, false, true);
+		SetWindowParams(1, false);
 #endif
 		return true;
 	}
@@ -1444,12 +1624,16 @@ bool CGraphicsBackend_SDL_GL::ResizeWindow(int w, int h, int RefreshRate)
 
 void CGraphicsBackend_SDL_GL::GetViewportSize(int &w, int &h)
 {
-	SDL_GL_GetDrawableSize(m_pWindow, &w, &h);
+	if(m_BackendType != EBackendType::BACKEND_TYPE_VULKAN)
+		SDL_GL_GetDrawableSize(m_pWindow, &w, &h);
+	else
+		SDL_Vulkan_GetDrawableSize(m_pWindow, &w, &h);
 }
 
 void CGraphicsBackend_SDL_GL::NotifyWindow()
 {
-#if SDL_MAJOR_VERSION > 2 || (SDL_MAJOR_VERSION == 2 && SDL_PATCHLEVEL >= 16)
+	// Minimum version 2.0.16, after version 2.0.22 the naming is changed to 2.24.0 etc.
+#if SDL_MAJOR_VERSION > 2 || (SDL_MAJOR_VERSION == 2 && SDL_MINOR_VERSION == 0 && SDL_PATCHLEVEL >= 16) || (SDL_MAJOR_VERSION == 2 && SDL_MINOR_VERSION > 0)
 	if(SDL_FlashWindow(m_pWindow, SDL_FlashOperation::SDL_FLASH_UNTIL_FOCUSED) != 0)
 	{
 		// fails if SDL hasn't implemented it
@@ -1458,13 +1642,13 @@ void CGraphicsBackend_SDL_GL::NotifyWindow()
 #endif
 }
 
-void CGraphicsBackend_SDL_GL::WindowDestroyNtf(uint32_t WindowID)
+void CGraphicsBackend_SDL_GL::WindowDestroyNtf(uint32_t WindowId)
 {
 }
 
-void CGraphicsBackend_SDL_GL::WindowCreateNtf(uint32_t WindowID)
+void CGraphicsBackend_SDL_GL::WindowCreateNtf(uint32_t WindowId)
 {
-	m_pWindow = SDL_GetWindowFromID(WindowID);
+	m_pWindow = SDL_GetWindowFromID(WindowId);
 }
 
 TGLBackendReadPresentedImageData &CGraphicsBackend_SDL_GL::GetReadPresentedImageDataFuncUnsafe()
@@ -1472,4 +1656,4 @@ TGLBackendReadPresentedImageData &CGraphicsBackend_SDL_GL::GetReadPresentedImage
 	return m_ReadPresentedImageDataFunc;
 }
 
-IGraphicsBackend *CreateGraphicsBackend() { return new CGraphicsBackend_SDL_GL; }
+IGraphicsBackend *CreateGraphicsBackend(TTranslateFunc &&TranslateFunc) { return new CGraphicsBackend_SDL_GL(std::move(TranslateFunc)); }
