@@ -3,10 +3,12 @@
 #ifndef ENGINE_CLIENT_CLIENT_H
 #define ENGINE_CLIENT_CLIENT_H
 
-#include <list>
+#include <deque>
 #include <memory>
+#include <mutex>
 
 #include <base/hash.h>
+
 #include <engine/client.h>
 #include <engine/client/checksum.h>
 #include <engine/client/friends.h>
@@ -14,113 +16,69 @@
 #include <engine/client/serverbrowser.h>
 #include <engine/client/updater.h>
 #include <engine/editor.h>
-#include <engine/engine.h>
 #include <engine/graphics.h>
 #include <engine/shared/config.h>
 #include <engine/shared/demo.h>
 #include <engine/shared/fifo.h>
 #include <engine/shared/http.h>
 #include <engine/shared/network.h>
+#include <engine/textrender.h>
 #include <engine/warning.h>
 
+#include "graph.h"
+#include "smooth_time.h"
+
 class CDemoEdit;
 class IDemoRecorder;
 class CMsgPacker;
 class CUnpacker;
 class IConfigManager;
 class IDiscord;
+class IEngine;
 class IEngineInput;
 class IEngineMap;
 class IEngineSound;
 class IFriends;
+class ILogger;
 class ISteam;
+class INotifications;
 class IStorage;
 class IUpdater;
 
-#define CONNECTLINK "ddnet:"
-
-class CGraph
-{
-public:
-	enum
-	{
-		MAX_VALUES = 128,
-	};
-
-private:
-	float m_Min, m_Max;
-	float m_MinRange, m_MaxRange;
-	float m_aValues[MAX_VALUES];
-	float m_aColors[MAX_VALUES][3];
-	size_t m_Index;
-
-public:
-	void Init(float Min, float Max);
-	void SetMin(float Min);
-	void SetMax(float Max);
-
-	void Scale();
-	void Add(float v, float r, float g, float b);
-	void InsertAt(size_t Index, float v, float r, float g, float b);
-	void Render(IGraphics *pGraphics, IGraphics::CTextureHandle FontTexture, float x, float y, float w, float h, const char *pDescription);
-};
-
-class CSmoothTime
-{
-	int64_t m_Snap;
-	int64_t m_Current;
-	int64_t m_Target;
-
-	int64_t m_SnapMargin;
-	int64_t m_CurrentMargin;
-	int64_t m_TargetMargin;
-
-	CGraph m_Graph;
-
-	int m_SpikeCounter;
-
-	float m_aAdjustSpeed[2]; // 0 = down, 1 = up
-public:
-	void Init(int64_t Target);
-	void SetAdjustSpeed(int Direction, float Value);
-
-	int64_t Get(int64_t Now);
-
-	void UpdateInt(int64_t Target);
-	void Update(CGraph *pGraph, int64_t Target, int TimeLeft, int AdjustDirection);
-
-	int64_t GetMargin(int64_t Now);
-	void UpdateMargin(int64_t TargetMargin);
-};
+#define CONNECTLINK_DOUBLE_SLASH "ddnet://"
+#define CONNECTLINK_NO_SLASH "ddnet:"
 
 class CServerCapabilities
 {
 public:
-	bool m_ChatTimeoutCode;
-	bool m_AnyPlayerFlag;
-	bool m_PingEx;
-	bool m_AllowDummy;
-	bool m_SyncWeaponInput;
+	bool m_ChatTimeoutCode = false;
+	bool m_AnyPlayerFlag = false;
+	bool m_PingEx = false;
+	bool m_AllowDummy = false;
+	bool m_SyncWeaponInput = false;
 };
 
 class CClient : public IClient, public CDemoPlayer::IListener
 {
 	// needed interfaces
-	IEngine *m_pEngine;
-	IEditor *m_pEditor;
-	IEngineInput *m_pInput;
-	IEngineGraphics *m_pGraphics;
-	IEngineSound *m_pSound;
-	IFavorites *m_pFavorites;
-	IGameClient *m_pGameClient;
-	IEngineMap *m_pMap;
-	IConfigManager *m_pConfigManager;
-	CConfig *m_pConfig;
-	IConsole *m_pConsole;
-	IStorage *m_pStorage;
-	IUpdater *m_pUpdater;
-	IDiscord *m_pDiscord;
-	ISteam *m_pSteam;
+	IConfigManager *m_pConfigManager = nullptr;
+	CConfig *m_pConfig = nullptr;
+	IConsole *m_pConsole = nullptr;
+	IDiscord *m_pDiscord = nullptr;
+	IEditor *m_pEditor = nullptr;
+	IEngine *m_pEngine = nullptr;
+	IFavorites *m_pFavorites = nullptr;
+	IGameClient *m_pGameClient = nullptr;
+	IEngineGraphics *m_pGraphics = nullptr;
+	IEngineInput *m_pInput = nullptr;
+	IEngineMap *m_pMap = nullptr;
+	IEngineSound *m_pSound = nullptr;
+	ISteam *m_pSteam = nullptr;
+	INotifications *m_pNotifications = nullptr;
+	IStorage *m_pStorage = nullptr;
+	IEngineTextRender *m_pTextRender = nullptr;
+	IUpdater *m_pUpdater = nullptr;
+	CHttp m_Http;
 
 	CNetClient m_aNetClient[NUM_CONNS];
 	CDemoPlayer m_DemoPlayer;
@@ -133,80 +91,80 @@ class CClient : public IClient, public CDemoPlayer::IListener
 	CFriends m_Friends;
 	CFriends m_Foes;
 
-	char m_aConnectAddressStr[MAX_SERVER_ADDRESSES * NETADDR_MAXSTRSIZE];
+	char m_aConnectAddressStr[MAX_SERVER_ADDRESSES * NETADDR_MAXSTRSIZE] = "";
 
-	CUuid m_ConnectionID;
+	CUuid m_ConnectionId = UUID_ZEROED;
+	bool m_Sixup;
 
 	bool m_HaveGlobalTcpAddr = false;
-	NETADDR m_GlobalTcpAddr;
+	NETADDR m_GlobalTcpAddr = NETADDR_ZEROED;
 
-	uint64_t m_aSnapshotParts[NUM_DUMMIES];
-	int64_t m_LocalStartTime;
+	uint64_t m_aSnapshotParts[NUM_DUMMIES] = {0, 0};
+	int64_t m_LocalStartTime = 0;
+	int64_t m_GlobalStartTime = 0;
 
 	IGraphics::CTextureHandle m_DebugFont;
-	int m_DebugSoundIndex = 0;
 
 	int64_t m_LastRenderTime;
-	float m_RenderFrameTimeLow;
-	float m_RenderFrameTimeHigh;
-	int m_RenderFrames;
 
-	int m_SnapCrcErrors;
-	bool m_AutoScreenshotRecycle;
-	bool m_AutoStatScreenshotRecycle;
-	bool m_AutoCSVRecycle;
-	bool m_EditorActive;
-	bool m_SoundInitFailed;
-	bool m_ResortServerBrowser;
+	int m_SnapCrcErrors = 0;
+	bool m_AutoScreenshotRecycle = false;
+	bool m_AutoStatScreenshotRecycle = false;
+	bool m_AutoCSVRecycle = false;
+	bool m_EditorActive = false;
 
-	int m_aAckGameTick[NUM_DUMMIES];
-	int m_aCurrentRecvTick[NUM_DUMMIES];
-	int m_aRconAuthed[NUM_DUMMIES];
-	char m_aRconPassword[32];
-	int m_UseTempRconCommands;
-	char m_aPassword[32];
-	bool m_SendPassword;
-	bool m_ButtonRender = false;
+	int m_aAckGameTick[NUM_DUMMIES] = {-1, -1};
+	int m_aCurrentRecvTick[NUM_DUMMIES] = {0, 0};
+	int m_aRconAuthed[NUM_DUMMIES] = {0, 0};
+	char m_aRconUsername[64] = "";
+	char m_aRconPassword[sizeof(g_Config.m_SvRconPassword)] = "";
+	int m_UseTempRconCommands = 0;
+	int m_ExpectedRconCommands = -1;
+	int m_GotRconCommands = 0;
+	char m_aPassword[sizeof(g_Config.m_Password)] = "";
+	bool m_SendPassword = false;
+
+	int m_ExpectedMaplistEntries = -1;
+	std::vector<std::string> m_vMaplistEntries;
 
 	// version-checking
-	char m_aVersionStr[10];
+	char m_aVersionStr[10] = "0";
 
 	// pinging
-	int64_t m_PingStartTime;
+	int64_t m_PingStartTime = 0;
 
-	char m_aCurrentMap[IO_MAX_PATH_LENGTH];
-	char m_aCurrentMapPath[IO_MAX_PATH_LENGTH];
+	char m_aCurrentMap[IO_MAX_PATH_LENGTH] = "";
+	char m_aCurrentMapPath[IO_MAX_PATH_LENGTH] = "";
 
-	char m_aTimeoutCodes[NUM_DUMMIES][32];
-	bool m_aCodeRunAfterJoin[NUM_DUMMIES];
-	bool m_GenerateTimeoutSeed;
+	char m_aTimeoutCodes[NUM_DUMMIES][32] = {"", ""};
+	bool m_aCodeRunAfterJoin[NUM_DUMMIES] = {false, false};
+	bool m_GenerateTimeoutSeed = true;
 
-	//
-	char m_aCmdConnect[256];
-	char m_aCmdPlayDemo[IO_MAX_PATH_LENGTH];
-	char m_aCmdEditMap[IO_MAX_PATH_LENGTH];
+	char m_aCmdConnect[256] = "";
+	char m_aCmdPlayDemo[IO_MAX_PATH_LENGTH] = "";
+	char m_aCmdEditMap[IO_MAX_PATH_LENGTH] = "";
 
 	// map download
-	std::shared_ptr<CHttpRequest> m_pMapdownloadTask;
-	char m_aMapdownloadFilename[256];
-	char m_aMapdownloadFilenameTemp[256];
-	char m_aMapdownloadName[256];
-	IOHANDLE m_MapdownloadFileTemp;
-	int m_MapdownloadChunk;
-	int m_MapdownloadCrc;
-	int m_MapdownloadAmount;
-	int m_MapdownloadTotalsize;
-	bool m_MapdownloadSha256Present;
-	SHA256_DIGEST m_MapdownloadSha256;
+	char m_aMapDownloadUrl[256] = "";
+	std::shared_ptr<CHttpRequest> m_pMapdownloadTask = nullptr;
+	char m_aMapdownloadFilename[256] = "";
+	char m_aMapdownloadFilenameTemp[256] = "";
+	char m_aMapdownloadName[256] = "";
+	IOHANDLE m_MapdownloadFileTemp = nullptr;
+	int m_MapdownloadChunk = 0;
+	int m_MapdownloadCrc = 0;
+	int m_MapdownloadAmount = -1;
+	int m_MapdownloadTotalsize = -1;
+	bool m_MapdownloadSha256Present = false;
+	SHA256_DIGEST m_MapdownloadSha256 = SHA256_ZEROED;
 
-	bool m_MapDetailsPresent;
-	char m_aMapDetailsName[256];
-	int m_MapDetailsCrc;
-	SHA256_DIGEST m_MapDetailsSha256;
-	char m_aMapDetailsUrl[256];
+	bool m_MapDetailsPresent = false;
+	char m_aMapDetailsName[256] = "";
+	int m_MapDetailsCrc = 0;
+	SHA256_DIGEST m_MapDetailsSha256 = SHA256_ZEROED;
+	char m_aMapDetailsUrl[256] = "";
 
-	char m_aDDNetInfoTmp[64];
-	std::shared_ptr<CHttpRequest> m_pDDNetInfoTask;
+	std::shared_ptr<CHttpRequest> m_pDDNetInfoTask = nullptr;
 
 	// time
 	CSmoothTime m_aGameTime[NUM_DUMMIES];
@@ -222,46 +180,51 @@ class CClient : public IClient, public CDemoPlayer::IListener
 		int64_t m_Time;
 	} m_aInputs[NUM_DUMMIES][200];
 
-	int m_aCurrentInput[NUM_DUMMIES];
-	bool m_LastDummy;
-	bool m_DummySendConnInfo;
+	int m_aCurrentInput[NUM_DUMMIES] = {0, 0};
+	bool m_LastDummy = false;
+	bool m_DummySendConnInfo = false;
+	bool m_DummyConnecting = false;
+	bool m_DummyConnected = false;
+	float m_LastDummyConnectTime = 0.0f;
+	bool m_DummyReconnectOnReload = false;
+	bool m_DummyDeactivateOnReconnect = false;
 
 	// graphs
 	CGraph m_InputtimeMarginGraph;
-	CGraph m_GametimeMarginGraph;
+	CGraph m_aGametimeMarginGraphs[NUM_DUMMIES];
 	CGraph m_FpsGraph;
 
 	// the game snapshots are modifiable by the game
 	CSnapshotStorage m_aSnapshotStorage[NUM_DUMMIES];
 	CSnapshotStorage::CHolder *m_aapSnapshots[NUM_DUMMIES][NUM_SNAPSHOT_TYPES];
 
-	int m_aReceivedSnapshots[NUM_DUMMIES];
+	int m_aReceivedSnapshots[NUM_DUMMIES] = {0, 0};
 	char m_aaSnapshotIncomingData[NUM_DUMMIES][CSnapshot::MAX_SIZE];
-	int m_aSnapshotIncomingDataSize[NUM_DUMMIES];
+	int m_aSnapshotIncomingDataSize[NUM_DUMMIES] = {0, 0};
 
 	CSnapshotStorage::CHolder m_aDemorecSnapshotHolders[NUM_SNAPSHOT_TYPES];
-	char *m_aaapDemorecSnapshotData[NUM_SNAPSHOT_TYPES][2][CSnapshot::MAX_SIZE];
+	char m_aaaDemorecSnapshotData[NUM_SNAPSHOT_TYPES][2][CSnapshot::MAX_SIZE];
 
 	CSnapshotDelta m_SnapshotDelta;
 
-	std::list<std::shared_ptr<CDemoEdit>> m_lpEditJobs;
+	std::deque<std::shared_ptr<CDemoEdit>> m_EditJobs;
 
 	//
-	bool m_CanReceiveServerCapabilities;
-	bool m_ServerSentCapabilities;
+	bool m_CanReceiveServerCapabilities = false;
+	bool m_ServerSentCapabilities = false;
 	CServerCapabilities m_ServerCapabilities;
 
-	bool ShouldSendChatTimeoutCodeHeuristic();
+	bool ServerCapAnyPlayerFlag() const override { return m_ServerCapabilities.m_AnyPlayerFlag; }
 
 	CServerInfo m_CurrentServerInfo;
-	int64_t m_CurrentServerInfoRequestTime; // >= 0 should request, == -1 got info
+	int64_t m_CurrentServerInfoRequestTime = -1; // >= 0 should request, == -1 got info
 
-	int m_CurrentServerPingInfoType;
-	int m_CurrentServerPingBasicToken;
-	int m_CurrentServerPingToken;
-	CUuid m_CurrentServerPingUuid;
-	int64_t m_CurrentServerCurrentPingTime; // >= 0 request running
-	int64_t m_CurrentServerNextPingTime; // >= 0 should request
+	int m_CurrentServerPingInfoType = -1;
+	int m_CurrentServerPingBasicToken = -1;
+	int m_CurrentServerPingToken = -1;
+	CUuid m_CurrentServerPingUuid = UUID_ZEROED;
+	int64_t m_CurrentServerCurrentPingTime = -1; // >= 0 request running
+	int64_t m_CurrentServerNextPingTime = -1; // >= 0 should request
 
 	// version info
 	struct CVersionInfo
@@ -273,25 +236,21 @@ class CClient : public IClient, public CDemoPlayer::IListener
 			STATE_READY,
 		};
 
-		int m_State;
-		class CHostLookup m_VersionServeraddr;
+		int m_State = STATE_INIT;
 	} m_VersionInfo;
 
-	static void GraphicsThreadProxy(void *pThis) { ((CClient *)pThis)->GraphicsThread(); }
-	void GraphicsThread();
-
+	std::mutex m_WarningsMutex;
 	std::vector<SWarning> m_vWarnings;
+	std::vector<SWarning> m_vQuittingWarnings;
 
-#if defined(CONF_FAMILY_UNIX)
 	CFifo m_Fifo;
-#endif
 
-	IOHANDLE m_BenchmarkFile;
-	int64_t m_BenchmarkStopTime;
+	IOHANDLE m_BenchmarkFile = nullptr;
+	int64_t m_BenchmarkStopTime = 0;
 
 	CChecksum m_Checksum;
-	int m_OwnExecutableSize = 0;
-	IOHANDLE m_OwnExecutable;
+	int64_t m_OwnExecutableSize = 0;
+	IOHANDLE m_OwnExecutable = nullptr;
 
 	// favorite command handling
 	bool m_FavoritesGroup = false;
@@ -303,18 +262,27 @@ class CClient : public IClient, public CDemoPlayer::IListener
 	int MaxLatencyTicks() const;
 	int PredictionMargin() const;
 
+	std::shared_ptr<ILogger> m_pFileLogger = nullptr;
+	std::shared_ptr<ILogger> m_pStdoutLogger = nullptr;
+
+	// For DummyName function
+	char m_aAutomaticDummyName[MAX_NAME_LENGTH];
+
 public:
+	IConfigManager *ConfigManager() { return m_pConfigManager; }
+	CConfig *Config() { return m_pConfig; }
+	IDiscord *Discord() { return m_pDiscord; }
 	IEngine *Engine() { return m_pEngine; }
+	IGameClient *GameClient() { return m_pGameClient; }
 	IEngineGraphics *Graphics() { return m_pGraphics; }
 	IEngineInput *Input() { return m_pInput; }
 	IEngineSound *Sound() { return m_pSound; }
-	IGameClient *GameClient() { return m_pGameClient; }
-	IConfigManager *ConfigManager() { return m_pConfigManager; }
-	CConfig *Config() { return m_pConfig; }
-	IStorage *Storage() { return m_pStorage; }
-	IUpdater *Updater() { return m_pUpdater; }
-	IDiscord *Discord() { return m_pDiscord; }
 	ISteam *Steam() { return m_pSteam; }
+	INotifications *Notifications() { return m_pNotifications; }
+	IStorage *Storage() { return m_pStorage; }
+	IEngineTextRender *TextRender() { return m_pTextRender; }
+	IUpdater *Updater() { return m_pUpdater; }
+	IHttp *Http() { return &m_Http; }
 
 	CClient();
 
@@ -323,23 +291,25 @@ public:
 	// Send via the currently active client (main/dummy)
 	int SendMsgActive(CMsgPacker *pMsg, int Flags) override;
 
-	void SendInfo();
+	void SendInfo(int Conn);
 	void SendEnterGame(int Conn);
-	void SendReady();
+	void SendReady(int Conn);
 	void SendMapRequest();
 
 	bool RconAuthed() const override { return m_aRconAuthed[g_Config.m_ClDummy] != 0; }
 	bool UseTempRconCommands() const override { return m_UseTempRconCommands != 0; }
-	void RconAuth(const char *pName, const char *pPassword) override;
+	void RconAuth(const char *pName, const char *pPassword, bool Dummy = g_Config.m_ClDummy) override;
 	void Rcon(const char *pCmd) override;
+	bool ReceivingRconCommands() const override { return m_ExpectedRconCommands > 0; }
+	float GotRconCommandsPercentage() const override;
+	bool ReceivingMaplist() const override { return m_ExpectedMaplistEntries > 0; }
+	float GotMaplistPercentage() const override;
+	const std::vector<std::string> &MaplistEntries() const override { return m_vMaplistEntries; }
 
 	bool ConnectionProblems() const override;
 
-	bool SoundInitFailed() const override { return m_SoundInitFailed; }
-
 	IGraphics::CTextureHandle GetDebugFont() const override { return m_DebugFont; }
 
-	void DirectInput(int *pInput, int Size);
 	void SendInput();
 
 	// TODO: OPT: do this a lot smarter!
@@ -348,7 +318,7 @@ public:
 	const char *LatestVersion() const override;
 
 	// ------ state handling -----
-	void SetState(EClientState s);
+	void SetState(EClientState State);
 
 	// called when the map is loaded and we should init for a new round
 	void OnEnterGame(bool Dummy);
@@ -360,55 +330,58 @@ public:
 
 	void DummyDisconnect(const char *pReason) override;
 	void DummyConnect() override;
-	bool DummyConnected() override;
-	bool DummyConnecting() override;
-	bool DummyAllowed() override;
-	int m_DummyConnected;
-	int m_LastDummyConnectTime;
+	bool DummyConnected() const override;
+	bool DummyConnecting() const override;
+	bool DummyConnectingDelayed() const override;
+	bool DummyAllowed() const override;
 
 	void GetServerInfo(CServerInfo *pServerInfo) const override;
 	void ServerInfoRequest();
 
-	int LoadData();
+	void LoadDebugFont();
 
 	// ---
 
 	int GetPredictionTime() override;
-	void *SnapGetItem(int SnapID, int Index, CSnapItem *pItem) const override;
-	int SnapItemSize(int SnapID, int Index) const override;
-	void SnapInvalidateItem(int SnapID, int Index) override;
-	void *SnapFindItem(int SnapID, int Type, int ID) const override;
-	int SnapNumItems(int SnapID) const override;
+	CSnapItem SnapGetItem(int SnapId, int Index) const override;
+	int GetPredictionTick() override;
+	const void *SnapFindItem(int SnapId, int Type, int Id) const override;
+	int SnapNumItems(int SnapId) const override;
 	void SnapSetStaticsize(int ItemType, int Size) override;
+	void SnapSetStaticsize7(int ItemType, int Size) override;
 
 	void Render();
-	void DebugRender();
+	void RenderDebug();
+	void RenderGraphs();
 
 	void Restart() override;
 	void Quit() override;
 
 	const char *PlayerName() const override;
-	const char *DummyName() const override;
+	const char *DummyName() override;
 	const char *ErrorString() const override;
 
 	const char *LoadMap(const char *pName, const char *pFilename, SHA256_DIGEST *pWantedSha256, unsigned WantedCrc);
 	const char *LoadMapSearch(const char *pMapName, SHA256_DIGEST *pWantedSha256, int WantedCrc);
 
+	int TranslateSysMsg(int *pMsgId, bool System, CUnpacker *pUnpacker, CPacker *pPacker, CNetChunk *pPacket, bool *pIsExMsg);
+
+	void PreprocessConnlessPacket7(CNetChunk *pPacket);
 	void ProcessConnlessPacket(CNetChunk *pPacket);
 	void ProcessServerInfo(int Type, NETADDR *pFrom, const void *pData, int DataSize);
 	void ProcessServerPacket(CNetChunk *pPacket, int Conn, bool Dummy);
 
 	int UnpackAndValidateSnapshot(CSnapshot *pFrom, CSnapshot *pTo);
 
-	void ResetMapDownload();
+	void ResetMapDownload(bool ResetActive);
 	void FinishMapDownload();
 
 	void RequestDDNetInfo() override;
-	void ResetDDNetInfo();
-	bool IsDDNetInfoChanged();
-	void FinishDDNetInfo();
+	void ResetDDNetInfoTask();
 	void LoadDDNetInfo();
 
+	bool IsSixup() const override { return m_Sixup; }
+
 	const NETADDR &ServerAddress() const override { return *m_aNetClient[CONN_MAIN].ServerAddress(); }
 	int ConnectNetTypes() const override;
 	const char *ConnectAddressString() const override { return m_aConnectAddressStr; }
@@ -428,6 +401,7 @@ public:
 
 	void Run();
 
+	bool InitNetworkClient(char *pError, size_t ErrorSize);
 	bool CtrlShiftKey(int Key, bool &Last);
 
 	static void Con_Connect(IConsole::IResult *pResult, void *pUserData);
@@ -438,18 +412,18 @@ public:
 	static void Con_DummyResetInput(IConsole::IResult *pResult, void *pUserData);
 
 	static void Con_Quit(IConsole::IResult *pResult, void *pUserData);
+	static void Con_Restart(IConsole::IResult *pResult, void *pUserData);
 	static void Con_DemoPlay(IConsole::IResult *pResult, void *pUserData);
 	static void Con_DemoSpeed(IConsole::IResult *pResult, void *pUserData);
 	static void Con_Minimize(IConsole::IResult *pResult, void *pUserData);
 	static void Con_Ping(IConsole::IResult *pResult, void *pUserData);
 	static void Con_Screenshot(IConsole::IResult *pResult, void *pUserData);
-	static void Con_Reset(IConsole::IResult *pResult, void *pUserData);
 
 #if defined(CONF_VIDEORECORDER)
-	static void StartVideo(IConsole::IResult *pResult, void *pUserData, const char *pVideoName);
+	void StartVideo(const char *pFilename, bool WithTimestamp);
 	static void Con_StartVideo(IConsole::IResult *pResult, void *pUserData);
 	static void Con_StopVideo(IConsole::IResult *pResult, void *pUserData);
-	const char *DemoPlayer_Render(const char *pFilename, int StorageType, const char *pVideoName, int SpeedIndex) override;
+	const char *DemoPlayer_Render(const char *pFilename, int StorageType, const char *pVideoName, int SpeedIndex, bool StartPaused = false) override;
 #endif
 
 	static void Con_Rcon(IConsole::IResult *pResult, void *pUserData);
@@ -469,10 +443,13 @@ public:
 	static void ConchainWindowBordered(IConsole::IResult *pResult, void *pUserData, IConsole::FCommandCallback pfnCallback, void *pCallbackUserData);
 	static void ConchainWindowScreen(IConsole::IResult *pResult, void *pUserData, IConsole::FCommandCallback pfnCallback, void *pCallbackUserData);
 	static void ConchainWindowVSync(IConsole::IResult *pResult, void *pUserData, IConsole::FCommandCallback pfnCallback, void *pCallbackUserData);
+	static void ConchainWindowResize(IConsole::IResult *pResult, void *pUserData, IConsole::FCommandCallback pfnCallback, void *pCallbackUserData);
 	static void ConchainTimeoutSeed(IConsole::IResult *pResult, void *pUserData, IConsole::FCommandCallback pfnCallback, void *pCallbackUserData);
-	static void ConchainLoglevel(IConsole::IResult *pResult, void *pUserData, IConsole::FCommandCallback pfnCallback, void *pCallbackUserData);
 	static void ConchainPassword(IConsole::IResult *pResult, void *pUserData, IConsole::FCommandCallback pfnCallback, void *pCallbackUserData);
 	static void ConchainReplays(IConsole::IResult *pResult, void *pUserData, IConsole::FCommandCallback pfnCallback, void *pCallbackUserData);
+	static void ConchainInputFifo(IConsole::IResult *pResult, void *pUserData, IConsole::FCommandCallback pfnCallback, void *pCallbackUserData);
+	static void ConchainLoglevel(IConsole::IResult *pResult, void *pUserData, IConsole::FCommandCallback pfnCallback, void *pCallbackUserData);
+	static void ConchainStdoutOutputLevel(IConsole::IResult *pResult, void *pUserData, IConsole::FCommandCallback pfnCallback, void *pCallbackUserData);
 
 	static void Con_DemoSlice(IConsole::IResult *pResult, void *pUserData);
 	static void Con_DemoSliceBegin(IConsole::IResult *pResult, void *pUserData);
@@ -482,10 +459,9 @@ public:
 	void RegisterCommands();
 
 	const char *DemoPlayer_Play(const char *pFilename, int StorageType) override;
-	void DemoRecorder_Start(const char *pFilename, bool WithTimestamp, int Recorder) override;
+	void DemoRecorder_Start(const char *pFilename, bool WithTimestamp, int Recorder, bool Verbose = false) override;
 	void DemoRecorder_HandleAutoStart() override;
-	void DemoRecorder_StartReplayRecorder();
-	void DemoRecorder_Stop(int Recorder, bool RemoveFile = false) override;
+	void DemoRecorder_UpdateReplayRecorder() override;
 	void DemoRecorder_AddDemoMarker(int Recorder);
 	IDemoRecorder *DemoRecorder(int Recorder) override;
 
@@ -509,10 +485,10 @@ public:
 
 	// gfx
 	void SwitchWindowScreen(int Index) override;
-	void SetWindowParams(int FullscreenMode, bool IsBorderless, bool AllowResizing) override;
+	void SetWindowParams(int FullscreenMode, bool IsBorderless) override;
 	void ToggleWindowVSync() override;
-	void LoadFont() override;
 	void Notify(const char *pTitle, const char *pMessage) override;
+	void OnWindowResize() override;
 	void BenchmarkQuit(int Seconds, const char *pFilename);
 
 	void UpdateAndSwap() override;
@@ -522,8 +498,6 @@ public:
 	void GenerateTimeoutSeed() override;
 	void GenerateTimeoutCodes(const NETADDR *pAddrs, int NumAddrs);
 
-	int GetCurrentRaceTime() override;
-
 	const char *GetCurrentMap() const override;
 	const char *GetCurrentMapPath() const override;
 	SHA256_DIGEST GetCurrentMapSha256() const override;
@@ -544,10 +518,24 @@ public:
 
 	void GetSmoothTick(int *pSmoothTick, float *pSmoothIntraTick, float MixAmount) override;
 
-	SWarning *GetCurWarning() override;
+	void AddWarning(const SWarning &Warning) override;
+	std::optional<SWarning> CurrentWarning() override;
+	std::vector<SWarning> &&QuittingWarnings() { return std::move(m_vQuittingWarnings); }
+
 	CChecksumData *ChecksumData() override { return &m_Checksum.m_Data; }
-	bool InfoTaskRunning() override { return m_pDDNetInfoTask != nullptr; }
 	int UdpConnectivity(int NetType) override;
+
+	bool ViewLink(const char *pLink) override;
+	bool ViewFile(const char *pFilename) override;
+
+#if defined(CONF_FAMILY_WINDOWS)
+	void ShellRegister() override;
+	void ShellUnregister() override;
+#endif
+
+	void ShowMessageBox(const char *pTitle, const char *pMessage, EMessageBoxType Type = MESSAGE_BOX_TYPE_ERROR) override;
+	void GetGpuInfoString(char (&aGpuInfo)[256]) override;
+	void SetLoggers(std::shared_ptr<ILogger> &&pFileLogger, std::shared_ptr<ILogger> &&pStdoutLogger);
 };
 
 #endif
