@@ -1,10 +1,15 @@
 #ifndef ENGINE_CLIENT_GRAPHICS_THREADED_H
 #define ENGINE_CLIENT_GRAPHICS_THREADED_H
 
+#include <base/system.h>
+
 #include <engine/graphics.h>
 #include <engine/shared/config.h>
 
+#include <atomic>
 #include <cstddef>
+#include <mutex>
+#include <string>
 #include <vector>
 
 constexpr int CMD_BUFFER_DATA_BUFFER_SIZE = 1024 * 1024 * 2;
@@ -29,7 +34,7 @@ class CCommandBuffer
 		~CBuffer()
 		{
 			delete[] m_pData;
-			m_pData = 0x0;
+			m_pData = nullptr;
 			m_Used = 0;
 			m_Size = 0;
 		}
@@ -46,7 +51,7 @@ class CCommandBuffer
 				Offset = Alignment - Offset;
 
 			if(Requested + Offset + m_Used > m_Size)
-				return 0;
+				return nullptr;
 
 			void *pPtr = &m_pData[m_Used + Offset];
 			m_Used += Requested + Offset;
@@ -73,7 +78,7 @@ public:
 
 	enum ECommandBufferCMD
 	{
-		// commadn groups
+		// command groups
 		CMDGROUP_CORE = 0, // commands that everyone has to implement
 		CMDGROUP_PLATFORM_GL = 10000, // commands specific to a platform
 		CMDGROUP_PLATFORM_SDL = 20000,
@@ -91,7 +96,6 @@ public:
 		// texture commands
 		CMD_TEXTURE_CREATE,
 		CMD_TEXTURE_DESTROY,
-		CMD_TEXTURE_UPDATE,
 		CMD_TEXT_TEXTURES_CREATE,
 		CMD_TEXT_TEXTURES_DESTROY,
 		CMD_TEXT_TEXTURE_UPDATE,
@@ -116,7 +120,6 @@ public:
 
 		CMD_RENDER_TILE_LAYER, // render a tilelayer
 		CMD_RENDER_BORDER_TILE, // render one tile multiple times
-		CMD_RENDER_BORDER_TILE_LINE, // render an amount of tiles multiple times
 		CMD_RENDER_QUAD_LAYER, // render a quad layer
 		CMD_RENDER_TEXT, // render text
 		CMD_RENDER_QUAD_CONTAINER, // render a quad buffer container
@@ -125,11 +128,11 @@ public:
 
 		// swap
 		CMD_SWAP,
-		CMD_FINISH,
 
 		// misc
 		CMD_MULTISAMPLING,
 		CMD_VSYNC,
+		CMD_TRY_SWAP_AND_READ_PIXEL,
 		CMD_TRY_SWAP_AND_SCREENSHOT,
 		CMD_UPDATE_VIEWPORT,
 
@@ -148,9 +151,7 @@ public:
 		TEXFLAG_NOMIPMAPS = 1,
 		TEXFLAG_TO_3D_TEXTURE = (1 << 3),
 		TEXFLAG_TO_2D_ARRAY_TEXTURE = (1 << 4),
-		TEXFLAG_TO_3D_TEXTURE_SINGLE_LAYER = (1 << 5),
-		TEXFLAG_TO_2D_ARRAY_TEXTURE_SINGLE_LAYER = (1 << 6),
-		TEXFLAG_NO_2D_TEXTURE = (1 << 7),
+		TEXFLAG_NO_2D_TEXTURE = (1 << 5),
 	};
 
 	enum
@@ -324,7 +325,7 @@ public:
 		int m_Stride;
 		int m_VertBufferBindingIndex;
 
-		int m_AttrCount;
+		size_t m_AttrCount;
 		SBufferContainerInfo::SAttribute *m_pAttributes;
 	};
 
@@ -338,7 +339,7 @@ public:
 		int m_Stride;
 		int m_VertBufferBindingIndex;
 
-		int m_AttrCount;
+		size_t m_AttrCount;
 		SBufferContainerInfo::SAttribute *m_pAttributes;
 	};
 
@@ -364,7 +365,7 @@ public:
 		SCommand_RenderTileLayer() :
 			SCommand(CMD_RENDER_TILE_LAYER) {}
 		SState m_State;
-		SColorf m_Color; // the color of the whole tilelayer -- already envelopped
+		SColorf m_Color; // the color of the whole tilelayer -- already enveloped
 
 		// the char offset of all indices that should be rendered, and the amount of renders
 		char **m_pIndicesOffsets;
@@ -379,29 +380,13 @@ public:
 		SCommand_RenderBorderTile() :
 			SCommand(CMD_RENDER_BORDER_TILE) {}
 		SState m_State;
-		SColorf m_Color; // the color of the whole tilelayer -- already envelopped
-		char *m_pIndicesOffset; // you should use the command buffer data to allocate vertices for this command
-		unsigned int m_DrawNum;
+		SColorf m_Color; // the color of the whole tilelayer -- already enveloped
+		char *m_pIndicesOffset;
+		uint32_t m_DrawNum;
 		int m_BufferContainerIndex;
 
 		vec2 m_Offset;
-		vec2 m_Dir;
-		int m_JumpIndex;
-	};
-
-	struct SCommand_RenderBorderTileLine : public SCommand
-	{
-		SCommand_RenderBorderTileLine() :
-			SCommand(CMD_RENDER_BORDER_TILE_LINE) {}
-		SState m_State;
-		SColorf m_Color; // the color of the whole tilelayer -- already envelopped
-		char *m_pIndicesOffset; // you should use the command buffer data to allocate vertices for this command
-		unsigned int m_IndexDrawNum;
-		unsigned int m_DrawNum;
-		int m_BufferContainerIndex;
-
-		vec2 m_Offset;
-		vec2 m_Dir;
+		vec2 m_Scale;
 	};
 
 	struct SCommand_RenderQuadLayer : public SCommand
@@ -412,7 +397,7 @@ public:
 
 		int m_BufferContainerIndex;
 		SQuadRenderInfo *m_pQuadInfo;
-		int m_QuadNum;
+		size_t m_QuadNum;
 		int m_QuadOffset;
 	};
 
@@ -480,12 +465,21 @@ public:
 		void *m_pOffset;
 	};
 
+	struct SCommand_TrySwapAndReadPixel : public SCommand
+	{
+		SCommand_TrySwapAndReadPixel() :
+			SCommand(CMD_TRY_SWAP_AND_READ_PIXEL) {}
+		ivec2 m_Position;
+		SColorf *m_pColor; // processor will fill this out
+		bool *m_pSwapped; // processor may set this to true, must be initialized to false by the caller
+	};
+
 	struct SCommand_TrySwapAndScreenshot : public SCommand
 	{
 		SCommand_TrySwapAndScreenshot() :
 			SCommand(CMD_TRY_SWAP_AND_SCREENSHOT) {}
 		CImageInfo *m_pImage; // processor will fill this out, the one who adds this command must free the data as well
-		bool *m_pSwapped;
+		bool *m_pSwapped; // processor may set this to true, must be initialized to false by the caller
 	};
 
 	struct SCommand_Swap : public SCommand
@@ -494,12 +488,6 @@ public:
 			SCommand(CMD_SWAP) {}
 	};
 
-	struct SCommand_Finish : public SCommand
-	{
-		SCommand_Finish() :
-			SCommand(CMD_FINISH) {}
-	};
-
 	struct SCommand_VSync : public SCommand
 	{
 		SCommand_VSync() :
@@ -539,29 +527,11 @@ public:
 		// texture information
 		int m_Slot;
 
-		int m_Width;
-		int m_Height;
-		int m_PixelSize;
-		int m_Format;
-		int m_StoreFormat;
+		size_t m_Width;
+		size_t m_Height;
 		int m_Flags;
-		void *m_pData; // will be freed by the command processor
-	};
-
-	struct SCommand_Texture_Update : public SCommand
-	{
-		SCommand_Texture_Update() :
-			SCommand(CMD_TEXTURE_UPDATE) {}
-
-		// texture information
-		int m_Slot;
-
-		int m_X;
-		int m_Y;
-		int m_Width;
-		int m_Height;
-		int m_Format;
-		void *m_pData; // will be freed by the command processor
+		// data must be in RGBA format
+		uint8_t *m_pData; // will be freed by the command processor
 	};
 
 	struct SCommand_Texture_Destroy : public SCommand
@@ -582,11 +552,11 @@ public:
 		int m_Slot;
 		int m_SlotOutline;
 
-		int m_Width;
-		int m_Height;
+		size_t m_Width;
+		size_t m_Height;
 
-		void *m_pTextData;
-		void *m_pTextOutlineData;
+		uint8_t *m_pTextData; // will be freed by the command processor
+		uint8_t *m_pTextOutlineData; // will be freed by the command processor
 	};
 
 	struct SCommand_TextTextures_Destroy : public SCommand
@@ -609,9 +579,9 @@ public:
 
 		int m_X;
 		int m_Y;
-		int m_Width;
-		int m_Height;
-		void *m_pData; // will be freed by the command processor
+		size_t m_Width;
+		size_t m_Height;
+		uint8_t *m_pData; // will be freed by the command processor
 	};
 
 	struct SCommand_WindowCreateNtf : public CCommandBuffer::SCommand
@@ -619,7 +589,7 @@ public:
 		SCommand_WindowCreateNtf() :
 			SCommand(CMD_WINDOW_CREATE_NTF) {}
 
-		uint32_t m_WindowID;
+		uint32_t m_WindowId;
 	};
 
 	struct SCommand_WindowDestroyNtf : public CCommandBuffer::SCommand
@@ -627,7 +597,7 @@ public:
 		SCommand_WindowDestroyNtf() :
 			SCommand(CMD_WINDOW_DESTROY_NTF) {}
 
-		uint32_t m_WindowID;
+		uint32_t m_WindowId;
 	};
 
 	//
@@ -710,11 +680,10 @@ public:
 		INITFLAG_VSYNC = 1 << 1,
 		INITFLAG_RESIZABLE = 1 << 2,
 		INITFLAG_BORDERLESS = 1 << 3,
-		INITFLAG_HIGHDPI = 1 << 4,
-		INITFLAG_DESKTOP_FULLSCREEN = 1 << 5,
+		INITFLAG_DESKTOP_FULLSCREEN = 1 << 4,
 	};
 
-	virtual ~IGraphicsBackend() {}
+	virtual ~IGraphicsBackend() = default;
 
 	virtual int Init(const char *pName, int *pScreen, int *pWidth, int *pHeight, int *pRefreshRate, int *pFsaaSamples, int Flags, int *pDesktopWidth, int *pDesktopHeight, int *pCurrentWidth, int *pCurrentHeight, class IStorage *pStorage) = 0;
 	virtual int Shutdown() = 0;
@@ -724,16 +693,17 @@ public:
 	virtual uint64_t StreamedMemoryUsage() const = 0;
 	virtual uint64_t StagingMemoryUsage() const = 0;
 
-	virtual const TTWGraphicsGPUList &GetGPUs() const = 0;
+	virtual const TTwGraphicsGpuList &GetGpus() const = 0;
 
-	virtual void GetVideoModes(CVideoMode *pModes, int MaxModes, int *pNumModes, int HiDPIScale, int MaxWindowWidth, int MaxWindowHeight, int Screen) = 0;
-	virtual void GetCurrentVideoMode(CVideoMode &CurMode, int HiDPIScale, int MaxWindowWidth, int MaxWindowHeight, int Screen) = 0;
+	virtual void GetVideoModes(CVideoMode *pModes, int MaxModes, int *pNumModes, float HiDPIScale, int MaxWindowWidth, int MaxWindowHeight, int Screen) = 0;
+	virtual void GetCurrentVideoMode(CVideoMode &CurMode, float HiDPIScale, int MaxWindowWidth, int MaxWindowHeight, int Screen) = 0;
 
 	virtual int GetNumScreens() const = 0;
+	virtual const char *GetScreenName(int Screen) const = 0;
 
 	virtual void Minimize() = 0;
 	virtual void Maximize() = 0;
-	virtual void SetWindowParams(int FullscreenMode, bool IsBorderless, bool AllowResizing) = 0;
+	virtual void SetWindowParams(int FullscreenMode, bool IsBorderless) = 0;
 	virtual bool SetWindowScreen(int Index) = 0;
 	virtual bool UpdateDisplayMode(int Index) = 0;
 	virtual int GetWindowScreen() = 0;
@@ -745,8 +715,8 @@ public:
 	virtual void GetViewportSize(int &w, int &h) = 0;
 	virtual void NotifyWindow() = 0;
 
-	virtual void WindowDestroyNtf(uint32_t WindowID) = 0;
-	virtual void WindowCreateNtf(uint32_t WindowID) = 0;
+	virtual void WindowDestroyNtf(uint32_t WindowId) = 0;
+	virtual void WindowCreateNtf(uint32_t WindowId) = 0;
 
 	virtual void RunBuffer(CCommandBuffer *pBuffer) = 0;
 	virtual void RunBufferSingleThreadedUnsafe(CCommandBuffer *pBuffer) = 0;
@@ -761,8 +731,9 @@ public:
 	virtual bool HasQuadBuffering() { return false; }
 	virtual bool HasTextBuffering() { return false; }
 	virtual bool HasQuadContainerBuffering() { return false; }
-	virtual bool Has2DTextureArrays() { return false; }
-	virtual const char *GetErrorString() { return NULL; }
+	virtual bool Uses2DTextureArrays() { return false; }
+	virtual bool HasTextureArraysSupport() { return false; }
+	virtual const char *GetErrorString() { return nullptr; }
 
 	virtual const char *GetVendorString() = 0;
 	virtual const char *GetVersionString() = 0;
@@ -770,6 +741,11 @@ public:
 
 	// be aware that this function should only be called from the graphics thread, and even then you should really know what you are doing
 	virtual TGLBackendReadPresentedImageData &GetReadPresentedImageDataFuncUnsafe() = 0;
+
+	virtual bool GetWarning(std::vector<std::string> &WarningStrings) = 0;
+
+	// returns true if the error msg was shown
+	virtual bool ShowMessageBox(unsigned Type, const char *pTitle, const char *pMsg) = 0;
 };
 
 class CGraphics_Threaded : public IEngineGraphics
@@ -789,7 +765,8 @@ class CGraphics_Threaded : public IEngineGraphics
 	bool m_GLQuadBufferingEnabled;
 	bool m_GLTextBufferingEnabled;
 	bool m_GLQuadContainerBufferingEnabled;
-	bool m_GLHasTextureArrays;
+	bool m_GLUses2DTextureArrays;
+	bool m_GLHasTextureArraysSupport;
 	bool m_GLUseTrianglesAsQuad;
 
 	CCommandBuffer *m_apCommandBuffers[NUM_CMDBUFFERS];
@@ -799,6 +776,7 @@ class CGraphics_Threaded : public IEngineGraphics
 	//
 	class IStorage *m_pStorage;
 	class IConsole *m_pConsole;
+	class IEngine *m_pEngine;
 
 	int m_CurIndex;
 
@@ -814,18 +792,17 @@ class CGraphics_Threaded : public IEngineGraphics
 	float m_Rotation;
 	int m_Drawing;
 	bool m_DoScreenshot;
-	char m_aScreenshotName[128];
+	char m_aScreenshotName[IO_MAX_PATH_LENGTH];
 
-	CTextureHandle m_InvalidTexture;
+	CTextureHandle m_NullTexture;
 
 	std::vector<int> m_vTextureIndices;
-	int m_FirstFreeTexture;
+	size_t m_FirstFreeTexture;
 	int m_TextureMemoryUsage;
 
-	std::vector<uint8_t> m_vSpriteHelper;
-
-	bool m_WarnPngliteIncompatibleImages = false;
+	std::atomic<bool> m_WarnPngliteIncompatibleImages = false;
 
+	std::mutex m_WarningsMutex;
 	std::vector<SWarning> m_vWarnings;
 
 	// is a non full windowed (in a sense that the viewport won't include the whole window),
@@ -875,16 +852,10 @@ class CGraphics_Threaded : public IEngineGraphics
 	std::vector<SQuadContainer> m_vQuadContainers;
 	int m_FirstFreeQuadContainer;
 
-	struct SWindowResizeListener
-	{
-		SWindowResizeListener(WINDOW_RESIZE_FUNC pFunc, void *pUser) :
-			m_pFunc(std::move(pFunc)), m_pUser(pUser) {}
-		WINDOW_RESIZE_FUNC m_pFunc;
-		void *m_pUser;
-	};
-	std::vector<SWindowResizeListener> m_vResizeListeners;
+	std::vector<WINDOW_RESIZE_FUNC> m_vResizeListeners;
+	std::vector<WINDOW_PROPS_CHANGED_FUNC> m_vPropChangeListeners;
 
-	void *AllocCommandBufferData(unsigned AllocSize);
+	void *AllocCommandBufferData(size_t AllocSize);
 
 	void AddVertices(int Count);
 	void AddVertices(int Count, CCommandBuffer::SVertex *pVertices);
@@ -893,8 +864,8 @@ class CGraphics_Threaded : public IEngineGraphics
 	template<typename TName>
 	void Rotate(const CCommandBuffer::SPoint &rCenter, TName *pPoints, int NumPoints)
 	{
-		float c = cosf(m_Rotation);
-		float s = sinf(m_Rotation);
+		float c = std::cos(m_Rotation);
+		float s = std::sin(m_Rotation);
 		float x, y;
 		int i;
 
@@ -908,24 +879,18 @@ class CGraphics_Threaded : public IEngineGraphics
 		}
 	}
 
-	template<typename TName, typename TFunc>
-	bool AddCmd(TName &Cmd, TFunc &&FailFunc, const char *pFailStr)
+	template<typename TName>
+	void AddCmd(
+		TName &Cmd, std::function<bool()> FailFunc = [] { return true; })
 	{
-		if(!m_pCommandBuffer->AddCommandUnsafe(Cmd))
-		{
-			// kick command buffer and try again
-			KickCommandBuffer();
+		if(m_pCommandBuffer->AddCommandUnsafe(Cmd))
+			return;
 
-			if(!FailFunc())
-				return false;
+		// kick command buffer and try again
+		KickCommandBuffer();
 
-			if(!m_pCommandBuffer->AddCommandUnsafe(Cmd))
-			{
-				dbg_msg("graphics", "%s", pFailStr);
-				return false;
-			}
-		}
-		return true;
+		dbg_assert(FailFunc(), "graphics: failed to run fail handler for command '%s'", typeid(TName).name());
+		dbg_assert(m_pCommandBuffer->AddCommandUnsafe(Cmd), "graphics: failed to add command '%s' to command buffer", typeid(TName).name());
 	}
 
 	void KickCommandBuffer();
@@ -934,6 +899,11 @@ class CGraphics_Threaded : public IEngineGraphics
 
 	void AdjustViewport(bool SendViewportChangeToBackend);
 
+	ivec2 m_ReadPixelPosition = ivec2(0, 0);
+	ColorRGBA *m_pReadPixelColor = nullptr;
+	void ReadPixelDirect(bool *pSwapped);
+	void ScreenshotDirect(bool *pSwapped);
+
 	int IssueInit();
 	int InitWindow();
 
@@ -955,7 +925,7 @@ public:
 	uint64_t StreamedMemoryUsage() const override;
 	uint64_t StagingMemoryUsage() const override;
 
-	const TTWGraphicsGPUList &GetGPUs() const override;
+	const TTwGraphicsGpuList &GetGpus() const override;
 
 	void MapScreen(float TopLeftX, float TopLeftY, float BottomRightX, float BottomRightY) override;
 	void GetScreen(float *pTopLeftX, float *pTopLeftY, float *pBottomRightX, float *pBottomRightY) override;
@@ -964,35 +934,31 @@ public:
 	void LinesEnd() override;
 	void LinesDraw(const CLineItem *pArray, int Num) override;
 
-	int UnloadTexture(IGraphics::CTextureHandle *pIndex) override;
-	IGraphics::CTextureHandle LoadTextureRaw(int Width, int Height, int Format, const void *pData, int StoreFormat, int Flags, const char *pTexName = NULL) override;
-	int LoadTextureRawSub(IGraphics::CTextureHandle TextureID, int x, int y, int Width, int Height, int Format, const void *pData) override;
+	IGraphics::CTextureHandle FindFreeTextureIndex();
+	void FreeTextureIndex(CTextureHandle *pIndex);
+	void UnloadTexture(IGraphics::CTextureHandle *pIndex) override;
+	void LoadTextureAddWarning(size_t Width, size_t Height, int Flags, const char *pTexName);
+	IGraphics::CTextureHandle LoadTextureRaw(const CImageInfo &Image, int Flags, const char *pTexName = nullptr) override;
+	IGraphics::CTextureHandle LoadTextureRawMove(CImageInfo &Image, int Flags, const char *pTexName = nullptr) override;
 
-	bool LoadTextTextures(int Width, int Height, CTextureHandle &TextTexture, CTextureHandle &TextOutlineTexture, void *pTextData, void *pTextOutlineData) override;
+	bool LoadTextTextures(size_t Width, size_t Height, CTextureHandle &TextTexture, CTextureHandle &TextOutlineTexture, uint8_t *pTextData, uint8_t *pTextOutlineData) override;
 	bool UnloadTextTextures(CTextureHandle &TextTexture, CTextureHandle &TextOutlineTexture) override;
-	bool UpdateTextTexture(CTextureHandle TextureID, int x, int y, int Width, int Height, const void *pData) override;
+	bool UpdateTextTexture(CTextureHandle TextureId, int x, int y, size_t Width, size_t Height, uint8_t *pData, bool IsMovedPointer) override;
 
-	CTextureHandle LoadSpriteTextureImpl(CImageInfo &FromImageInfo, int x, int y, int w, int h);
-	CTextureHandle LoadSpriteTexture(CImageInfo &FromImageInfo, struct CDataSprite *pSprite) override;
-	CTextureHandle LoadSpriteTexture(CImageInfo &FromImageInfo, struct client_data7::CDataSprite *pSprite) override;
+	CTextureHandle LoadSpriteTexture(const CImageInfo &FromImageInfo, const struct CDataSprite *pSprite) override;
 
-	bool IsImageSubFullyTransparent(CImageInfo &FromImageInfo, int x, int y, int w, int h) override;
-	bool IsSpriteTextureFullyTransparent(CImageInfo &FromImageInfo, struct client_data7::CDataSprite *pSprite) override;
+	bool IsImageSubFullyTransparent(const CImageInfo &FromImageInfo, int x, int y, int w, int h) override;
+	bool IsSpriteTextureFullyTransparent(const CImageInfo &FromImageInfo, const struct CDataSprite *pSprite) override;
 
 	// simple uncompressed RGBA loaders
-	IGraphics::CTextureHandle LoadTexture(const char *pFilename, int StorageType, int StoreFormat, int Flags) override;
-	int LoadPNG(CImageInfo *pImg, const char *pFilename, int StorageType) override;
-	void FreePNG(CImageInfo *pImg) override;
+	IGraphics::CTextureHandle LoadTexture(const char *pFilename, int StorageType, int Flags = 0) override;
+	bool LoadPng(CImageInfo &Image, const char *pFilename, int StorageType) override;
+	bool LoadPng(CImageInfo &Image, const uint8_t *pData, size_t DataSize, const char *pContextName) override;
 
-	bool CheckImageDivisibility(const char *pFileName, CImageInfo &Img, int DivX, int DivY, bool AllowResize) override;
-	bool IsImageFormatRGBA(const char *pFileName, CImageInfo &Img) override;
+	bool CheckImageDivisibility(const char *pContextName, CImageInfo &Image, int DivX, int DivY, bool AllowResize) override;
+	bool IsImageFormatRgba(const char *pContextName, const CImageInfo &Image) override;
 
-	void CopyTextureBufferSub(uint8_t *pDestBuffer, uint8_t *pSourceBuffer, int FullWidth, int FullHeight, int ColorChannelCount, int SubOffsetX, int SubOffsetY, int SubCopyWidth, int SubCopyHeight) override;
-	void CopyTextureFromTextureBufferSub(uint8_t *pDestBuffer, int DestWidth, int DestHeight, uint8_t *pSourceBuffer, int SrcWidth, int SrcHeight, int ColorChannelCount, int SrcSubOffsetX, int SrcSubOffsetY, int SrcSubCopyWidth, int SrcSubCopyHeight) override;
-
-	bool ScreenshotDirect();
-
-	void TextureSet(CTextureHandle TextureID) override;
+	void TextureSet(CTextureHandle TextureId) override;
 
 	void Clear(float r, float g, float b, bool ForceClearNow = false) override;
 
@@ -1013,14 +979,14 @@ public:
 		pVert->m_Color = m_aColor[ColorIndex];
 	}
 
-	void SetColorVertex(const CColorVertex *pArray, int Num) override;
+	void SetColorVertex(const CColorVertex *pArray, size_t Num) override;
 	void SetColor(float r, float g, float b, float a) override;
 	void SetColor(ColorRGBA Color) override;
 	void SetColor4(ColorRGBA TopLeft, ColorRGBA TopRight, ColorRGBA BottomLeft, ColorRGBA BottomRight) override;
 
 	// go through all vertices and change their color (only works for quads)
 	void ChangeColorOfCurrentQuadVertices(float r, float g, float b, float a) override;
-	void ChangeColorOfQuadVertices(int QuadOffset, unsigned char r, unsigned char g, unsigned char b, unsigned char a) override;
+	void ChangeColorOfQuadVertices(size_t QuadOffset, unsigned char r, unsigned char g, unsigned char b, unsigned char a) override;
 
 	void QuadsSetSubset(float TlU, float TlV, float BrU, float BrV) override;
 	void QuadsSetSubsetFree(
@@ -1134,23 +1100,13 @@ public:
 	void DrawRect4(float x, float y, float w, float h, ColorRGBA ColorTopLeft, ColorRGBA ColorTopRight, ColorRGBA ColorBottomLeft, ColorRGBA ColorBottomRight, int Corners, float Rounding) override;
 	void DrawCircle(float CenterX, float CenterY, float Radius, int Segments) override;
 
-	const GL_STexCoord *GetCurTextureCoordinates() override
-	{
-		return m_aTexture;
-	}
-
-	const GL_SColor *GetCurColor() override
-	{
-		return m_aColor;
-	}
-
 	int CreateQuadContainer(bool AutomaticUpload = true) override;
 	void QuadContainerChangeAutomaticUpload(int ContainerIndex, bool AutomaticUpload) override;
 	void QuadContainerUpload(int ContainerIndex) override;
 	int QuadContainerAddQuads(int ContainerIndex, CQuadItem *pArray, int Num) override;
 	int QuadContainerAddQuads(int ContainerIndex, CFreeformItem *pArray, int Num) override;
 	void QuadContainerReset(int ContainerIndex) override;
-	void DeleteQuadContainer(int ContainerIndex) override;
+	void DeleteQuadContainer(int &ContainerIndex) override;
 	void RenderQuadContainer(int ContainerIndex, int QuadDrawNum) override;
 	void RenderQuadContainer(int ContainerIndex, int QuadOffset, int QuadDrawNum, bool ChangeWrapMode = true) override;
 	void RenderQuadContainerEx(int ContainerIndex, int QuadOffset, int QuadDrawNum, float X, float Y, float ScaleX = 1.f, float ScaleY = 1.f) override;
@@ -1158,9 +1114,9 @@ public:
 	void RenderQuadContainerAsSpriteMultiple(int ContainerIndex, int QuadOffset, int DrawCount, SRenderSpriteInfo *pRenderInfo) override;
 
 	template<typename TName>
-	void FlushVerticesImpl(bool KeepVertices, int &PrimType, int &PrimCount, int &NumVerts, TName &Command, size_t VertSize)
+	void FlushVerticesImpl(bool KeepVertices, int &PrimType, size_t &PrimCount, size_t &NumVerts, TName &Command, size_t VertSize)
 	{
-		Command.m_pVertices = NULL;
+		Command.m_pVertices = nullptr;
 		if(m_NumVertices == 0)
 			return;
 
@@ -1195,40 +1151,16 @@ public:
 		else
 			return;
 
-		Command.m_pVertices = (decltype(Command.m_pVertices))m_pCommandBuffer->AllocData(VertSize * NumVerts);
-		if(Command.m_pVertices == NULL)
-		{
-			// kick command buffer and try again
-			KickCommandBuffer();
-
-			Command.m_pVertices = (decltype(Command.m_pVertices))m_pCommandBuffer->AllocData(VertSize * NumVerts);
-			if(Command.m_pVertices == NULL)
-			{
-				dbg_msg("graphics", "failed to allocate data for vertices");
-				return;
-			}
-		}
-
+		Command.m_pVertices = (decltype(Command.m_pVertices))AllocCommandBufferData(VertSize * NumVerts);
 		Command.m_State = m_State;
 
 		Command.m_PrimType = PrimType;
 		Command.m_PrimCount = PrimCount;
 
-		if(
-			!AddCmd(
-				Command, [&] {
-					Command.m_pVertices = (decltype(Command.m_pVertices))m_pCommandBuffer->AllocData(VertSize * NumVerts);
-					if(Command.m_pVertices == NULL)
-					{
-						dbg_msg("graphics", "failed to allocate data for vertices");
-						return false;
-					}
-					return true;
-				},
-				"failed to allocate memory for render command"))
-		{
-			return;
-		}
+		AddCmd(Command, [&] {
+			Command.m_pVertices = (decltype(Command.m_pVertices))m_pCommandBuffer->AllocData(VertSize * NumVerts);
+			return Command.m_pVertices != nullptr;
+		});
 
 		m_pCommandBuffer->AddRenderCalls(1);
 	}
@@ -1237,9 +1169,8 @@ public:
 	void FlushVerticesTex3D() override;
 
 	void RenderTileLayer(int BufferContainerIndex, const ColorRGBA &Color, char **pOffsets, unsigned int *pIndicedVertexDrawNum, size_t NumIndicesOffset) override;
-	void RenderBorderTiles(int BufferContainerIndex, const ColorRGBA &Color, char *pIndexBufferOffset, const vec2 &Offset, const vec2 &Dir, int JumpIndex, unsigned int DrawNum) override;
-	void RenderBorderTileLines(int BufferContainerIndex, const ColorRGBA &Color, char *pIndexBufferOffset, const vec2 &Offset, const vec2 &Dir, unsigned int IndexDrawNum, unsigned int RedrawNum) override;
-	void RenderQuadLayer(int BufferContainerIndex, SQuadRenderInfo *pQuadInfo, int QuadNum, int QuadOffset) override;
+	virtual void RenderBorderTiles(int BufferContainerIndex, const ColorRGBA &Color, char *pIndexBufferOffset, const vec2 &Offset, const vec2 &Scale, uint32_t DrawNum) override;
+	void RenderQuadLayer(int BufferContainerIndex, SQuadRenderInfo *pQuadInfo, size_t QuadNum, int QuadOffset) override;
 	void RenderText(int BufferContainerIndex, int TextQuadNum, int TextureSize, int TextureTextIndex, int TextureTextOutlineIndex, const ColorRGBA &TextColor, const ColorRGBA &TextOutlineColor) override;
 
 	// modern GL functions
@@ -1251,25 +1182,29 @@ public:
 
 	int CreateBufferContainer(SBufferContainerInfo *pContainerInfo) override;
 	// destroying all buffer objects means, that all referenced VBOs are destroyed automatically, so the user does not need to save references to them
-	void DeleteBufferContainer(int ContainerIndex, bool DestroyAllBO = true) override;
+	void DeleteBufferContainer(int &ContainerIndex, bool DestroyAllBO = true) override;
 	void UpdateBufferContainerInternal(int ContainerIndex, SBufferContainerInfo *pContainerInfo);
 	void IndicesNumRequiredNotify(unsigned int RequiredIndicesCount) override;
 
 	int GetNumScreens() const override;
+	const char *GetScreenName(int Screen) const override;
+
 	void Minimize() override;
 	void Maximize() override;
 	void WarnPngliteIncompatibleImages(bool Warn) override;
-	void SetWindowParams(int FullscreenMode, bool IsBorderless, bool AllowResizing) override;
+	void SetWindowParams(int FullscreenMode, bool IsBorderless) override;
 	bool SetWindowScreen(int Index) override;
 	void Move(int x, int y) override;
-	void Resize(int w, int h, int RefreshRate) override;
+	bool Resize(int w, int h, int RefreshRate) override;
+	void ResizeToScreen() override;
 	void GotResized(int w, int h, int RefreshRate) override;
 	void UpdateViewport(int X, int Y, int W, int H, bool ByResize) override;
-	void AddWindowResizeListener(WINDOW_RESIZE_FUNC pFunc, void *pUser) override;
+	void AddWindowResizeListener(WINDOW_RESIZE_FUNC pFunc) override;
+	void AddWindowPropChangeListener(WINDOW_PROPS_CHANGED_FUNC pFunc) override;
 	int GetWindowScreen() override;
 
-	void WindowDestroyNtf(uint32_t WindowID) override;
-	void WindowCreateNtf(uint32_t WindowID) override;
+	void WindowDestroyNtf(uint32_t WindowId) override;
+	void WindowCreateNtf(uint32_t WindowId) override;
 
 	int WindowActive() override;
 	int WindowOpen() override;
@@ -1280,6 +1215,7 @@ public:
 	int Init() override;
 	void Shutdown() override;
 
+	void ReadPixel(ivec2 Position, ColorRGBA *pColor) override;
 	void TakeScreenshot(const char *pFilename) override;
 	void TakeCustomScreenshot(const char *pFilename) override;
 	void Swap() override;
@@ -1287,6 +1223,7 @@ public:
 	bool SetMultiSampling(uint32_t ReqMultiSamplingCount, uint32_t &MultiSamplingCountBackend) override;
 
 	int GetVideoModes(CVideoMode *pModes, int MaxModes, int Screen) override;
+	void GetCurrentVideoMode(CVideoMode &CurMode, int Screen) override;
 
 	virtual int GetDesktopScreenWidth() const { return g_Config.m_GfxDesktopWidth; }
 	virtual int GetDesktopScreenHeight() const { return g_Config.m_GfxDesktopHeight; }
@@ -1296,7 +1233,11 @@ public:
 	bool IsIdle() const override;
 	void WaitForIdle() override;
 
-	SWarning *GetCurWarning() override;
+	void AddWarning(const SWarning &Warning);
+	std::optional<SWarning> CurrentWarning() override;
+
+	bool ShowMessageBox(unsigned Type, const char *pTitle, const char *pMsg) override;
+	bool IsBackendInitialized() override;
 
 	bool GetDriverVersion(EGraphicsDriverAgeType DriverAgeType, int &Major, int &Minor, int &Patch, const char *&pName, EBackendType BackendType) override { return m_pBackend->GetDriverVersion(DriverAgeType, Major, Minor, Patch, pName, BackendType); }
 	bool IsConfigModernAPI() override { return m_pBackend->IsConfigModernAPI(); }
@@ -1304,7 +1245,8 @@ public:
 	bool IsQuadBufferingEnabled() override { return m_GLQuadBufferingEnabled; }
 	bool IsTextBufferingEnabled() override { return m_GLTextBufferingEnabled; }
 	bool IsQuadContainerBufferingEnabled() override { return m_GLQuadContainerBufferingEnabled; }
-	bool HasTextureArrays() override { return m_GLHasTextureArrays; }
+	bool Uses2DTextureArrays() override { return m_GLUses2DTextureArrays; }
+	bool HasTextureArraysSupport() override { return m_GLHasTextureArraysSupport; }
 
 	const char *GetVendorString() override;
 	const char *GetVersionString() override;
@@ -1313,6 +1255,7 @@ public:
 	TGLBackendReadPresentedImageData &GetReadPresentedImageDataFuncUnsafe() override;
 };
 
-extern IGraphicsBackend *CreateGraphicsBackend();
+typedef std::function<const char *(const char *, const char *)> TTranslateFunc;
+extern IGraphicsBackend *CreateGraphicsBackend(TTranslateFunc &&TranslateFunc);
 
 #endif // ENGINE_CLIENT_GRAPHICS_THREADED_H
