@@ -1,11 +1,11 @@
 #ifndef ENGINE_CLIENT_SERVERBROWSER_HTTP_H
 #define ENGINE_CLIENT_SERVERBROWSER_HTTP_H
-#include <base/system.h>
+#include <base/types.h>
 
 class CServerInfo;
-class IConsole;
 class IEngine;
 class IStorage;
+class IHttp;
 
 class IServerBrowserHttp
 {
@@ -21,9 +21,7 @@ public:
 
 	virtual int NumServers() const = 0;
 	virtual const CServerInfo &Server(int Index) const = 0;
-	virtual int NumLegacyServers() const = 0;
-	virtual const NETADDR &LegacyServer(int Index) const = 0;
 };
 
-IServerBrowserHttp *CreateServerBrowserHttp(IEngine *pEngine, IConsole *pConsole, IStorage *pStorage, const char *pPreviousBestUrl);
+IServerBrowserHttp *CreateServerBrowserHttp(IEngine *pEngine, IStorage *pStorage, IHttp *pHttp, const char *pPreviousBestUrl);
 #endif // ENGINE_CLIENT_SERVERBROWSER_HTTP_H
