@@ -1,8 +1,13 @@
 #ifndef ENGINE_CLIENT_UPDATER_H
 #define ENGINE_CLIENT_UPDATER_H
 
+#include <base/detect.h>
+#include <base/lock.h>
+
 #include <engine/updater.h>
-#include <map>
+
+#include <forward_list>
+#include <memory>
 #include <string>
 
 #define CLIENT_EXEC "DDNet"
@@ -21,6 +26,9 @@
 #define PLAT_NAME CONF_PLATFORM_STRING "-unsupported"
 #endif
 #else
+#if defined(AUTOUPDATE)
+#error Compiling with autoupdater on an unsupported platform
+#endif
 #define PLAT_EXT ""
 #define PLAT_NAME "unsupported-unsupported"
 #endif
@@ -31,6 +39,8 @@
 #define PLAT_CLIENT_EXEC CLIENT_EXEC PLAT_EXT
 #define PLAT_SERVER_EXEC SERVER_EXEC PLAT_EXT
 
+class CUpdaterFetchTask;
+
 class CUpdater : public IUpdater
 {
 	friend class CUpdaterFetchTask;
@@ -38,45 +48,50 @@ class CUpdater : public IUpdater
 	class IClient *m_pClient;
 	class IStorage *m_pStorage;
 	class IEngine *m_pEngine;
+	class CHttp *m_pHttp;
 
-	LOCK m_Lock;
+	CLock m_Lock;
 
-	int m_State;
+	EUpdaterState m_State GUARDED_BY(m_Lock);
 	char m_aStatus[256] GUARDED_BY(m_Lock);
 	int m_Percent GUARDED_BY(m_Lock);
-	char m_aLastFile[256];
 	char m_aClientExecTmp[64];
 	char m_aServerExecTmp[64];
 
+	std::forward_list<std::pair<std::string, bool>> m_FileJobs;
+	std::shared_ptr<CUpdaterFetchTask> m_pCurrentTask;
+	decltype(m_FileJobs)::iterator m_CurrentJob;
+
 	bool m_ClientUpdate;
 	bool m_ServerUpdate;
 
-	std::map<std::string, bool> m_FileJobs;
+	bool m_ClientFetched;
+	bool m_ServerFetched;
 
 	void AddFileJob(const char *pFile, bool Job);
-	void FetchFile(const char *pFile, const char *pDestPath = nullptr);
+	void FetchFile(const char *pFile, const char *pDestPath = nullptr) REQUIRES(!m_Lock);
 	bool MoveFile(const char *pFile);
 
-	void ParseUpdate();
-	void PerformUpdate();
-	void CommitUpdate();
+	void ParseUpdate() REQUIRES(!m_Lock);
+	void PerformUpdate() REQUIRES(!m_Lock);
+	void RunningUpdate() REQUIRES(!m_Lock);
+	void CommitUpdate() REQUIRES(!m_Lock);
 
 	bool ReplaceClient();
 	bool ReplaceServer();
 
-	void SetCurrentState(int NewState) REQUIRES(!m_Lock);
+	void SetCurrentState(EUpdaterState NewState) REQUIRES(!m_Lock);
 
 public:
 	CUpdater();
-	~CUpdater();
 
-	int GetCurrentState() override REQUIRES(!m_Lock);
+	EUpdaterState GetCurrentState() override REQUIRES(!m_Lock);
 	void GetCurrentFile(char *pBuf, int BufSize) override REQUIRES(!m_Lock);
 	int GetCurrentPercent() override REQUIRES(!m_Lock);
 
-	void InitiateUpdate() override;
-	void Init();
-	void Update() override;
+	void InitiateUpdate() REQUIRES(!m_Lock) override;
+	void Init(CHttp *pHttp);
+	void Update() REQUIRES(!m_Lock) override;
 };
 
 #endif
