@@ -4,40 +4,24 @@
 #define ENGINE_ENGINE_H
 
 #include "kernel.h"
-#include <engine/shared/jobs.h>
+
+#include <memory>
 
 class CFutureLogger;
+class IJob;
 class ILogger;
 
-class CHostLookup : public IJob
-{
-private:
-	void Run() override;
-
-public:
-	CHostLookup();
-	CHostLookup(const char *pHostname, int Nettype);
-
-	int m_Result;
-	char m_aHostname[128];
-	int m_Nettype;
-	NETADDR m_Addr;
-};
-
 class IEngine : public IInterface
 {
-	MACRO_INTERFACE("engine", 0)
-
-protected:
-	class CJobPool m_JobPool;
+	MACRO_INTERFACE("engine")
 
 public:
 	virtual ~IEngine() = default;
 
 	virtual void Init() = 0;
 	virtual void AddJob(std::shared_ptr<IJob> pJob) = 0;
-	virtual void SetAdditionalLogger(std::unique_ptr<ILogger> &&pLogger) = 0;
-	static void RunJobBlocking(IJob *pJob);
+	virtual void ShutdownJobs() = 0;
+	virtual void SetAdditionalLogger(std::shared_ptr<ILogger> &&pLogger) = 0;
 };
 
 extern IEngine *CreateEngine(const char *pAppname, std::shared_ptr<CFutureLogger> pFutureLogger, int Jobs);
