@@ -17,7 +17,7 @@ struct CFriendInfo
 
 class IFriends : public IInterface
 {
-	MACRO_INTERFACE("friends", 0)
+	MACRO_INTERFACE("friends")
 public:
 	enum
 	{
@@ -25,7 +25,7 @@ public:
 		FRIEND_CLAN,
 		FRIEND_PLAYER,
 
-		MAX_FRIENDS = 1024,
+		MAX_FRIENDS = 4096,
 	};
 
 	virtual void Init(bool Foes = false) = 0;
