@@ -5,6 +5,7 @@
 
 #include "kernel.h"
 #include <base/hash.h>
+#include <base/types.h>
 
 enum
 {
@@ -13,30 +14,35 @@ enum
 
 class IMap : public IInterface
 {
-	MACRO_INTERFACE("map", 0)
+	MACRO_INTERFACE("map")
 public:
+	virtual int GetDataSize(int Index) const = 0;
 	virtual void *GetData(int Index) = 0;
-	virtual int GetDataSize(int Index) = 0;
 	virtual void *GetDataSwapped(int Index) = 0;
+	virtual const char *GetDataString(int Index) = 0;
 	virtual void UnloadData(int Index) = 0;
-	virtual void *GetItem(int Index, int *pType, int *pID) = 0;
+	virtual int NumData() const = 0;
+
 	virtual int GetItemSize(int Index) = 0;
+	virtual void *GetItem(int Index, int *pType = nullptr, int *pId = nullptr) = 0;
 	virtual void GetType(int Type, int *pStart, int *pNum) = 0;
-	virtual void *FindItem(int Type, int ID) = 0;
-	virtual int NumItems() = 0;
+	virtual int FindItemIndex(int Type, int Id) = 0;
+	virtual void *FindItem(int Type, int Id) = 0;
+	virtual int NumItems() const = 0;
 };
 
 class IEngineMap : public IMap
 {
-	MACRO_INTERFACE("enginemap", 0)
+	MACRO_INTERFACE("enginemap")
 public:
 	virtual bool Load(const char *pMapName) = 0;
-	virtual bool IsLoaded() = 0;
 	virtual void Unload() = 0;
-	virtual SHA256_DIGEST Sha256() = 0;
-	virtual unsigned Crc() = 0;
-	virtual int MapSize() = 0;
-	virtual IOHANDLE File() = 0;
+	virtual bool IsLoaded() const = 0;
+	virtual IOHANDLE File() const = 0;
+
+	virtual SHA256_DIGEST Sha256() const = 0;
+	virtual unsigned Crc() const = 0;
+	virtual int MapSize() const = 0;
 };
 
 extern IEngineMap *CreateEngineMap();
