@@ -1,5 +1,6 @@
 #include "authmanager.h"
 #include <base/hash_ctxt.h>
+#include <base/system.h>
 #include <engine/shared/config.h>
 #include <game/generated/protocol.h>
 
@@ -65,21 +66,21 @@ int CAuthManager::AddKey(const char *pIdent, const char *pPw, int AuthLevel)
 	return AddKeyHash(pIdent, HashPassword(pPw, aSalt), aSalt, AuthLevel);
 }
 
-int CAuthManager::RemoveKey(int Slot)
+void CAuthManager::RemoveKey(int Slot)
 {
 	m_vKeys.erase(m_vKeys.begin() + Slot);
+	// Update indices of default keys
 	for(int &Default : m_aDefault)
 	{
 		if(Default == Slot)
 		{
 			Default = -1;
 		}
-		else if(Default == (int)m_vKeys.size())
+		else if(Default > Slot)
 		{
-			Default = Slot;
+			--Default;
 		}
 	}
-	return m_vKeys.size();
 }
 
 int CAuthManager::FindKey(const char *pIdent) const
@@ -115,10 +116,15 @@ int CAuthManager::KeyLevel(int Slot) const
 const char *CAuthManager::KeyIdent(int Slot) const
 {
 	if(Slot < 0 || Slot >= (int)m_vKeys.size())
-		return NULL;
+		return nullptr;
 	return m_vKeys[Slot].m_aIdent;
 }
 
+bool CAuthManager::IsValidIdent(const char *pIdent) const
+{
+	return str_length(pIdent) < (int)sizeof(CKey().m_aIdent);
+}
+
 void CAuthManager::UpdateKeyHash(int Slot, MD5_DIGEST Hash, const unsigned char *pSalt, int AuthLevel)
 {
 	if(Slot < 0 || Slot >= (int)m_vKeys.size())
