@@ -1,11 +1,14 @@
 #include "connection.h"
 
-#include <engine/shared/protocol.h>
+IDbConnection::IDbConnection(const char *pPrefix)
+{
+	str_copy(m_aPrefix, pPrefix);
+}
 
-void IDbConnection::FormatCreateRace(char *aBuf, unsigned int BufferSize)
+void IDbConnection::FormatCreateRace(char *aBuf, unsigned int BufferSize, bool Backup) const
 {
 	str_format(aBuf, BufferSize,
-		"CREATE TABLE IF NOT EXISTS %s_race ("
+		"CREATE TABLE IF NOT EXISTS %s_race%s ("
 		"  Map VARCHAR(128) COLLATE %s NOT NULL, "
 		"  Name VARCHAR(%d) COLLATE %s NOT NULL, "
 		"  Timestamp TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP, "
@@ -20,30 +23,32 @@ void IDbConnection::FormatCreateRace(char *aBuf, unsigned int BufferSize)
 		"  cp19 FLOAT DEFAULT 0, cp20 FLOAT DEFAULT 0, cp21 FLOAT DEFAULT 0, "
 		"  cp22 FLOAT DEFAULT 0, cp23 FLOAT DEFAULT 0, cp24 FLOAT DEFAULT 0, "
 		"  cp25 FLOAT DEFAULT 0, "
-		"  GameID VARCHAR(64), "
+		"  GameId VARCHAR(64), "
 		"  DDNet7 BOOL DEFAULT FALSE, "
 		"  PRIMARY KEY (Map, Name, Time, Timestamp, Server)"
 		")",
-		GetPrefix(), BinaryCollate(), MAX_NAME_LENGTH, BinaryCollate());
+		GetPrefix(), Backup ? "_backup" : "",
+		BinaryCollate(), MAX_NAME_LENGTH_SQL, BinaryCollate());
 }
 
-void IDbConnection::FormatCreateTeamrace(char *aBuf, unsigned int BufferSize, const char *pIdType)
+void IDbConnection::FormatCreateTeamrace(char *aBuf, unsigned int BufferSize, const char *pIdType, bool Backup) const
 {
 	str_format(aBuf, BufferSize,
-		"CREATE TABLE IF NOT EXISTS %s_teamrace ("
+		"CREATE TABLE IF NOT EXISTS %s_teamrace%s ("
 		"  Map VARCHAR(128) COLLATE %s NOT NULL, "
 		"  Name VARCHAR(%d) COLLATE %s NOT NULL, "
 		"  Timestamp TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP, "
 		"  Time FLOAT DEFAULT 0, "
 		"  ID %s NOT NULL, " // VARBINARY(16) for MySQL and BLOB for SQLite
-		"  GameID VARCHAR(64), "
+		"  GameId VARCHAR(64), "
 		"  DDNet7 BOOL DEFAULT FALSE, "
-		"  PRIMARY KEY (ID, Name)"
+		"  PRIMARY KEY (Id, Name)"
 		")",
-		GetPrefix(), BinaryCollate(), MAX_NAME_LENGTH, BinaryCollate(), pIdType);
+		GetPrefix(), Backup ? "_backup" : "",
+		BinaryCollate(), MAX_NAME_LENGTH_SQL, BinaryCollate(), pIdType);
 }
 
-void IDbConnection::FormatCreateMaps(char *aBuf, unsigned int BufferSize)
+void IDbConnection::FormatCreateMaps(char *aBuf, unsigned int BufferSize) const
 {
 	str_format(aBuf, BufferSize,
 		"CREATE TABLE IF NOT EXISTS %s_maps ("
@@ -58,23 +63,24 @@ void IDbConnection::FormatCreateMaps(char *aBuf, unsigned int BufferSize)
 		GetPrefix(), BinaryCollate(), BinaryCollate(), BinaryCollate());
 }
 
-void IDbConnection::FormatCreateSaves(char *aBuf, unsigned int BufferSize)
+void IDbConnection::FormatCreateSaves(char *aBuf, unsigned int BufferSize, bool Backup) const
 {
 	str_format(aBuf, BufferSize,
-		"CREATE TABLE IF NOT EXISTS %s_saves ("
+		"CREATE TABLE IF NOT EXISTS %s_saves%s ("
 		"  Savegame TEXT COLLATE %s NOT NULL, "
 		"  Map VARCHAR(128) COLLATE %s NOT NULL, "
 		"  Code VARCHAR(128) COLLATE %s NOT NULL, "
 		"  Timestamp TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP, "
 		"  Server CHAR(4), "
 		"  DDNet7 BOOL DEFAULT FALSE, "
-		"  SaveID VARCHAR(36) DEFAULT NULL, "
+		"  SaveId VARCHAR(36) DEFAULT NULL, "
 		"  PRIMARY KEY (Map, Code)"
 		")",
-		GetPrefix(), BinaryCollate(), BinaryCollate(), BinaryCollate());
+		GetPrefix(), Backup ? "_backup" : "",
+		BinaryCollate(), BinaryCollate(), BinaryCollate());
 }
 
-void IDbConnection::FormatCreatePoints(char *aBuf, unsigned int BufferSize)
+void IDbConnection::FormatCreatePoints(char *aBuf, unsigned int BufferSize) const
 {
 	str_format(aBuf, BufferSize,
 		"CREATE TABLE IF NOT EXISTS %s_points ("
@@ -82,5 +88,5 @@ void IDbConnection::FormatCreatePoints(char *aBuf, unsigned int BufferSize)
 		"  Points INT DEFAULT 0, "
 		"  PRIMARY KEY (Name)"
 		")",
-		GetPrefix(), MAX_NAME_LENGTH, BinaryCollate());
+		GetPrefix(), MAX_NAME_LENGTH_SQL, BinaryCollate());
 }
