@@ -1,5 +1,7 @@
 #include "connection.h"
 
+#include <engine/server/databases/connection_pool.h>
+
 #if defined(CONF_MYSQL)
 #include <mysql.h>
 
@@ -25,6 +27,11 @@ enum
 std::atomic_int g_MysqlState = {MYSQLSTATE_UNINITIALIZED};
 std::atomic_int g_MysqlNumConnections;
 
+bool MysqlAvailable()
+{
+	return true;
+}
+
 int MysqlInit()
 {
 	dbg_assert(mysql_thread_safe(), "MySQL library without thread safety");
@@ -58,20 +65,10 @@ void MysqlUninit()
 class CMysqlConnection : public IDbConnection
 {
 public:
-	CMysqlConnection(
-		const char *pDatabase,
-		const char *pPrefix,
-		const char *pUser,
-		const char *pPass,
-		const char *pIp,
-		const char *pBindaddr,
-		int Port,
-		bool Setup);
+	explicit CMysqlConnection(CMysqlConfig m_Config);
 	~CMysqlConnection();
 	void Print(IConsole *pConsole, const char *pMode) override;
 
-	CMysqlConnection *Copy() override;
-
 	const char *BinaryCollate() const override { return "utf8mb4_bin"; }
 	void ToUnixTimestamp(const char *pTimestamp, char *aBuf, unsigned int BufferSize) override;
 	const char *InsertTimestampAsUtc() const override { return "?"; }
@@ -92,6 +89,7 @@ public:
 	void BindInt(int Idx, int Value) override;
 	void BindInt64(int Idx, int64_t Value) override;
 	void BindFloat(int Idx, float Value) override;
+	void BindNull(int Idx) override;
 
 	void Print() override {}
 	bool Step(bool *pEnd, char *pError, int ErrorSize) override;
@@ -134,14 +132,8 @@ private:
 	std::vector<MYSQL_BIND> m_vStmtParameters;
 	std::vector<UParameterExtra> m_vStmtParameterExtras;
 
-	// copy of config vars
-	char m_aDatabase[64];
-	char m_aUser[64];
-	char m_aPass[64];
-	char m_aIp[64];
-	char m_aBindaddr[128];
-	int m_Port;
-	bool m_Setup;
+	// copy of m_Config vars
+	CMysqlConfig m_Config;
 
 	std::atomic_bool m_InUse;
 };
@@ -151,18 +143,9 @@ void CMysqlConnection::CStmtDeleter::operator()(MYSQL_STMT *pStmt) const
 	mysql_stmt_close(pStmt);
 }
 
-CMysqlConnection::CMysqlConnection(
-	const char *pDatabase,
-	const char *pPrefix,
-	const char *pUser,
-	const char *pPass,
-	const char *pIp,
-	const char *pBindaddr,
-	int Port,
-	bool Setup) :
-	IDbConnection(pPrefix),
-	m_Port(Port),
-	m_Setup(Setup),
+CMysqlConnection::CMysqlConnection(CMysqlConfig Config) :
+	IDbConnection(Config.m_aPrefix),
+	m_Config(Config),
 	m_InUse(false)
 {
 	g_MysqlNumConnections += 1;
@@ -171,12 +154,6 @@ CMysqlConnection::CMysqlConnection(
 	mem_zero(m_aErrorDetail, sizeof(m_aErrorDetail));
 	mem_zero(&m_Mysql, sizeof(m_Mysql));
 	mysql_init(&m_Mysql);
-
-	str_copy(m_aDatabase, pDatabase, sizeof(m_aDatabase));
-	str_copy(m_aUser, pUser, sizeof(m_aUser));
-	str_copy(m_aPass, pPass, sizeof(m_aPass));
-	str_copy(m_aIp, pIp, sizeof(m_aIp));
-	str_copy(m_aBindaddr, pBindaddr, sizeof(m_aBindaddr));
 }
 
 CMysqlConnection::~CMysqlConnection()
@@ -215,15 +192,10 @@ void CMysqlConnection::Print(IConsole *pConsole, const char *pMode)
 	char aBuf[512];
 	str_format(aBuf, sizeof(aBuf),
 		"MySQL-%s: DB: '%s' Prefix: '%s' User: '%s' IP: <{'%s'}> Port: %d",
-		pMode, m_aDatabase, GetPrefix(), m_aUser, m_aIp, m_Port);
+		pMode, m_Config.m_aDatabase, GetPrefix(), m_Config.m_aUser, m_Config.m_aIp, m_Config.m_Port);
 	pConsole->Print(IConsole::OUTPUT_LEVEL_STANDARD, "server", aBuf);
 }
 
-CMysqlConnection *CMysqlConnection::Copy()
-{
-	return new CMysqlConnection(m_aDatabase, GetPrefix(), m_aUser, m_aPass, m_aIp, m_aBindaddr, m_Port, m_Setup);
-}
-
 void CMysqlConnection::ToUnixTimestamp(const char *pTimestamp, char *aBuf, unsigned int BufferSize)
 {
 	str_format(aBuf, BufferSize, "UNIX_TIMESTAMP(%s)", pTimestamp);
@@ -255,7 +227,7 @@ bool CMysqlConnection::ConnectImpl()
 			StoreErrorStmt("free_result");
 			dbg_msg("mysql", "can't free last result %s", m_aErrorDetail);
 		}
-		if(!mysql_select_db(&m_Mysql, m_aDatabase))
+		if(!mysql_select_db(&m_Mysql, m_Config.m_aDatabase))
 		{
 			// Success.
 			return false;
@@ -277,12 +249,12 @@ bool CMysqlConnection::ConnectImpl()
 	mysql_options(&m_Mysql, MYSQL_OPT_WRITE_TIMEOUT, &OptWriteTimeout);
 	mysql_options(&m_Mysql, MYSQL_OPT_RECONNECT, &OptReconnect);
 	mysql_options(&m_Mysql, MYSQL_SET_CHARSET_NAME, "utf8mb4");
-	if(m_aBindaddr[0] != '\0')
+	if(m_Config.m_aBindaddr[0] != '\0')
 	{
-		mysql_options(&m_Mysql, MYSQL_OPT_BIND, m_aBindaddr);
+		mysql_options(&m_Mysql, MYSQL_OPT_BIND, m_Config.m_aBindaddr);
 	}
 
-	if(!mysql_real_connect(&m_Mysql, m_aIp, m_aUser, m_aPass, nullptr, m_Port, nullptr, CLIENT_IGNORE_SIGPIPE))
+	if(!mysql_real_connect(&m_Mysql, m_Config.m_aIp, m_Config.m_aUser, m_Config.m_aPass, nullptr, m_Config.m_Port, nullptr, CLIENT_IGNORE_SIGPIPE))
 	{
 		StoreErrorMysql("real_connect");
 		return true;
@@ -297,11 +269,11 @@ bool CMysqlConnection::ConnectImpl()
 		return true;
 	}
 
-	if(m_Setup)
+	if(m_Config.m_Setup)
 	{
 		char aCreateDatabase[1024];
 		// create database
-		str_format(aCreateDatabase, sizeof(aCreateDatabase), "CREATE DATABASE IF NOT EXISTS %s CHARACTER SET utf8mb4", m_aDatabase);
+		str_format(aCreateDatabase, sizeof(aCreateDatabase), "CREATE DATABASE IF NOT EXISTS %s CHARACTER SET utf8mb4", m_Config.m_aDatabase);
 		if(PrepareAndExecuteStatement(aCreateDatabase))
 		{
 			return true;
@@ -309,23 +281,23 @@ bool CMysqlConnection::ConnectImpl()
 	}
 
 	// Connect to specific database
-	if(mysql_select_db(&m_Mysql, m_aDatabase))
+	if(mysql_select_db(&m_Mysql, m_Config.m_aDatabase))
 	{
 		StoreErrorMysql("select_db");
 		return true;
 	}
 
-	if(m_Setup)
+	if(m_Config.m_Setup)
 	{
 		char aCreateRace[1024];
 		char aCreateTeamrace[1024];
 		char aCreateMaps[1024];
 		char aCreateSaves[1024];
 		char aCreatePoints[1024];
-		FormatCreateRace(aCreateRace, sizeof(aCreateRace));
-		FormatCreateTeamrace(aCreateTeamrace, sizeof(aCreateTeamrace), "VARBINARY(16)");
+		FormatCreateRace(aCreateRace, sizeof(aCreateRace), /* Backup */ false);
+		FormatCreateTeamrace(aCreateTeamrace, sizeof(aCreateTeamrace), "VARBINARY(16)", /* Backup */ false);
 		FormatCreateMaps(aCreateMaps, sizeof(aCreateMaps));
-		FormatCreateSaves(aCreateSaves, sizeof(aCreateSaves));
+		FormatCreateSaves(aCreateSaves, sizeof(aCreateSaves), /* Backup */ false);
 		FormatCreatePoints(aCreatePoints, sizeof(aCreatePoints));
 
 		if(PrepareAndExecuteStatement(aCreateRace) ||
@@ -336,7 +308,7 @@ bool CMysqlConnection::ConnectImpl()
 		{
 			return true;
 		}
-		m_Setup = false;
+		m_Config.m_Setup = false;
 	}
 	dbg_msg("mysql", "connection established");
 	return false;
@@ -359,8 +331,11 @@ bool CMysqlConnection::PrepareStatement(const char *pStmt, char *pError, int Err
 	unsigned NumParameters = mysql_stmt_param_count(m_pStmt.get());
 	m_vStmtParameters.resize(NumParameters);
 	m_vStmtParameterExtras.resize(NumParameters);
-	mem_zero(&m_vStmtParameters[0], sizeof(m_vStmtParameters[0]) * m_vStmtParameters.size());
-	mem_zero(&m_vStmtParameterExtras[0], sizeof(m_vStmtParameterExtras[0]) * m_vStmtParameterExtras.size());
+	if(NumParameters)
+	{
+		mem_zero(m_vStmtParameters.data(), sizeof(m_vStmtParameters[0]) * m_vStmtParameters.size());
+		mem_zero(m_vStmtParameterExtras.data(), sizeof(m_vStmtParameterExtras[0]) * m_vStmtParameterExtras.size());
+	}
 	return false;
 }
 
@@ -450,12 +425,28 @@ void CMysqlConnection::BindFloat(int Idx, float Value)
 	pParam->error = nullptr;
 }
 
+void CMysqlConnection::BindNull(int Idx)
+{
+	m_NewQuery = true;
+	Idx -= 1;
+	dbg_assert(0 <= Idx && Idx < (int)m_vStmtParameters.size(), "index out of bounds");
+
+	MYSQL_BIND *pParam = &m_vStmtParameters[Idx];
+	pParam->buffer_type = MYSQL_TYPE_NULL;
+	pParam->buffer = nullptr;
+	pParam->buffer_length = 0;
+	pParam->length = nullptr;
+	pParam->is_null = nullptr;
+	pParam->is_unsigned = false;
+	pParam->error = nullptr;
+}
+
 bool CMysqlConnection::Step(bool *pEnd, char *pError, int ErrorSize)
 {
 	if(m_NewQuery)
 	{
 		m_NewQuery = false;
-		if(mysql_stmt_bind_param(m_pStmt.get(), &m_vStmtParameters[0]))
+		if(mysql_stmt_bind_param(m_pStmt.get(), m_vStmtParameters.data()))
 		{
 			StoreErrorStmt("bind_param");
 			str_copy(pError, m_aErrorDetail, ErrorSize);
@@ -486,7 +477,7 @@ bool CMysqlConnection::ExecuteUpdate(int *pNumUpdated, char *pError, int ErrorSi
 	if(m_NewQuery)
 	{
 		m_NewQuery = false;
-		if(mysql_stmt_bind_param(m_pStmt.get(), &m_vStmtParameters[0]))
+		if(mysql_stmt_bind_param(m_pStmt.get(), m_vStmtParameters.data()))
 		{
 			StoreErrorStmt("bind_param");
 			str_copy(pError, m_aErrorDetail, ErrorSize);
@@ -618,7 +609,7 @@ void CMysqlConnection::GetString(int Col, char *pBuffer, int BufferSize)
 
 	for(int i = 0; i < BufferSize; i++)
 	{
-		pBuffer[i] = 'a';
+		pBuffer[i] = '\0';
 	}
 
 	MYSQL_BIND Bind;
@@ -628,7 +619,8 @@ void CMysqlConnection::GetString(int Col, char *pBuffer, int BufferSize)
 	mem_zero(&Bind, sizeof(Bind));
 	Bind.buffer_type = MYSQL_TYPE_STRING;
 	Bind.buffer = pBuffer;
-	Bind.buffer_length = BufferSize;
+	// leave one character for null-termination
+	Bind.buffer_length = BufferSize - 1;
 	Bind.length = &Length;
 	Bind.is_null = &IsNull;
 	Bind.is_unsigned = false;
@@ -645,7 +637,7 @@ void CMysqlConnection::GetString(int Col, char *pBuffer, int BufferSize)
 	}
 	if(Error)
 	{
-		dbg_assert(0, "error getting string: truncation occured");
+		dbg_assert(0, "error getting string: truncation occurred");
 	}
 }
 
@@ -677,7 +669,7 @@ int CMysqlConnection::GetBlob(int Col, unsigned char *pBuffer, int BufferSize)
 	}
 	if(Error)
 	{
-		dbg_assert(0, "error getting blob: truncation occured");
+		dbg_assert(0, "error getting blob: truncation occurred");
 	}
 	return Length;
 }
@@ -713,19 +705,15 @@ bool CMysqlConnection::AddPoints(const char *pPlayer, int Points, char *pError,
 	return ExecuteUpdate(&NumUpdated, pError, ErrorSize);
 }
 
-std::unique_ptr<IDbConnection> CreateMysqlConnection(
-	const char *pDatabase,
-	const char *pPrefix,
-	const char *pUser,
-	const char *pPass,
-	const char *pIp,
-	const char *pBindaddr,
-	int Port,
-	bool Setup)
+std::unique_ptr<IDbConnection> CreateMysqlConnection(CMysqlConfig Config)
 {
-	return std::make_unique<CMysqlConnection>(pDatabase, pPrefix, pUser, pPass, pIp, pBindaddr, Port, Setup);
+	return std::make_unique<CMysqlConnection>(Config);
 }
 #else
+bool MysqlAvailable()
+{
+	return false;
+}
 int MysqlInit()
 {
 	return 0;
@@ -733,15 +721,7 @@ int MysqlInit()
 void MysqlUninit()
 {
 }
-std::unique_ptr<IDbConnection> CreateMysqlConnection(
-	const char *pDatabase,
-	const char *pPrefix,
-	const char *pUser,
-	const char *pPass,
-	const char *pIp,
-	const char *pBindaddr,
-	int Port,
-	bool Setup)
+std::unique_ptr<IDbConnection> CreateMysqlConnection(CMysqlConfig Config)
 {
 	return nullptr;
 }
