@@ -11,11 +11,9 @@ class CSqliteConnection : public IDbConnection
 {
 public:
 	CSqliteConnection(const char *pFilename, bool Setup);
-	virtual ~CSqliteConnection();
+	~CSqliteConnection() override;
 	void Print(IConsole *pConsole, const char *pMode) override;
 
-	CSqliteConnection *Copy() override;
-
 	const char *BinaryCollate() const override { return "BINARY"; }
 	void ToUnixTimestamp(const char *pTimestamp, char *aBuf, unsigned int BufferSize) override;
 	const char *InsertTimestampAsUtc() const override { return "DATETIME(?, 'utc')"; }
@@ -39,6 +37,7 @@ public:
 	void BindInt(int Idx, int Value) override;
 	void BindInt64(int Idx, int64_t Value) override;
 	void BindFloat(int Idx, float Value) override;
+	void BindNull(int Idx) override;
 
 	void Print() override;
 	bool Step(bool *pEnd, char *pError, int ErrorSize) override;
@@ -54,6 +53,9 @@ public:
 
 	bool AddPoints(const char *pPlayer, int Points, char *pError, int ErrorSize) override;
 
+	// fail safe
+	bool CreateFailsafeTables();
+
 private:
 	// copy of config vars
 	char m_aFilename[IO_MAX_PATH_LENGTH];
@@ -64,6 +66,8 @@ private:
 	bool m_Done; // no more rows available for Step
 	// returns false, if the query succeeded
 	bool Execute(const char *pQuery, char *pError, int ErrorSize);
+	// returns true on failure
+	bool ConnectImpl(char *pError, int ErrorSize);
 
 	// returns true if an error was formatted
 	bool FormatError(int Result, char *pError, int ErrorSize);
@@ -105,18 +109,22 @@ void CSqliteConnection::ToUnixTimestamp(const char *pTimestamp, char *aBuf, unsi
 	str_format(aBuf, BufferSize, "strftime('%%s', %s)", pTimestamp);
 }
 
-CSqliteConnection *CSqliteConnection::Copy()
-{
-	return new CSqliteConnection(m_aFilename, m_Setup);
-}
-
 bool CSqliteConnection::Connect(char *pError, int ErrorSize)
 {
 	if(m_InUse.exchange(true))
 	{
-		dbg_assert(0, "Tried connecting while the connection is in use");
+		dbg_assert(false, "Tried connecting while the connection is in use");
 	}
+	if(ConnectImpl(pError, ErrorSize))
+	{
+		m_InUse.store(false);
+		return true;
+	}
+	return false;
+}
 
+bool CSqliteConnection::ConnectImpl(char *pError, int ErrorSize)
+{
 	if(m_pDb != nullptr)
 	{
 		return false;
@@ -139,20 +147,32 @@ bool CSqliteConnection::Connect(char *pError, int ErrorSize)
 
 	if(m_Setup)
 	{
+		if(Execute("PRAGMA journal_mode=WAL", pError, ErrorSize))
+			return true;
 		char aBuf[1024];
-		FormatCreateRace(aBuf, sizeof(aBuf));
+		FormatCreateRace(aBuf, sizeof(aBuf), /* Backup */ false);
 		if(Execute(aBuf, pError, ErrorSize))
 			return true;
-		FormatCreateTeamrace(aBuf, sizeof(aBuf), "BLOB");
+		FormatCreateTeamrace(aBuf, sizeof(aBuf), "BLOB", /* Backup */ false);
 		if(Execute(aBuf, pError, ErrorSize))
 			return true;
 		FormatCreateMaps(aBuf, sizeof(aBuf));
 		if(Execute(aBuf, pError, ErrorSize))
 			return true;
-		FormatCreateSaves(aBuf, sizeof(aBuf));
+		FormatCreateSaves(aBuf, sizeof(aBuf), /* Backup */ false);
 		if(Execute(aBuf, pError, ErrorSize))
 			return true;
 		FormatCreatePoints(aBuf, sizeof(aBuf));
+		if(Execute(aBuf, pError, ErrorSize))
+			return true;
+
+		FormatCreateRace(aBuf, sizeof(aBuf), /* Backup */ true);
+		if(Execute(aBuf, pError, ErrorSize))
+			return true;
+		FormatCreateTeamrace(aBuf, sizeof(aBuf), "BLOB", /* Backup */ true);
+		if(Execute(aBuf, pError, ErrorSize))
+			return true;
+		FormatCreateSaves(aBuf, sizeof(aBuf), /* Backup */ true);
 		if(Execute(aBuf, pError, ErrorSize))
 			return true;
 		m_Setup = false;
@@ -178,7 +198,7 @@ bool CSqliteConnection::PrepareStatement(const char *pStmt, char *pError, int Er
 		pStmt,
 		-1, // pStmt can be any length
 		&m_pStmt,
-		NULL);
+		nullptr);
 	if(FormatError(Result, pError, ErrorSize))
 	{
 		return true;
@@ -189,14 +209,14 @@ bool CSqliteConnection::PrepareStatement(const char *pStmt, char *pError, int Er
 
 void CSqliteConnection::BindString(int Idx, const char *pString)
 {
-	int Result = sqlite3_bind_text(m_pStmt, Idx, pString, -1, NULL);
+	int Result = sqlite3_bind_text(m_pStmt, Idx, pString, -1, nullptr);
 	AssertNoError(Result);
 	m_Done = false;
 }
 
 void CSqliteConnection::BindBlob(int Idx, unsigned char *pBlob, int Size)
 {
-	int Result = sqlite3_bind_blob(m_pStmt, Idx, pBlob, Size, NULL);
+	int Result = sqlite3_bind_blob(m_pStmt, Idx, pBlob, Size, nullptr);
 	AssertNoError(Result);
 	m_Done = false;
 }
@@ -222,6 +242,13 @@ void CSqliteConnection::BindFloat(int Idx, float Value)
 	m_Done = false;
 }
 
+void CSqliteConnection::BindNull(int Idx)
+{
+	int Result = sqlite3_bind_null(m_pStmt, Idx);
+	AssertNoError(Result);
+	m_Done = false;
+}
+
 // Keep support for SQLite < 3.14 on older Linux distributions. MinGW does not
 // support __attribute__((weak)): https://sourceware.org/bugzilla/show_bug.cgi?id=9687
 #if defined(__GNUC__) && !defined(__MINGW32__)
@@ -336,7 +363,7 @@ const char *CSqliteConnection::MedianMapTime(char *pBuffer, int BufferSize) cons
 bool CSqliteConnection::Execute(const char *pQuery, char *pError, int ErrorSize)
 {
 	char *pErrorMsg;
-	int Result = sqlite3_exec(m_pDb, pQuery, NULL, NULL, &pErrorMsg);
+	int Result = sqlite3_exec(m_pDb, pQuery, nullptr, nullptr, &pErrorMsg);
 	if(Result != SQLITE_OK)
 	{
 		str_format(pError, ErrorSize, "error executing query: '%s'", pErrorMsg);
