@@ -11,6 +11,10 @@ CServerLogger::CServerLogger(CServer *pServer) :
 
 void CServerLogger::Log(const CLogMessage *pMessage)
 {
+	if(m_Filter.Filters(pMessage))
+	{
+		return;
+	}
 	m_PendingLock.lock();
 	if(m_MainThread == std::this_thread::get_id())
 	{
@@ -26,7 +30,8 @@ void CServerLogger::Log(const CLogMessage *pMessage)
 			m_vPending.clear();
 		}
 		m_PendingLock.unlock();
-		m_pServer->SendLogLine(pMessage);
+		if(m_pServer)
+			m_pServer->SendLogLine(pMessage);
 	}
 	else
 	{
