@@ -19,85 +19,68 @@
 
 // todo: rework this
 
-const char *CConsole::CResult::GetString(unsigned Index)
+const char *CConsole::CResult::GetString(unsigned Index) const
 {
 	if(Index >= m_NumArgs)
 		return "";
 	return m_apArgs[Index];
 }
 
-int CConsole::CResult::GetInteger(unsigned Index)
+int CConsole::CResult::GetInteger(unsigned Index) const
 {
 	if(Index >= m_NumArgs)
 		return 0;
 	return str_toint(m_apArgs[Index]);
 }
 
-float CConsole::CResult::GetFloat(unsigned Index)
+float CConsole::CResult::GetFloat(unsigned Index) const
 {
 	if(Index >= m_NumArgs)
 		return 0.0f;
 	return str_tofloat(m_apArgs[Index]);
 }
 
-ColorHSLA CConsole::CResult::GetColor(unsigned Index, bool Light)
+std::optional<ColorHSLA> CConsole::CResult::GetColor(unsigned Index, float DarkestLighting) const
 {
-	ColorHSLA Hsla = ColorHSLA(0, 0, 0);
 	if(Index >= m_NumArgs)
-		return Hsla;
+		return std::nullopt;
 
 	const char *pStr = m_apArgs[Index];
 	if(str_isallnum(pStr) || ((pStr[0] == '-' || pStr[0] == '+') && str_isallnum(pStr + 1))) // Teeworlds Color (Packed HSL)
 	{
-		Hsla = ColorHSLA(str_toulong_base(pStr, 10), true);
-		if(Light)
-			Hsla = Hsla.UnclampLighting();
+		unsigned long Value = str_toulong_base(pStr, 10);
+		if(Value == std::numeric_limits<unsigned long>::max())
+			return std::nullopt;
+		return ColorHSLA(Value, true).UnclampLighting(DarkestLighting);
 	}
-	else if(*pStr == '$') // Hex RGB
+	else if(*pStr == '$') // Hex RGB/RGBA
 	{
-		ColorRGBA Rgba = ColorRGBA(0, 0, 0, 1);
-		int Len = str_length(pStr);
-		if(Len == 4)
-		{
-			unsigned Num = str_toulong_base(pStr + 1, 16);
-			Rgba.r = (((Num >> 8) & 0x0F) + ((Num >> 4) & 0xF0)) / 255.0f;
-			Rgba.g = (((Num >> 4) & 0x0F) + ((Num >> 0) & 0xF0)) / 255.0f;
-			Rgba.b = (((Num >> 0) & 0x0F) + ((Num << 4) & 0xF0)) / 255.0f;
-		}
-		else if(Len == 7)
-		{
-			unsigned Num = str_toulong_base(pStr + 1, 16);
-			Rgba.r = ((Num >> 16) & 0xFF) / 255.0f;
-			Rgba.g = ((Num >> 8) & 0xFF) / 255.0f;
-			Rgba.b = ((Num >> 0) & 0xFF) / 255.0f;
-		}
+		auto ParsedColor = color_parse<ColorRGBA>(pStr + 1);
+		if(ParsedColor)
+			return color_cast<ColorHSLA>(ParsedColor.value());
 		else
-		{
-			return Hsla;
-		}
-
-		Hsla = color_cast<ColorHSLA>(Rgba);
+			return std::nullopt;
 	}
 	else if(!str_comp_nocase(pStr, "red"))
-		Hsla = ColorHSLA(0.0f / 6.0f, 1, .5f);
+		return ColorHSLA(0.0f / 6.0f, 1, .5f);
 	else if(!str_comp_nocase(pStr, "yellow"))
-		Hsla = ColorHSLA(1.0f / 6.0f, 1, .5f);
+		return ColorHSLA(1.0f / 6.0f, 1, .5f);
 	else if(!str_comp_nocase(pStr, "green"))
-		Hsla = ColorHSLA(2.0f / 6.0f, 1, .5f);
+		return ColorHSLA(2.0f / 6.0f, 1, .5f);
 	else if(!str_comp_nocase(pStr, "cyan"))
-		Hsla = ColorHSLA(3.0f / 6.0f, 1, .5f);
+		return ColorHSLA(3.0f / 6.0f, 1, .5f);
 	else if(!str_comp_nocase(pStr, "blue"))
-		Hsla = ColorHSLA(4.0f / 6.0f, 1, .5f);
+		return ColorHSLA(4.0f / 6.0f, 1, .5f);
 	else if(!str_comp_nocase(pStr, "magenta"))
-		Hsla = ColorHSLA(5.0f / 6.0f, 1, .5f);
+		return ColorHSLA(5.0f / 6.0f, 1, .5f);
 	else if(!str_comp_nocase(pStr, "white"))
-		Hsla = ColorHSLA(0, 0, 1);
+		return ColorHSLA(0, 0, 1);
 	else if(!str_comp_nocase(pStr, "gray"))
-		Hsla = ColorHSLA(0, 0, .5f);
+		return ColorHSLA(0, 0, .5f);
 	else if(!str_comp_nocase(pStr, "black"))
-		Hsla = ColorHSLA(0, 0, 0);
+		return ColorHSLA(0, 0, 0);
 
-	return Hsla;
+	return std::nullopt;
 }
 
 const IConsole::CCommandInfo *CConsole::CCommand::NextCommandInfo(int AccessLevel, int FlagMask) const
@@ -120,7 +103,7 @@ const IConsole::CCommandInfo *CConsole::FirstCommandInfo(int AccessLevel, int Fl
 			return pCommand;
 	}
 
-	return 0;
+	return nullptr;
 }
 
 // the maximum number of tokens occurs in a string of length CONSOLE_MAX_STR_LENGTH with tokens size 1 separated by single spaces
@@ -150,12 +133,12 @@ int CConsole::ParseStart(CResult *pResult, const char *pString, int Length)
 	return 0;
 }
 
-int CConsole::ParseArgs(CResult *pResult, const char *pFormat)
+int CConsole::ParseArgs(CResult *pResult, const char *pFormat, bool IsColor)
 {
 	char Command = *pFormat;
 	char *pStr;
 	int Optional = 0;
-	int Error = 0;
+	int Error = PARSEARGS_OK;
 
 	pResult->ResetVictim();
 
@@ -176,7 +159,7 @@ int CConsole::ParseArgs(CResult *pResult, const char *pFormat)
 			{
 				if(!Optional)
 				{
-					Error = 1;
+					Error = PARSEARGS_MISSING_VALUE;
 					break;
 				}
 
@@ -212,7 +195,7 @@ int CConsole::ParseArgs(CResult *pResult, const char *pFormat)
 							pStr++; // skip due to escape
 					}
 					else if(pStr[0] == 0)
-						return 1; // return error
+						return PARSEARGS_MISSING_VALUE; // return error
 
 					*pDst = *pStr;
 					pDst++;
@@ -226,7 +209,7 @@ int CConsole::ParseArgs(CResult *pResult, const char *pFormat)
 			}
 			else
 			{
-				char *pVictim = 0;
+				char *pVictim = nullptr;
 
 				pResult->AddArgument(pStr);
 				if(Command == 'v')
@@ -236,13 +219,7 @@ int CConsole::ParseArgs(CResult *pResult, const char *pFormat)
 
 				if(Command == 'r') // rest of the string
 					break;
-				else if(Command == 'v') // validate victim
-					pStr = str_skip_to_whitespace(pStr);
-				else if(Command == 'i') // validate int
-					pStr = str_skip_to_whitespace(pStr);
-				else if(Command == 'f') // validate float
-					pStr = str_skip_to_whitespace(pStr);
-				else if(Command == 's') // validate string
+				else if(Command == 'v' || Command == 'i' || Command == 'f' || Command == 's')
 					pStr = str_skip_to_whitespace(pStr);
 
 				if(pStr[0] != 0) // check for end of string
@@ -251,6 +228,32 @@ int CConsole::ParseArgs(CResult *pResult, const char *pFormat)
 					pStr++;
 				}
 
+				// validate args
+				if(Command == 'i')
+				{
+					// don't validate colors here
+					if(!IsColor)
+					{
+						int Value;
+						if(!str_toint(pResult->GetString(pResult->NumArguments() - 1), &Value) ||
+							Value == std::numeric_limits<int>::max() || Value == std::numeric_limits<int>::min())
+						{
+							Error = PARSEARGS_INVALID_INTEGER;
+							break;
+						}
+					}
+				}
+				else if(Command == 'f')
+				{
+					float Value;
+					if(!str_tofloat(pResult->GetString(pResult->NumArguments() - 1), &Value) ||
+						Value == std::numeric_limits<float>::max() || Value == std::numeric_limits<float>::min())
+					{
+						Error = PARSEARGS_INVALID_FLOAT;
+						break;
+					}
+				}
+
 				if(pVictim)
 				{
 					pResult->SetVictim(pVictim);
@@ -290,15 +293,6 @@ char CConsole::NextParam(const char *&pFormat)
 	return *pFormat;
 }
 
-char *CConsole::Format(char *pBuf, int Size, const char *pFrom, const char *pStr)
-{
-	char aTimeBuf[80];
-	str_timestamp_format(aTimeBuf, sizeof(aTimeBuf), FORMAT_TIME);
-
-	str_format(pBuf, Size, "[%s][%s]: %s", aTimeBuf, pFrom, pStr);
-	return pBuf;
-}
-
 LEVEL IConsole::ToLogLevel(int Level)
 {
 	switch(Level)
@@ -314,6 +308,15 @@ LEVEL IConsole::ToLogLevel(int Level)
 	return LEVEL_INFO;
 }
 
+int IConsole::ToLogLevelFilter(int Level)
+{
+	if(!(-3 <= Level && Level <= 2))
+	{
+		dbg_assert(0, "invalid log level filter");
+	}
+	return Level + 2;
+}
+
 LOG_COLOR ColorToLogColor(ColorRGBA Color)
 {
 	return LOG_COLOR{
@@ -322,11 +325,11 @@ LOG_COLOR ColorToLogColor(ColorRGBA Color)
 		(uint8_t)(Color.b * 255.0)};
 }
 
-void CConsole::Print(int Level, const char *pFrom, const char *pStr, ColorRGBA PrintColor)
+void CConsole::Print(int Level, const char *pFrom, const char *pStr, ColorRGBA PrintColor) const
 {
 	LEVEL LogLevel = IConsole::ToLogLevel(Level);
-	// if the color is pure white, use default terminal color
-	if(mem_comp(&PrintColor, &gs_ConsoleDefaultColor, sizeof(ColorRGBA)) != 0)
+	// if console colors are not enabled or if the color is pure white, use default terminal color
+	if(g_Config.m_ConsoleEnableColors && PrintColor != gs_ConsoleDefaultColor)
 	{
 		log_log_color(LogLevel, ColorToLogColor(PrintColor), pFrom, "%s", pStr);
 	}
@@ -373,9 +376,9 @@ bool CConsole::LineIsValid(const char *pStr)
 
 	do
 	{
-		CResult Result;
+		CResult Result(-1);
 		const char *pEnd = pStr;
-		const char *pNextPart = 0;
+		const char *pNextPart = nullptr;
 		int InString = 0;
 
 		while(*pEnd)
@@ -414,7 +417,7 @@ bool CConsole::LineIsValid(const char *pStr)
 	return true;
 }
 
-void CConsole::ExecuteLineStroked(int Stroke, const char *pStr, int ClientID, bool InterpretSemicolons)
+void CConsole::ExecuteLineStroked(int Stroke, const char *pStr, int ClientId, bool InterpretSemicolons)
 {
 	const char *pWithoutPrefix = str_startswith(pStr, "mc;");
 	if(pWithoutPrefix)
@@ -424,10 +427,9 @@ void CConsole::ExecuteLineStroked(int Stroke, const char *pStr, int ClientID, bo
 	}
 	while(pStr && *pStr)
 	{
-		CResult Result;
-		Result.m_ClientID = ClientID;
+		CResult Result(ClientId);
 		const char *pEnd = pStr;
-		const char *pNextPart = 0;
+		const char *pNextPart = nullptr;
 		int InString = 0;
 
 		while(*pEnd)
@@ -459,24 +461,28 @@ void CConsole::ExecuteLineStroked(int Stroke, const char *pStr, int ClientID, bo
 		if(!*Result.m_pCommand)
 			return;
 
-		CCommand *pCommand = FindCommand(Result.m_pCommand, m_FlagMask);
+		CCommand *pCommand;
+		if(ClientId == IConsole::CLIENT_ID_GAME)
+			pCommand = FindCommand(Result.m_pCommand, m_FlagMask | CFGFLAG_GAME);
+		else
+			pCommand = FindCommand(Result.m_pCommand, m_FlagMask);
 
 		if(pCommand)
 		{
-			if(ClientID == IConsole::CLIENT_ID_GAME && !(pCommand->m_Flags & CFGFLAG_GAME))
+			if(ClientId == IConsole::CLIENT_ID_GAME && !(pCommand->m_Flags & CFGFLAG_GAME))
 			{
 				if(Stroke)
 				{
-					char aBuf[96];
+					char aBuf[CMDLINE_LENGTH + 64];
 					str_format(aBuf, sizeof(aBuf), "Command '%s' cannot be executed from a map.", Result.m_pCommand);
 					Print(OUTPUT_LEVEL_STANDARD, "console", aBuf);
 				}
 			}
-			else if(ClientID == IConsole::CLIENT_ID_NO_GAME && pCommand->m_Flags & CFGFLAG_GAME)
+			else if(ClientId == IConsole::CLIENT_ID_NO_GAME && pCommand->m_Flags & CFGFLAG_GAME)
 			{
 				if(Stroke)
 				{
-					char aBuf[96];
+					char aBuf[CMDLINE_LENGTH + 64];
 					str_format(aBuf, sizeof(aBuf), "Command '%s' cannot be executed from a non-map config file.", Result.m_pCommand);
 					Print(OUTPUT_LEVEL_STANDARD, "console", aBuf);
 					str_format(aBuf, sizeof(aBuf), "Hint: Put the command in '%s.cfg' instead of '%s.map.cfg' ", g_Config.m_SvMap, g_Config.m_SvMap);
@@ -495,31 +501,44 @@ void CConsole::ExecuteLineStroked(int Stroke, const char *pStr, int ClientID, bo
 
 				if(Stroke || IsStrokeCommand)
 				{
-					if(ParseArgs(&Result, pCommand->m_pParams))
+					bool IsColor = false;
 					{
-						char aBuf[256];
-						str_format(aBuf, sizeof(aBuf), "Invalid arguments... Usage: %s %s", pCommand->m_pName, pCommand->m_pParams);
+						FCommandCallback pfnCallback = pCommand->m_pfnCallback;
+						void *pUserData = pCommand->m_pUserData;
+						TraverseChain(&pfnCallback, &pUserData);
+						IsColor = pfnCallback == &SColorConfigVariable::CommandCallback;
+					}
+
+					if(int Error = ParseArgs(&Result, pCommand->m_pParams, IsColor))
+					{
+						char aBuf[CMDLINE_LENGTH + 64];
+						if(Error == PARSEARGS_INVALID_INTEGER)
+							str_format(aBuf, sizeof(aBuf), "%s is not a valid integer.", Result.GetString(Result.NumArguments() - 1));
+						else if(Error == PARSEARGS_INVALID_FLOAT)
+							str_format(aBuf, sizeof(aBuf), "%s is not a valid decimal number.", Result.GetString(Result.NumArguments() - 1));
+						else
+							str_format(aBuf, sizeof(aBuf), "Invalid arguments. Usage: %s %s", pCommand->m_pName, pCommand->m_pParams);
 						Print(OUTPUT_LEVEL_STANDARD, "chatresp", aBuf);
 					}
 					else if(m_StoreCommands && pCommand->m_Flags & CFGFLAG_STORE)
 					{
-						m_ExecutionQueue.AddEntry();
-						m_ExecutionQueue.m_pLast->m_pfnCommandCallback = pCommand->m_pfnCallback;
-						m_ExecutionQueue.m_pLast->m_pCommandUserData = pCommand->m_pUserData;
-						m_ExecutionQueue.m_pLast->m_Result = Result;
+						m_vExecutionQueue.emplace_back(pCommand, Result);
 					}
 					else
 					{
 						if(pCommand->m_Flags & CMDFLAG_TEST && !g_Config.m_SvTestingCommands)
+						{
+							Print(OUTPUT_LEVEL_STANDARD, "console", "Test commands aren't allowed, enable them with 'sv_test_cmds 1' in your initial config.");
 							return;
+						}
 
 						if(m_pfnTeeHistorianCommandCallback && !(pCommand->m_Flags & CFGFLAG_NONTEEHISTORIC))
 						{
-							m_pfnTeeHistorianCommandCallback(ClientID, m_FlagMask, pCommand->m_pName, &Result, m_pTeeHistorianCommandUserdata);
+							m_pfnTeeHistorianCommandCallback(ClientId, m_FlagMask, pCommand->m_pName, &Result, m_pTeeHistorianCommandUserdata);
 						}
 
 						if(Result.GetVictim() == CResult::VICTIM_ME)
-							Result.SetVictim(ClientID);
+							Result.SetVictim(ClientId);
 
 						if(Result.HasVictim() && Result.GetVictim() == CResult::VICTIM_ALL)
 						{
@@ -541,17 +560,22 @@ void CConsole::ExecuteLineStroked(int Stroke, const char *pStr, int ClientID, bo
 			}
 			else if(Stroke)
 			{
-				char aBuf[256];
+				char aBuf[CMDLINE_LENGTH + 32];
 				str_format(aBuf, sizeof(aBuf), "Access for command %s denied.", Result.m_pCommand);
 				Print(OUTPUT_LEVEL_STANDARD, "console", aBuf);
 			}
 		}
 		else if(Stroke)
 		{
-			if(!m_pfnUnknownCommandCallback(Result.m_pCommand, m_pUnknownCommandUserdata))
+			// Pass the original string to the unknown command callback instead of the parsed command, as the latter
+			// ends at the first whitespace, which breaks for unknown commands (filenames) containing spaces.
+			if(!m_pfnUnknownCommandCallback(pStr, m_pUnknownCommandUserdata))
 			{
-				char aBuf[256];
-				str_format(aBuf, sizeof(aBuf), "No such command: %s.", Result.m_pCommand);
+				char aBuf[CMDLINE_LENGTH + 32];
+				if(m_FlagMask & CFGFLAG_CHAT)
+					str_format(aBuf, sizeof(aBuf), "No such command: %s. Use /cmdlist for a list of all commands.", Result.m_pCommand);
+				else
+					str_format(aBuf, sizeof(aBuf), "No such command: %s.", Result.m_pCommand);
 				Print(OUTPUT_LEVEL_STANDARD, "chatresp", aBuf);
 			}
 		}
@@ -588,32 +612,36 @@ CConsole::CCommand *CConsole::FindCommand(const char *pName, int FlagMask)
 		}
 	}
 
-	return 0x0;
+	return nullptr;
 }
 
-void CConsole::ExecuteLine(const char *pStr, int ClientID, bool InterpretSemicolons)
+void CConsole::ExecuteLine(const char *pStr, int ClientId, bool InterpretSemicolons)
 {
-	CConsole::ExecuteLineStroked(1, pStr, ClientID, InterpretSemicolons); // press it
-	CConsole::ExecuteLineStroked(0, pStr, ClientID, InterpretSemicolons); // then release it
+	CConsole::ExecuteLineStroked(1, pStr, ClientId, InterpretSemicolons); // press it
+	CConsole::ExecuteLineStroked(0, pStr, ClientId, InterpretSemicolons); // then release it
 }
 
-void CConsole::ExecuteLineFlag(const char *pStr, int FlagMask, int ClientID, bool InterpretSemicolons)
+void CConsole::ExecuteLineFlag(const char *pStr, int FlagMask, int ClientId, bool InterpretSemicolons)
 {
 	int Temp = m_FlagMask;
 	m_FlagMask = FlagMask;
-	ExecuteLine(pStr, ClientID, InterpretSemicolons);
+	ExecuteLine(pStr, ClientId, InterpretSemicolons);
 	m_FlagMask = Temp;
 }
 
-void CConsole::ExecuteFile(const char *pFilename, int ClientID, bool LogFailure, int StorageType)
+bool CConsole::ExecuteFile(const char *pFilename, int ClientId, bool LogFailure, int StorageType)
 {
-	// make sure that this isn't being executed already
+	int Count = 0;
+	// make sure that this isn't being executed already and that recursion limit isn't met
 	for(CExecFile *pCur = m_pFirstExec; pCur; pCur = pCur->m_pPrev)
-		if(str_comp(pFilename, pCur->m_pFilename) == 0)
-			return;
+	{
+		Count++;
 
+		if(str_comp(pFilename, pCur->m_pFilename) == 0 || Count > FILE_RECURSION_LIMIT)
+			return false;
+	}
 	if(!m_pStorage)
-		return;
+		return false;
 
 	// push this one to the stack
 	CExecFile ThisFile;
@@ -623,22 +651,20 @@ void CConsole::ExecuteFile(const char *pFilename, int ClientID, bool LogFailure,
 	m_pFirstExec = &ThisFile;
 
 	// exec the file
-	IOHANDLE File = m_pStorage->OpenFile(pFilename, IOFLAG_READ | IOFLAG_SKIP_BOM, StorageType);
-
-	char aBuf[128];
-	if(File)
+	CLineReader LineReader;
+	bool Success = false;
+	char aBuf[32 + IO_MAX_PATH_LENGTH];
+	if(LineReader.OpenFile(m_pStorage->OpenFile(pFilename, IOFLAG_READ, StorageType)))
 	{
-		char *pLine;
-		CLineReader Reader;
-
 		str_format(aBuf, sizeof(aBuf), "executing '%s'", pFilename);
 		Print(IConsole::OUTPUT_LEVEL_STANDARD, "console", aBuf);
-		Reader.Init(File);
 
-		while((pLine = Reader.Get()))
-			ExecuteLine(pLine, ClientID);
+		while(const char *pLine = LineReader.Get())
+		{
+			ExecuteLine(pLine, ClientId);
+		}
 
-		io_close(File);
+		Success = true;
 	}
 	else if(LogFailure)
 	{
@@ -647,6 +673,7 @@ void CConsole::ExecuteFile(const char *pFilename, int ClientID, bool LogFailure,
 	}
 
 	m_pFirstExec = pPrev;
+	return Success;
 }
 
 void CConsole::Con_Echo(IResult *pResult, void *pUserData)
@@ -662,7 +689,7 @@ void CConsole::Con_Exec(IResult *pResult, void *pUserData)
 void CConsole::ConCommandAccess(IResult *pResult, void *pUser)
 {
 	CConsole *pConsole = static_cast<CConsole *>(pUser);
-	char aBuf[128];
+	char aBuf[CMDLINE_LENGTH + 64];
 	CCommand *pCommand = pConsole->FindCommand(pResult->GetString(0), CFGFLAG_SERVER);
 	if(pCommand)
 	{
@@ -707,9 +734,9 @@ void CConsole::ConCommandStatus(IResult *pResult, void *pUser)
 				if(Used > 0)
 				{
 					Used += 2;
-					str_append(aBuf, ", ", sizeof(aBuf));
+					str_append(aBuf, ", ");
 				}
-				str_append(aBuf, pCommand->m_pName, sizeof(aBuf));
+				str_append(aBuf, pCommand->m_pName);
 				Used += Length;
 			}
 			else
@@ -728,143 +755,13 @@ void CConsole::ConCommandStatus(IResult *pResult, void *pUser)
 void CConsole::ConUserCommandStatus(IResult *pResult, void *pUser)
 {
 	CConsole *pConsole = static_cast<CConsole *>(pUser);
-	CResult Result;
+	CResult Result(pResult->m_ClientId);
 	Result.m_pCommand = "access_status";
 	char aBuf[4];
-	str_format(aBuf, sizeof(aBuf), "%d", IConsole::ACCESS_LEVEL_USER);
+	str_format(aBuf, sizeof(aBuf), "%d", (int)IConsole::ACCESS_LEVEL_USER);
 	Result.AddArgument(aBuf);
 
-	pConsole->ConCommandStatus(&Result, pConsole);
-}
-
-struct CIntVariableData
-{
-	IConsole *m_pConsole;
-	int *m_pVariable;
-	int m_Min;
-	int m_Max;
-	int m_OldValue;
-};
-
-struct CColVariableData
-{
-	IConsole *m_pConsole;
-	unsigned *m_pVariable;
-	bool m_Light;
-	bool m_Alpha;
-	unsigned m_OldValue;
-};
-
-struct CStrVariableData
-{
-	IConsole *m_pConsole;
-	char *m_pStr;
-	int m_MaxSize;
-	char *m_pOldValue;
-};
-
-static void IntVariableCommand(IConsole::IResult *pResult, void *pUserData)
-{
-	CIntVariableData *pData = (CIntVariableData *)pUserData;
-
-	if(pResult->NumArguments())
-	{
-		int Val = pResult->GetInteger(0);
-
-		// do clamping
-		if(pData->m_Min != pData->m_Max)
-		{
-			if(Val < pData->m_Min)
-				Val = pData->m_Min;
-			if(pData->m_Max != 0 && Val > pData->m_Max)
-				Val = pData->m_Max;
-		}
-
-		*(pData->m_pVariable) = Val;
-		if(pResult->m_ClientID != IConsole::CLIENT_ID_GAME)
-			pData->m_OldValue = Val;
-	}
-	else
-	{
-		char aBuf[32];
-		str_format(aBuf, sizeof(aBuf), "Value: %d", *(pData->m_pVariable));
-		pData->m_pConsole->Print(IConsole::OUTPUT_LEVEL_STANDARD, "console", aBuf);
-	}
-}
-
-static void ColVariableCommand(IConsole::IResult *pResult, void *pUserData)
-{
-	CColVariableData *pData = (CColVariableData *)pUserData;
-
-	if(pResult->NumArguments())
-	{
-		ColorHSLA Col = pResult->GetColor(0, pData->m_Light);
-		int Val = Col.Pack(pData->m_Light ? 0.5f : 0.0f, pData->m_Alpha);
-
-		*(pData->m_pVariable) = Val;
-		if(pResult->m_ClientID != IConsole::CLIENT_ID_GAME)
-			pData->m_OldValue = Val;
-	}
-	else
-	{
-		char aBuf[256];
-		str_format(aBuf, sizeof(aBuf), "Value: %u", *(pData->m_pVariable));
-		pData->m_pConsole->Print(IConsole::OUTPUT_LEVEL_STANDARD, "console", aBuf);
-
-		ColorHSLA Hsla(*(pData->m_pVariable), true);
-		if(pData->m_Light)
-			Hsla = Hsla.UnclampLighting();
-		str_format(aBuf, sizeof(aBuf), "H: %d°, S: %d%%, L: %d%%", round_truncate(Hsla.h * 360), round_truncate(Hsla.s * 100), round_truncate(Hsla.l * 100));
-		pData->m_pConsole->Print(IConsole::OUTPUT_LEVEL_STANDARD, "console", aBuf);
-
-		ColorRGBA Rgba = color_cast<ColorRGBA>(Hsla);
-		str_format(aBuf, sizeof(aBuf), "R: %d, G: %d, B: %d, #%06X", round_truncate(Rgba.r * 255), round_truncate(Rgba.g * 255), round_truncate(Rgba.b * 255), Rgba.Pack(false));
-		pData->m_pConsole->Print(IConsole::OUTPUT_LEVEL_STANDARD, "console", aBuf);
-
-		if(pData->m_Alpha)
-		{
-			str_format(aBuf, sizeof(aBuf), "A: %d%%", round_truncate(Hsla.a * 100));
-			pData->m_pConsole->Print(IConsole::OUTPUT_LEVEL_STANDARD, "console", aBuf);
-		}
-	}
-}
-
-static void StrVariableCommand(IConsole::IResult *pResult, void *pUserData)
-{
-	CStrVariableData *pData = (CStrVariableData *)pUserData;
-
-	if(pResult->NumArguments())
-	{
-		const char *pString = pResult->GetString(0);
-		if(!str_utf8_check(pString))
-		{
-			char aTemp[4];
-			int Length = 0;
-			while(*pString)
-			{
-				int Size = str_utf8_encode(aTemp, static_cast<unsigned char>(*pString++));
-				if(Length + Size < pData->m_MaxSize)
-				{
-					mem_copy(pData->m_pStr + Length, aTemp, Size);
-					Length += Size;
-				}
-				else
-					break;
-			}
-			pData->m_pStr[Length] = 0;
-		}
-		else
-			str_copy(pData->m_pStr, pString, pData->m_MaxSize);
-
-		if(pResult->m_ClientID != IConsole::CLIENT_ID_GAME)
-			str_copy(pData->m_pOldValue, pData->m_pStr, pData->m_MaxSize);
-	}
-	else
-	{
-		char aBuf[1024];
-		str_format(aBuf, sizeof(aBuf), "Value: %s", pData->m_pStr);
-		pData->m_pConsole->Print(IConsole::OUTPUT_LEVEL_STANDARD, "console", aBuf);
-	}
+	CConsole::ConCommandStatus(&Result, pConsole);
 }
 
 void CConsole::TraverseChain(FCommandCallback *ppfnCallback, void **ppUserData)
@@ -877,109 +774,26 @@ void CConsole::TraverseChain(FCommandCallback *ppfnCallback, void **ppUserData)
 	}
 }
 
-void CConsole::ConToggle(IConsole::IResult *pResult, void *pUser)
-{
-	CConsole *pConsole = static_cast<CConsole *>(pUser);
-	char aBuf[128] = {0};
-	CCommand *pCommand = pConsole->FindCommand(pResult->GetString(0), pConsole->m_FlagMask);
-	if(pCommand)
-	{
-		FCommandCallback pfnCallback = pCommand->m_pfnCallback;
-		void *pUserData = pCommand->m_pUserData;
-		TraverseChain(&pfnCallback, &pUserData);
-		if(pfnCallback == IntVariableCommand)
-		{
-			CIntVariableData *pData = static_cast<CIntVariableData *>(pUserData);
-			int Val = *(pData->m_pVariable) == pResult->GetInteger(1) ? pResult->GetInteger(2) : pResult->GetInteger(1);
-			str_format(aBuf, sizeof(aBuf), "%s %i", pResult->GetString(0), Val);
-			pConsole->ExecuteLine(aBuf);
-			aBuf[0] = 0;
-		}
-		else if(pfnCallback == StrVariableCommand)
-		{
-			CStrVariableData *pData = static_cast<CStrVariableData *>(pUserData);
-			const char *pStr = !str_comp(pData->m_pStr, pResult->GetString(1)) ? pResult->GetString(2) : pResult->GetString(1);
-			str_format(aBuf, sizeof(aBuf), "%s \"", pResult->GetString(0));
-			char *pDst = aBuf + str_length(aBuf);
-			str_escape(&pDst, pStr, aBuf + sizeof(aBuf));
-			str_append(aBuf, "\"", sizeof(aBuf));
-			pConsole->ExecuteLine(aBuf);
-			aBuf[0] = 0;
-		}
-		else if(pfnCallback == ColVariableCommand)
-		{
-			CColVariableData *pData = static_cast<CColVariableData *>(pUserData);
-			bool Light = pData->m_Light;
-			float Darkest = Light ? 0.5f : 0.0f;
-			bool Alpha = pData->m_Alpha;
-			unsigned Cur = *pData->m_pVariable;
-			ColorHSLA Val = Cur == pResult->GetColor(1, Light).Pack(Darkest, Alpha) ? pResult->GetColor(2, Light) : pResult->GetColor(1, Light);
-
-			str_format(aBuf, sizeof(aBuf), "%s %u", pResult->GetString(0), Val.Pack(Darkest, Alpha));
-			pConsole->ExecuteLine(aBuf);
-			aBuf[0] = 0;
-		}
-		else
-			str_format(aBuf, sizeof(aBuf), "Invalid command: '%s'.", pResult->GetString(0));
-	}
-	else
-		str_format(aBuf, sizeof(aBuf), "No such command: '%s'.", pResult->GetString(0));
-
-	if(aBuf[0] != 0)
-		pConsole->Print(OUTPUT_LEVEL_STANDARD, "console", aBuf);
-}
-
-void CConsole::ConToggleStroke(IConsole::IResult *pResult, void *pUser)
-{
-	CConsole *pConsole = static_cast<CConsole *>(pUser);
-	char aBuf[128] = {0};
-	CCommand *pCommand = pConsole->FindCommand(pResult->GetString(1), pConsole->m_FlagMask);
-	if(pCommand)
-	{
-		FCommandCallback pfnCallback = pCommand->m_pfnCallback;
-		void *pUserData = pCommand->m_pUserData;
-		TraverseChain(&pfnCallback, &pUserData);
-		if(pfnCallback == IntVariableCommand)
-		{
-			int Val = pResult->GetInteger(0) == 0 ? pResult->GetInteger(3) : pResult->GetInteger(2);
-			str_format(aBuf, sizeof(aBuf), "%s %i", pResult->GetString(1), Val);
-			pConsole->ExecuteLine(aBuf);
-			aBuf[0] = 0;
-		}
-		else
-			str_format(aBuf, sizeof(aBuf), "Invalid command: '%s'.", pResult->GetString(1));
-	}
-	else
-		str_format(aBuf, sizeof(aBuf), "No such command: '%s'.", pResult->GetString(1));
-
-	if(aBuf[0] != 0)
-		pConsole->Print(OUTPUT_LEVEL_STANDARD, "console", aBuf);
-}
-
 CConsole::CConsole(int FlagMask)
 {
 	m_FlagMask = FlagMask;
 	m_AccessLevel = ACCESS_LEVEL_ADMIN;
-	m_pRecycleList = 0;
+	m_pRecycleList = nullptr;
 	m_TempCommands.Reset();
 	m_StoreCommands = true;
 	m_apStrokeStr[0] = "0";
 	m_apStrokeStr[1] = "1";
-	m_ExecutionQueue.Reset();
-	m_pFirstCommand = 0;
-	m_pFirstExec = 0;
-	m_pfnTeeHistorianCommandCallback = 0;
-	m_pTeeHistorianCommandUserdata = 0;
+	m_pFirstCommand = nullptr;
+	m_pFirstExec = nullptr;
+	m_pfnTeeHistorianCommandCallback = nullptr;
+	m_pTeeHistorianCommandUserdata = nullptr;
 
-	m_pStorage = 0;
+	m_pStorage = nullptr;
 
 	// register some basic commands
 	Register("echo", "r[text]", CFGFLAG_SERVER, Con_Echo, this, "Echo the text");
 	Register("exec", "r[file]", CFGFLAG_SERVER | CFGFLAG_CLIENT, Con_Exec, this, "Execute the specified file");
 
-	Register("toggle", "s[config-option] i[value 1] i[value 2]", CFGFLAG_SERVER | CFGFLAG_CLIENT, ConToggle, this, "Toggle config value");
-	Register("+toggle", "s[config-option] i[value 1] i[value 2]", CFGFLAG_CLIENT, ConToggleStroke, this, "Toggle config value via keypress");
-
 	Register("access_level", "s[command] ?i[accesslevel]", CFGFLAG_SERVER, ConCommandAccess, this, "Specify command accessibility (admin = 0, moderator = 1, helper = 2, all = 3)");
 	Register("access_status", "i[accesslevel]", CFGFLAG_SERVER, ConCommandStatus, this, "List all commands which are accessible for admin = 0, moderator = 1, helper = 2, all = 3");
 	Register("cmdlist", "", CFGFLAG_SERVER | CFGFLAG_CHAT, ConUserCommandStatus, this, "List all commands which are accessible for users");
@@ -1016,35 +830,7 @@ CConsole::~CConsole()
 
 void CConsole::Init()
 {
-	m_pConfig = Kernel()->RequestInterface<IConfigManager>()->Values();
 	m_pStorage = Kernel()->RequestInterface<IStorage>();
-
-// TODO: this should disappear
-#define MACRO_CONFIG_INT(Name, ScriptName, Def, Min, Max, Flags, Desc) \
-	{ \
-		static CIntVariableData Data = {this, &g_Config.m_##Name, Min, Max, Def}; \
-		Register(#ScriptName, "?i", Flags, IntVariableCommand, &Data, Desc " (default: " #Def ", min: " #Min ", max: " #Max ")"); \
-	}
-
-#define MACRO_CONFIG_COL(Name, ScriptName, Def, Flags, Desc) \
-	{ \
-		static CColVariableData Data = {this, &g_Config.m_##Name, static_cast<bool>((Flags)&CFGFLAG_COLLIGHT), \
-			static_cast<bool>((Flags)&CFGFLAG_COLALPHA), Def}; \
-		Register(#ScriptName, "?i", Flags, ColVariableCommand, &Data, Desc " (default: " #Def ")"); \
-	}
-
-#define MACRO_CONFIG_STR(Name, ScriptName, Len, Def, Flags, Desc) \
-	{ \
-		static char s_aOldValue[Len] = Def; \
-		static CStrVariableData Data = {this, g_Config.m_##Name, Len, s_aOldValue}; \
-		Register(#ScriptName, "?r", Flags, StrVariableCommand, &Data, Desc " (default: " #Def ", max length: " #Len ")"); \
-	}
-
-#include "config_variables.h"
-
-#undef MACRO_CONFIG_INT
-#undef MACRO_CONFIG_COL
-#undef MACRO_CONFIG_STR
 }
 
 void CConsole::ParseArguments(int NumArgs, const char **ppArguments)
@@ -1078,7 +864,7 @@ void CConsole::AddCommandSorted(CCommand *pCommand)
 		if(m_pFirstCommand && m_pFirstCommand->m_pNext)
 			pCommand->m_pNext = m_pFirstCommand;
 		else
-			pCommand->m_pNext = 0;
+			pCommand->m_pNext = nullptr;
 		m_pFirstCommand = pCommand;
 	}
 	else
@@ -1100,7 +886,7 @@ void CConsole::Register(const char *pName, const char *pParams,
 {
 	CCommand *pCommand = FindCommand(pName, Flags);
 	bool DoAdd = false;
-	if(pCommand == 0)
+	if(pCommand == nullptr)
 	{
 		pCommand = new CCommand();
 		DoAdd = true;
@@ -1148,8 +934,8 @@ void CConsole::RegisterTemp(const char *pName, const char *pParams, int Flags, c
 		pCommand->m_pParams = pMem;
 	}
 
-	pCommand->m_pfnCallback = 0;
-	pCommand->m_pUserData = 0;
+	pCommand->m_pfnCallback = nullptr;
+	pCommand->m_pUserData = nullptr;
 	pCommand->m_Flags = Flags;
 	pCommand->m_Temp = true;
 
@@ -1161,7 +947,7 @@ void CConsole::DeregisterTemp(const char *pName)
 	if(!m_pFirstCommand)
 		return;
 
-	CCommand *pRemoved = 0;
+	CCommand *pRemoved = nullptr;
 
 	// remove temp entry from command list
 	if(m_pFirstCommand->m_Temp && str_comp(m_pFirstCommand->m_pName, pName) == 0)
@@ -1207,7 +993,7 @@ void CConsole::DeregisterTempAll()
 	}
 
 	m_TempCommands.Reset();
-	m_pRecycleList = 0;
+	m_pRecycleList = nullptr;
 }
 
 void CConsole::Con_Chain(IResult *pResult, void *pUserData)
@@ -1245,9 +1031,11 @@ void CConsole::StoreCommands(bool Store)
 {
 	if(!Store)
 	{
-		for(CExecutionQueue::CQueueEntry *pEntry = m_ExecutionQueue.m_pFirst; pEntry; pEntry = pEntry->m_pNext)
-			pEntry->m_pfnCommandCallback(&pEntry->m_Result, pEntry->m_pCommandUserData);
-		m_ExecutionQueue.Reset();
+		for(CExecutionQueueEntry &Entry : m_vExecutionQueue)
+		{
+			Entry.m_pCommand->m_pfnCallback(&Entry.m_Result, Entry.m_pCommand->m_pUserData);
+		}
+		m_vExecutionQueue.clear();
 	}
 	m_StoreCommands = Store;
 }
@@ -1263,49 +1051,12 @@ const IConsole::CCommandInfo *CConsole::GetCommandInfo(const char *pName, int Fl
 		}
 	}
 
-	return 0;
+	return nullptr;
 }
 
-extern IConsole *CreateConsole(int FlagMask) { return new CConsole(FlagMask); }
+std::unique_ptr<IConsole> CreateConsole(int FlagMask) { return std::make_unique<CConsole>(FlagMask); }
 
-void CConsole::ResetServerGameSettings()
-{
-#define MACRO_CONFIG_INT(Name, ScriptName, Def, Min, Max, Flags, Desc) \
-	{ \
-		if(((Flags) & (CFGFLAG_SERVER | CFGFLAG_GAME)) == (CFGFLAG_SERVER | CFGFLAG_GAME)) \
-		{ \
-			CCommand *pCommand = FindCommand(#ScriptName, CFGFLAG_SERVER); \
-			void *pUserData = pCommand->m_pUserData; \
-			FCommandCallback pfnCallback = pCommand->m_pfnCallback; \
-			TraverseChain(&pfnCallback, &pUserData); \
-			CIntVariableData *pData = (CIntVariableData *)pUserData; \
-			*pData->m_pVariable = pData->m_OldValue; \
-		} \
-	}
-
-#define MACRO_CONFIG_COL(Name, ScriptName, Def, Save, Desc) MACRO_CONFIG_INT(Name, ScriptName, Def, 0, 0, Save, Desc)
-
-#define MACRO_CONFIG_STR(Name, ScriptName, Len, Def, Flags, Desc) \
-	{ \
-		if(((Flags) & (CFGFLAG_SERVER | CFGFLAG_GAME)) == (CFGFLAG_SERVER | CFGFLAG_GAME)) \
-		{ \
-			CCommand *pCommand = FindCommand(#ScriptName, CFGFLAG_SERVER); \
-			void *pUserData = pCommand->m_pUserData; \
-			FCommandCallback pfnCallback = pCommand->m_pfnCallback; \
-			TraverseChain(&pfnCallback, &pUserData); \
-			CStrVariableData *pData = (CStrVariableData *)pUserData; \
-			str_copy(pData->m_pOldValue, pData->m_pStr, pData->m_MaxSize); \
-		} \
-	}
-
-#include "config_variables.h"
-
-#undef MACRO_CONFIG_INT
-#undef MACRO_CONFIG_COL
-#undef MACRO_CONFIG_STR
-}
-
-int CConsole::CResult::GetVictim()
+int CConsole::CResult::GetVictim() const
 {
 	return m_Victim;
 }
@@ -1315,7 +1066,7 @@ void CConsole::CResult::ResetVictim()
 	m_Victim = VICTIM_NONE;
 }
 
-bool CConsole::CResult::HasVictim()
+bool CConsole::CResult::HasVictim() const
 {
 	return m_Victim != VICTIM_NONE;
 }
