@@ -5,6 +5,7 @@
 
 #include "memheap.h"
 #include <base/math.h>
+#include <base/system.h>
 #include <engine/console.h>
 #include <engine/storage.h>
 
@@ -46,8 +47,7 @@ class CConsole : public IConsole
 	};
 
 	CExecFile *m_pFirstExec;
-	class CConfig *m_pConfig;
-	class IStorage *m_pStorage;
+	IStorage *m_pStorage;
 	int m_AccessLevel;
 
 	CCommand *m_pRecycleList;
@@ -58,12 +58,10 @@ class CConsole : public IConsole
 	static void Con_Chain(IResult *pResult, void *pUserData);
 	static void Con_Echo(IResult *pResult, void *pUserData);
 	static void Con_Exec(IResult *pResult, void *pUserData);
-	static void ConToggle(IResult *pResult, void *pUser);
-	static void ConToggleStroke(IResult *pResult, void *pUser);
 	static void ConCommandAccess(IResult *pResult, void *pUser);
 	static void ConCommandStatus(IConsole::IResult *pResult, void *pUser);
 
-	void ExecuteLineStroked(int Stroke, const char *pStr, int ClientID = -1, bool InterpretSemicolons = true) override;
+	void ExecuteLineStroked(int Stroke, const char *pStr, int ClientId = -1, bool InterpretSemicolons = true) override;
 
 	FTeeHistorianCommandCallback m_pfnTeeHistorianCommandCallback;
 	void *m_pTeeHistorianCommandUserdata;
@@ -86,27 +84,23 @@ class CConsole : public IConsole
 		const char *m_pCommand;
 		const char *m_apArgs[MAX_PARTS];
 
-		CResult()
-
+		CResult(int ClientId) :
+			IResult(ClientId)
 		{
 			mem_zero(m_aStringStorage, sizeof(m_aStringStorage));
-			m_pArgsStart = 0;
-			m_pCommand = 0;
+			m_pArgsStart = nullptr;
+			m_pCommand = nullptr;
 			mem_zero(m_apArgs, sizeof(m_apArgs));
 		}
 
-		CResult &operator=(const CResult &Other)
+		CResult(const CResult &Other) :
+			IResult(Other)
 		{
-			if(this != &Other)
-			{
-				IResult::operator=(Other);
-				mem_copy(m_aStringStorage, Other.m_aStringStorage, sizeof(m_aStringStorage));
-				m_pArgsStart = m_aStringStorage + (Other.m_pArgsStart - Other.m_aStringStorage);
-				m_pCommand = m_aStringStorage + (Other.m_pCommand - Other.m_aStringStorage);
-				for(unsigned i = 0; i < Other.m_NumArgs; ++i)
-					m_apArgs[i] = m_aStringStorage + (Other.m_apArgs[i] - Other.m_aStringStorage);
-			}
-			return *this;
+			mem_copy(m_aStringStorage, Other.m_aStringStorage, sizeof(m_aStringStorage));
+			m_pArgsStart = m_aStringStorage + (Other.m_pArgsStart - Other.m_aStringStorage);
+			m_pCommand = m_aStringStorage + (Other.m_pCommand - Other.m_aStringStorage);
+			for(unsigned i = 0; i < Other.m_NumArgs; ++i)
+				m_apArgs[i] = m_aStringStorage + (Other.m_apArgs[i] - Other.m_aStringStorage);
 		}
 
 		void AddArgument(const char *pArg)
@@ -114,10 +108,10 @@ class CConsole : public IConsole
 			m_apArgs[m_NumArgs++] = pArg;
 		}
 
-		const char *GetString(unsigned Index) override;
-		int GetInteger(unsigned Index) override;
-		float GetFloat(unsigned Index) override;
-		ColorHSLA GetColor(unsigned Index, bool Light) override;
+		const char *GetString(unsigned Index) const override;
+		int GetInteger(unsigned Index) const override;
+		float GetFloat(unsigned Index) const override;
+		std::optional<ColorHSLA> GetColor(unsigned Index, float DarkestLighting) const override;
 
 		void RemoveArgument(unsigned Index) override
 		{
@@ -125,7 +119,7 @@ class CConsole : public IConsole
 			for(unsigned i = Index; i < m_NumArgs - 1; i++)
 				m_apArgs[i] = m_apArgs[i + 1];
 
-			m_apArgs[m_NumArgs--] = 0;
+			m_apArgs[m_NumArgs--] = nullptr;
 		}
 
 		// DDRace
@@ -139,14 +133,23 @@ class CConsole : public IConsole
 
 		int m_Victim;
 		void ResetVictim();
-		bool HasVictim();
+		bool HasVictim() const;
 		void SetVictim(int Victim);
 		void SetVictim(const char *pVictim);
-		int GetVictim() override;
+		int GetVictim() const override;
 	};
 
 	int ParseStart(CResult *pResult, const char *pString, int Length);
-	int ParseArgs(CResult *pResult, const char *pFormat);
+
+	enum
+	{
+		PARSEARGS_OK = 0,
+		PARSEARGS_MISSING_VALUE,
+		PARSEARGS_INVALID_INTEGER,
+		PARSEARGS_INVALID_FLOAT,
+	};
+
+	int ParseArgs(CResult *pResult, const char *pFormat, bool IsColor = false);
 
 	/*
 	this function will set pFormat to the next parameter (i,s,r,v,?) it contains and
@@ -156,43 +159,23 @@ class CConsole : public IConsole
 	*/
 	char NextParam(const char *&pFormat);
 
-	class CExecutionQueue
+	class CExecutionQueueEntry
 	{
-		CHeap m_Queue;
-
 	public:
-		struct CQueueEntry
-		{
-			CQueueEntry *m_pNext;
-			FCommandCallback m_pfnCommandCallback;
-			void *m_pCommandUserData;
-			CResult m_Result;
-		} * m_pFirst, *m_pLast;
-
-		void AddEntry()
-		{
-			CQueueEntry *pEntry = static_cast<CQueueEntry *>(m_Queue.Allocate(sizeof(CQueueEntry)));
-			pEntry->m_pNext = 0;
-			if(!m_pFirst)
-				m_pFirst = pEntry;
-			if(m_pLast)
-				m_pLast->m_pNext = pEntry;
-			m_pLast = pEntry;
-			(void)new(&(pEntry->m_Result)) CResult;
-		}
-		void Reset()
-		{
-			m_Queue.Reset();
-			m_pFirst = m_pLast = 0;
-		}
-	} m_ExecutionQueue;
+		CCommand *m_pCommand;
+		CResult m_Result;
+		CExecutionQueueEntry(CCommand *pCommand, CResult Result) :
+			m_pCommand(pCommand),
+			m_Result(Result) {}
+	};
+	std::vector<CExecutionQueueEntry> m_vExecutionQueue;
 
 	void AddCommandSorted(CCommand *pCommand);
 	CCommand *FindCommand(const char *pName, int FlagMask);
 
-public:
-	CConfig *Config() { return m_pConfig; }
+	bool m_Cheated;
 
+public:
 	CConsole(int FlagMask);
 	~CConsole();
 
@@ -210,21 +193,24 @@ public:
 	void StoreCommands(bool Store) override;
 
 	bool LineIsValid(const char *pStr) override;
-	void ExecuteLine(const char *pStr, int ClientID = -1, bool InterpretSemicolons = true) override;
-	void ExecuteLineFlag(const char *pStr, int FlagMask, int ClientID = -1, bool InterpretSemicolons = true) override;
-	void ExecuteFile(const char *pFilename, int ClientID = -1, bool LogFailure = false, int StorageType = IStorage::TYPE_ALL) override;
+	void ExecuteLine(const char *pStr, int ClientId = -1, bool InterpretSemicolons = true) override;
+	void ExecuteLineFlag(const char *pStr, int FlagMask, int ClientId = -1, bool InterpretSemicolons = true) override;
+	bool ExecuteFile(const char *pFilename, int ClientId = -1, bool LogFailure = false, int StorageType = IStorage::TYPE_ALL) override;
 
-	char *Format(char *pBuf, int Size, const char *pFrom, const char *pStr) override;
-	void Print(int Level, const char *pFrom, const char *pStr, ColorRGBA PrintColor = gs_ConsoleDefaultColor) override;
+	void Print(int Level, const char *pFrom, const char *pStr, ColorRGBA PrintColor = gs_ConsoleDefaultColor) const override;
 	void SetTeeHistorianCommandCallback(FTeeHistorianCommandCallback pfnCallback, void *pUser) override;
 	void SetUnknownCommandCallback(FUnknownCommandCallback pfnCallback, void *pUser) override;
 	void InitChecksum(CChecksumData *pData) const override;
 
 	void SetAccessLevel(int AccessLevel) override { m_AccessLevel = clamp(AccessLevel, (int)(ACCESS_LEVEL_ADMIN), (int)(ACCESS_LEVEL_USER)); }
-	void ResetServerGameSettings() override;
+
 	// DDRace
 
 	static void ConUserCommandStatus(IConsole::IResult *pResult, void *pUser);
+
+	bool Cheated() const override { return m_Cheated; }
+
+	int FlagMask() const override { return m_FlagMask; }
 	void SetFlagMask(int FlagMask) override { m_FlagMask = FlagMask; }
 };
 
