@@ -1,7 +1,7 @@
 #ifndef ENGINE_SHARED_CSV_H
 #define ENGINE_SHARED_CSV_H
 
-#include <base/system.h>
+#include <base/types.h>
 
 void CsvWrite(IOHANDLE File, int NumColumns, const char *const *ppColumns);
 
