@@ -38,13 +38,13 @@ class CEcon
 
 	bool m_Ready;
 	int m_PrintCBIndex;
-	int m_UserClientID;
+	int m_UserClientId;
 
 	static void SendLineCB(const char *pLine, void *pUserData, ColorRGBA PrintColor = {1, 1, 1, 1});
 	static void ConLogout(IConsole::IResult *pResult, void *pUserData);
 
-	static int NewClientCallback(int ClientID, void *pUser);
-	static int DelClientCallback(int ClientID, const char *pReason, void *pUser);
+	static int NewClientCallback(int ClientId, void *pUser);
+	static int DelClientCallback(int ClientId, const char *pReason, void *pUser);
 
 public:
 	CEcon();
@@ -52,7 +52,7 @@ public:
 
 	void Init(CConfig *pConfig, IConsole *pConsole, CNetBan *pNetBan);
 	void Update();
-	void Send(int ClientID, const char *pLine);
+	void Send(int ClientId, const char *pLine);
 	void Shutdown();
 };
 
