@@ -7,33 +7,22 @@
 #include <engine/console.h>
 #include <engine/engine.h>
 #include <engine/shared/config.h>
+#include <engine/shared/jobs.h>
 #include <engine/shared/network.h>
 #include <engine/storage.h>
 
-CHostLookup::CHostLookup() = default;
-
-CHostLookup::CHostLookup(const char *pHostname, int Nettype)
-{
-	str_copy(m_aHostname, pHostname);
-	m_Nettype = Nettype;
-}
-
-void CHostLookup::Run()
-{
-	m_Result = net_host_lookup(m_aHostname, &m_Addr, m_Nettype);
-}
-
 class CEngine : public IEngine
 {
-public:
 	IConsole *m_pConsole;
 	IStorage *m_pStorage;
-	bool m_Logging;
 
+	bool m_Logging;
 	std::shared_ptr<CFutureLogger> m_pFutureLogger;
 
 	char m_aAppName[256];
 
+	CJobPool m_JobPool;
+
 	static void Con_DbgLognetwork(IConsole::IResult *pResult, void *pUserData)
 	{
 		CEngine *pEngine = static_cast<CEngine *>(pUserData);
@@ -56,21 +45,21 @@ public:
 		}
 	}
 
+public:
 	CEngine(bool Test, const char *pAppname, std::shared_ptr<CFutureLogger> pFutureLogger, int Jobs) :
 		m_pFutureLogger(std::move(pFutureLogger))
 	{
 		str_copy(m_aAppName, pAppname);
 		if(!Test)
 		{
-			//
 			dbg_msg("engine", "running on %s-%s-%s", CONF_FAMILY_STRING, CONF_PLATFORM_STRING, CONF_ARCH_STRING);
-#ifdef CONF_ARCH_ENDIAN_LITTLE
-			dbg_msg("engine", "arch is little endian");
-#elif defined(CONF_ARCH_ENDIAN_BIG)
-			dbg_msg("engine", "arch is big endian");
-#else
-			dbg_msg("engine", "unknown endian");
-#endif
+			dbg_msg("engine", "arch is %s", CONF_ARCH_ENDIAN_STRING);
+
+			char aVersionStr[128];
+			if(os_version_str(aVersionStr, sizeof(aVersionStr)))
+			{
+				dbg_msg("engine", "operating system version: %s", aVersionStr);
+			}
 
 			// init the network
 			net_init();
@@ -84,7 +73,7 @@ public:
 
 	~CEngine() override
 	{
-		m_JobPool.Destroy();
+		CNetBase::CloseLog();
 	}
 
 	void Init() override
@@ -95,8 +84,6 @@ public:
 		if(!m_pConsole || !m_pStorage)
 			return;
 
-		char aFullPath[IO_MAX_PATH_LENGTH];
-		m_pStorage->GetCompletePath(IStorage::TYPE_SAVE, "dumps/", aFullPath, sizeof(aFullPath));
 		m_pConsole->Register("dbg_lognetwork", "", CFGFLAG_SERVER | CFGFLAG_CLIENT, Con_DbgLognetwork, this, "Log the network");
 	}
 
@@ -107,16 +94,16 @@ public:
 		m_JobPool.Add(std::move(pJob));
 	}
 
-	void SetAdditionalLogger(std::unique_ptr<ILogger> &&pLogger) override
+	void ShutdownJobs() override
 	{
-		m_pFutureLogger->Set(std::move(pLogger));
+		m_JobPool.Shutdown();
+	}
+
+	void SetAdditionalLogger(std::shared_ptr<ILogger> &&pLogger) override
+	{
+		m_pFutureLogger->Set(pLogger);
 	}
 };
 
-void IEngine::RunJobBlocking(IJob *pJob)
-{
-	CJobPool::RunBlocking(pJob);
-}
-
 IEngine *CreateEngine(const char *pAppname, std::shared_ptr<CFutureLogger> pFutureLogger, int Jobs) { return new CEngine(false, pAppname, std::move(pFutureLogger), Jobs); }
 IEngine *CreateTestEngine(const char *pAppname, int Jobs) { return new CEngine(true, pAppname, nullptr, Jobs); }
