@@ -3,7 +3,7 @@
 #ifndef ENGINE_SHARED_MAP_H
 #define ENGINE_SHARED_MAP_H
 
-#include <base/system.h>
+#include <base/types.h>
 
 #include "datafile.h"
 #include <engine/map.h>
@@ -15,29 +15,32 @@ class CMap : public IEngineMap
 public:
 	CMap();
 
-	void *GetData(int Index) override;
-	int GetDataSize(int Index) override;
-	void *GetDataSwapped(int Index) override;
-	void UnloadData(int Index) override;
-	void *GetItem(int Index, int *pType, int *pID) override;
-	int GetItemSize(int Index) override;
-	void GetType(int Type, int *pStart, int *pNum) override;
-	void *FindItem(int Type, int ID) override;
-	int NumItems() override;
+	CDataFileReader *GetReader() { return &m_DataFile; }
 
-	void Unload() override;
+	int GetDataSize(int Index) const override;
+	void *GetData(int Index) override;
+	void *GetDataSwapped(int Index) override;
+	const char *GetDataString(int Index) override;
+	void UnloadData(int Index) override;
+	int NumData() const override;
+
+	int GetItemSize(int Index) override;
+	void *GetItem(int Index, int *pType = nullptr, int *pId = nullptr) override;
+	void GetType(int Type, int *pStart, int *pNum) override;
+	int FindItemIndex(int Type, int Id) override;
+	void *FindItem(int Type, int Id) override;
+	int NumItems() const override;
 
 	bool Load(const char *pMapName) override;
+	void Unload() override;
+	bool IsLoaded() const override;
+	IOHANDLE File() const override;
 
-	bool IsLoaded() override;
+	SHA256_DIGEST Sha256() const override;
+	unsigned Crc() const override;
+	int MapSize() const override;
 
-	SHA256_DIGEST Sha256() override;
-
-	unsigned Crc() override;
-
-	int MapSize() override;
-
-	IOHANDLE File() override;
+	static void ExtractTiles(class CTile *pDest, size_t DestSize, const class CTile *pSrc, size_t SrcSize);
 };
 
 #endif
