@@ -44,36 +44,13 @@ int CNetBan::CNetHash::MakeHashArray(const NETADDR *pAddr, CNetHash aHash[17])
 }
 
 template<class T, int HashCount>
-typename CNetBan::CBan<T> *CNetBan::CBanPool<T, HashCount>::Add(const T *pData, const CBanInfo *pInfo, const CNetHash *pNetHash)
+void CNetBan::CBanPool<T, HashCount>::InsertUsed(CBan<T> *pBan)
 {
-	if(!m_pFirstFree)
-		return 0;
-
-	// create new ban
-	CBan<T> *pBan = m_pFirstFree;
-	pBan->m_Data = *pData;
-	pBan->m_Info = *pInfo;
-	pBan->m_NetHash = *pNetHash;
-	if(pBan->m_pNext)
-		pBan->m_pNext->m_pPrev = pBan->m_pPrev;
-	if(pBan->m_pPrev)
-		pBan->m_pPrev->m_pNext = pBan->m_pNext;
-	else
-		m_pFirstFree = pBan->m_pNext;
-
-	// add it to the hash list
-	if(m_aapHashList[pNetHash->m_HashIndex][pNetHash->m_Hash])
-		m_aapHashList[pNetHash->m_HashIndex][pNetHash->m_Hash]->m_pHashPrev = pBan;
-	pBan->m_pHashPrev = 0;
-	pBan->m_pHashNext = m_aapHashList[pNetHash->m_HashIndex][pNetHash->m_Hash];
-	m_aapHashList[pNetHash->m_HashIndex][pNetHash->m_Hash] = pBan;
-
-	// insert it into the used list
 	if(m_pFirstUsed)
 	{
 		for(CBan<T> *p = m_pFirstUsed;; p = p->m_pNext)
 		{
-			if(p->m_Info.m_Expires == CBanInfo::EXPIRES_NEVER || (pInfo->m_Expires != CBanInfo::EXPIRES_NEVER && pInfo->m_Expires <= p->m_Info.m_Expires))
+			if(p->m_Info.m_Expires == CBanInfo::EXPIRES_NEVER || (pBan->m_Info.m_Expires != CBanInfo::EXPIRES_NEVER && pBan->m_Info.m_Expires <= p->m_Info.m_Expires))
 			{
 				// insert before
 				pBan->m_pNext = p;
@@ -101,6 +78,35 @@ typename CNetBan::CBan<T> *CNetBan::CBanPool<T, HashCount>::Add(const T *pData,
 		m_pFirstUsed = pBan;
 		pBan->m_pNext = pBan->m_pPrev = 0;
 	}
+}
+
+template<class T, int HashCount>
+typename CNetBan::CBan<T> *CNetBan::CBanPool<T, HashCount>::Add(const T *pData, const CBanInfo *pInfo, const CNetHash *pNetHash)
+{
+	if(!m_pFirstFree)
+		return nullptr;
+
+	// create new ban
+	CBan<T> *pBan = m_pFirstFree;
+	pBan->m_Data = *pData;
+	pBan->m_Info = *pInfo;
+	pBan->m_NetHash = *pNetHash;
+	if(pBan->m_pNext)
+		pBan->m_pNext->m_pPrev = pBan->m_pPrev;
+	if(pBan->m_pPrev)
+		pBan->m_pPrev->m_pNext = pBan->m_pNext;
+	else
+		m_pFirstFree = pBan->m_pNext;
+
+	// add it to the hash list
+	if(m_aapHashList[pNetHash->m_HashIndex][pNetHash->m_Hash])
+		m_aapHashList[pNetHash->m_HashIndex][pNetHash->m_Hash]->m_pHashPrev = pBan;
+	pBan->m_pHashPrev = 0;
+	pBan->m_pHashNext = m_aapHashList[pNetHash->m_HashIndex][pNetHash->m_Hash];
+	m_aapHashList[pNetHash->m_HashIndex][pNetHash->m_Hash] = pBan;
+
+	// insert it into the used list
+	InsertUsed(pBan);
 
 	// update ban count
 	++m_CountUsed;
@@ -111,7 +117,7 @@ typename CNetBan::CBan<T> *CNetBan::CBanPool<T, HashCount>::Add(const T *pData,
 template<class T, int HashCount>
 int CNetBan::CBanPool<T, HashCount>::Remove(CBan<T> *pBan)
 {
-	if(pBan == 0)
+	if(pBan == nullptr)
 		return -1;
 
 	// remove from hash list
@@ -158,38 +164,7 @@ void CNetBan::CBanPool<T, HashCount>::Update(CBan<CDataType> *pBan, const CBanIn
 		m_pFirstUsed = pBan->m_pNext;
 
 	// insert it into the used list
-	if(m_pFirstUsed)
-	{
-		for(CBan<T> *p = m_pFirstUsed;; p = p->m_pNext)
-		{
-			if(p->m_Info.m_Expires == CBanInfo::EXPIRES_NEVER || (pInfo->m_Expires != CBanInfo::EXPIRES_NEVER && pInfo->m_Expires <= p->m_Info.m_Expires))
-			{
-				// insert before
-				pBan->m_pNext = p;
-				pBan->m_pPrev = p->m_pPrev;
-				if(p->m_pPrev)
-					p->m_pPrev->m_pNext = pBan;
-				else
-					m_pFirstUsed = pBan;
-				p->m_pPrev = pBan;
-				break;
-			}
-
-			if(!p->m_pNext)
-			{
-				// last entry
-				p->m_pNext = pBan;
-				pBan->m_pPrev = p;
-				pBan->m_pNext = 0;
-				break;
-			}
-		}
-	}
-	else
-	{
-		m_pFirstUsed = pBan;
-		pBan->m_pNext = pBan->m_pPrev = 0;
-	}
+	InsertUsed(pBan);
 }
 
 void CNetBan::UnbanAll()
@@ -221,7 +196,7 @@ template<class T, int HashCount>
 typename CNetBan::CBan<T> *CNetBan::CBanPool<T, HashCount>::Get(int Index) const
 {
 	if(Index < 0 || Index >= Num())
-		return 0;
+		return nullptr;
 
 	for(CNetBan::CBan<T> *pBan = m_pFirstUsed; pBan; pBan = pBan->m_pNext, --Index)
 	{
@@ -229,24 +204,25 @@ typename CNetBan::CBan<T> *CNetBan::CBanPool<T, HashCount>::Get(int Index) const
 			return pBan;
 	}
 
-	return 0;
+	return nullptr;
 }
 
 template<class T>
-int CNetBan::Ban(T *pBanPool, const typename T::CDataType *pData, int Seconds, const char *pReason)
+int CNetBan::Ban(T *pBanPool, const typename T::CDataType *pData, int Seconds, const char *pReason, bool VerbatimReason)
 {
 	// do not ban localhost
-	if(NetMatch(pData, &m_LocalhostIPV4) || NetMatch(pData, &m_LocalhostIPV6))
+	if(NetMatch(pData, &m_LocalhostIpV4) || NetMatch(pData, &m_LocalhostIpV6))
 	{
 		Console()->Print(IConsole::OUTPUT_LEVEL_STANDARD, "net_ban", "ban failed (localhost)");
 		return -1;
 	}
 
-	int Stamp = Seconds > 0 ? time_timestamp() + Seconds : CBanInfo::EXPIRES_NEVER;
+	int64_t Stamp = Seconds > 0 ? time_timestamp() + Seconds : static_cast<int64_t>(CBanInfo::EXPIRES_NEVER);
 
 	// set up info
 	CBanInfo Info = {0};
 	Info.m_Expires = Stamp;
+	Info.m_VerbatimReason = VerbatimReason;
 	str_copy(Info.m_aReason, pReason);
 
 	// check if it already exists
@@ -256,7 +232,7 @@ int CNetBan::Ban(T *pBanPool, const typename T::CDataType *pData, int Seconds, c
 	{
 		// adjust the ban
 		pBanPool->Update(pBan, &Info);
-		char aBuf[128];
+		char aBuf[256];
 		MakeBanInfo(pBan, aBuf, sizeof(aBuf), MSGTYPE_LIST);
 		Console()->Print(IConsole::OUTPUT_LEVEL_STANDARD, "net_ban", aBuf);
 		return 1;
@@ -266,7 +242,7 @@ int CNetBan::Ban(T *pBanPool, const typename T::CDataType *pData, int Seconds, c
 	pBan = pBanPool->Add(pData, &Info, &NetHash);
 	if(pBan)
 	{
-		char aBuf[128];
+		char aBuf[256];
 		MakeBanInfo(pBan, aBuf, sizeof(aBuf), MSGTYPE_BANADD);
 		Console()->Print(IConsole::OUTPUT_LEVEL_STANDARD, "net_ban", aBuf);
 		return 0;
@@ -301,21 +277,22 @@ void CNetBan::Init(IConsole *pConsole, IStorage *pStorage)
 	m_BanAddrPool.Reset();
 	m_BanRangePool.Reset();
 
-	net_host_lookup("localhost", &m_LocalhostIPV4, NETTYPE_IPV4);
-	net_host_lookup("localhost", &m_LocalhostIPV6, NETTYPE_IPV6);
+	net_host_lookup("localhost", &m_LocalhostIpV4, NETTYPE_IPV4);
+	net_host_lookup("localhost", &m_LocalhostIpV6, NETTYPE_IPV6);
 
 	Console()->Register("ban", "s[ip|id] ?i[minutes] r[reason]", CFGFLAG_SERVER | CFGFLAG_MASTER | CFGFLAG_STORE, ConBan, this, "Ban ip for x minutes for any reason");
 	Console()->Register("ban_range", "s[first ip] s[last ip] ?i[minutes] r[reason]", CFGFLAG_SERVER | CFGFLAG_MASTER | CFGFLAG_STORE, ConBanRange, this, "Ban ip range for x minutes for any reason");
 	Console()->Register("unban", "s[ip|entry]", CFGFLAG_SERVER | CFGFLAG_MASTER | CFGFLAG_STORE, ConUnban, this, "Unban ip/banlist entry");
 	Console()->Register("unban_range", "s[first ip] s[last ip]", CFGFLAG_SERVER | CFGFLAG_MASTER | CFGFLAG_STORE, ConUnbanRange, this, "Unban ip range");
 	Console()->Register("unban_all", "", CFGFLAG_SERVER | CFGFLAG_MASTER | CFGFLAG_STORE, ConUnbanAll, this, "Unban all entries");
-	Console()->Register("bans", "?i[page]", CFGFLAG_SERVER | CFGFLAG_MASTER, ConBans, this, "Show banlist (page 0 by default, 20 entries per page)");
+	Console()->Register("bans", "?i[page]", CFGFLAG_SERVER | CFGFLAG_MASTER, ConBans, this, "Show banlist (page 1 by default, 20 entries per page)");
+	Console()->Register("bans_find", "s[ip]", CFGFLAG_SERVER | CFGFLAG_MASTER, ConBansFind, this, "Find all ban records for the specified IP address");
 	Console()->Register("bans_save", "s[file]", CFGFLAG_SERVER | CFGFLAG_MASTER | CFGFLAG_STORE, ConBansSave, this, "Save banlist in a file");
 }
 
 void CNetBan::Update()
 {
-	int Now = time_timestamp();
+	int64_t Now = time_timestamp();
 
 	// remove expired bans
 	char aBuf[256], aNetStr[256];
@@ -333,15 +310,15 @@ void CNetBan::Update()
 	}
 }
 
-int CNetBan::BanAddr(const NETADDR *pAddr, int Seconds, const char *pReason)
+int CNetBan::BanAddr(const NETADDR *pAddr, int Seconds, const char *pReason, bool VerbatimReason)
 {
-	return Ban(&m_BanAddrPool, pAddr, Seconds, pReason);
+	return Ban(&m_BanAddrPool, pAddr, Seconds, pReason, VerbatimReason);
 }
 
 int CNetBan::BanRange(const CNetRange *pRange, int Seconds, const char *pReason)
 {
 	if(pRange->IsValid())
-		return Ban(&m_BanRangePool, pRange, Seconds, pReason);
+		return Ban(&m_BanRangePool, pRange, Seconds, pReason, false);
 
 	Console()->Print(IConsole::OUTPUT_LEVEL_STANDARD, "net_ban", "ban failed (invalid range)");
 	return -1;
@@ -439,7 +416,7 @@ void CNetBan::ConBan(IConsole::IResult *pResult, void *pUser)
 
 	NETADDR Addr;
 	if(net_addr_from_str(&Addr, pStr) == 0)
-		pThis->BanAddr(&Addr, Minutes * 60, pReason);
+		pThis->BanAddr(&Addr, Minutes * 60, pReason, false);
 	else
 		pThis->Console()->Print(IConsole::OUTPUT_LEVEL_STANDARD, "net_ban", "ban error (invalid network address)");
 }
@@ -503,13 +480,31 @@ void CNetBan::ConBans(IConsole::IResult *pResult, void *pUser)
 {
 	CNetBan *pThis = static_cast<CNetBan *>(pUser);
 
-	int Page = pResult->NumArguments() > 0 ? pResult->GetInteger(0) : 0;
+	int Page = pResult->NumArguments() > 0 ? pResult->GetInteger(0) : 1;
 	static const int s_EntriesPerPage = 20;
-	const int Start = Page * s_EntriesPerPage;
-	const int End = (Page + 1) * s_EntriesPerPage;
+	const int Start = (Page - 1) * s_EntriesPerPage;
+	const int End = Page * s_EntriesPerPage;
+	const int NumBans = pThis->m_BanAddrPool.Num() + pThis->m_BanRangePool.Num();
+	const int NumPages = NumBans / s_EntriesPerPage + 1;
+
+	char aBuf[256], aMsg[256];
+
+	if(NumBans == 0)
+	{
+		pThis->Console()->Print(IConsole::OUTPUT_LEVEL_STANDARD, "net_ban", "The ban list is empty.");
+
+		return;
+	}
+
+	if(Page <= 0 || Page > NumPages)
+	{
+		str_format(aMsg, sizeof(aMsg), "Invalid page number. There %s %d %s available.", NumPages == 1 ? "is" : "are", NumPages, NumPages == 1 ? "page" : "pages");
+		pThis->Console()->Print(IConsole::OUTPUT_LEVEL_STANDARD, "net_ban", aMsg);
+
+		return;
+	}
 
 	int Count = 0;
-	char aBuf[256], aMsg[256];
 	for(CBanAddr *pBan = pThis->m_BanAddrPool.First(); pBan; pBan = pBan->m_pNext, Count++)
 	{
 		if(Count < Start || Count >= End)
@@ -530,7 +525,56 @@ void CNetBan::ConBans(IConsole::IResult *pResult, void *pUser)
 		str_format(aMsg, sizeof(aMsg), "#%i %s", Count, aBuf);
 		pThis->Console()->Print(IConsole::OUTPUT_LEVEL_STANDARD, "net_ban", aMsg);
 	}
-	str_format(aMsg, sizeof(aMsg), "%d %s, showing entries %d - %d", Count, Count == 1 ? "ban" : "bans", Start, End - 1);
+	str_format(aMsg, sizeof(aMsg), "%d %s, showing entries %d - %d (page %d/%d)", Count, Count == 1 ? "ban" : "bans", Start, End > Count ? Count - 1 : End - 1, Page, NumPages);
+	pThis->Console()->Print(IConsole::OUTPUT_LEVEL_STANDARD, "net_ban", aMsg);
+}
+
+void CNetBan::ConBansFind(IConsole::IResult *pResult, void *pUser)
+{
+	CNetBan *pThis = static_cast<CNetBan *>(pUser);
+
+	const char *pStr = pResult->GetString(0);
+	char aBuf[256], aMsg[256];
+
+	NETADDR Addr;
+	if(net_addr_from_str(&Addr, pStr) != 0)
+	{
+		pThis->Console()->Print(IConsole::OUTPUT_LEVEL_STANDARD, "net_ban", "bans_find error (invalid network address)");
+		return;
+	}
+
+	int Count = 0;
+	int Found = 0;
+	// Check first for bans
+	for(CBanAddr *pBan = pThis->m_BanAddrPool.First(); pBan; pBan = pBan->m_pNext, Count++)
+	{
+		if(NetComp(&pBan->m_Data, &Addr) == 0)
+		{
+			pThis->MakeBanInfo(pBan, aBuf, sizeof(aBuf), MSGTYPE_LIST);
+			str_format(aMsg, sizeof(aMsg), "#%i %s", Count, aBuf);
+			pThis->Console()->Print(IConsole::OUTPUT_LEVEL_STANDARD, "net_ban", aMsg);
+
+			Found++;
+		}
+	}
+	// check ban ranges
+	for(CBanRange *pBan = pThis->m_BanRangePool.First(); pBan; pBan = pBan->m_pNext, Count++)
+	{
+		if(pThis->NetMatch(&pBan->m_Data, &Addr))
+		{
+			pThis->MakeBanInfo(pBan, aBuf, sizeof(aBuf), MSGTYPE_LIST);
+			str_format(aMsg, sizeof(aMsg), "#%i %s", Count, aBuf);
+			pThis->Console()->Print(IConsole::OUTPUT_LEVEL_STANDARD, "net_ban", aMsg);
+
+			Found++;
+		}
+	}
+
+	if(Found)
+		str_format(aMsg, sizeof(aMsg), "%i ban records found.", Found);
+	else
+		str_copy(aMsg, "No ban records found.", sizeof(aMsg));
+
 	pThis->Console()->Print(IConsole::OUTPUT_LEVEL_STANDARD, "net_ban", aMsg);
 }
 
@@ -547,7 +591,7 @@ void CNetBan::ConBansSave(IConsole::IResult *pResult, void *pUser)
 		return;
 	}
 
-	int Now = time_timestamp();
+	int64_t Now = time_timestamp();
 	char aAddrStr1[NETADDR_MAXSTRSIZE], aAddrStr2[NETADDR_MAXSTRSIZE];
 	for(CBanAddr *pBan = pThis->m_BanAddrPool.First(); pBan; pBan = pBan->m_pNext)
 	{
