@@ -1,6 +1,7 @@
 /* (c) Magnus Auvinen. See licence.txt in the root of the distribution for more information. */
 /* If you are missing that file, acquire a complete release at teeworlds.com.                */
 #include <base/system.h>
+#include <base/types.h>
 
 #include "config.h"
 #include "huffman.h"
@@ -8,16 +9,26 @@
 
 const unsigned char SECURITY_TOKEN_MAGIC[4] = {'T', 'K', 'E', 'N'};
 
+SECURITY_TOKEN ToSecurityToken(const unsigned char *pData)
+{
+	return bytes_be_to_uint(pData);
+}
+
+void WriteSecurityToken(unsigned char *pData, SECURITY_TOKEN Token)
+{
+	uint_to_bytes_be(pData, Token);
+}
+
 void CNetRecvUnpacker::Clear()
 {
 	m_Valid = false;
 }
 
-void CNetRecvUnpacker::Start(const NETADDR *pAddr, CNetConnection *pConnection, int ClientID)
+void CNetRecvUnpacker::Start(const NETADDR *pAddr, CNetConnection *pConnection, int ClientId)
 {
 	m_Addr = *pAddr;
 	m_pConnection = pConnection;
-	m_ClientID = ClientID;
+	m_ClientId = ClientId;
 	m_CurrentChunk = 0;
 	m_Valid = true;
 }
@@ -82,7 +93,7 @@ int CNetRecvUnpacker::FetchChunk(CNetChunk *pChunk)
 		}
 
 		// fill in the info
-		pChunk->m_ClientID = m_ClientID;
+		pChunk->m_ClientId = m_ClientId;
 		pChunk->m_Address = m_Addr;
 		pChunk->m_Flags = Header.m_Flags;
 		pChunk->m_DataSize = Header.m_Size;
@@ -111,6 +122,19 @@ void CNetBase::SendPacketConnless(NETSOCKET Socket, NETADDR *pAddr, const void *
 	net_udp_send(Socket, pAddr, aBuffer, DataSize + DATA_OFFSET);
 }
 
+void CNetBase::SendPacketConnlessWithToken7(NETSOCKET Socket, NETADDR *pAddr, const void *pData, int DataSize, SECURITY_TOKEN Token, SECURITY_TOKEN ResponseToken)
+{
+	unsigned char aBuffer[NET_MAX_PACKETSIZE];
+	aBuffer[0] = (NET_PACKETFLAG_CONNLESS << 2) | 1;
+
+	const int DATA_OFFSET = 9;
+
+	WriteSecurityToken(aBuffer + 1, Token);
+	WriteSecurityToken(aBuffer + 5, ResponseToken);
+	mem_copy(aBuffer + DATA_OFFSET, pData, DataSize);
+	net_udp_send(Socket, pAddr, aBuffer, DataSize + DATA_OFFSET);
+}
+
 void CNetBase::SendPacket(NETSOCKET Socket, NETADDR *pAddr, CNetPacketConstruct *pPacket, SECURITY_TOKEN SecurityToken, bool Sixup, bool NoCompress)
 {
 	unsigned char aBuffer[NET_MAX_PACKETSIZE];
@@ -131,13 +155,13 @@ void CNetBase::SendPacket(NETSOCKET Socket, NETADDR *pAddr, CNetPacketConstruct
 	if(Sixup)
 	{
 		HeaderSize += sizeof(SecurityToken);
-		mem_copy(&aBuffer[3], &SecurityToken, sizeof(SecurityToken));
+		WriteSecurityToken(aBuffer + 3, SecurityToken);
 	}
 	else if(SecurityToken != NET_SECURITY_TOKEN_UNSUPPORTED)
 	{
 		// append security token
 		// if SecurityToken is NET_SECURITY_TOKEN_UNKNOWN we will still append it hoping to negotiate it
-		mem_copy(&pPacket->m_aChunkData[pPacket->m_DataSize], &SecurityToken, sizeof(SecurityToken));
+		WriteSecurityToken(pPacket->m_aChunkData + pPacket->m_DataSize, SecurityToken);
 		pPacket->m_DataSize += sizeof(SecurityToken);
 	}
 
@@ -215,14 +239,16 @@ int CNetBase::UnpackPacket(unsigned char *pBuffer, int Size, CNetPacketConstruct
 	if(pPacket->m_Flags & NET_PACKETFLAG_CONNLESS)
 	{
 		Sixup = (pBuffer[0] & 0x3) == 1;
+		if(Sixup && (pSecurityToken == nullptr || pResponseToken == nullptr))
+			return -1;
 		int Offset = Sixup ? 9 : 6;
 		if(Size < Offset)
 			return -1;
 
 		if(Sixup)
 		{
-			mem_copy(pSecurityToken, &pBuffer[1], sizeof(*pSecurityToken));
-			mem_copy(pResponseToken, &pBuffer[5], sizeof(*pResponseToken));
+			*pSecurityToken = ToSecurityToken(pBuffer + 1);
+			*pResponseToken = ToSecurityToken(pBuffer + 5);
 		}
 
 		pPacket->m_Flags = NET_PACKETFLAG_CONNLESS;
@@ -241,6 +267,8 @@ int CNetBase::UnpackPacket(unsigned char *pBuffer, int Size, CNetPacketConstruct
 	{
 		if(pPacket->m_Flags & NET_PACKETFLAG_UNUSED)
 			Sixup = true;
+		if(Sixup && pSecurityToken == nullptr)
+			return -1;
 		int DataStart = Sixup ? 7 : NET_PACKETHEADERSIZE;
 		if(Size < DataStart)
 			return -1;
@@ -260,7 +288,7 @@ int CNetBase::UnpackPacket(unsigned char *pBuffer, int Size, CNetPacketConstruct
 				Flags |= NET_PACKETFLAG_COMPRESSION;
 			pPacket->m_Flags = Flags;
 
-			mem_copy(pSecurityToken, &pBuffer[3], sizeof(*pSecurityToken));
+			*pSecurityToken = ToSecurityToken(pBuffer + 3);
 		}
 
 		if(pPacket->m_Flags & NET_PACKETFLAG_COMPRESSION)
@@ -284,6 +312,18 @@ int CNetBase::UnpackPacket(unsigned char *pBuffer, int Size, CNetPacketConstruct
 		return -1;
 	}
 
+	// set the response token (a bit hacky because this function shouldn't know about control packets)
+	if(pPacket->m_Flags & NET_PACKETFLAG_CONTROL)
+	{
+		if(pPacket->m_DataSize >= 5) // control byte + token
+		{
+			if(pPacket->m_aChunkData[0] == NET_CTRLMSG_CONNECT || pPacket->m_aChunkData[0] == NET_CTRLMSG_TOKEN)
+			{
+				*pResponseToken = ToSecurityToken(&pPacket->m_aChunkData[1]);
+			}
+		}
+	}
+
 	// log the data
 	if(ms_DataLogRecv)
 	{
@@ -313,7 +353,20 @@ void CNetBase::SendControlMsg(NETSOCKET Socket, NETADDR *pAddr, int Ack, int Con
 	CNetBase::SendPacket(Socket, pAddr, &Construct, SecurityToken, Sixup, true);
 }
 
-unsigned char *CNetChunkHeader::Pack(unsigned char *pData, int Split)
+void CNetBase::SendControlMsgWithToken7(NETSOCKET Socket, NETADDR *pAddr, TOKEN Token, int Ack, int ControlMsg, TOKEN MyToken, bool Extended)
+{
+	dbg_assert((Token & ~NET_TOKEN_MASK) == 0, "token out of range");
+	dbg_assert((MyToken & ~NET_TOKEN_MASK) == 0, "resp token out of range");
+
+	unsigned char s_aRequestTokenBuf[NET_TOKENREQUEST_DATASIZE];
+	s_aRequestTokenBuf[0] = (MyToken >> 24) & 0xff;
+	s_aRequestTokenBuf[1] = (MyToken >> 16) & 0xff;
+	s_aRequestTokenBuf[2] = (MyToken >> 8) & 0xff;
+	s_aRequestTokenBuf[3] = (MyToken)&0xff;
+	CNetBase::SendControlMsg(Socket, pAddr, 0, ControlMsg, s_aRequestTokenBuf, Extended ? sizeof(s_aRequestTokenBuf) : 4, Token, true);
+}
+
+unsigned char *CNetChunkHeader::Pack(unsigned char *pData, int Split) const
 {
 	pData[0] = ((m_Flags & 3) << 6) | ((m_Size >> Split) & 0x3f);
 	pData[1] = (m_Size & ((1 << Split) - 1));
@@ -358,8 +411,8 @@ bool CNetBase::IsSeqInBackroom(int Seq, int Ack)
 	return false;
 }
 
-IOHANDLE CNetBase::ms_DataLogSent = 0;
-IOHANDLE CNetBase::ms_DataLogRecv = 0;
+IOHANDLE CNetBase::ms_DataLogSent = nullptr;
+IOHANDLE CNetBase::ms_DataLogRecv = nullptr;
 CHuffman CNetBase::ms_Huffman;
 
 void CNetBase::OpenLog(IOHANDLE DataLogSent, IOHANDLE DataLogRecv)
@@ -387,14 +440,14 @@ void CNetBase::CloseLog()
 	{
 		dbg_msg("network", "stopped logging sent packages");
 		io_close(ms_DataLogSent);
-		ms_DataLogSent = 0;
+		ms_DataLogSent = nullptr;
 	}
 
 	if(ms_DataLogRecv)
 	{
 		dbg_msg("network", "stopped logging recv packages");
 		io_close(ms_DataLogRecv);
-		ms_DataLogRecv = 0;
+		ms_DataLogRecv = nullptr;
 	}
 }
 
@@ -412,3 +465,115 @@ void CNetBase::Init()
 {
 	ms_Huffman.Init();
 }
+
+void CNetTokenCache::Init(NETSOCKET Socket)
+{
+	m_Socket = Socket;
+}
+
+void CNetTokenCache::SendPacketConnless(CNetChunk *pChunk)
+{
+	TOKEN Token = GetToken(&pChunk->m_Address);
+
+	if(Token != NET_TOKEN_NONE)
+	{
+		CNetBase::SendPacketConnlessWithToken7(m_Socket, &pChunk->m_Address, pChunk->m_pData, pChunk->m_DataSize, Token, GenerateToken());
+	}
+	else
+	{
+		FetchToken(&pChunk->m_Address);
+
+		CConnlessPacketInfo ConnlessPacket;
+		ConnlessPacket.m_Addr = pChunk->m_Address;
+		ConnlessPacket.m_Addr.type = pChunk->m_Address.type & ~(NETTYPE_IPV4 | NETTYPE_IPV6);
+		mem_copy(ConnlessPacket.m_aData, pChunk->m_pData, pChunk->m_DataSize);
+		ConnlessPacket.m_DataSize = pChunk->m_DataSize;
+		ConnlessPacket.m_Expiry = time_get() + time_freq() * NET_TOKENCACHE_PACKETEXPIRY;
+
+		unsigned int NetType = pChunk->m_Address.type;
+		auto SavePacketFor = [&](unsigned int Type) {
+			if(NetType & Type)
+			{
+				ConnlessPacket.m_Addr.type |= Type;
+				m_ConnlessPackets.push_back(ConnlessPacket);
+				ConnlessPacket.m_Addr.type &= ~Type;
+			}
+		};
+
+		SavePacketFor(NETTYPE_IPV4);
+		SavePacketFor(NETTYPE_IPV6);
+	}
+}
+
+void CNetTokenCache::FetchToken(NETADDR *pAddr)
+{
+	CNetBase::SendControlMsgWithToken7(m_Socket, pAddr, NET_TOKEN_NONE, 0, NET_CTRLMSG_TOKEN, GenerateToken(), true);
+}
+
+void CNetTokenCache::AddToken(const NETADDR *pAddr, TOKEN Token)
+{
+	if(Token == NET_TOKEN_NONE)
+		return;
+
+	NETADDR NullAddr = NETADDR_ZEROED;
+	NullAddr.port = pAddr->port;
+	NullAddr.type = pAddr->type | NETTYPE_LINK_BROADCAST;
+
+	for(auto Iter = m_ConnlessPackets.begin(); Iter != m_ConnlessPackets.end();)
+	{
+		if(Iter->m_Addr == NullAddr)
+		{
+			CNetBase::SendPacketConnlessWithToken7(m_Socket, &Iter->m_Addr, Iter->m_aData, Iter->m_DataSize, Token, GenerateToken());
+
+			Iter = m_ConnlessPackets.erase(Iter);
+		}
+		else
+		{
+			Iter++;
+		}
+	}
+
+	CAddressInfo Info;
+	Info.m_Addr = *pAddr,
+	Info.m_Token = Token,
+	Info.m_Expiry = time_get() + (time_freq() * NET_TOKENCACHE_ADDRESSEXPIRY);
+
+	m_TokenCache.push_back(Info);
+}
+
+TOKEN CNetTokenCache::GetToken(const NETADDR *pAddr)
+{
+	for(const auto &AddrInfo : m_TokenCache)
+	{
+		if(AddrInfo.m_Addr == *pAddr)
+		{
+			return AddrInfo.m_Token;
+		}
+	}
+
+	return NET_TOKEN_NONE;
+}
+
+TOKEN CNetTokenCache::GenerateToken()
+{
+	TOKEN Token;
+	secure_random_fill(&Token, sizeof(Token));
+	return Token;
+}
+
+void CNetTokenCache::Update()
+{
+	int64_t Now = time_get();
+
+	m_TokenCache.erase(
+		std::remove_if(m_TokenCache.begin(), m_TokenCache.end(), [&](const CAddressInfo &Info) {
+			return Info.m_Expiry <= Now;
+		}),
+		m_TokenCache.end());
+
+	m_ConnlessPackets.erase(
+		std::remove_if(m_ConnlessPackets.begin(), m_ConnlessPackets.end(), [&](CConnlessPacketInfo &Packet) {
+			return Packet.m_Expiry <= Now;
+		}),
+		m_ConnlessPackets.end());
+}
