@@ -9,9 +9,11 @@
 #include <base/math.h>
 #include <base/system.h>
 
+#include <array>
+
 class CHuffman;
-class CMsgPacker;
 class CNetBan;
+class CPacker;
 
 /*
 
@@ -19,9 +21,19 @@ CURRENT:
 	packet header: 3 bytes
 		unsigned char flags_ack; // 6bit flags, 2bit ack
 			0.6:   ORNCaaAA
-			0.6.5: ORNCTUAA
+			0.6.5: ORNCT-AA
 			0.7:   --NORCAA
 
+		O = flag compression
+		R = flag resend
+		N = flag connless
+		C = flag control
+		T = flag token (0.6.5 only not supported by ddnet)
+		- = unused, should be zero
+		a = should be zero otherwise it messes up the ack number
+		A = bit of ack number
+
+
 		unsigned char ack; // 8 bit ack
 		unsigned char num_chunks; // 8 bit chunks
 
@@ -37,7 +49,6 @@ CURRENT:
 
 enum
 {
-	NETFLAG_ALLOWSTATELESS = 1,
 	NETSENDFLAG_VITAL = 1,
 	NETSENDFLAG_CONNLESS = 2,
 	NETSENDFLAG_FLUSH = 4,
@@ -46,29 +57,24 @@ enum
 	NETSTATE_OFFLINE = 0,
 	NETSTATE_CONNECTING,
 	NETSTATE_ONLINE,
-
-	NETBANTYPE_SOFT = 1,
-	NETBANTYPE_DROP = 2
 };
 
 enum
 {
-	NET_VERSION = 2,
-
 	NET_MAX_PACKETSIZE = 1400,
 	NET_MAX_PAYLOAD = NET_MAX_PACKETSIZE - 6,
-	NET_MAX_CHUNKHEADERSIZE = 5,
+	NET_MAX_CHUNKHEADERSIZE = 3,
 	NET_PACKETHEADERSIZE = 3,
 	NET_MAX_CLIENTS = 64,
 	NET_MAX_CONSOLE_CLIENTS = 4,
 	NET_MAX_SEQUENCE = 1 << 10,
-	NET_SEQUENCE_MASK = NET_MAX_SEQUENCE - 1,
 
 	NET_CONNSTATE_OFFLINE = 0,
-	NET_CONNSTATE_CONNECT = 1,
-	NET_CONNSTATE_PENDING = 2,
-	NET_CONNSTATE_ONLINE = 3,
-	NET_CONNSTATE_ERROR = 4,
+	NET_CONNSTATE_TOKEN = 1,
+	NET_CONNSTATE_CONNECT = 2,
+	NET_CONNSTATE_PENDING = 3,
+	NET_CONNSTATE_ONLINE = 4,
+	NET_CONNSTATE_ERROR = 5,
 
 	NET_PACKETFLAG_UNUSED = 1 << 0,
 	NET_PACKETFLAG_TOKEN = 1 << 1,
@@ -87,17 +93,29 @@ enum
 	NET_CTRLMSG_CONNECTACCEPT = 2,
 	NET_CTRLMSG_ACCEPT = 3,
 	NET_CTRLMSG_CLOSE = 4,
+	NET_CTRLMSG_TOKEN = 5,
 
 	NET_CONN_BUFFERSIZE = 1024 * 32,
 
 	NET_CONNLIMIT_IPS = 16,
 
-	NET_ENUM_TERMINATOR
+	NET_TOKENCACHE_ADDRESSEXPIRY = 64,
+	NET_TOKENCACHE_PACKETEXPIRY = 5,
+};
+enum
+{
+	NET_TOKEN_MAX = 0xffffffff,
+	NET_TOKEN_NONE = NET_TOKEN_MAX,
+	NET_TOKEN_MASK = NET_TOKEN_MAX,
+
+	NET_TOKENREQUEST_DATASIZE = 512,
 };
 
 typedef int SECURITY_TOKEN;
+typedef unsigned int TOKEN;
 
-SECURITY_TOKEN ToSecurityToken(unsigned char *pData);
+SECURITY_TOKEN ToSecurityToken(const unsigned char *pData);
+void WriteSecurityToken(unsigned char *pData, SECURITY_TOKEN Token);
 
 extern const unsigned char SECURITY_TOKEN_MAGIC[4];
 
@@ -107,17 +125,17 @@ enum
 	NET_SECURITY_TOKEN_UNSUPPORTED = 0,
 };
 
-typedef int (*NETFUNC_DELCLIENT)(int ClientID, const char *pReason, void *pUser);
-typedef int (*NETFUNC_NEWCLIENT_CON)(int ClientID, void *pUser);
-typedef int (*NETFUNC_NEWCLIENT)(int ClientID, void *pUser, bool Sixup);
-typedef int (*NETFUNC_NEWCLIENT_NOAUTH)(int ClientID, void *pUser);
-typedef int (*NETFUNC_CLIENTREJOIN)(int ClientID, void *pUser);
+typedef int (*NETFUNC_DELCLIENT)(int ClientId, const char *pReason, void *pUser);
+typedef int (*NETFUNC_NEWCLIENT_CON)(int ClientId, void *pUser);
+typedef int (*NETFUNC_NEWCLIENT)(int ClientId, void *pUser, bool Sixup);
+typedef int (*NETFUNC_NEWCLIENT_NOAUTH)(int ClientId, void *pUser);
+typedef int (*NETFUNC_CLIENTREJOIN)(int ClientId, void *pUser);
 
 struct CNetChunk
 {
 	// -1 means that it's a stateless packet
 	// 0 on the client means the server
-	int m_ClientID;
+	int m_ClientId;
 	NETADDR m_Address; // only used when client_id == -1
 	int m_Flags;
 	int m_DataSize;
@@ -133,7 +151,7 @@ public:
 	int m_Size;
 	int m_Sequence;
 
-	unsigned char *Pack(unsigned char *pData, int Split = 4);
+	unsigned char *Pack(unsigned char *pData, int Split = 4) const;
 	unsigned char *Unpack(unsigned char *pData, int Split = 4);
 };
 
@@ -216,7 +234,10 @@ private:
 	unsigned short m_PeerAck;
 	unsigned m_State;
 
+public:
 	SECURITY_TOKEN m_SecurityToken;
+
+private:
 	int m_RemoteClosed;
 	bool m_BlockCloseMsg;
 	bool m_UnknownSeq;
@@ -237,7 +258,16 @@ private:
 	NETSOCKET m_Socket;
 	NETSTATS m_Stats;
 
+	std::array<char, NETADDR_MAXSTRSIZE> m_aPeerAddrStr;
+	std::array<char, NETADDR_MAXSTRSIZE> m_aPeerAddrStrNoPort;
+	// client 0.7
+	static TOKEN GenerateToken7(const NETADDR *pPeerAddr);
+	class CNetBase *m_pNetBase;
+	bool IsSixup() { return m_Sixup; }
+
 	//
+	void SetPeerAddr(const NETADDR *pAddr);
+	void ClearPeerAddr();
 	void ResetStats();
 	void SetError(const char *pString);
 	void AckChunks(int Ack);
@@ -245,6 +275,7 @@ private:
 	int QueueChunkEx(int Flags, int DataSize, const void *pData, int Sequence);
 	void SendConnect();
 	void SendControl(int ControlMsg, const void *pExtra, int ExtraSize);
+	void SendControlWithToken7(int ControlMsg, SECURITY_TOKEN ResponseToken);
 	void ResendChunk(CNetChunkResend *pResend);
 	void Resend();
 
@@ -252,21 +283,28 @@ public:
 	bool m_TimeoutProtected;
 	bool m_TimeoutSituation;
 
+	void SetToken7(TOKEN Token);
+
 	void Reset(bool Rejoin = false);
 	void Init(NETSOCKET Socket, bool BlockCloseMsg);
 	int Connect(const NETADDR *pAddr, int NumAddrs);
+	int Connect7(const NETADDR *pAddr, int NumAddrs);
 	void Disconnect(const char *pReason);
 
 	int Update();
 	int Flush();
 
-	int Feed(CNetPacketConstruct *pPacket, NETADDR *pAddr, SECURITY_TOKEN SecurityToken = NET_SECURITY_TOKEN_UNSUPPORTED);
+	int Feed(CNetPacketConstruct *pPacket, NETADDR *pAddr, SECURITY_TOKEN SecurityToken = NET_SECURITY_TOKEN_UNSUPPORTED, SECURITY_TOKEN ResponseToken = NET_SECURITY_TOKEN_UNSUPPORTED);
 	int QueueChunk(int Flags, int DataSize, const void *pData);
 
 	const char *ErrorString();
 	void SignalResend();
 	int State() const { return m_State; }
 	const NETADDR *PeerAddress() const { return &m_PeerAddr; }
+	const std::array<char, NETADDR_MAXSTRSIZE> &PeerAddressString(bool IncludePort) const
+	{
+		return IncludePort ? m_aPeerAddrStr : m_aPeerAddrStrNoPort;
+	}
 	void ConnectAddresses(const NETADDR **ppAddrs, int *pNumAddrs) const
 	{
 		*ppAddrs = m_aConnectAddrs;
@@ -288,7 +326,7 @@ public:
 	void SetTimedOut(const NETADDR *pAddr, int Sequence, int Ack, SECURITY_TOKEN SecurityToken, CStaticRingBuffer<CNetChunkResend, NET_CONN_BUFFERSIZE> *pResendBuffer, bool Sixup);
 
 	// anti spoof
-	void DirectInit(NETADDR &Addr, SECURITY_TOKEN SecurityToken, SECURITY_TOKEN Token, bool Sixup);
+	void DirectInit(const NETADDR &Addr, SECURITY_TOKEN SecurityToken, SECURITY_TOKEN Token, bool Sixup);
 	void SetUnknownSeq() { m_UnknownSeq = true; }
 	void SetSequence(int Sequence) { m_Sequence = Sequence; }
 
@@ -334,13 +372,13 @@ public:
 	NETADDR m_Addr;
 	CNetConnection *m_pConnection;
 	int m_CurrentChunk;
-	int m_ClientID;
+	int m_ClientId;
 	CNetPacketConstruct m_Data;
 	unsigned char m_aBuffer[NET_MAX_PACKETSIZE];
 
 	CNetRecvUnpacker() { Clear(); }
 	void Clear();
-	void Start(const NETADDR *pAddr, CNetConnection *pConnection, int ClientID);
+	void Start(const NETADDR *pAddr, CNetConnection *pConnection, int ClientId);
 	int FetchChunk(CNetChunk *pChunk);
 };
 
@@ -365,7 +403,7 @@ class CNetServer
 	CNetBan *m_pNetBan;
 	CSlot m_aSlots[NET_MAX_CLIENTS];
 	int m_MaxClients;
-	int m_MaxClientsPerIP;
+	int m_MaxClientsPerIp;
 
 	NETFUNC_NEWCLIENT m_pfnNewClient;
 	NETFUNC_NEWCLIENT_NOAUTH m_pfnNewClientNoAuth;
@@ -388,7 +426,7 @@ class CNetServer
 	void OnTokenCtrlMsg(NETADDR &Addr, int ControlMsg, const CNetPacketConstruct &Packet);
 	int OnSixupCtrlMsg(NETADDR &Addr, CNetChunk *pChunk, int ControlMsg, const CNetPacketConstruct &Packet, SECURITY_TOKEN &ResponseToken, SECURITY_TOKEN Token);
 	void OnPreConnMsg(NETADDR &Addr, CNetPacketConstruct &Packet);
-	void OnConnCtrlMsg(NETADDR &Addr, int ClientID, int ControlMsg, const CNetPacketConstruct &Packet);
+	void OnConnCtrlMsg(NETADDR &Addr, int ClientId, int ControlMsg, const CNetPacketConstruct &Packet);
 	bool ClientExists(const NETADDR &Addr) { return GetClientSlot(Addr) != -1; }
 	int GetClientSlot(const NETADDR &Addr);
 	void SendControl(NETADDR &Addr, int ControlMsg, const void *pExtra, int ExtraSize, SECURITY_TOKEN SecurityToken);
@@ -396,14 +434,14 @@ class CNetServer
 	int TryAcceptClient(NETADDR &Addr, SECURITY_TOKEN SecurityToken, bool VanillaAuth = false, bool Sixup = false, SECURITY_TOKEN Token = 0);
 	int NumClientsWithAddr(NETADDR Addr);
 	bool Connlimit(NETADDR Addr);
-	void SendMsgs(NETADDR &Addr, const CMsgPacker *apMsgs[], int Num);
+	void SendMsgs(NETADDR &Addr, const CPacker **ppMsgs, int Num);
 
 public:
 	int SetCallbacks(NETFUNC_NEWCLIENT pfnNewClient, NETFUNC_DELCLIENT pfnDelClient, void *pUser);
 	int SetCallbacks(NETFUNC_NEWCLIENT pfnNewClient, NETFUNC_NEWCLIENT_NOAUTH pfnNewClientNoAuth, NETFUNC_CLIENTREJOIN pfnClientRejoin, NETFUNC_DELCLIENT pfnDelClient, void *pUser);
 
 	//
-	bool Open(NETADDR BindAddr, CNetBan *pNetBan, int MaxClients, int MaxClientsPerIP);
+	bool Open(NETADDR BindAddr, CNetBan *pNetBan, int MaxClients, int MaxClientsPerIp);
 	int Close();
 
 	//
@@ -412,11 +450,12 @@ public:
 	int Update();
 
 	//
-	int Drop(int ClientID, const char *pReason);
+	int Drop(int ClientId, const char *pReason);
 
 	// status requests
-	const NETADDR *ClientAddr(int ClientID) const { return m_aSlots[ClientID].m_Connection.PeerAddress(); }
-	bool HasSecurityToken(int ClientID) const { return m_aSlots[ClientID].m_Connection.SecurityToken() != NET_SECURITY_TOKEN_UNSUPPORTED; }
+	const NETADDR *ClientAddr(int ClientId) const { return m_aSlots[ClientId].m_Connection.PeerAddress(); }
+	const std::array<char, NETADDR_MAXSTRSIZE> &ClientAddrString(int ClientId, bool IncludePort) const { return m_aSlots[ClientId].m_Connection.PeerAddressString(IncludePort); }
+	bool HasSecurityToken(int ClientId) const { return m_aSlots[ClientId].m_Connection.SecurityToken() != NET_SECURITY_TOKEN_UNSUPPORTED; }
 	NETADDR Address() const { return m_Address; }
 	NETSOCKET Socket() const { return m_Socket; }
 	CNetBan *NetBan() const { return m_pNetBan; }
@@ -424,15 +463,14 @@ public:
 	int MaxClients() const { return m_MaxClients; }
 
 	void SendTokenSixup(NETADDR &Addr, SECURITY_TOKEN Token);
-	int SendConnlessSixup(CNetChunk *pChunk, SECURITY_TOKEN ResponseToken);
 
 	//
-	void SetMaxClientsPerIP(int Max);
-	bool SetTimedOut(int ClientID, int OrigID);
-	void SetTimeoutProtected(int ClientID);
+	void SetMaxClientsPerIp(int Max);
+	bool SetTimedOut(int ClientId, int OrigId);
+	void SetTimeoutProtected(int ClientId);
 
-	int ResetErrorString(int ClientID);
-	const char *ErrorString(int ClientID);
+	int ResetErrorString(int ClientId);
+	const char *ErrorString(int ClientId);
 
 	// anti spoof
 	SECURITY_TOKEN GetGlobalToken();
@@ -466,24 +504,59 @@ public:
 	int Close();
 
 	//
-	int Recv(char *pLine, int MaxLength, int *pClientID = nullptr);
-	int Send(int ClientID, const char *pLine);
+	int Recv(char *pLine, int MaxLength, int *pClientId = nullptr);
+	int Send(int ClientId, const char *pLine);
 	int Update();
 
 	//
 	int AcceptClient(NETSOCKET Socket, const NETADDR *pAddr);
-	int Drop(int ClientID, const char *pReason);
+	int Drop(int ClientId, const char *pReason);
 
 	// status requests
-	const NETADDR *ClientAddr(int ClientID) const { return m_aSlots[ClientID].m_Connection.PeerAddress(); }
+	const NETADDR *ClientAddr(int ClientId) const { return m_aSlots[ClientId].m_Connection.PeerAddress(); }
 	CNetBan *NetBan() const { return m_pNetBan; }
 };
 
+class CNetTokenCache
+{
+public:
+	void Init(NETSOCKET Socket);
+	void SendPacketConnless(CNetChunk *pChunk);
+	void FetchToken(NETADDR *pAddr);
+	void AddToken(const NETADDR *pAddr, TOKEN Token);
+	TOKEN GetToken(const NETADDR *pAddr);
+	TOKEN GenerateToken();
+	void Update();
+
+private:
+	class CConnlessPacketInfo
+	{
+	public:
+		NETADDR m_Addr;
+		int m_DataSize;
+		unsigned char m_aData[NET_MAX_PAYLOAD];
+		int64_t m_Expiry;
+	};
+
+	class CAddressInfo
+	{
+	public:
+		NETADDR m_Addr;
+		TOKEN m_Token;
+		int64_t m_Expiry;
+	};
+
+	NETSOCKET m_Socket;
+	std::vector<CAddressInfo> m_TokenCache;
+	std::vector<CConnlessPacketInfo> m_ConnlessPackets;
+};
+
 // client side
 class CNetClient
 {
 	CNetConnection m_Connection;
 	CNetRecvUnpacker m_RecvUnpacker;
+	CNetTokenCache m_TokenCache;
 
 	CStun *m_pStun = nullptr;
 
@@ -496,9 +569,10 @@ public:
 	// connection state
 	int Disconnect(const char *pReason);
 	int Connect(const NETADDR *pAddr, int NumAddrs);
+	int Connect7(const NETADDR *pAddr, int NumAddrs);
 
 	// communication
-	int Recv(CNetChunk *pChunk);
+	int Recv(CNetChunk *pChunk, SECURITY_TOKEN *pResponseToken, bool Sixup);
 	int Send(CNetChunk *pChunk);
 
 	// pumping
@@ -515,15 +589,13 @@ public:
 	int GotProblems(int64_t MaxLatency) const;
 	const char *ErrorString() const;
 
-	bool SecurityTokenUnknown() { return m_Connection.SecurityToken() == NET_SECURITY_TOKEN_UNKNOWN; }
-
 	// stun
 	void FeedStunServer(NETADDR StunServer);
 	void RefreshStun();
 	CONNECTIVITY GetConnectivity(int NetType, NETADDR *pGlobalAddr);
 };
 
-// TODO: both, fix these. This feels like a junk class for stuff that doesn't fit anywere
+// TODO: both, fix these. This feels like a junk class for stuff that doesn't fit anywhere
 class CNetBase
 {
 	static IOHANDLE ms_DataLogSent;
@@ -538,7 +610,9 @@ public:
 	static int Decompress(const void *pData, int DataSize, void *pOutput, int OutputSize);
 
 	static void SendControlMsg(NETSOCKET Socket, NETADDR *pAddr, int Ack, int ControlMsg, const void *pExtra, int ExtraSize, SECURITY_TOKEN SecurityToken, bool Sixup = false);
+	static void SendControlMsgWithToken7(NETSOCKET Socket, NETADDR *pAddr, TOKEN Token, int Ack, int ControlMsg, TOKEN MyToken, bool Extended);
 	static void SendPacketConnless(NETSOCKET Socket, NETADDR *pAddr, const void *pData, int DataSize, bool Extended, unsigned char aExtra[4]);
+	static void SendPacketConnlessWithToken7(NETSOCKET Socket, NETADDR *pAddr, const void *pData, int DataSize, SECURITY_TOKEN Token, SECURITY_TOKEN ResponseToken);
 	static void SendPacket(NETSOCKET Socket, NETADDR *pAddr, CNetPacketConstruct *pPacket, SECURITY_TOKEN SecurityToken, bool Sixup = false, bool NoCompress = false);
 
 	static int UnpackPacket(unsigned char *pBuffer, int Size, CNetPacketConstruct *pPacket, bool &Sixup, SECURITY_TOKEN *pSecurityToken = nullptr, SECURITY_TOKEN *pResponseToken = nullptr);
