@@ -8,7 +8,7 @@
 //
 // Example:
 //
-// 1) `i-unfreeze-you@ddnet.tw`
+// 1) `i-unfreeze-you@ddnet.org`
 // 2) `creeper@minetee`
 //
 // The first example applies if you own the `ddnet.org` domain, that is, if you
@@ -32,3 +32,11 @@ UUID(NETMSG_PONGEX, "pong@ddnet.tw")
 UUID(NETMSG_CHECKSUM_REQUEST, "checksum-request@ddnet.tw")
 UUID(NETMSG_CHECKSUM_RESPONSE, "checksum-response@ddnet.tw")
 UUID(NETMSG_CHECKSUM_ERROR, "checksum-error@ddnet.tw")
+UUID(NETMSG_REDIRECT, "redirect@ddnet.org")
+UUID(NETMSG_RCON_CMD_GROUP_START, "rcon-cmd-group-start@ddnet.org")
+UUID(NETMSG_RCON_CMD_GROUP_END, "rcon-cmd-group-end@ddnet.org")
+UUID(NETMSG_MAP_RELOAD, "map-reload@ddnet.org")
+UUID(NETMSG_RECONNECT, "reconnect@ddnet.org")
+UUID(NETMSG_MAPLIST_ADD, "sv-maplist-add@ddnet.org")
+UUID(NETMSG_MAPLIST_GROUP_START, "sv-maplist-start@ddnet.org")
+UUID(NETMSG_MAPLIST_GROUP_END, "sv-maplist-end@ddnet.org")
