@@ -16,3 +16,8 @@ UUID(TEEHISTORIAN_LOAD_FAILURE, "teehistorian-load-failure@ddnet.tw")
 UUID(TEEHISTORIAN_PLAYER_TEAM, "teehistorian-player-team@ddnet.tw")
 UUID(TEEHISTORIAN_TEAM_PRACTICE, "teehistorian-team-practice@ddnet.tw")
 UUID(TEEHISTORIAN_PLAYER_READY, "teehistorian-player-ready@ddnet.tw")
+UUID(TEEHISTORIAN_PLAYER_REJOIN, "teehistorian-rejoinver6@ddnet.org")
+UUID(TEEHISTORIAN_ANTIBOT, "teehistorian-antibot@ddnet.org")
+UUID(TEEHISTORIAN_PLAYER_NAME, "teehistorian-player-name@ddnet.org")
+UUID(TEEHISTORIAN_PLAYER_FINISH, "teehistorian-player-finish@ddnet.org")
+UUID(TEEHISTORIAN_TEAM_FINISH, "teehistorian-team-finish@ddnet.org")
