@@ -2,7 +2,7 @@
 
 #include "video.h"
 
-IVideo *IVideo::ms_pCurrentVideo = 0;
+IVideo *IVideo::ms_pCurrentVideo = nullptr;
 
 int64_t IVideo::ms_Time = 0;
 float IVideo::ms_LocalTime = 0;
