@@ -3,14 +3,16 @@
 
 #include <base/system.h>
 
-typedef void (*ISoundMixFunc)(short *pFinalOut, unsigned Frames);
+#include <functional>
+
+typedef std::function<void(short *pFinalOut, unsigned Frames)> ISoundMixFunc;
 
 class IVideo
 {
 public:
 	virtual ~IVideo(){};
 
-	virtual void Start() = 0;
+	virtual bool Start() = 0;
 	virtual void Stop() = 0;
 	virtual void Pause(bool Pause) = 0;
 	virtual bool IsRecording() = 0;
