@@ -1,11 +1,13 @@
 #ifndef ENGINE_STEAM_H
 #define ENGINE_STEAM_H
 
+#include <base/types.h>
+
 #include "kernel.h"
 
 class ISteam : public IInterface
 {
-	MACRO_INTERFACE("steam", 0)
+	MACRO_INTERFACE("steam")
 public:
 	// Returns NULL if the name cannot be determined.
 	virtual const char *GetPlayerName() = 0;
@@ -18,7 +20,7 @@ public:
 	virtual void Update() = 0;
 
 	virtual void ClearGameInfo() = 0;
-	virtual void SetGameInfo(NETADDR ServerAddr, const char *pMapName) = 0;
+	virtual void SetGameInfo(const NETADDR &ServerAddr, const char *pMapName, bool AnnounceAddr) = 0;
 };
 
 ISteam *CreateSteam();
