@@ -5,7 +5,7 @@
 
 #include <game/client/components/maplayers.h>
 
-#include <stdint.h>
+#include <cstdint>
 
 class CLayers;
 class CMapImages;
@@ -15,7 +15,7 @@ class CMapImages;
 
 class CBackgroundEngineMap : public CMap
 {
-	MACRO_INTERFACE("background_enginemap", 0)
+	MACRO_INTERFACE("background_enginemap")
 };
 
 class CBackground : public CMapLayers
@@ -25,9 +25,6 @@ protected:
 	bool m_Loaded;
 	char m_aMapName[MAX_MAP_LENGTH];
 
-	//to avoid spam when in menu
-	int64_t m_LastLoad;
-
 	//to avoid memory leak when switching to %current%
 	CBackgroundEngineMap *m_pBackgroundMap;
 	CLayers *m_pBackgroundLayers;
@@ -35,6 +32,8 @@ protected:
 
 	virtual CBackgroundEngineMap *CreateBGMap();
 
+	const char *LoadingTitle() const override;
+
 public:
 	CBackground(int MapType = CMapLayers::TYPE_BACKGROUND_FORCE, bool OnlineOnly = true);
 	virtual ~CBackground();
@@ -45,6 +44,7 @@ public:
 	virtual void OnRender() override;
 
 	void LoadBackground();
+	const char *MapName() const { return m_aMapName; }
 };
 
 #endif
