@@ -11,46 +11,45 @@
 
 #include <game/client/component.h>
 #include <game/client/lineinput.h>
+#include <game/client/render.h>
 #include <game/client/skin.h>
+#include <game/generated/protocol7.h>
+
+constexpr auto SAVES_FILE = "ddnet-saves.txt";
 
 class CChat : public CComponent
 {
-	CLineInput m_Input;
-
-	static constexpr float CHAT_WIDTH = 200.0f;
 	static constexpr float CHAT_HEIGHT_FULL = 200.0f;
 	static constexpr float CHAT_HEIGHT_MIN = 50.0f;
+	static constexpr float CHAT_FONTSIZE_WIDTH_RATIO = 2.5f;
 
 	enum
 	{
-		MAX_LINES = 25
+		MAX_LINES = 64,
+		MAX_LINE_LENGTH = 256
 	};
 
+	CLineInputBuffered<MAX_LINE_LENGTH> m_Input;
 	struct CLine
 	{
 		int64_t m_Time;
 		float m_aYOffset[2];
-		int m_ClientID;
+		int m_ClientId;
 		int m_TeamNumber;
 		bool m_Team;
 		bool m_Whisper;
 		int m_NameColor;
 		char m_aName[64];
-		char m_aText[512];
+		char m_aText[MAX_LINE_LENGTH];
 		bool m_Friend;
 		bool m_Highlighted;
+		std::optional<ColorRGBA> m_CustomColor;
 
-		int m_TextContainerIndex;
+		STextContainerIndex m_TextContainerIndex;
 		int m_QuadContainerIndex;
 
-		char m_aSkinName[std::size(g_Config.m_ClPlayerSkin)];
-		CSkin::SSkinTextures m_RenderSkin;
-		CSkin::SSkinMetrics m_RenderSkinMetrics;
-		bool m_CustomColoredSkin;
-		ColorRGBA m_ColorBody;
-		ColorRGBA m_ColorFeet;
+		std::shared_ptr<CManagedTeeRenderInfo> m_pManagedTeeRenderInfo;
 
-		bool m_HasRenderTee;
 		float m_TextYOffset;
 
 		int m_TimesRepeated;
@@ -62,9 +61,12 @@ class CChat : public CComponent
 	CLine m_aLines[MAX_LINES];
 	int m_CurrentLine;
 
-	// chat
 	enum
 	{
+		// client IDs for special messages
+		CLIENT_MSG = -2,
+		SERVER_MSG = -1,
+
 		MODE_NONE = 0,
 		MODE_ALL,
 		MODE_TEAM,
@@ -77,17 +79,15 @@ class CChat : public CComponent
 
 	int m_Mode;
 	bool m_Show;
-	bool m_InputUpdate;
-	int m_ChatStringOffset;
-	int m_OldChatStringLength;
 	bool m_CompletionUsed;
 	int m_CompletionChosen;
-	char m_aCompletionBuffer[256];
+	char m_aCompletionBuffer[MAX_LINE_LENGTH];
 	int m_PlaceholderOffset;
 	int m_PlaceholderLength;
+	static char ms_aDisplayText[MAX_LINE_LENGTH];
 	struct CRateablePlayer
 	{
-		int ClientID;
+		int ClientId;
 		int Score;
 	};
 	CRateablePlayer m_aPlayerCompletionList[MAX_CLIENTS];
@@ -95,22 +95,25 @@ class CChat : public CComponent
 
 	struct CCommand
 	{
-		const char *m_pName;
-		const char *m_pParams;
+		char m_aName[IConsole::TEMPCMD_NAME_LENGTH];
+		char m_aParams[IConsole::TEMPCMD_PARAMS_LENGTH];
+		char m_aHelpText[IConsole::TEMPCMD_HELP_LENGTH];
 
 		CCommand() = default;
-		CCommand(const char *pName, const char *pParams) :
-			m_pName(pName), m_pParams(pParams)
+		CCommand(const char *pName, const char *pParams, const char *pHelpText)
 		{
+			str_copy(m_aName, pName);
+			str_copy(m_aParams, pParams);
+			str_copy(m_aHelpText, pHelpText);
 		}
 
-		bool operator<(const CCommand &Other) const { return str_comp(m_pName, Other.m_pName) < 0; }
-		bool operator<=(const CCommand &Other) const { return str_comp(m_pName, Other.m_pName) <= 0; }
-		bool operator==(const CCommand &Other) const { return str_comp(m_pName, Other.m_pName) == 0; }
+		bool operator<(const CCommand &Other) const { return str_comp(m_aName, Other.m_aName) < 0; }
+		bool operator<=(const CCommand &Other) const { return str_comp(m_aName, Other.m_aName) <= 0; }
+		bool operator==(const CCommand &Other) const { return str_comp(m_aName, Other.m_aName) == 0; }
 	};
 
 	std::vector<CCommand> m_vCommands;
-	bool m_ReverseTAB;
+	bool m_CommandsNeedSorting;
 
 	struct CHistoryEntry
 	{
@@ -122,14 +125,22 @@ class CChat : public CComponent
 	int m_PendingChatCounter;
 	int64_t m_LastChatSend;
 	int64_t m_aLastSoundPlayed[CHAT_NUM];
+	bool m_IsInputCensored;
+	char m_aCurrentInputText[MAX_LINE_LENGTH];
+	bool m_EditingNewLine;
+
+	bool m_ServerSupportsCommandInfo;
 
 	static void ConSay(IConsole::IResult *pResult, void *pUserData);
 	static void ConSayTeam(IConsole::IResult *pResult, void *pUserData);
 	static void ConChat(IConsole::IResult *pResult, void *pUserData);
 	static void ConShowChat(IConsole::IResult *pResult, void *pUserData);
 	static void ConEcho(IConsole::IResult *pResult, void *pUserData);
+	static void ConClearChat(IConsole::IResult *pResult, void *pUserData);
 
 	static void ConchainChatOld(IConsole::IResult *pResult, void *pUserData, IConsole::FCommandCallback pfnCallback, void *pCallbackUserData);
+	static void ConchainChatFontSize(IConsole::IResult *pResult, void *pUserData, IConsole::FCommandCallback pfnCallback, void *pCallbackUserData);
+	static void ConchainChatWidth(IConsole::IResult *pResult, void *pUserData, IConsole::FCommandCallback pfnCallback, void *pCallbackUserData);
 
 	bool LineShouldHighlight(const char *pLine, const char *pName);
 	void StoreSave(const char *pText);
@@ -138,35 +149,54 @@ public:
 	CChat();
 	int Sizeof() const override { return sizeof(*this); }
 
-	static constexpr float MESSAGE_PADDING_X = 5.0f;
-	static constexpr float MESSAGE_TEE_SIZE = 7.0f;
 	static constexpr float MESSAGE_TEE_PADDING_RIGHT = 0.5f;
-	static constexpr float FONT_SIZE = 6.0f;
-	static constexpr float MESSAGE_PADDING_Y = 1.0f;
-	static constexpr float MESSAGE_ROUNDING = 3.0f;
-	static_assert(FONT_SIZE + MESSAGE_PADDING_Y >= MESSAGE_ROUNDING * 2.0f, "Corners for background chat are too huge for this combination of font size and message padding.");
 
 	bool IsActive() const { return m_Mode != MODE_NONE; }
-	void AddLine(int ClientID, int Team, const char *pLine);
+	void AddLine(int ClientId, int Team, const char *pLine);
 	void EnableMode(int Team);
 	void DisableMode();
-	void Say(int Team, const char *pLine);
-	void SayChat(const char *pLine);
-	void RegisterCommand(const char *pName, const char *pParams, int flags, const char *pHelp);
+	void RegisterCommand(const char *pName, const char *pParams, const char *pHelpText);
+	void UnregisterCommand(const char *pName);
 	void Echo(const char *pString);
 
 	void OnWindowResize() override;
 	void OnConsoleInit() override;
 	void OnStateChange(int NewState, int OldState) override;
 	void OnRender() override;
-	void RefindSkins();
-	void OnPrepareLines();
+	void OnPrepareLines(float y);
 	void Reset();
 	void OnRelease() override;
 	void OnMessage(int MsgType, void *pRawMsg) override;
-	bool OnInput(IInput::CEvent Event) override;
+	bool OnInput(const IInput::CEvent &Event) override;
 	void OnInit() override;
 
 	void RebuildChat();
+	void ClearLines();
+
+	void EnsureCoherentFontSize() const;
+	void EnsureCoherentWidth() const;
+
+	float FontSize() const { return g_Config.m_ClChatFontSize / 10.0f; }
+	float MessagePaddingX() const { return FontSize() * (5 / 6.f); }
+	float MessagePaddingY() const { return FontSize() * (1 / 6.f); }
+	float MessageTeeSize() const { return FontSize() * (7 / 6.f); }
+	float MessageRounding() const { return FontSize() * (1 / 2.f); }
+
+	// ----- send functions -----
+
+	// Sends a chat message to the server.
+	//
+	// @param Team MODE_ALL=0 MODE_TEAM=1
+	// @param pLine the chat message
+	void SendChat(int Team, const char *pLine);
+
+	// Sends a chat message to the server.
+	//
+	// It uses a queue with a maximum of 3 entries
+	// that ensures there is a minimum delay of one second
+	// between sent messages.
+	//
+	// It uses team or public chat depending on m_Mode.
+	void SendChatQueued(const char *pLine);
 };
 #endif
