@@ -16,8 +16,8 @@
 void CCountryFlags::LoadCountryflagsIndexfile()
 {
 	const char *pFilename = "countryflags/index.txt";
-	IOHANDLE File = Storage()->OpenFile(pFilename, IOFLAG_READ | IOFLAG_SKIP_BOM, IStorage::TYPE_ALL);
-	if(!File)
+	CLineReader LineReader;
+	if(!LineReader.OpenFile(Storage()->OpenFile(pFilename, IOFLAG_READ, IStorage::TYPE_ALL)))
 	{
 		char aBuf[128];
 		str_format(aBuf, sizeof(aBuf), "couldn't open index file '%s'", pFilename);
@@ -26,16 +26,13 @@ void CCountryFlags::LoadCountryflagsIndexfile()
 	}
 
 	char aOrigin[128];
-	CLineReader LineReader;
-	LineReader.Init(File);
-	char *pLine;
-	while((pLine = LineReader.Get()))
+	while(const char *pLine = LineReader.Get())
 	{
 		if(!str_length(pLine) || pLine[0] == '#') // skip empty lines and comments
 			continue;
 
 		str_copy(aOrigin, pLine);
-		char *pReplacement = LineReader.Get();
+		const char *pReplacement = LineReader.Get();
 		if(!pReplacement)
 		{
 			Console()->Print(IConsole::OUTPUT_LEVEL_ADDINFO, "countryflags", "unexpected end of index file");
@@ -63,7 +60,7 @@ void CCountryFlags::LoadCountryflagsIndexfile()
 		char aBuf[128];
 		CImageInfo Info;
 		str_format(aBuf, sizeof(aBuf), "countryflags/%s.png", aOrigin);
-		if(!Graphics()->LoadPNG(&Info, aBuf, IStorage::TYPE_ALL))
+		if(!Graphics()->LoadPng(Info, aBuf, IStorage::TYPE_ALL))
 		{
 			char aMsg[128];
 			str_format(aMsg, sizeof(aMsg), "failed to load '%s'", aBuf);
@@ -75,8 +72,7 @@ void CCountryFlags::LoadCountryflagsIndexfile()
 		CCountryFlag CountryFlag;
 		CountryFlag.m_CountryCode = CountryCode;
 		str_copy(CountryFlag.m_aCountryCodeString, aOrigin);
-		CountryFlag.m_Texture = Graphics()->LoadTextureRaw(Info.m_Width, Info.m_Height, Info.m_Format, Info.m_pData, Info.m_Format, 0, aOrigin);
-		Graphics()->FreePNG(&Info);
+		CountryFlag.m_Texture = Graphics()->LoadTextureRawMove(Info, 0, aBuf);
 
 		if(g_Config.m_Debug)
 		{
@@ -85,7 +81,7 @@ void CCountryFlags::LoadCountryflagsIndexfile()
 		}
 		m_vCountryFlags.push_back(CountryFlag);
 	}
-	io_close(File);
+
 	std::sort(m_vCountryFlags.begin(), m_vCountryFlags.end());
 
 	// find index of default item
@@ -143,13 +139,17 @@ const CCountryFlags::CCountryFlag *CCountryFlags::GetByIndex(size_t Index) const
 	return &m_vCountryFlags[Index % m_vCountryFlags.size()];
 }
 
-void CCountryFlags::Render(int CountryCode, const ColorRGBA *pColor, float x, float y, float w, float h)
+void CCountryFlags::Render(const CCountryFlag *pFlag, ColorRGBA Color, float x, float y, float w, float h)
 {
-	const CCountryFlag *pFlag = GetByCountryCode(CountryCode);
 	if(pFlag->m_Texture.IsValid())
 	{
 		Graphics()->TextureSet(pFlag->m_Texture);
-		Graphics()->SetColor(*pColor);
+		Graphics()->SetColor(Color);
 		Graphics()->RenderQuadContainerEx(m_FlagsQuadContainerIndex, 0, -1, x, y, w, h);
 	}
 }
+
+void CCountryFlags::Render(int CountryCode, ColorRGBA Color, float x, float y, float w, float h)
+{
+	Render(GetByCountryCode(CountryCode), Color, x, y, w, h);
+}
