@@ -2,18 +2,19 @@
 /* If you are missing that file, acquire a complete release at teeworlds.com.                */
 #ifndef GAME_CLIENT_COMPONENTS_DAMAGEIND_H
 #define GAME_CLIENT_COMPONENTS_DAMAGEIND_H
+#include <base/color.h>
 #include <base/vmath.h>
 #include <game/client/component.h>
 
 class CDamageInd : public CComponent
 {
-	int64_t m_Lastupdate;
 	struct CItem
 	{
 		vec2 m_Pos;
 		vec2 m_Dir;
-		float m_StartTime;
+		float m_RemainingLife;
 		float m_StartAngle;
+		ColorRGBA m_Color;
 	};
 
 	enum
@@ -24,17 +25,14 @@ class CDamageInd : public CComponent
 	CItem m_aItems[MAX_ITEMS];
 	int m_NumItems;
 
-	CItem *CreateI();
-	void DestroyI(CItem *pItem);
-
 	int m_DmgIndQuadContainerIndex;
 
 public:
 	CDamageInd();
 	virtual int Sizeof() const override { return sizeof(*this); }
 
-	void Create(vec2 Pos, vec2 Dir);
-	void Reset();
+	void Create(vec2 Pos, vec2 Dir, float Alpha);
+	virtual void OnReset() override;
 	virtual void OnRender() override;
 	virtual void OnInit() override;
 };
