@@ -15,8 +15,6 @@
 
 #include "effects.h"
 
-inline vec2 RandomDir() { return normalize(vec2(random_float() - 0.5f, random_float() - 0.5f)); }
-
 CEffects::CEffects()
 {
 	m_Add5hz = false;
@@ -24,7 +22,7 @@ CEffects::CEffects()
 	m_Add100hz = false;
 }
 
-void CEffects::AirJump(vec2 Pos)
+void CEffects::AirJump(vec2 Pos, float Alpha)
 {
 	CParticle p;
 	p.SetDefault();
@@ -34,11 +32,13 @@ void CEffects::AirJump(vec2 Pos)
 	p.m_LifeSpan = 0.5f;
 	p.m_StartSize = 48.0f;
 	p.m_EndSize = 0;
-	p.m_Rot = random_float() * pi * 2;
+	p.m_Rot = random_angle();
 	p.m_Rotspeed = pi * 2;
 	p.m_Gravity = 500;
 	p.m_Friction = 0.7f;
 	p.m_FlowAffected = 0.0f;
+	p.m_Color.a = Alpha;
+	p.m_StartAlpha = Alpha;
 	m_pClient->m_Particles.Add(CParticles::GROUP_GENERAL, &p);
 
 	p.m_Pos = Pos + vec2(6.0f, 16.0f);
@@ -48,17 +48,12 @@ void CEffects::AirJump(vec2 Pos)
 		m_pClient->m_Sounds.PlayAt(CSounds::CHN_WORLD, SOUND_PLAYER_AIRJUMP, 1.0f, Pos);
 }
 
-void CEffects::DamageIndicator(vec2 Pos, vec2 Dir)
+void CEffects::DamageIndicator(vec2 Pos, vec2 Dir, float Alpha)
 {
-	m_pClient->m_DamageInd.Create(Pos, Dir);
+	m_pClient->m_DamageInd.Create(Pos, Dir, Alpha);
 }
 
-void CEffects::ResetDamageIndicator()
-{
-	m_pClient->m_DamageInd.Reset();
-}
-
-void CEffects::PowerupShine(vec2 Pos, vec2 Size)
+void CEffects::PowerupShine(vec2 Pos, vec2 Size, float Alpha)
 {
 	if(!m_Add50hz)
 		return;
@@ -66,20 +61,22 @@ void CEffects::PowerupShine(vec2 Pos, vec2 Size)
 	CParticle p;
 	p.SetDefault();
 	p.m_Spr = SPRITE_PART_SLICE;
-	p.m_Pos = Pos + vec2((random_float() - 0.5f) * Size.x, (random_float() - 0.5f) * Size.y);
+	p.m_Pos = Pos + vec2(random_float(-0.5f, 0.5f), random_float(-0.5f, 0.5f)) * Size;
 	p.m_Vel = vec2(0, 0);
 	p.m_LifeSpan = 0.5f;
 	p.m_StartSize = 16.0f;
 	p.m_EndSize = 0;
-	p.m_Rot = random_float() * pi * 2;
+	p.m_Rot = random_angle();
 	p.m_Rotspeed = pi * 2;
 	p.m_Gravity = 500;
 	p.m_Friction = 0.9f;
 	p.m_FlowAffected = 0.0f;
+	p.m_Color.a = Alpha;
+	p.m_StartAlpha = Alpha;
 	m_pClient->m_Particles.Add(CParticles::GROUP_GENERAL, &p);
 }
 
-void CEffects::FreezingFlakes(vec2 Pos, vec2 Size)
+void CEffects::FreezingFlakes(vec2 Pos, vec2 Size, float Alpha)
 {
 	if(!m_Add5hz)
 		return;
@@ -87,23 +84,45 @@ void CEffects::FreezingFlakes(vec2 Pos, vec2 Size)
 	CParticle p;
 	p.SetDefault();
 	p.m_Spr = SPRITE_PART_SNOWFLAKE;
-	p.m_Pos = Pos + vec2((random_float() - 0.5f) * Size.x, (random_float() - 0.5f) * Size.y);
+	p.m_Pos = Pos + vec2(random_float(-0.5f, 0.5f), random_float(-0.5f, 0.5f)) * Size;
 	p.m_Vel = vec2(0, 0);
 	p.m_LifeSpan = 1.5f;
-	p.m_StartSize = (random_float() + 0.5f) * 16.0f;
+	p.m_StartSize = random_float(0.5f, 1.5f) * 16.0f;
 	p.m_EndSize = p.m_StartSize * 0.5f;
 	p.m_UseAlphaFading = true;
 	p.m_StartAlpha = 1.0f;
 	p.m_EndAlpha = 0.0f;
-	p.m_Rot = random_float() * pi * 2;
+	p.m_Rot = random_angle();
 	p.m_Rotspeed = pi;
-	p.m_Gravity = random_float() * 250.0f;
+	p.m_Gravity = random_float(250.0f);
 	p.m_Friction = 0.9f;
 	p.m_FlowAffected = 0.0f;
 	p.m_Collides = false;
+	p.m_Color.a = Alpha;
+	p.m_StartAlpha = Alpha;
 	m_pClient->m_Particles.Add(CParticles::GROUP_EXTRA, &p);
 }
 
+void CEffects::SparkleTrail(vec2 Pos, float Alpha)
+{
+	if(!m_Add50hz)
+		return;
+
+	CParticle p;
+	p.SetDefault();
+	p.m_Spr = SPRITE_PART_SPARKLE;
+	p.m_Pos = Pos + random_direction() * random_float(40.0f);
+	p.m_Vel = vec2(0, 0);
+	p.m_LifeSpan = 0.5f;
+	p.m_StartSize = 0.0f;
+	p.m_EndSize = random_float(20.0f, 30.0f);
+	p.m_UseAlphaFading = true;
+	p.m_StartAlpha = Alpha;
+	p.m_EndAlpha = std::min(0.2f, Alpha);
+	p.m_Collides = false;
+	m_pClient->m_Particles.Add(CParticles::GROUP_TRAIL_EXTRA, &p);
+}
+
 void CEffects::SmokeTrail(vec2 Pos, vec2 Vel, float Alpha, float TimePassed)
 {
 	if(!m_Add50hz && TimePassed < 0.001f)
@@ -113,17 +132,18 @@ void CEffects::SmokeTrail(vec2 Pos, vec2 Vel, float Alpha, float TimePassed)
 	p.SetDefault();
 	p.m_Spr = SPRITE_PART_SMOKE;
 	p.m_Pos = Pos;
-	p.m_Vel = Vel + RandomDir() * 50.0f;
-	p.m_LifeSpan = 0.5f + random_float() * 0.5f;
-	p.m_StartSize = 12.0f + random_float() * 8;
+	p.m_Vel = Vel + random_direction() * 50.0f;
+	p.m_LifeSpan = random_float(0.5f, 1.0f);
+	p.m_StartSize = random_float(12.0f, 20.0f);
 	p.m_EndSize = 0;
 	p.m_Friction = 0.7f;
-	p.m_Gravity = random_float() * -500.0f;
-	p.m_Color.a *= Alpha;
+	p.m_Gravity = random_float(-500.0f);
+	p.m_Color.a = Alpha;
+	p.m_StartAlpha = Alpha;
 	m_pClient->m_Particles.Add(CParticles::GROUP_PROJECTILE_TRAIL, &p, TimePassed);
 }
 
-void CEffects::SkidTrail(vec2 Pos, vec2 Vel)
+void CEffects::SkidTrail(vec2 Pos, vec2 Vel, float Alpha)
 {
 	if(!m_Add100hz)
 		return;
@@ -132,13 +152,14 @@ void CEffects::SkidTrail(vec2 Pos, vec2 Vel)
 	p.SetDefault();
 	p.m_Spr = SPRITE_PART_SMOKE;
 	p.m_Pos = Pos;
-	p.m_Vel = Vel + RandomDir() * 50.0f;
-	p.m_LifeSpan = 0.5f + random_float() * 0.5f;
-	p.m_StartSize = 24.0f + random_float() * 12;
+	p.m_Vel = Vel + random_direction() * 50.0f;
+	p.m_LifeSpan = random_float(0.5f, 1.0f);
+	p.m_StartSize = random_float(24.0f, 36.0f);
 	p.m_EndSize = 0;
 	p.m_Friction = 0.7f;
-	p.m_Gravity = random_float() * -500.0f;
-	p.m_Color = ColorRGBA(0.75f, 0.75f, 0.75f, 1.0f);
+	p.m_Gravity = random_float(-500.0f);
+	p.m_Color = ColorRGBA(0.75f, 0.75f, 0.75f, Alpha);
+	p.m_StartAlpha = Alpha;
 	m_pClient->m_Particles.Add(CParticles::GROUP_GENERAL, &p);
 }
 
@@ -151,15 +172,16 @@ void CEffects::BulletTrail(vec2 Pos, float Alpha, float TimePassed)
 	p.SetDefault();
 	p.m_Spr = SPRITE_PART_BALL;
 	p.m_Pos = Pos;
-	p.m_LifeSpan = 0.25f + random_float() * 0.25f;
+	p.m_LifeSpan = random_float(0.25f, 0.5f);
 	p.m_StartSize = 8.0f;
 	p.m_EndSize = 0;
 	p.m_Friction = 0.7f;
 	p.m_Color.a *= Alpha;
+	p.m_StartAlpha = Alpha;
 	m_pClient->m_Particles.Add(CParticles::GROUP_PROJECTILE_TRAIL, &p, TimePassed);
 }
 
-void CEffects::PlayerSpawn(vec2 Pos)
+void CEffects::PlayerSpawn(vec2 Pos, float Alpha)
 {
 	for(int i = 0; i < 32; i++)
 	{
@@ -167,26 +189,27 @@ void CEffects::PlayerSpawn(vec2 Pos)
 		p.SetDefault();
 		p.m_Spr = SPRITE_PART_SHELL;
 		p.m_Pos = Pos;
-		p.m_Vel = RandomDir() * (powf(random_float(), 3) * 600.0f);
-		p.m_LifeSpan = 0.3f + random_float() * 0.3f;
-		p.m_StartSize = 64.0f + random_float() * 32;
+		p.m_Vel = random_direction() * (std::pow(random_float(), 3) * 600.0f);
+		p.m_LifeSpan = random_float(0.3f, 0.6f);
+		p.m_StartSize = random_float(64.0f, 96.0f);
 		p.m_EndSize = 0;
-		p.m_Rot = random_float() * pi * 2;
+		p.m_Rot = random_angle();
 		p.m_Rotspeed = random_float();
-		p.m_Gravity = random_float() * -400.0f;
+		p.m_Gravity = random_float(-400.0f);
 		p.m_Friction = 0.7f;
-		p.m_Color = ColorRGBA(0xb5 / 255.0f, 0x50 / 255.0f, 0xcb / 255.0f, 1.0f);
+		p.m_Color = ColorRGBA(0xb5 / 255.0f, 0x50 / 255.0f, 0xcb / 255.0f, Alpha);
+		p.m_StartAlpha = Alpha;
 		m_pClient->m_Particles.Add(CParticles::GROUP_GENERAL, &p);
 	}
 	if(g_Config.m_SndGame)
 		m_pClient->m_Sounds.PlayAt(CSounds::CHN_WORLD, SOUND_PLAYER_SPAWN, 1.0f, Pos);
 }
 
-void CEffects::PlayerDeath(vec2 Pos, int ClientID)
+void CEffects::PlayerDeath(vec2 Pos, int ClientId, float Alpha)
 {
 	ColorRGBA BloodColor(1.0f, 1.0f, 1.0f);
 
-	if(ClientID >= 0)
+	if(ClientId >= 0)
 	{
 		// Use m_RenderInfo.m_CustomColoredSkin instead of m_UseCustomColor
 		// m_UseCustomColor says if the player's skin has a custom color (value sent from the client side)
@@ -194,11 +217,27 @@ void CEffects::PlayerDeath(vec2 Pos, int ClientID)
 		// m_RenderInfo.m_CustomColoredSkin Defines if in the context of the game the color is being customized,
 		// Using this value if the game is teams (red and blue), this value will be true even if the skin is with the normal color.
 		// And will use the team body color to create player death effect instead of tee color
-		if(m_pClient->m_aClients[ClientID].m_RenderInfo.m_CustomColoredSkin)
-			BloodColor = m_pClient->m_aClients[ClientID].m_RenderInfo.m_ColorBody;
+		if(m_pClient->Client()->IsSixup())
+		{
+			if(m_pClient->m_aClients[ClientId].m_RenderInfo.m_aSixup[g_Config.m_ClDummy].m_aUseCustomColors[protocol7::SKINPART_BODY])
+			{
+				BloodColor = m_pClient->m_aClients[ClientId].m_RenderInfo.m_aSixup[g_Config.m_ClDummy].m_aColors[protocol7::SKINPART_BODY];
+			}
+			else
+			{
+				BloodColor = m_pClient->m_aClients[ClientId].m_RenderInfo.m_aSixup[g_Config.m_ClDummy].m_BloodColor;
+			}
+		}
 		else
 		{
-			BloodColor = m_pClient->m_aClients[ClientID].m_RenderInfo.m_BloodColor;
+			if(m_pClient->m_aClients[ClientId].m_RenderInfo.m_CustomColoredSkin)
+			{
+				BloodColor = m_pClient->m_aClients[ClientId].m_RenderInfo.m_ColorBody;
+			}
+			else
+			{
+				BloodColor = m_pClient->m_aClients[ClientId].m_RenderInfo.m_BloodColor;
+			}
 		}
 	}
 
@@ -208,21 +247,76 @@ void CEffects::PlayerDeath(vec2 Pos, int ClientID)
 		p.SetDefault();
 		p.m_Spr = SPRITE_PART_SPLAT01 + (rand() % 3);
 		p.m_Pos = Pos;
-		p.m_Vel = RandomDir() * ((random_float() + 0.1f) * 900.0f);
-		p.m_LifeSpan = 0.3f + random_float() * 0.3f;
-		p.m_StartSize = 24.0f + random_float() * 16;
+		p.m_Vel = random_direction() * (random_float(0.1f, 1.1f) * 900.0f);
+		p.m_LifeSpan = random_float(0.3f, 0.6f);
+		p.m_StartSize = random_float(24.0f, 40.0f);
 		p.m_EndSize = 0;
-		p.m_Rot = random_float() * pi * 2;
-		p.m_Rotspeed = (random_float() - 0.5f) * pi;
+		p.m_Rot = random_angle();
+		p.m_Rotspeed = random_float(-0.5f, 0.5f) * pi;
 		p.m_Gravity = 800.0f;
 		p.m_Friction = 0.8f;
-		ColorRGBA c = BloodColor.v4() * (0.75f + random_float() * 0.25f);
-		p.m_Color = ColorRGBA(c.r, c.g, c.b, 0.75f);
+		ColorRGBA c = BloodColor.v4() * random_float(0.75f, 1.0f);
+		p.m_Color = ColorRGBA(c.r, c.g, c.b, 0.75f * Alpha);
+		p.m_StartAlpha = Alpha;
 		m_pClient->m_Particles.Add(CParticles::GROUP_GENERAL, &p);
 	}
 }
 
-void CEffects::Explosion(vec2 Pos)
+void CEffects::Confetti(vec2 Pos, float Alpha)
+{
+	ColorRGBA Red(1.0f, 0.4f, 0.4f);
+	ColorRGBA Green(0.4f, 1.0f, 0.4f);
+	ColorRGBA Blue(0.4f, 0.4f, 1.0f);
+	ColorRGBA Yellow(1.0f, 1.0f, 0.4f);
+	ColorRGBA Cyan(0.4f, 1.0f, 1.0f);
+	ColorRGBA Magenta(1.0f, 0.4f, 1.0f);
+
+	ColorRGBA aConfettiColors[] = {Red, Green, Blue, Yellow, Cyan, Magenta};
+
+	// powerful confettis
+	for(int i = 0; i < 32; i++)
+	{
+		CParticle p;
+		p.SetDefault();
+		p.m_Spr = SPRITE_PART_SPLAT01 + (rand() % 3);
+		p.m_Pos = Pos;
+		p.m_Vel = direction(-0.5f * pi + random_float(-0.2f, 0.2f)) * random_float(0.01f, 1.0f) * 2000.0f;
+		p.m_LifeSpan = random_float(1.0f, 1.2f);
+		p.m_StartSize = random_float(12.0f, 24.0f);
+		p.m_EndSize = 0;
+		p.m_Rot = random_angle();
+		p.m_Rotspeed = random_float(-0.5f, 0.5f) * pi;
+		p.m_Gravity = -700.0f;
+		p.m_Friction = 0.6f;
+		ColorRGBA c = aConfettiColors[(rand() % std::size(aConfettiColors))];
+		p.m_Color = c.WithMultipliedAlpha(0.75f * Alpha);
+		p.m_StartAlpha = Alpha;
+		m_pClient->m_Particles.Add(CParticles::GROUP_GENERAL, &p);
+	}
+
+	// broader confettis
+	for(int i = 0; i < 32; i++)
+	{
+		CParticle p;
+		p.SetDefault();
+		p.m_Spr = SPRITE_PART_SPLAT01 + (rand() % 3);
+		p.m_Pos = Pos;
+		p.m_Vel = direction(-0.5f * pi + random_float(-0.8f, 0.8f)) * random_float(0.01f, 1.0f) * 1500.0f;
+		p.m_LifeSpan = random_float(0.8f, 1.0f);
+		p.m_StartSize = random_float(12.0f, 24.0f);
+		p.m_EndSize = 0;
+		p.m_Rot = random_angle();
+		p.m_Rotspeed = random_float(-0.5f, 0.5f) * pi;
+		p.m_Gravity = -700.0f;
+		p.m_Friction = 0.6f;
+		ColorRGBA c = aConfettiColors[(rand() % std::size(aConfettiColors))];
+		p.m_Color = c.WithMultipliedAlpha(0.75f * Alpha);
+		p.m_StartAlpha = Alpha;
+		m_pClient->m_Particles.Add(CParticles::GROUP_GENERAL, &p);
+	}
+}
+
+void CEffects::Explosion(vec2 Pos, float Alpha)
 {
 	// add to flow
 	for(int y = -8; y <= 8; y++)
@@ -243,27 +337,53 @@ void CEffects::Explosion(vec2 Pos)
 	p.m_LifeSpan = 0.4f;
 	p.m_StartSize = 150.0f;
 	p.m_EndSize = 0;
-	p.m_Rot = random_float() * pi * 2;
+	p.m_Rot = random_angle();
+	p.m_Color.a = Alpha;
+	p.m_StartAlpha = Alpha;
 	m_pClient->m_Particles.Add(CParticles::GROUP_EXPLOSIONS, &p);
 
+	// Nudge position slightly to edge of closest tile so the
+	// smoke doesn't get stuck inside the tile.
+	if(Collision()->CheckPoint(Pos))
+	{
+		const vec2 DistanceToTopLeft = Pos - vec2(round_truncate(Pos.x / 32), round_truncate(Pos.y / 32)) * 32;
+
+		vec2 CheckOffset;
+		CheckOffset.x = (DistanceToTopLeft.x > 16 ? 32 : -1);
+		CheckOffset.y = (DistanceToTopLeft.y > 16 ? 32 : -1);
+		CheckOffset -= DistanceToTopLeft;
+
+		for(vec2 Mask : {vec2(1.0f, 0.0f), vec2(0.0f, 1.0f), vec2(1.0f, 1.0f)})
+		{
+			const vec2 NewPos = Pos + CheckOffset * Mask;
+			if(!Collision()->CheckPoint(NewPos))
+			{
+				Pos = NewPos;
+				break;
+			}
+		}
+	}
+
 	// add the smoke
 	for(int i = 0; i < 24; i++)
 	{
 		p.SetDefault();
 		p.m_Spr = SPRITE_PART_SMOKE;
 		p.m_Pos = Pos;
-		p.m_Vel = RandomDir() * ((1.0f + random_float() * 0.2f) * 1000.0f);
-		p.m_LifeSpan = 0.5f + random_float() * 0.4f;
-		p.m_StartSize = 32.0f + random_float() * 8;
+		p.m_Vel = random_direction() * (random_float(1.0f, 1.2f) * 1000.0f);
+		p.m_LifeSpan = random_float(0.5f, 0.9f);
+		p.m_StartSize = random_float(32.0f, 40.0f);
 		p.m_EndSize = 0;
-		p.m_Gravity = random_float() * -800.0f;
+		p.m_Gravity = random_float(-800.0f);
 		p.m_Friction = 0.4f;
 		p.m_Color = mix(vec4(0.75f, 0.75f, 0.75f, 1.0f), vec4(0.5f, 0.5f, 0.5f, 1.0f), random_float());
+		p.m_Color.a *= Alpha;
+		p.m_StartAlpha = p.m_Color.a;
 		m_pClient->m_Particles.Add(CParticles::GROUP_GENERAL, &p);
 	}
 }
 
-void CEffects::HammerHit(vec2 Pos)
+void CEffects::HammerHit(vec2 Pos, float Alpha)
 {
 	// add the explosion
 	CParticle p;
@@ -273,7 +393,9 @@ void CEffects::HammerHit(vec2 Pos)
 	p.m_LifeSpan = 0.3f;
 	p.m_StartSize = 120.0f;
 	p.m_EndSize = 0;
-	p.m_Rot = random_float() * pi * 2;
+	p.m_Rot = random_angle();
+	p.m_Color.a = Alpha;
+	p.m_StartAlpha = Alpha;
 	m_pClient->m_Particles.Add(CParticles::GROUP_EXPLOSIONS, &p);
 	if(g_Config.m_SndGame)
 		m_pClient->m_Sounds.PlayAt(CSounds::CHN_WORLD, SOUND_HAMMER_HIT, 1.0f, Pos);
