@@ -2,6 +2,9 @@
 /* If you are missing that file, acquire a complete release at teeworlds.com.                */
 #ifndef GAME_CLIENT_COMPONENTS_EFFECTS_H
 #define GAME_CLIENT_COMPONENTS_EFFECTS_H
+
+#include <base/vmath.h>
+
 #include <game/client/component.h>
 
 class CEffects : public CComponent
@@ -18,16 +21,17 @@ public:
 
 	void BulletTrail(vec2 Pos, float Alpha = 1.f, float TimePassed = 0.f);
 	void SmokeTrail(vec2 Pos, vec2 Vel, float Alpha = 1.f, float TimePassed = 0.f);
-	void SkidTrail(vec2 Pos, vec2 Vel);
-	void Explosion(vec2 Pos);
-	void HammerHit(vec2 Pos);
-	void AirJump(vec2 Pos);
-	void DamageIndicator(vec2 Pos, vec2 Dir);
-	void ResetDamageIndicator();
-	void PlayerSpawn(vec2 Pos);
-	void PlayerDeath(vec2 Pos, int ClientID);
-	void PowerupShine(vec2 Pos, vec2 Size);
-	void FreezingFlakes(vec2 Pos, vec2 Size);
+	void SkidTrail(vec2 Pos, vec2 Vel, float Alpha = 1.0f);
+	void Explosion(vec2 Pos, float Alpha = 1.0f);
+	void HammerHit(vec2 Pos, float Alpha = 1.0f);
+	void AirJump(vec2 Pos, float Alpha = 1.0f);
+	void DamageIndicator(vec2 Pos, vec2 Dir, float Alpha = 1.0f);
+	void PlayerSpawn(vec2 Pos, float Alpha = 1.0f);
+	void PlayerDeath(vec2 Pos, int ClientId, float Alpha = 1.0f);
+	void PowerupShine(vec2 Pos, vec2 Size, float Alpha = 1.0f);
+	void FreezingFlakes(vec2 Pos, vec2 Size, float Alpha = 1.0f);
+	void SparkleTrail(vec2 Pos, float Alpha = 1.0f);
+	void Confetti(vec2 Pos, float Alpha = 1.0f);
 
 	void Update();
 };
