@@ -3,7 +3,10 @@
 #ifndef GAME_CLIENT_COMPONENTS_EMOTICON_H
 #define GAME_CLIENT_COMPONENTS_EMOTICON_H
 #include <base/vmath.h>
+#include <engine/console.h>
+
 #include <game/client/component.h>
+#include <game/client/ui.h>
 
 class CEmoticon : public CComponent
 {
@@ -14,6 +17,9 @@ class CEmoticon : public CComponent
 	int m_SelectedEmote;
 	int m_SelectedEyeEmote;
 
+	CUi::CTouchState m_TouchState;
+	bool m_TouchPressedOutside;
+
 	static void ConKeyEmoticon(IConsole::IResult *pResult, void *pUserData);
 	static void ConEmote(IConsole::IResult *pResult, void *pUserData);
 
@@ -26,9 +32,12 @@ public:
 	virtual void OnRender() override;
 	virtual void OnRelease() override;
 	virtual bool OnCursorMove(float x, float y, IInput::ECursorType CursorType) override;
+	virtual bool OnInput(const IInput::CEvent &Event) override;
 
 	void Emote(int Emoticon);
 	void EyeEmote(int EyeEmote);
+
+	bool IsActive() const { return m_Active; }
 };
 
 #endif
