@@ -46,7 +46,6 @@ public:
 	virtual void OnMessage(int MsgType, void *pRawMsg) override;
 	virtual void OnMapLoad() override;
 	virtual void OnShutdown() override;
-
-	void OnNewSnapshot();
+	virtual void OnNewSnapshot() override;
 };
 #endif
