@@ -3,26 +3,30 @@
 #ifndef GAME_CLIENT_COMPONENTS_SPECTATOR_H
 #define GAME_CLIENT_COMPONENTS_SPECTATOR_H
 #include <base/vmath.h>
+#include <engine/console.h>
 
 #include <game/client/component.h>
+#include <game/client/ui.h>
 
 class CSpectator : public CComponent
 {
 	enum
 	{
+		MULTI_VIEW = -4,
 		NO_SELECTION = -3,
 	};
 
 	bool m_Active;
 	bool m_WasActive;
 
-	int m_SelectedSpectatorID;
+	int m_SelectedSpectatorId;
 	vec2 m_SelectorMouse;
 
-	float m_OldMouseX;
-	float m_OldMouseY;
+	CUi::CTouchState m_TouchState;
 
-	bool CanChangeSpectator();
+	float m_MultiViewActivateDelay;
+
+	bool CanChangeSpectatorId();
 	void SpectateNext(bool Reverse);
 
 	static void ConKeySpectator(IConsole::IResult *pResult, void *pUserData);
@@ -30,6 +34,7 @@ class CSpectator : public CComponent
 	static void ConSpectateNext(IConsole::IResult *pResult, void *pUserData);
 	static void ConSpectatePrevious(IConsole::IResult *pResult, void *pUserData);
 	static void ConSpectateClosest(IConsole::IResult *pResult, void *pUserData);
+	static void ConMultiView(IConsole::IResult *pResult, void *pUserData);
 
 public:
 	CSpectator();
@@ -37,11 +42,15 @@ public:
 
 	virtual void OnConsoleInit() override;
 	virtual bool OnCursorMove(float x, float y, IInput::ECursorType CursorType) override;
+	virtual bool OnInput(const IInput::CEvent &Event) override;
 	virtual void OnRender() override;
 	virtual void OnRelease() override;
 	virtual void OnReset() override;
 
-	void Spectate(int SpectatorID);
+	void Spectate(int SpectatorId);
+	void SpectateClosest();
+
+	bool IsActive() const { return m_Active; }
 };
 
 #endif
