@@ -6,13 +6,7 @@
 #include <game/client/prediction/entity.h>
 
 #include <game/gamecore.h>
-
-enum
-{
-	WEAPON_GAME = -3, // team switching etc
-	WEAPON_SELF = -2, // console kill command
-	WEAPON_WORLD = -1, // death tiles etc
-};
+#include <game/generated/protocol.h>
 
 enum
 {
@@ -33,9 +27,6 @@ class CCharacter : public CEntity
 public:
 	~CCharacter();
 
-	//character's size
-	static const int ms_PhysSize = 28;
-
 	void PreTick() override;
 	void Tick() override;
 	void TickDeferred() override;
@@ -54,6 +45,8 @@ public:
 
 	void OnPredictedInput(CNetObj_PlayerInput *pNewInput);
 	void OnDirectInput(CNetObj_PlayerInput *pNewInput);
+	void ReleaseHook();
+	void ResetHook();
 	void ResetInput();
 	void FireWeapon();
 
@@ -63,6 +56,12 @@ public:
 	void GiveNinja();
 	void RemoveNinja();
 
+	void ResetVelocity();
+	void SetVelocity(vec2 NewVelocity);
+	void SetRawVelocity(vec2 NewVelocity);
+	void AddVelocity(vec2 Addition);
+	void ApplyMoveRestrictions();
+
 	bool m_IsLocal;
 
 	CTeamsCore *TeamsCore();
@@ -71,12 +70,11 @@ public:
 	bool UnFreeze();
 	void GiveAllWeapons();
 	int Team();
-	bool CanCollide(int ClientID);
-	bool SameTeam(int ClientID);
+	bool CanCollide(int ClientId) override;
+	bool SameTeam(int ClientId);
 	bool m_NinjaJetpack;
 	int m_FreezeTime;
 	bool m_FrozenLastTick;
-	int m_TuneZone;
 	vec2 m_PrevPos;
 	vec2 m_PrevPrevPos;
 	int m_TeleCheckpoint;
@@ -84,7 +82,6 @@ public:
 	int m_TileIndex;
 	int m_TileFIndex;
 
-	int m_MoveRestrictions;
 	bool m_LastRefillJumps;
 
 	// Setters/Getters because i don't want to modify vanilla vars access modifiers
@@ -94,7 +91,7 @@ public:
 	void SetActiveWeapon(int ActiveWeap);
 	CCharacterCore GetCore() { return m_Core; }
 	void SetCore(CCharacterCore Core) { m_Core = Core; }
-	CCharacterCore *Core() { return &m_Core; }
+	const CCharacterCore *Core() const { return &m_Core; }
 	bool GetWeaponGot(int Type) { return m_Core.m_aWeapons[Type].m_Got; }
 	void SetWeaponGot(int Type, bool Value) { m_Core.m_aWeapons[Type].m_Got = Value; }
 	int GetWeaponAmmo(int Type) { return m_Core.m_aWeapons[Type].m_Ammo; }
@@ -102,8 +99,8 @@ public:
 	void SetNinjaActivationDir(vec2 ActivationDir) { m_Core.m_Ninja.m_ActivationDir = ActivationDir; }
 	void SetNinjaActivationTick(int ActivationTick) { m_Core.m_Ninja.m_ActivationTick = ActivationTick; }
 	void SetNinjaCurrentMoveTime(int CurrentMoveTime) { m_Core.m_Ninja.m_CurrentMoveTime = CurrentMoveTime; }
-	int GetCID() { return m_ID; }
-	void SetInput(CNetObj_PlayerInput *pNewInput)
+	int GetCid() { return m_Id; }
+	void SetInput(const CNetObj_PlayerInput *pNewInput)
 	{
 		m_LatestInput = m_Input = *pNewInput;
 		// it is not allowed to aim in the center
@@ -114,9 +111,9 @@ public:
 	};
 	int GetJumped() { return m_Core.m_Jumped; }
 	int GetAttackTick() { return m_AttackTick; }
-	int GetStrongWeakID() { return m_StrongWeakID; }
+	int GetStrongWeakId() { return m_StrongWeakId; }
 
-	CCharacter(CGameWorld *pGameWorld, int ID, CNetObj_Character *pChar, CNetObj_DDNetCharacter *pExtended = 0);
+	CCharacter(CGameWorld *pGameWorld, int Id, CNetObj_Character *pChar, CNetObj_DDNetCharacter *pExtended = nullptr);
 	void Read(CNetObj_Character *pChar, CNetObj_DDNetCharacter *pExtended, bool IsLocal);
 	void SetCoreWorld(CGameWorld *pGameWorld);
 
@@ -126,9 +123,11 @@ public:
 	int m_GameTeam;
 	bool m_CanMoveInFreeze;
 
-	bool Match(CCharacter *pChar);
+	bool Match(CCharacter *pChar) const;
 	void ResetPrediction();
 	void SetTuneZone(int Zone);
+	int GetOverriddenTuneZone() const;
+	int GetPureTuneZone() const;
 
 	bool HammerHitDisabled() { return m_Core.m_HammerHitDisabled; }
 	bool ShotgunHitDisabled() { return m_Core.m_ShotgunHitDisabled; }
@@ -148,6 +147,8 @@ private:
 	int m_ReloadTimer;
 	int m_AttackTick;
 
+	int m_MoveRestrictions;
+
 	// these are non-heldback inputs
 	CNetObj_PlayerInput m_LatestPrevInput;
 	CNetObj_PlayerInput m_LatestInput;
@@ -159,6 +160,10 @@ private:
 
 	int m_NumInputs;
 
+	// tune
+	int m_TuneZone;
+	int m_TuneZoneOverride;
+
 	// the player core for the physics
 	CCharacterCore m_Core;
 
@@ -173,7 +178,7 @@ private:
 
 	CTuningParams *CharacterTuning();
 
-	int m_StrongWeakID;
+	int m_StrongWeakId;
 
 	int m_LastWeaponSwitchTick;
 	int m_LastTuneZoneTick;
