@@ -2,22 +2,25 @@
 /* If you are missing that file, acquire a complete release at teeworlds.com.                */
 #include "pickup.h"
 #include "character.h"
+#include <game/client/pickup_data.h>
 #include <game/collision.h>
 #include <game/generated/protocol.h>
 #include <game/mapitems.h>
 
+static constexpr int gs_PickupPhysSize = 14;
+
 void CPickup::Tick()
 {
 	Move();
 	// Check if a player intersected us
-	CCharacter *apEnts[MAX_CLIENTS];
-	int Num = GameWorld()->FindEntities(m_Pos, 20.0f, (CEntity **)apEnts, MAX_CLIENTS, CGameWorld::ENTTYPE_CHARACTER);
+	CEntity *apEnts[MAX_CLIENTS];
+	int Num = GameWorld()->FindEntities(m_Pos, GetProximityRadius() + ms_CollisionExtraSize, apEnts, MAX_CLIENTS, CGameWorld::ENTTYPE_CHARACTER);
 	for(int i = 0; i < Num; ++i)
 	{
-		CCharacter *pChr = apEnts[i];
+		auto *pChr = static_cast<CCharacter *>(apEnts[i]);
 		if(pChr)
 		{
-			if(GameWorld()->m_WorldConfig.m_IsVanilla && distance(m_Pos, pChr->m_Pos) >= 20.0f * 2) // pickup distance is shorter on vanilla due to using ClosestEntity
+			if(GameWorld()->m_WorldConfig.m_IsVanilla && distance(m_Pos, pChr->m_Pos) >= (GetProximityRadius() + ms_CollisionExtraSize) * 2) // pickup distance is shorter on vanilla due to using ClosestEntity
 				continue;
 			if(m_Layer == LAYER_SWITCH && m_Number > 0 && m_Number < (int)Switchers().size() && !Switchers()[m_Number].m_aStatus[pChr->Team()])
 				continue;
@@ -128,37 +131,27 @@ void CPickup::Tick()
 
 void CPickup::Move()
 {
-	if(GameWorld()->GameTick() % int(GameWorld()->GameTickSpeed() * 0.15f) == 0)
+	if(GameWorld()->GameTick() % (int)(GameWorld()->GameTickSpeed() * 0.15f) == 0)
 	{
-		int Flags;
-		int index = Collision()->IsMover(m_Pos.x, m_Pos.y, &Flags);
-		if(index)
+		if(Collision()->MoverSpeed(m_Pos.x, m_Pos.y, &m_Core))
 		{
 			m_IsCoreActive = true;
-			m_Core = Collision()->CpSpeed(index, Flags);
 		}
 		m_Pos += m_Core;
 	}
 }
 
-CPickup::CPickup(CGameWorld *pGameWorld, int ID, CNetObj_Pickup *pPickup, const CNetObj_EntityEx *pEntEx) :
-	CEntity(pGameWorld, CGameWorld::ENTTYPE_PICKUP)
+CPickup::CPickup(CGameWorld *pGameWorld, int Id, const CPickupData *pPickup) :
+	CEntity(pGameWorld, CGameWorld::ENTTYPE_PICKUP, vec2(0, 0), gs_PickupPhysSize)
 {
-	m_Pos.x = pPickup->m_X;
-	m_Pos.y = pPickup->m_Y;
+	m_Pos = pPickup->m_Pos;
 	m_Type = pPickup->m_Type;
 	m_Subtype = pPickup->m_Subtype;
 	m_Core = vec2(0.f, 0.f);
 	m_IsCoreActive = false;
-	m_ID = ID;
-	m_Layer = LAYER_GAME;
-	m_Number = 0;
-
-	if(pEntEx)
-	{
-		m_Layer = pEntEx->m_Layer;
-		m_Number = pEntEx->m_SwitchNumber;
-	}
+	m_Id = Id;
+	m_Number = pPickup->m_SwitchNumber;
+	m_Layer = m_Number > 0 ? LAYER_SWITCH : LAYER_GAME;
 }
 
 void CPickup::FillInfo(CNetObj_Pickup *pPickup)
