@@ -5,16 +5,23 @@
 
 #include <game/client/prediction/entity.h>
 
+class CPickupData;
+
 class CPickup : public CEntity
 {
 public:
+	static const int ms_CollisionExtraSize = 6;
+
 	void Tick() override;
 
-	CPickup(CGameWorld *pGameWorld, int ID, CNetObj_Pickup *pPickup, const CNetObj_EntityEx *pEntEx = 0);
+	CPickup(CGameWorld *pGameWorld, int Id, const CPickupData *pPickup);
 	void FillInfo(CNetObj_Pickup *pPickup);
 	bool Match(CPickup *pPickup);
 	bool InDDNetTile() { return m_IsCoreActive; }
 
+	int Type() const { return m_Type; }
+	int Subtype() const { return m_Subtype; }
+
 private:
 	int m_Type;
 	int m_Subtype;
