@@ -7,20 +7,25 @@
 #include <game/teamscore.h>
 
 #include <list>
+#include <vector>
 
 class CCollision;
 class CCharacter;
 class CEntity;
+class CMapBugs;
 
 class CGameWorld
 {
-	friend CCharacter;
-
 public:
 	enum
 	{
 		ENTTYPE_PROJECTILE = 0,
 		ENTTYPE_LASER,
+		ENTTYPE_DOOR,
+		ENTTYPE_DRAGGER,
+		ENTTYPE_LIGHT,
+		ENTTYPE_GUN,
+		ENTTYPE_PLASMA,
 		ENTTYPE_PICKUP,
 		ENTTYPE_FLAG,
 		ENTTYPE_CHARACTER,
@@ -36,32 +41,32 @@ public:
 	CEntity *FindFirst(int Type);
 	CEntity *FindLast(int Type);
 	int FindEntities(vec2 Pos, float Radius, CEntity **ppEnts, int Max, int Type);
-	CCharacter *IntersectCharacter(vec2 Pos0, vec2 Pos1, float Radius, vec2 &NewPos, CCharacter *pNotThis = nullptr, int CollideWith = -1, CCharacter *pThisOnly = nullptr);
+	CCharacter *IntersectCharacter(vec2 Pos0, vec2 Pos1, float Radius, vec2 &NewPos, const CCharacter *pNotThis = nullptr, int CollideWith = -1, const CCharacter *pThisOnly = nullptr);
+	CEntity *IntersectEntity(vec2 Pos0, vec2 Pos1, float Radius, int Type, vec2 &NewPos, const CEntity *pNotThis = nullptr, int CollideWith = -1, const CEntity *pThisOnly = nullptr);
 	void InsertEntity(CEntity *pEntity, bool Last = false);
 	void RemoveEntity(CEntity *pEntity);
 	void RemoveCharacter(CCharacter *pChar);
 	void Tick();
 
 	// DDRace
-	void ReleaseHooked(int ClientID);
-	std::list<CCharacter *> IntersectedCharacters(vec2 Pos0, vec2 Pos1, float Radius, CEntity *pNotThis = nullptr);
+	void ReleaseHooked(int ClientId);
+	std::vector<CCharacter *> IntersectedCharacters(vec2 Pos0, vec2 Pos1, float Radius, const CEntity *pNotThis = nullptr);
 
 	int m_GameTick;
-	int m_GameTickSpeed;
 	CCollision *m_pCollision;
 
 	// getter for server variables
 	int GameTick() { return m_GameTick; }
-	int GameTickSpeed() { return m_GameTickSpeed; }
+	int GameTickSpeed() { return SERVER_TICK_SPEED; }
 	CCollision *Collision() { return m_pCollision; }
 	CTeamsCore *Teams() { return &m_Teams; }
 	std::vector<SSwitchers> &Switchers() { return m_Core.m_vSwitchers; }
 	CTuningParams *Tuning();
-	CEntity *GetEntity(int ID, int EntityType);
-	CCharacter *GetCharacterByID(int ID) { return (ID >= 0 && ID < MAX_CLIENTS) ? m_apCharacters[ID] : nullptr; }
+	CEntity *GetEntity(int Id, int EntityType);
+	CCharacter *GetCharacterById(int Id) { return (Id >= 0 && Id < MAX_CLIENTS) ? m_apCharacters[Id] : nullptr; }
 
 	// from gamecontext
-	void CreateExplosion(vec2 Pos, int Owner, int Weapon, bool NoDamage, int ActivatedTeam, int64_t Mask);
+	void CreateExplosion(vec2 Pos, int Owner, int Weapon, bool NoDamage, int ActivatedTeam, CClientMask Mask);
 
 	// for client side prediction
 	struct
@@ -84,19 +89,25 @@ public:
 	CGameWorld *m_pParent;
 	CGameWorld *m_pChild;
 
-	void OnModified();
-	void NetObjBegin();
-	void NetCharAdd(int ObjID, CNetObj_Character *pChar, CNetObj_DDNetCharacter *pExtended, int GameTeam, bool IsLocal);
-	void NetObjAdd(int ObjID, int ObjType, const void *pObjData, const CNetObj_EntityEx *pDataEx);
-	void NetObjEnd(int LocalID);
+	int m_LocalClientId;
+
+	bool IsLocalTeam(int OwnerId) const;
+	void OnModified() const;
+	void NetObjBegin(CTeamsCore Teams, int LocalClientId);
+	void NetCharAdd(int ObjId, CNetObj_Character *pChar, CNetObj_DDNetCharacter *pExtended, int GameTeam, bool IsLocal);
+	void NetObjAdd(int ObjId, int ObjType, const void *pObjData, const CNetObj_EntityEx *pDataEx);
+	void NetObjEnd();
 	void CopyWorld(CGameWorld *pFrom);
-	CEntity *FindMatch(int ObjID, int ObjType, const void *pObjData);
+	CEntity *FindMatch(int ObjId, int ObjType, const void *pObjData);
 	void Clear();
 
 	CTuningParams *m_pTuningList;
 	CTuningParams *TuningList() { return m_pTuningList; }
 	CTuningParams *GetTuning(int i) { return &TuningList()[i]; }
 
+	const CMapBugs *m_pMapBugs;
+	bool EmulateBug(int Bug) const;
+
 private:
 	void RemoveEntities();
 
@@ -109,31 +120,36 @@ private:
 class CCharOrder
 {
 public:
-	std::list<int> m_IDs; // reverse of the order in the gameworld, since entities will be inserted in reverse
+	std::list<int> m_Ids; // reverse of the order in the gameworld, since entities will be inserted in reverse
 	CCharOrder()
 	{
+		Reset();
+	}
+	void Reset()
+	{
+		m_Ids.clear();
 		for(int i = 0; i < MAX_CLIENTS; i++)
-			m_IDs.push_back(i);
+			m_Ids.push_back(i);
 	}
 	void GiveStrong(int c)
 	{
 		if(0 <= c && c < MAX_CLIENTS)
 		{
-			m_IDs.remove(c);
-			m_IDs.push_front(c);
+			m_Ids.remove(c);
+			m_Ids.push_front(c);
 		}
 	}
 	void GiveWeak(int c)
 	{
 		if(0 <= c && c < MAX_CLIENTS)
 		{
-			m_IDs.remove(c);
-			m_IDs.push_back(c);
+			m_Ids.remove(c);
+			m_Ids.push_back(c);
 		}
 	}
 	bool HasStrongAgainst(int From, int To)
 	{
-		for(int i : m_IDs)
+		for(int i : m_Ids)
 		{
 			if(i == To)
 				return false;
