@@ -1,12 +1,39 @@
 #include <cctype>
-#include <list>
+#include <vector>
 
 #include <game/client/gameclient.h>
+#include <game/collision.h>
 #include <game/mapitems.h>
 
 #include "race.h"
 
-int CRaceHelper::ms_aFlagIndex[2] = {-1, -1};
+void CRaceHelper::Init(const CGameClient *pGameClient)
+{
+	m_pGameClient = pGameClient;
+
+	m_aFlagIndex[TEAM_RED] = -1;
+	m_aFlagIndex[TEAM_BLUE] = -1;
+
+	const CTile *pGameTiles = m_pGameClient->Collision()->GameLayer();
+	const int MapSize = m_pGameClient->Collision()->GetWidth() * m_pGameClient->Collision()->GetHeight();
+	for(int Index = 0; Index < MapSize; Index++)
+	{
+		const int EntityIndex = pGameTiles[Index].m_Index - ENTITY_OFFSET;
+		if(EntityIndex == ENTITY_FLAGSTAND_RED)
+		{
+			m_aFlagIndex[TEAM_RED] = Index;
+			if(m_aFlagIndex[TEAM_BLUE] != -1)
+				break; // Found both flags
+		}
+		else if(EntityIndex == ENTITY_FLAGSTAND_BLUE)
+		{
+			m_aFlagIndex[TEAM_BLUE] = Index;
+			if(m_aFlagIndex[TEAM_RED] != -1)
+				break; // Found both flags
+		}
+		Index += pGameTiles[Index].m_Skip;
+	}
+}
 
 int CRaceHelper::TimeFromSecondsStr(const char *pStr)
 {
@@ -21,7 +48,7 @@ int CRaceHelper::TimeFromSecondsStr(const char *pStr)
 	{
 		pStr++;
 		static const int s_aMult[3] = {100, 10, 1};
-		for(int i = 0; isdigit(pStr[i]) && i < 3; i++)
+		for(size_t i = 0; i < std::size(s_aMult) && isdigit(pStr[i]); i++)
 			Time += (pStr[i] - '0') * s_aMult[i];
 	}
 	return Time;
@@ -66,30 +93,32 @@ int CRaceHelper::TimeFromFinishMessage(const char *pStr, char *pNameBuf, int Nam
 	return TimeFromStr(pFinished + str_length(s_pFinishedStr));
 }
 
-bool CRaceHelper::IsStart(CGameClient *pClient, vec2 Prev, vec2 Pos)
+bool CRaceHelper::IsStart(vec2 Prev, vec2 Pos) const
 {
-	CCollision *pCollision = pClient->Collision();
-	if(pClient->m_GameInfo.m_FlagStartsRace)
+	if(m_pGameClient->m_GameInfo.m_FlagStartsRace)
 	{
-		int EnemyTeam = pClient->m_aClients[pClient->m_Snap.m_LocalClientID].m_Team ^ 1;
-		return ms_aFlagIndex[EnemyTeam] != -1 && distance(Pos, pCollision->GetPos(ms_aFlagIndex[EnemyTeam])) < 32;
+		int EnemyTeam = m_pGameClient->m_aClients[m_pGameClient->m_Snap.m_LocalClientId].m_Team ^ 1;
+		return m_aFlagIndex[EnemyTeam] != -1 && distance(Pos, m_pGameClient->Collision()->GetPos(m_aFlagIndex[EnemyTeam])) < 32;
 	}
 	else
 	{
-		std::list<int> Indices = pCollision->GetMapIndices(Prev, Pos);
-		if(!Indices.empty())
-			for(int &Indice : Indices)
+		std::vector<int> vIndices = m_pGameClient->Collision()->GetMapIndices(Prev, Pos);
+		if(!vIndices.empty())
+		{
+			for(const int Index : vIndices)
 			{
-				if(pCollision->GetTileIndex(Indice) == TILE_START)
+				if(m_pGameClient->Collision()->GetTileIndex(Index) == TILE_START)
 					return true;
-				if(pCollision->GetFTileIndex(Indice) == TILE_START)
+				if(m_pGameClient->Collision()->GetFrontTileIndex(Index) == TILE_START)
 					return true;
 			}
+		}
 		else
 		{
-			if(pCollision->GetTileIndex(pCollision->GetPureMapIndex(Pos)) == TILE_START)
+			const int Index = m_pGameClient->Collision()->GetPureMapIndex(Pos);
+			if(m_pGameClient->Collision()->GetTileIndex(Index) == TILE_START)
 				return true;
-			if(pCollision->GetFTileIndex(pCollision->GetPureMapIndex(Pos)) == TILE_START)
+			if(m_pGameClient->Collision()->GetFrontTileIndex(Index) == TILE_START)
 				return true;
 		}
 	}
