@@ -5,8 +5,11 @@
 #include <engine/shared/linereader.h>
 #include <engine/storage.h>
 
+#include <game/editor/mapitems/layer_tiles.h>
+#include <game/mapitems.h>
+
 #include "auto_map.h"
-#include "editor.h"
+#include "editor_actions.h"
 
 // Based on triple32inc from https://github.com/skeeto/hash-prospector/tree/79a6074062a84907df6e45b756134b74e2956760
 static uint32_t HashUInt32(uint32_t Num)
@@ -39,29 +42,28 @@ static int HashLocation(uint32_t Seed, uint32_t Run, uint32_t Rule, uint32_t X,
 
 CAutoMapper::CAutoMapper(CEditor *pEditor)
 {
-	m_pEditor = pEditor;
-	m_FileLoaded = false;
+	OnInit(pEditor);
 }
 
 void CAutoMapper::Load(const char *pTileName)
 {
-	char aPath[256];
-	str_format(aPath, sizeof(aPath), "editor/%s.rules", pTileName);
-	IOHANDLE RulesFile = m_pEditor->Storage()->OpenFile(aPath, IOFLAG_READ | IOFLAG_SKIP_BOM, IStorage::TYPE_ALL);
-	if(!RulesFile)
-		return;
-
+	char aPath[IO_MAX_PATH_LENGTH];
+	str_format(aPath, sizeof(aPath), "editor/automap/%s.rules", pTileName);
 	CLineReader LineReader;
-	LineReader.Init(RulesFile);
+	if(!LineReader.OpenFile(Storage()->OpenFile(aPath, IOFLAG_READ, IStorage::TYPE_ALL)))
+	{
+		char aBuf[IO_MAX_PATH_LENGTH + 32];
+		str_format(aBuf, sizeof(aBuf), "failed to load %s", aPath);
+		Console()->Print(IConsole::OUTPUT_LEVEL_DEBUG, "editor/automap", aBuf);
+		return;
+	}
 
 	CConfiguration *pCurrentConf = nullptr;
 	CRun *pCurrentRun = nullptr;
 	CIndexRule *pCurrentIndex = nullptr;
 
-	char aBuf[256];
-
 	// read each line
-	while(char *pLine = LineReader.Get())
+	while(const char *pLine = LineReader.Get())
 	{
 		// skip blank/empty lines as well as comments
 		if(str_length(pLine) > 0 && pLine[0] != '#' && pLine[0] != '\n' && pLine[0] != '\r' && pLine[0] != '\t' && pLine[0] != '\v' && pLine[0] != ' ')
@@ -77,16 +79,16 @@ void CAutoMapper::Load(const char *pTileName)
 				NewConf.m_EndX = 0;
 				NewConf.m_EndY = 0;
 				m_vConfigs.push_back(NewConf);
-				int ConfigurationID = m_vConfigs.size() - 1;
-				pCurrentConf = &m_vConfigs[ConfigurationID];
-				str_copy(pCurrentConf->m_aName, pLine, str_length(pLine));
+				int ConfigurationId = m_vConfigs.size() - 1;
+				pCurrentConf = &m_vConfigs[ConfigurationId];
+				str_copy(pCurrentConf->m_aName, pLine, minimum<int>(sizeof(pCurrentConf->m_aName), str_length(pLine)));
 
 				// add start run
 				CRun NewRun;
 				NewRun.m_AutomapCopy = true;
 				pCurrentConf->m_vRuns.push_back(NewRun);
-				int RunID = pCurrentConf->m_vRuns.size() - 1;
-				pCurrentRun = &pCurrentConf->m_vRuns[RunID];
+				int RunId = pCurrentConf->m_vRuns.size() - 1;
+				pCurrentRun = &pCurrentConf->m_vRuns[RunId];
 			}
 			else if(str_startswith(pLine, "NewRun") && pCurrentConf)
 			{
@@ -94,21 +96,20 @@ void CAutoMapper::Load(const char *pTileName)
 				CRun NewRun;
 				NewRun.m_AutomapCopy = true;
 				pCurrentConf->m_vRuns.push_back(NewRun);
-				int RunID = pCurrentConf->m_vRuns.size() - 1;
-				pCurrentRun = &pCurrentConf->m_vRuns[RunID];
+				int RunId = pCurrentConf->m_vRuns.size() - 1;
+				pCurrentRun = &pCurrentConf->m_vRuns[RunId];
 			}
 			else if(str_startswith(pLine, "Index") && pCurrentRun)
 			{
 				// new index
-				int ID = 0;
+				CIndexRule NewIndexRule;
+
 				char aOrientation1[128] = "";
 				char aOrientation2[128] = "";
 				char aOrientation3[128] = "";
 
-				sscanf(pLine, "Index %d %127s %127s %127s", &ID, aOrientation1, aOrientation2, aOrientation3);
+				sscanf(pLine, "Index %d %127s %127s %127s", &NewIndexRule.m_Id, aOrientation1, aOrientation2, aOrientation3);
 
-				CIndexRule NewIndexRule;
-				NewIndexRule.m_ID = ID;
 				NewIndexRule.m_Flag = 0;
 				NewIndexRule.m_RandomProbability = 1.0f;
 				NewIndexRule.m_DefaultRule = true;
@@ -116,39 +117,18 @@ void CAutoMapper::Load(const char *pTileName)
 				NewIndexRule.m_SkipFull = false;
 
 				if(str_length(aOrientation1) > 0)
-				{
-					if(!str_comp(aOrientation1, "XFLIP"))
-						NewIndexRule.m_Flag |= TILEFLAG_VFLIP;
-					else if(!str_comp(aOrientation1, "YFLIP"))
-						NewIndexRule.m_Flag |= TILEFLAG_HFLIP;
-					else if(!str_comp(aOrientation1, "ROTATE"))
-						NewIndexRule.m_Flag |= TILEFLAG_ROTATE;
-				}
+					NewIndexRule.m_Flag = CheckIndexFlag(NewIndexRule.m_Flag, aOrientation1, false);
 
 				if(str_length(aOrientation2) > 0)
-				{
-					if(!str_comp(aOrientation2, "XFLIP"))
-						NewIndexRule.m_Flag |= TILEFLAG_VFLIP;
-					else if(!str_comp(aOrientation2, "YFLIP"))
-						NewIndexRule.m_Flag |= TILEFLAG_HFLIP;
-					else if(!str_comp(aOrientation2, "ROTATE"))
-						NewIndexRule.m_Flag |= TILEFLAG_ROTATE;
-				}
+					NewIndexRule.m_Flag = CheckIndexFlag(NewIndexRule.m_Flag, aOrientation2, false);
 
 				if(str_length(aOrientation3) > 0)
-				{
-					if(!str_comp(aOrientation3, "XFLIP"))
-						NewIndexRule.m_Flag |= TILEFLAG_VFLIP;
-					else if(!str_comp(aOrientation3, "YFLIP"))
-						NewIndexRule.m_Flag |= TILEFLAG_HFLIP;
-					else if(!str_comp(aOrientation3, "ROTATE"))
-						NewIndexRule.m_Flag |= TILEFLAG_ROTATE;
-				}
+					NewIndexRule.m_Flag = CheckIndexFlag(NewIndexRule.m_Flag, aOrientation3, false);
 
 				// add the index rule object and make it current
 				pCurrentRun->m_vIndexRules.push_back(NewIndexRule);
-				int IndexRuleID = pCurrentRun->m_vIndexRules.size() - 1;
-				pCurrentIndex = &pCurrentRun->m_vIndexRules[IndexRuleID];
+				int IndexRuleId = pCurrentRun->m_vIndexRules.size() - 1;
+				pCurrentIndex = &pCurrentRun->m_vIndexRules[IndexRuleId];
 			}
 			else if(str_startswith(pLine, "Pos") && pCurrentIndex)
 			{
@@ -169,9 +149,9 @@ void CAutoMapper::Load(const char *pTileName)
 				{
 					Value = CPosRule::NOTINDEX;
 					CIndexInfo NewIndexInfo1 = {0, 0, false};
-					//CIndexInfo NewIndexInfo2 = {-1, 0};
+					// CIndexInfo NewIndexInfo2 = {-1, 0};
 					vNewIndexList.push_back(NewIndexInfo1);
-					//vNewIndexList.push_back(NewIndexInfo2);
+					// vNewIndexList.push_back(NewIndexInfo2);
 				}
 				else if(!str_comp(aValue, "INDEX") || !str_comp(aValue, "NOTINDEX"))
 				{
@@ -183,15 +163,14 @@ void CAutoMapper::Load(const char *pTileName)
 					int pWord = 4;
 					while(true)
 					{
-						int ID = 0;
+						CIndexInfo NewIndexInfo;
+
 						char aOrientation1[128] = "";
 						char aOrientation2[128] = "";
 						char aOrientation3[128] = "";
 						char aOrientation4[128] = "";
-						sscanf(str_trim_words(pLine, pWord), "%d %127s %127s %127s %127s", &ID, aOrientation1, aOrientation2, aOrientation3, aOrientation4);
+						sscanf(str_trim_words(pLine, pWord), "%d %127s %127s %127s %127s", &NewIndexInfo.m_Id, aOrientation1, aOrientation2, aOrientation3, aOrientation4);
 
-						CIndexInfo NewIndexInfo;
-						NewIndexInfo.m_ID = ID;
 						NewIndexInfo.m_Flag = 0;
 						NewIndexInfo.m_TestFlag = false;
 
@@ -203,17 +182,8 @@ void CAutoMapper::Load(const char *pTileName)
 						}
 						else if(str_length(aOrientation1) > 0)
 						{
-							NewIndexInfo.m_TestFlag = true;
-							if(!str_comp(aOrientation1, "XFLIP"))
-								NewIndexInfo.m_Flag = TILEFLAG_VFLIP;
-							else if(!str_comp(aOrientation1, "YFLIP"))
-								NewIndexInfo.m_Flag = TILEFLAG_HFLIP;
-							else if(!str_comp(aOrientation1, "ROTATE"))
-								NewIndexInfo.m_Flag = TILEFLAG_ROTATE;
-							else if(!str_comp(aOrientation1, "NONE"))
-								NewIndexInfo.m_Flag = 0;
-							else
-								NewIndexInfo.m_TestFlag = false;
+							NewIndexInfo.m_Flag = CheckIndexFlag(NewIndexInfo.m_Flag, aOrientation1, true);
+							NewIndexInfo.m_TestFlag = !(NewIndexInfo.m_Flag == 0 && str_comp(aOrientation1, "NONE"));
 						}
 						else
 						{
@@ -229,12 +199,7 @@ void CAutoMapper::Load(const char *pTileName)
 						}
 						else if(str_length(aOrientation2) > 0 && NewIndexInfo.m_Flag != 0)
 						{
-							if(!str_comp(aOrientation2, "XFLIP"))
-								NewIndexInfo.m_Flag |= TILEFLAG_VFLIP;
-							else if(!str_comp(aOrientation2, "YFLIP"))
-								NewIndexInfo.m_Flag |= TILEFLAG_HFLIP;
-							else if(!str_comp(aOrientation2, "ROTATE"))
-								NewIndexInfo.m_Flag |= TILEFLAG_ROTATE;
+							NewIndexInfo.m_Flag = CheckIndexFlag(NewIndexInfo.m_Flag, aOrientation2, false);
 						}
 						else
 						{
@@ -250,12 +215,7 @@ void CAutoMapper::Load(const char *pTileName)
 						}
 						else if(str_length(aOrientation3) > 0 && NewIndexInfo.m_Flag != 0)
 						{
-							if(!str_comp(aOrientation3, "XFLIP"))
-								NewIndexInfo.m_Flag |= TILEFLAG_VFLIP;
-							else if(!str_comp(aOrientation3, "YFLIP"))
-								NewIndexInfo.m_Flag |= TILEFLAG_HFLIP;
-							else if(!str_comp(aOrientation3, "ROTATE"))
-								NewIndexInfo.m_Flag |= TILEFLAG_ROTATE;
+							NewIndexInfo.m_Flag = CheckIndexFlag(NewIndexInfo.m_Flag, aOrientation3, false);
 						}
 						else
 						{
@@ -291,10 +251,18 @@ void CAutoMapper::Load(const char *pTileName)
 					{
 						for(const auto &Index : vNewIndexList)
 						{
-							if(Value == CPosRule::INDEX && Index.m_ID == 0)
+							if(Index.m_Id == 0 && Value == CPosRule::INDEX)
+							{
+								// Skip full tiles if we have a rule "POS 0 0 INDEX 0"
+								// because that forces the tile to be empty
 								pCurrentIndex->m_SkipFull = true;
-							else
+							}
+							else if((Index.m_Id > 0 && Value == CPosRule::INDEX) || (Index.m_Id == 0 && Value == CPosRule::NOTINDEX))
+							{
+								// Skip empty tiles if we have a rule "POS 0 0 INDEX i" where i > 0
+								// or if we have a rule "POS 0 0 NOTINDEX 0"
 								pCurrentIndex->m_SkipEmpty = true;
+							}
 						}
 					}
 				}
@@ -313,6 +281,16 @@ void CAutoMapper::Load(const char *pTileName)
 					pCurrentIndex->m_RandomProbability = 1.0f / Value;
 				}
 			}
+			else if(str_startswith(pLine, "Modulo") && pCurrentIndex)
+			{
+				CModuloRule NewModuloRule;
+				sscanf(pLine, "Modulo %d %d %d %d", &NewModuloRule.m_ModX, &NewModuloRule.m_ModY, &NewModuloRule.m_OffsetX, &NewModuloRule.m_OffsetY);
+				if(NewModuloRule.m_ModX == 0)
+					NewModuloRule.m_ModX = 1;
+				if(NewModuloRule.m_ModY == 0)
+					NewModuloRule.m_ModY = 1;
+				pCurrentIndex->m_vModuloRules.push_back(NewModuloRule);
+			}
 			else if(str_startswith(pLine, "NoDefaultRule") && pCurrentIndex)
 			{
 				pCurrentIndex->m_DefaultRule = false;
@@ -332,14 +310,25 @@ void CAutoMapper::Load(const char *pTileName)
 			for(auto &IndexRule : Run.m_vIndexRules)
 			{
 				bool Found = false;
+
+				// Search for the exact rule "POS 0 0 INDEX 0" which corresponds to the default rule
 				for(const auto &Rule : IndexRule.m_vRules)
 				{
-					if(Rule.m_X == 0 && Rule.m_Y == 0)
+					if(Rule.m_X == 0 && Rule.m_Y == 0 && Rule.m_Value == CPosRule::INDEX)
 					{
-						Found = true;
+						for(const auto &Index : Rule.m_vIndexList)
+						{
+							if(Index.m_Id == 0)
+								Found = true;
+						}
 						break;
 					}
+
+					if(Found)
+						break;
 				}
+
+				// If the default rule was not found, and we require it, then add it
 				if(!Found && IndexRule.m_DefaultRule)
 				{
 					std::vector<CIndexInfo> vNewIndexList;
@@ -351,6 +340,7 @@ void CAutoMapper::Load(const char *pTileName)
 					IndexRule.m_SkipEmpty = true;
 					IndexRule.m_SkipFull = false;
 				}
+
 				if(IndexRule.m_SkipEmpty && IndexRule.m_SkipFull)
 				{
 					IndexRule.m_SkipEmpty = false;
@@ -360,14 +350,33 @@ void CAutoMapper::Load(const char *pTileName)
 		}
 	}
 
-	io_close(RulesFile);
-
+	char aBuf[IO_MAX_PATH_LENGTH + 16];
 	str_format(aBuf, sizeof(aBuf), "loaded %s", aPath);
-	m_pEditor->Console()->Print(IConsole::OUTPUT_LEVEL_DEBUG, "editor", aBuf);
+	Console()->Print(IConsole::OUTPUT_LEVEL_DEBUG, "editor/automap", aBuf);
 
 	m_FileLoaded = true;
 }
 
+void CAutoMapper::Unload()
+{
+	m_FileLoaded = false;
+	m_vConfigs.clear();
+}
+
+int CAutoMapper::CheckIndexFlag(int Flag, const char *pFlag, bool CheckNone) const
+{
+	if(!str_comp(pFlag, "XFLIP"))
+		Flag |= TILEFLAG_XFLIP;
+	else if(!str_comp(pFlag, "YFLIP"))
+		Flag |= TILEFLAG_YFLIP;
+	else if(!str_comp(pFlag, "ROTATE"))
+		Flag |= TILEFLAG_ROTATE;
+	else if(!str_comp(pFlag, "NONE") && CheckNone)
+		Flag = 0;
+
+	return Flag;
+}
+
 const char *CAutoMapper::GetConfigName(int Index)
 {
 	if(Index < 0 || Index >= (int)m_vConfigs.size())
@@ -376,9 +385,9 @@ const char *CAutoMapper::GetConfigName(int Index)
 	return m_vConfigs[Index].m_aName;
 }
 
-void CAutoMapper::ProceedLocalized(CLayerTiles *pLayer, int ConfigID, int Seed, int X, int Y, int Width, int Height)
+void CAutoMapper::ProceedLocalized(CLayerTiles *pLayer, CLayerTiles *pGameLayer, int ReferenceId, int ConfigId, int Seed, int X, int Y, int Width, int Height)
 {
-	if(!m_FileLoaded || pLayer->m_Readonly || ConfigID < 0 || ConfigID >= (int)m_vConfigs.size())
+	if(!m_FileLoaded || pLayer->m_Readonly || ConfigId < 0 || ConfigId >= (int)m_vConfigs.size())
 		return;
 
 	if(Width < 0)
@@ -387,7 +396,7 @@ void CAutoMapper::ProceedLocalized(CLayerTiles *pLayer, int ConfigID, int Seed,
 	if(Height < 0)
 		Height = pLayer->m_Height;
 
-	CConfiguration *pConf = &m_vConfigs[ConfigID];
+	CConfiguration *pConf = &m_vConfigs[ConfigId];
 
 	int CommitFromX = clamp(X + pConf->m_StartX, 0, pLayer->m_Width);
 	int CommitFromY = clamp(Y + pConf->m_StartY, 0, pLayer->m_Height);
@@ -399,86 +408,131 @@ void CAutoMapper::ProceedLocalized(CLayerTiles *pLayer, int ConfigID, int Seed,
 	int UpdateToX = clamp(X + Width + 3 * pConf->m_EndX, 0, pLayer->m_Width);
 	int UpdateToY = clamp(Y + Height + 3 * pConf->m_EndY, 0, pLayer->m_Height);
 
-	CLayerTiles *pUpdateLayer = new CLayerTiles(UpdateToX - UpdateFromX, UpdateToY - UpdateFromY);
+	CLayerTiles *pUpdateLayer = new CLayerTiles(Editor(), UpdateToX - UpdateFromX, UpdateToY - UpdateFromY);
+	CLayerTiles *pUpdateGame = new CLayerTiles(Editor(), UpdateToX - UpdateFromX, UpdateToY - UpdateFromY);
 
 	for(int y = UpdateFromY; y < UpdateToY; y++)
 	{
 		for(int x = UpdateFromX; x < UpdateToX; x++)
 		{
-			CTile *pIn = &pLayer->m_pTiles[y * pLayer->m_Width + x];
-			CTile *pOut = &pUpdateLayer->m_pTiles[(y - UpdateFromY) * pUpdateLayer->m_Width + x - UpdateFromX];
-			pOut->m_Index = pIn->m_Index;
-			pOut->m_Flags = pIn->m_Flags;
+			const CTile *pInLayer = &pLayer->m_pTiles[y * pLayer->m_Width + x];
+			CTile *pOutLayer = &pUpdateLayer->m_pTiles[(y - UpdateFromY) * pUpdateLayer->m_Width + x - UpdateFromX];
+			pOutLayer->m_Index = pInLayer->m_Index;
+			pOutLayer->m_Flags = pInLayer->m_Flags;
+
+			const CTile *pInGame = &pGameLayer->m_pTiles[y * pGameLayer->m_Width + x];
+			CTile *pOutGame = &pUpdateGame->m_pTiles[(y - UpdateFromY) * pUpdateGame->m_Width + x - UpdateFromX];
+			pOutGame->m_Index = pInGame->m_Index;
+			pOutGame->m_Flags = pInGame->m_Flags;
 		}
 	}
 
-	Proceed(pUpdateLayer, ConfigID, Seed, UpdateFromX, UpdateFromY);
+	Proceed(pUpdateLayer, pUpdateGame, ReferenceId, ConfigId, Seed, UpdateFromX, UpdateFromY);
 
 	for(int y = CommitFromY; y < CommitToY; y++)
 	{
 		for(int x = CommitFromX; x < CommitToX; x++)
 		{
-			CTile *pIn = &pUpdateLayer->m_pTiles[(y - UpdateFromY) * pUpdateLayer->m_Width + x - UpdateFromX];
-			CTile *pOut = &pLayer->m_pTiles[y * pLayer->m_Width + x];
-			pOut->m_Index = pIn->m_Index;
-			pOut->m_Flags = pIn->m_Flags;
+			const CTile *pInLayer = &pUpdateLayer->m_pTiles[(y - UpdateFromY) * pUpdateLayer->m_Width + x - UpdateFromX];
+			CTile *pOutLayer = &pLayer->m_pTiles[y * pLayer->m_Width + x];
+			CTile PreviousLayer = *pOutLayer;
+			pOutLayer->m_Index = pInLayer->m_Index;
+			pOutLayer->m_Flags = pInLayer->m_Flags;
+			pLayer->RecordStateChange(x, y, PreviousLayer, *pOutLayer);
+
+			const CTile *pInGame = &pUpdateGame->m_pTiles[(y - UpdateFromY) * pUpdateGame->m_Width + x - UpdateFromX];
+			CTile *pOutGame = &pGameLayer->m_pTiles[y * pGameLayer->m_Width + x];
+			CTile PreviousGame = *pOutGame;
+			pOutGame->m_Index = pInGame->m_Index;
+			pOutGame->m_Flags = pInGame->m_Flags;
+			pGameLayer->RecordStateChange(x, y, PreviousGame, *pOutGame);
 		}
 	}
 
 	delete pUpdateLayer;
+	delete pUpdateGame;
 }
 
-void CAutoMapper::Proceed(CLayerTiles *pLayer, int ConfigID, int Seed, int SeedOffsetX, int SeedOffsetY)
+void CAutoMapper::Proceed(CLayerTiles *pLayer, CLayerTiles *pGameLayer, int ReferenceId, int ConfigId, int Seed, int SeedOffsetX, int SeedOffsetY)
 {
-	if(!m_FileLoaded || pLayer->m_Readonly || ConfigID < 0 || ConfigID >= (int)m_vConfigs.size())
+	if(!m_FileLoaded || pLayer->m_Readonly || ConfigId < 0 || ConfigId >= (int)m_vConfigs.size())
 		return;
 
 	if(Seed == 0)
 		Seed = rand();
 
-	CConfiguration *pConf = &m_vConfigs[ConfigID];
+	CConfiguration *pConf = &m_vConfigs[ConfigId];
+	pLayer->ClearHistory();
+
+	const int LayerWidth = pLayer->m_Width;
+	const int LayerHeight = pLayer->m_Height;
+
+	static const int s_aTileIndex[] = {TILE_SOLID, TILE_DEATH, TILE_NOHOOK, TILE_FREEZE, TILE_UNFREEZE, TILE_DFREEZE, TILE_DUNFREEZE, TILE_LFREEZE, TILE_LUNFREEZE};
+
+	static_assert(std::size(g_apAutoMapReferenceNames) == std::size(s_aTileIndex) + 1, "g_apAutoMapReferenceNames and s_aTileIndex must include the same items");
 
 	// for every run: copy tiles, automap, overwrite tiles
 	for(size_t h = 0; h < pConf->m_vRuns.size(); ++h)
 	{
 		CRun *pRun = &pConf->m_vRuns[h];
+		bool IsFilterable = h == 0 && ReferenceId >= 0;
 
 		// don't make copy if it's requested
 		CLayerTiles *pReadLayer;
+		CLayerTiles *pBuffer = IsFilterable ? pGameLayer : pLayer;
 		if(pRun->m_AutomapCopy)
 		{
-			pReadLayer = new CLayerTiles(pLayer->m_Width, pLayer->m_Height);
+			pReadLayer = new CLayerTiles(Editor(), LayerWidth, LayerHeight);
 
-			for(int y = 0; y < pLayer->m_Height; y++)
+			int LoopWidth = IsFilterable ? std::min(pGameLayer->m_Width, LayerWidth) : LayerWidth;
+			int LoopHeight = IsFilterable ? std::min(pGameLayer->m_Height, LayerHeight) : LayerHeight;
+
+			for(int y = 0; y < LoopHeight; y++)
 			{
-				for(int x = 0; x < pLayer->m_Width; x++)
+				for(int x = 0; x < LoopWidth; x++)
 				{
-					CTile *pIn = &pLayer->m_pTiles[y * pLayer->m_Width + x];
-					CTile *pOut = &pReadLayer->m_pTiles[y * pLayer->m_Width + x];
-					pOut->m_Index = pIn->m_Index;
+					const CTile *pIn = &pBuffer->m_pTiles[y * pBuffer->m_Width + x];
+					CTile *pOut = &pReadLayer->m_pTiles[y * LayerWidth + x];
+					if(h == 0 && ReferenceId >= 1 && pIn->m_Index != s_aTileIndex[ReferenceId - 1])
+						pOut->m_Index = 0;
+					else
+						pOut->m_Index = pIn->m_Index;
 					pOut->m_Flags = pIn->m_Flags;
 				}
 			}
 		}
 		else
 		{
-			pReadLayer = pLayer;
+			pReadLayer = pBuffer;
 		}
 
 		// auto map
-		for(int y = 0; y < pLayer->m_Height; y++)
+		for(int y = 0; y < LayerHeight; y++)
 		{
-			for(int x = 0; x < pLayer->m_Width; x++)
+			for(int x = 0; x < LayerWidth; x++)
 			{
-				CTile *pTile = &(pLayer->m_pTiles[y * pLayer->m_Width + x]);
-				m_pEditor->m_Map.m_Modified = true;
+				CTile *pTile = &(pLayer->m_pTiles[y * LayerWidth + x]);
+				const CTile *pReadTile = &(pReadLayer->m_pTiles[y * LayerWidth + x]);
+				Editor()->m_Map.OnModify();
 
 				for(size_t i = 0; i < pRun->m_vIndexRules.size(); ++i)
 				{
 					CIndexRule *pIndexRule = &pRun->m_vIndexRules[i];
-					if(pIndexRule->m_SkipEmpty && pTile->m_Index == 0) // skip empty tiles
-						continue;
-					if(pIndexRule->m_SkipFull && pTile->m_Index != 0) // skip full tiles
+					if(pReadTile->m_Index == 0)
+					{
+						if(pTile->m_Index != 0 && IsFilterable) // TODO: This is a lazy workaround
+						{
+							CTile Previous = *pTile;
+							pTile->m_Index = 0;
+							pTile->m_Flags = pIndexRule->m_Flag;
+							pLayer->RecordStateChange(x, y, Previous, *pTile);
+							continue;
+						}
+
+						if(pIndexRule->m_SkipEmpty) // skip empty tiles
+							continue;
+					}
+					if(pIndexRule->m_SkipFull && pReadTile->m_Index != 0) // skip full tiles
 						continue;
 
 					bool RespectRules = true;
@@ -489,11 +543,11 @@ void CAutoMapper::Proceed(CLayerTiles *pLayer, int ConfigID, int Seed, int SeedO
 						int CheckIndex, CheckFlags;
 						int CheckX = x + pRule->m_X;
 						int CheckY = y + pRule->m_Y;
-						if(CheckX >= 0 && CheckX < pLayer->m_Width && CheckY >= 0 && CheckY < pLayer->m_Height)
+						if(CheckX >= 0 && CheckX < LayerWidth && CheckY >= 0 && CheckY < LayerHeight)
 						{
-							int CheckTile = CheckY * pLayer->m_Width + CheckX;
+							int CheckTile = CheckY * LayerWidth + CheckX;
 							CheckIndex = pReadLayer->m_pTiles[CheckTile].m_Index;
-							CheckFlags = pReadLayer->m_pTiles[CheckTile].m_Flags & (TILEFLAG_ROTATE | TILEFLAG_VFLIP | TILEFLAG_HFLIP);
+							CheckFlags = pReadLayer->m_pTiles[CheckTile].m_Flags & (TILEFLAG_ROTATE | TILEFLAG_XFLIP | TILEFLAG_YFLIP);
 						}
 						else
 						{
@@ -504,9 +558,9 @@ void CAutoMapper::Proceed(CLayerTiles *pLayer, int ConfigID, int Seed, int SeedO
 						if(pRule->m_Value == CPosRule::INDEX)
 						{
 							RespectRules = false;
-							for(auto &Index : pRule->m_vIndexList)
+							for(const auto &Index : pRule->m_vIndexList)
 							{
-								if(CheckIndex == Index.m_ID && (!Index.m_TestFlag || CheckFlags == Index.m_Flag))
+								if(CheckIndex == Index.m_Id && (!Index.m_TestFlag || CheckFlags == Index.m_Flag))
 								{
 									RespectRules = true;
 									break;
@@ -515,9 +569,9 @@ void CAutoMapper::Proceed(CLayerTiles *pLayer, int ConfigID, int Seed, int SeedO
 						}
 						else if(pRule->m_Value == CPosRule::NOTINDEX)
 						{
-							for(auto &Index : pRule->m_vIndexList)
+							for(const auto &Index : pRule->m_vIndexList)
 							{
-								if(CheckIndex == Index.m_ID && (!Index.m_TestFlag || CheckFlags == Index.m_Flag))
+								if(CheckIndex == Index.m_Id && (!Index.m_TestFlag || CheckFlags == Index.m_Flag))
 								{
 									RespectRules = false;
 									break;
@@ -526,11 +580,21 @@ void CAutoMapper::Proceed(CLayerTiles *pLayer, int ConfigID, int Seed, int SeedO
 						}
 					}
 
-					if(RespectRules &&
+					bool PassesModuloCheck;
+					if(pIndexRule->m_vModuloRules.empty())
+						PassesModuloCheck = true;
+					else
+						PassesModuloCheck = std::any_of(pIndexRule->m_vModuloRules.cbegin(), pIndexRule->m_vModuloRules.cend(), [&](const CModuloRule &ModuloRule) {
+							return (x + SeedOffsetX + ModuloRule.m_OffsetX) % ModuloRule.m_ModX == 0 && (y + SeedOffsetY + ModuloRule.m_OffsetY) % ModuloRule.m_ModY == 0;
+						});
+
+					if(RespectRules && PassesModuloCheck &&
 						(pIndexRule->m_RandomProbability >= 1.0f || HashLocation(Seed, h, i, x + SeedOffsetX, y + SeedOffsetY) < HASH_MAX * pIndexRule->m_RandomProbability))
 					{
-						pTile->m_Index = pIndexRule->m_ID;
+						CTile Previous = *pTile;
+						pTile->m_Index = pIndexRule->m_Id;
 						pTile->m_Flags = pIndexRule->m_Flag;
+						pLayer->RecordStateChange(x, y, Previous, *pTile);
 					}
 				}
 			}
