@@ -4,9 +4,27 @@
 #define GAME_LOCALIZATION_H
 
 #include <base/system.h> // GNUC_ATTRIBUTE
+
 #include <engine/shared/memheap.h>
+
+#include <string>
 #include <vector>
 
+class CLanguage
+{
+public:
+	CLanguage() = default;
+	CLanguage(const char *pName, const char *pFileName, int Code, const std::vector<std::string> &vLanguageCodes) :
+		m_Name(pName), m_FileName(pFileName), m_CountryCode(Code), m_vLanguageCodes(vLanguageCodes) {}
+
+	std::string m_Name;
+	std::string m_FileName;
+	int m_CountryCode;
+	std::vector<std::string> m_vLanguageCodes;
+
+	bool operator<(const CLanguage &Other) const { return m_Name < Other.m_Name; }
+};
+
 class CLocalizationDatabase
 {
 	class CString
@@ -27,44 +45,23 @@ class CLocalizationDatabase
 		bool operator==(const CString &Other) const { return m_Hash == Other.m_Hash && m_ContextHash == Other.m_ContextHash; }
 	};
 
+	std::vector<CLanguage> m_vLanguages;
 	std::vector<CString> m_vStrings;
 	CHeap m_StringsHeap;
-	int m_VersionCounter;
-	int m_CurrentVersion;
 
 public:
-	CLocalizationDatabase();
+	void LoadIndexfile(class IStorage *pStorage, class IConsole *pConsole);
+	const std::vector<CLanguage> &Languages() const { return m_vLanguages; }
+	void SelectDefaultLanguage(class IConsole *pConsole, char *pFilename, size_t Length) const;
 
 	bool Load(const char *pFilename, class IStorage *pStorage, class IConsole *pConsole);
 
-	int Version() const { return m_CurrentVersion; }
-
 	void AddString(const char *pOrgStr, const char *pNewStr, const char *pContext);
 	const char *FindString(unsigned Hash, unsigned ContextHash) const;
 };
 
 extern CLocalizationDatabase g_Localization;
 
-class CLocConstString
-{
-	const char *m_pDefaultStr;
-	const char *m_pCurrentStr;
-	unsigned m_Hash;
-	unsigned m_ContextHash;
-	int m_Version;
-
-public:
-	CLocConstString(const char *pStr, const char *pContext = "");
-	void Reload();
-
-	inline operator const char *()
-	{
-		if(m_Version != g_Localization.Version())
-			Reload();
-		return m_pCurrentStr;
-	}
-};
-
 extern const char *Localize(const char *pStr, const char *pContext = "")
 	GNUC_ATTRIBUTE((format_arg(1)));
 #endif
