@@ -11,24 +11,23 @@ enum
 	NUM_BUGS,
 };
 
-enum
+enum class EMapBugUpdate
 {
-	MAPBUGUPDATE_OK,
-	MAPBUGUPDATE_NOTFOUND,
-	MAPBUGUPDATE_OVERRIDDEN,
+	OK,
+	NOTFOUND,
+	OVERRIDDEN,
 };
 
 class CMapBugs
 {
-	friend CMapBugs GetMapBugs(const char *pName, int Size, SHA256_DIGEST Sha256);
-	void *m_pData;
-	unsigned int m_Extra;
+	void *m_pData = nullptr;
+	unsigned int m_Extra = 0;
 
 public:
+	static CMapBugs Create(const char *pName, int Size, SHA256_DIGEST Sha256);
 	bool Contains(int Bug) const;
-	int Update(const char *pBug);
+	EMapBugUpdate Update(const char *pBug);
 	void Dump() const;
 };
 
-CMapBugs GetMapBugs(const char *pName, int Size, SHA256_DIGEST Sha256);
 #endif // GAME_MAPBUGS_H
