@@ -57,9 +57,28 @@ bool IsValidTeleTile(int Index)
 		Index == TILE_TELECHECKINEVIL);
 }
 
+bool IsTeleTileCheckpoint(int Index)
+{
+	return Index == TILE_TELECHECK || Index == TILE_TELECHECKOUT;
+}
+
+bool IsTeleTileNumberUsed(int Index, bool Checkpoint)
+{
+	if(Checkpoint)
+		return IsTeleTileCheckpoint(Index);
+	return !IsTeleTileCheckpoint(Index) && Index != TILE_TELECHECKIN &&
+	       Index != TILE_TELECHECKINEVIL;
+}
+
+bool IsTeleTileNumberUsedAny(int Index)
+{
+	return Index != TILE_TELECHECKIN &&
+	       Index != TILE_TELECHECKINEVIL;
+}
+
 bool IsValidSpeedupTile(int Index)
 {
-	return Index == TILE_BOOST;
+	return Index == TILE_SPEED_BOOST_OLD || Index == TILE_SPEED_BOOST;
 }
 
 bool IsValidSwitchTile(int Index)
@@ -81,6 +100,28 @@ bool IsValidSwitchTile(int Index)
 		(IsValidEntity(Index) && Index >= ENTITY_OFFSET + ENTITY_ARMOR_1));
 }
 
+bool IsSwitchTileFlagsUsed(int Index)
+{
+	return Index != TILE_FREEZE &&
+	       Index != TILE_DFREEZE &&
+	       Index != TILE_DUNFREEZE;
+}
+
+bool IsSwitchTileNumberUsed(int Index)
+{
+	return Index != TILE_JUMP &&
+	       Index != TILE_HIT_ENABLE &&
+	       Index != TILE_HIT_DISABLE &&
+	       Index != TILE_ALLOW_TELE_GUN &&
+	       Index != TILE_ALLOW_BLUE_TELE_GUN;
+}
+
+bool IsSwitchTileDelayUsed(int Index)
+{
+	return Index != TILE_DFREEZE &&
+	       Index != TILE_DUNFREEZE;
+}
+
 bool IsValidTuneTile(int Index)
 {
 	return Index == TILE_TUNE;
@@ -122,3 +163,13 @@ bool IsCreditsTile(int TileIndex)
 		(TILE_CREDITS_7 == TileIndex) ||
 		(TILE_CREDITS_8 == TileIndex));
 }
+
+int PackColor(CColor Color)
+{
+	int Res = 0;
+	Res |= Color.r << 24;
+	Res |= Color.g << 16;
+	Res |= Color.b << 8;
+	Res |= Color.a;
+	return Res;
+}
