@@ -16,6 +16,7 @@
 CDragger::CDragger(CGameWorld *pGameWorld, vec2 Pos, float Strength, bool IgnoreWalls, int Layer, int Number) :
 	CEntity(pGameWorld, CGameWorld::ENTTYPE_LASER)
 {
+	m_Core = vec2(0.0f, 0.0f);
 	m_Pos = Pos;
 	m_Strength = Strength;
 	m_IgnoreWalls = IgnoreWalls;
@@ -33,15 +34,10 @@ CDragger::CDragger(CGameWorld *pGameWorld, vec2 Pos, float Strength, bool Ignore
 
 void CDragger::Tick()
 {
-	if(Server()->Tick() % int(Server()->TickSpeed() * 0.15f) == 0)
+	if(Server()->Tick() % (int)(Server()->TickSpeed() * 0.15f) == 0)
 	{
-		int Flags;
 		m_EvalTick = Server()->Tick();
-		int index = GameServer()->Collision()->IsMover(m_Pos.x, m_Pos.y, &Flags);
-		if(index)
-		{
-			m_Core = GameServer()->Collision()->CpSpeed(index, Flags);
-		}
+		GameServer()->Collision()->MoverSpeed(m_Pos.x, m_Pos.y, &m_Core);
 		m_Pos += m_Core;
 
 		// Adopt the new position for all outgoing laser beams
@@ -60,12 +56,12 @@ void CDragger::Tick()
 void CDragger::LookForPlayersToDrag()
 {
 	// Create a list of players who are in the range of the dragger
-	CCharacter *apPlayersInRange[MAX_CLIENTS];
+	CEntity *apPlayersInRange[MAX_CLIENTS];
 	mem_zero(apPlayersInRange, sizeof(apPlayersInRange));
 
 	int NumPlayersInRange = GameServer()->m_World.FindEntities(m_Pos,
 		g_Config.m_SvDraggerRange - CCharacterCore::PhysicalSize(),
-		(CEntity **)apPlayersInRange, MAX_CLIENTS, CGameWorld::ENTTYPE_CHARACTER);
+		apPlayersInRange, MAX_CLIENTS, CGameWorld::ENTTYPE_CHARACTER);
 
 	// The closest player (within range) in a team is selected as the target
 	int aClosestTargetIdInTeam[MAX_CLIENTS];
@@ -82,7 +78,7 @@ void CDragger::LookForPlayersToDrag()
 
 	for(int i = 0; i < NumPlayersInRange; i++)
 	{
-		CCharacter *pTarget = apPlayersInRange[i];
+		CCharacter *pTarget = static_cast<CCharacter *>(apPlayersInRange[i]);
 		const int &TargetTeam = pTarget->Team();
 
 		// Do not create a dragger beam for super player
@@ -100,11 +96,11 @@ void CDragger::LookForPlayersToDrag()
 		// Dragger beams can be created only for reachable, alive players
 		int IsReachable =
 			m_IgnoreWalls ?
-				!GameServer()->Collision()->IntersectNoLaserNW(m_Pos, pTarget->m_Pos, 0, 0) :
-				!GameServer()->Collision()->IntersectNoLaser(m_Pos, pTarget->m_Pos, 0, 0);
+				!GameServer()->Collision()->IntersectNoLaserNoWalls(m_Pos, pTarget->m_Pos, nullptr, nullptr) :
+				!GameServer()->Collision()->IntersectNoLaser(m_Pos, pTarget->m_Pos, nullptr, nullptr);
 		if(IsReachable && pTarget->IsAlive())
 		{
-			const int &TargetClientId = pTarget->GetPlayer()->GetCID();
+			const int &TargetClientId = pTarget->GetPlayer()->GetCid();
 			// Solo players are dragged independently from the rest of the team
 			if(pTarget->Teams()->m_Core.GetSolo(TargetClientId))
 			{
@@ -155,9 +151,34 @@ void CDragger::LookForPlayersToDrag()
 	}
 }
 
-void CDragger::RemoveDraggerBeam(int ClientID)
+void CDragger::RemoveDraggerBeam(int ClientId)
 {
-	m_apDraggerBeam[ClientID] = nullptr;
+	m_apDraggerBeam[ClientId] = nullptr;
+}
+
+bool CDragger::WillDraggerBeamUseDraggerId(int TargetClientId, int SnappingClientId)
+{
+	// For each snapping client, this must return true for at most one target (i.e. only one of the dragger beams),
+	// in which case the dragger itself must not be snapped
+	CCharacter *pTargetChar = GameServer()->GetPlayerChar(TargetClientId);
+	CCharacter *pSnapChar = GameServer()->GetPlayerChar(SnappingClientId);
+	if(pTargetChar && pSnapChar && m_apDraggerBeam[TargetClientId] != nullptr)
+	{
+		const int SnapTeam = pSnapChar->Team();
+		const int TargetTeam = pTargetChar->Team();
+		if(SnapTeam == TargetTeam && SnapTeam < MAX_CLIENTS)
+		{
+			if(pSnapChar->Teams()->m_Core.GetSolo(SnappingClientId) || m_aTargetIdInTeam[SnapTeam] < 0)
+			{
+				return SnappingClientId == TargetClientId;
+			}
+			else
+			{
+				return m_aTargetIdInTeam[SnapTeam] == TargetClientId;
+			}
+		}
+	}
+	return false;
 }
 
 void CDragger::Reset()
@@ -171,33 +192,23 @@ void CDragger::Snap(int SnappingClient)
 	if(NetworkClipped(SnappingClient))
 		return;
 
-	// Send the dragger in its resting position if the player would not otherwise see a dragger beam
+	// Send the dragger in its resting position if the player would not otherwise see a dragger beam within its own team
 	for(int i = 0; i < MAX_CLIENTS; i++)
 	{
-		if(m_apDraggerBeam[i] != nullptr)
+		if(WillDraggerBeamUseDraggerId(i, SnappingClient))
 		{
-			CCharacter *pChar = GameServer()->GetPlayerChar(i);
-			if(pChar && pChar->CanSnapCharacter(SnappingClient))
-			{
-				return;
-			}
+			return;
 		}
 	}
 
 	int SnappingClientVersion = GameServer()->GetClientVersion(SnappingClient);
 
-	CNetObj_EntityEx *pEntData = 0;
-	if(SnappingClientVersion >= VERSION_DDNET_SWITCH)
+	int Subtype = (m_IgnoreWalls ? 1 : 0) | (clamp(round_to_int(m_Strength - 1.f), 0, 2) << 1);
+
+	int StartTick;
+	if(SnappingClientVersion >= VERSION_DDNET_ENTITY_NETOBJS)
 	{
-		pEntData = static_cast<CNetObj_EntityEx *>(Server()->SnapNewItem(NETOBJTYPE_ENTITYEX, GetID(),
-			sizeof(CNetObj_EntityEx)));
-		if(pEntData)
-		{
-			pEntData->m_SwitchNumber = m_Number;
-			pEntData->m_Layer = m_Layer;
-			pEntData->m_EntityClass = clamp(ENTITYCLASS_DRAGGER_WEAK + round_to_int(m_Strength) - 1,
-				(int)ENTITYCLASS_DRAGGER_WEAK, (int)ENTITYCLASS_DRAGGER_STRONG);
-		}
+		StartTick = -1;
 	}
 	else
 	{
@@ -206,37 +217,30 @@ void CDragger::Snap(int SnappingClient)
 		if(SnappingClient != SERVER_DEMO_CLIENT &&
 			(GameServer()->m_apPlayers[SnappingClient]->GetTeam() == TEAM_SPECTATORS ||
 				GameServer()->m_apPlayers[SnappingClient]->IsPaused()) &&
-			GameServer()->m_apPlayers[SnappingClient]->m_SpectatorID != SPEC_FREEVIEW)
-			pChar = GameServer()->GetPlayerChar(GameServer()->m_apPlayers[SnappingClient]->m_SpectatorID);
+			GameServer()->m_apPlayers[SnappingClient]->m_SpectatorId != SPEC_FREEVIEW)
+			pChar = GameServer()->GetPlayerChar(GameServer()->m_apPlayers[SnappingClient]->m_SpectatorId);
 
 		int Tick = (Server()->Tick() % Server()->TickSpeed()) % 11;
 		if(pChar && m_Layer == LAYER_SWITCH && m_Number > 0 &&
 			!Switchers()[m_Number].m_aStatus[pChar->Team()] && !Tick)
 			return;
-	}
 
-	CNetObj_Laser *pObjLaser = static_cast<CNetObj_Laser *>(Server()->SnapNewItem(
-		NETOBJTYPE_LASER, GetID(), sizeof(CNetObj_Laser)));
-
-	if(!pObjLaser)
-		return;
-
-	pObjLaser->m_X = (int)m_Pos.x;
-	pObjLaser->m_Y = (int)m_Pos.y;
-	pObjLaser->m_FromX = (int)m_Pos.x;
-	pObjLaser->m_FromY = (int)m_Pos.y;
-
-	if(pEntData)
-	{
-		pObjLaser->m_StartTick = 0;
-	}
-	else
-	{
-		int StartTick = m_EvalTick;
+		StartTick = m_EvalTick;
 		if(StartTick < Server()->Tick() - 4)
 			StartTick = Server()->Tick() - 4;
 		else if(StartTick > Server()->Tick())
 			StartTick = Server()->Tick();
-		pObjLaser->m_StartTick = StartTick;
+	}
+
+	GameServer()->SnapLaserObject(CSnapContext(SnappingClientVersion), GetId(),
+		m_Pos, m_Pos, StartTick, -1, LASERTYPE_DRAGGER, Subtype, m_Number);
+}
+
+void CDragger::SwapClients(int Client1, int Client2)
+{
+	std::swap(m_apDraggerBeam[Client1], m_apDraggerBeam[Client2]);
+	for(int &TargetId : m_aTargetIdInTeam)
+	{
+		TargetId = TargetId == Client1 ? Client2 : TargetId == Client2 ? Client1 : TargetId;
 	}
 }
