@@ -8,6 +8,8 @@
 class CPickup : public CEntity
 {
 public:
+	static const int ms_CollisionExtraSize = 6;
+
 	CPickup(CGameWorld *pGameWorld, int Type, int SubType = 0, int Layer = 0, int Number = 0);
 
 	void Reset() override;
@@ -15,10 +17,12 @@ public:
 	void TickPaused() override;
 	void Snap(int SnappingClient) override;
 
+	int Type() const { return m_Type; }
+	int Subtype() const { return m_Subtype; }
+
 private:
 	int m_Type;
 	int m_Subtype;
-	//int m_SpawnTick;
 
 	// DDRace
 
