@@ -12,14 +12,14 @@
 const float PLASMA_ACCEL = 1.1f;
 
 CPlasma::CPlasma(CGameWorld *pGameWorld, vec2 Pos, vec2 Dir, bool Freeze,
-	bool Explosive, int ForClientID) :
+	bool Explosive, int ForClientId) :
 	CEntity(pGameWorld, CGameWorld::ENTTYPE_LASER)
 {
 	m_Pos = Pos;
 	m_Core = Dir;
 	m_Freeze = Freeze;
 	m_Explosive = Explosive;
-	m_ForClientID = ForClientID;
+	m_ForClientId = ForClientId;
 	m_EvalTick = Server()->Tick();
 	m_LifeTime = Server()->TickSpeed() * 1.5f;
 
@@ -34,7 +34,7 @@ void CPlasma::Tick()
 		Reset();
 		return;
 	}
-	CCharacter *pTarget = GameServer()->GetPlayerChar(m_ForClientID);
+	CCharacter *pTarget = GameServer()->GetPlayerChar(m_ForClientId);
 	// Without a target, a plasma bullet has no reason to live
 	if(!pTarget)
 	{
@@ -58,7 +58,7 @@ bool CPlasma::HitCharacter(CCharacter *pTarget)
 {
 	vec2 IntersectPos;
 	CCharacter *pHitPlayer = GameServer()->m_World.IntersectCharacter(
-		m_Pos, m_Pos + m_Core, 0.0f, IntersectPos, 0, m_ForClientID);
+		m_Pos, m_Pos + m_Core, 0.0f, IntersectPos, nullptr, m_ForClientId);
 	if(!pHitPlayer)
 	{
 		return false;
@@ -76,7 +76,7 @@ bool CPlasma::HitCharacter(CCharacter *pTarget)
 		// Plasma Turrets are very precise weapons only one tee gets speed from it,
 		// other tees near the explosion remain unaffected
 		GameServer()->CreateExplosion(
-			m_Pos, m_ForClientID, WEAPON_GRENADE, true, pTarget->Team(), pTarget->TeamMask());
+			m_Pos, m_ForClientId, WEAPON_GRENADE, true, pTarget->Team(), pTarget->TeamMask());
 	}
 	Reset();
 	return true;
@@ -85,14 +85,14 @@ bool CPlasma::HitCharacter(CCharacter *pTarget)
 bool CPlasma::HitObstacle(CCharacter *pTarget)
 {
 	// Check if the plasma bullet is stopped by a solid block or a laser stopper
-	int HasIntersection = GameServer()->Collision()->IntersectNoLaser(m_Pos, m_Pos + m_Core, 0, 0);
+	int HasIntersection = GameServer()->Collision()->IntersectNoLaser(m_Pos, m_Pos + m_Core, nullptr, nullptr);
 	if(HasIntersection)
 	{
 		if(m_Explosive)
 		{
 			// Even in the case of an explosion due to a collision with obstacles, only one player is affected
 			GameServer()->CreateExplosion(
-				m_Pos, m_ForClientID, WEAPON_GRENADE, true, pTarget->Team(), pTarget->TeamMask());
+				m_Pos, m_ForClientId, WEAPON_GRENADE, true, pTarget->Team(), pTarget->TeamMask());
 		}
 		Reset();
 		return true;
@@ -108,7 +108,7 @@ void CPlasma::Reset()
 void CPlasma::Snap(int SnappingClient)
 {
 	// Only players who can see the targeted player can see the plasma bullet
-	CCharacter *pTarget = GameServer()->GetPlayerChar(m_ForClientID);
+	CCharacter *pTarget = GameServer()->GetPlayerChar(m_ForClientId);
 	if(!pTarget || !pTarget->CanSnapCharacter(SnappingClient))
 	{
 		return;
@@ -118,14 +118,14 @@ void CPlasma::Snap(int SnappingClient)
 	if(NetworkClipped(SnappingClient))
 		return;
 
-	CNetObj_Laser *pObj = static_cast<CNetObj_Laser *>(Server()->SnapNewItem(
-		NETOBJTYPE_LASER, GetID(), sizeof(CNetObj_Laser)));
-	if(!pObj)
-		return;
+	int SnappingClientVersion = GameServer()->GetClientVersion(SnappingClient);
 
-	pObj->m_X = (int)m_Pos.x;
-	pObj->m_Y = (int)m_Pos.y;
-	pObj->m_FromX = (int)m_Pos.x;
-	pObj->m_FromY = (int)m_Pos.y;
-	pObj->m_StartTick = m_EvalTick;
+	int Subtype = (m_Explosive ? 1 : 0) | (m_Freeze ? 2 : 0);
+	GameServer()->SnapLaserObject(CSnapContext(SnappingClientVersion), GetId(),
+		m_Pos, m_Pos, m_EvalTick, -1, LASERTYPE_PLASMA, Subtype, m_Number);
+}
+
+void CPlasma::SwapClients(int Client1, int Client2)
+{
+	m_ForClientId = m_ForClientId == Client1 ? Client2 : m_ForClientId == Client2 ? Client1 : m_ForClientId;
 }
