@@ -17,8 +17,8 @@ public:
 		int Span,
 		bool Freeze,
 		bool Explosive,
-		float Force,
 		int SoundImpact,
+		vec2 InitDir,
 		int Layer = 0,
 		int Number = 0);
 
@@ -38,7 +38,6 @@ private:
 	int m_Type;
 	//int m_Damage;
 	int m_SoundImpact;
-	float m_Force;
 	int m_StartTick;
 	bool m_Explosive;
 
@@ -48,10 +47,17 @@ private:
 	bool m_Freeze;
 	int m_TuneZone;
 	bool m_BelongsToPracticeTeam;
+	int m_DDRaceTeam;
+	bool m_IsSolo;
+	vec2 m_InitDir;
 
 public:
 	void SetBouncing(int Value);
-	bool FillExtraInfo(CNetObj_DDNetProjectile *pProj);
+	bool FillExtraInfoLegacy(CNetObj_DDRaceProjectile *pProj);
+	void FillExtraInfo(CNetObj_DDNetProjectile *pProj);
+
+	bool CanCollide(int ClientId) override;
+	int GetOwnerId() const override { return m_Owner; }
 };
 
 #endif
