@@ -5,8 +5,10 @@
 
 #include <base/vmath.h>
 
-#include "alloc.h"
+#include <game/alloc.h>
+
 #include "gameworld.h"
+#include "save.h"
 
 class CCollision;
 class CGameContext;
@@ -28,7 +30,7 @@ private:
 	CGameWorld *m_pGameWorld;
 	CCollision *m_pCCollision;
 
-	int m_ID;
+	int m_Id;
 	int m_ObjType;
 
 	/*
@@ -49,7 +51,7 @@ public: // TODO: Maybe make protected
 	vec2 m_Pos;
 
 	/* Getters */
-	int GetID() const { return m_ID; }
+	int GetId() const { return m_Id; }
 
 	/* Constructor */
 	CEntity(CGameWorld *pGameWorld, int Objtype, vec2 Pos = vec2(0, 0), int ProximityRadius = 0);
@@ -60,6 +62,9 @@ public: // TODO: Maybe make protected
 	/* Objects */
 	std::vector<SSwitchers> &Switchers() { return m_pGameWorld->m_Core.m_vSwitchers; }
 	CGameWorld *GameWorld() { return m_pGameWorld; }
+	CTuningParams *Tuning() { return GameWorld()->Tuning(); }
+	CTuningParams *TuningList() { return GameWorld()->TuningList(); }
+	CTuningParams *GetTuning(int i) { return GameWorld()->GetTuning(i); }
 	class CConfig *Config() { return m_pGameWorld->Config(); }
 	class CGameContext *GameServer() { return m_pGameWorld->GameServer(); }
 	class IServer *Server() { return m_pGameWorld->Server(); }
@@ -86,13 +91,6 @@ public: // TODO: Maybe make protected
 	*/
 	virtual void Reset() {}
 
-	/*
-		Function: PreTick
-			Called to progress the entity before the next tick.
-			Can be used to prepare variables for all clients before the next tick is executed.
-	*/
-	virtual void PreTick() {}
-
 	/*
 		Function: Tick
 			Called to progress the entity to the next tick. Updates
@@ -125,6 +123,12 @@ public: // TODO: Maybe make protected
 	*/
 	virtual void Snap(int SnappingClient) {}
 
+	/*
+		Function: PostSnap
+			Called after all clients received their snapshot.
+	*/
+	virtual void PostSnap() {}
+
 	/*
 		Function: SwapClients
 			Called when two players have swapped their client ids.
@@ -135,6 +139,25 @@ public: // TODO: Maybe make protected
 	*/
 	virtual void SwapClients(int Client1, int Client2) {}
 
+	/*
+		Function: BlocksSave
+			Called to check if a team can be saved
+
+		Arguments:
+			ClientId - Client ID
+	*/
+	virtual ESaveResult BlocksSave(int ClientId) { return ESaveResult::SUCCESS; }
+
+	/*
+		Function GetOwnerId
+		Returns:
+			ClientId of the initiator from this entity. -1 created by map.
+			This is used by save/load to remove related entities to the tee.
+			CCharacter should not return the PlayerId, because they get
+			handled separately in save/load code.
+	*/
+	virtual int GetOwnerId() const { return -1; }
+
 	/*
 		Function: NetworkClipped
 			Performs a series of test to see if a client can see the
@@ -154,6 +177,7 @@ public: // TODO: Maybe make protected
 	bool NetworkClippedLine(int SnappingClient, vec2 StartPos, vec2 EndPos) const;
 
 	bool GameLayerClipped(vec2 CheckPos);
+	virtual bool CanCollide(int ClientId) { return true; };
 
 	// DDRace
 
