@@ -13,7 +13,7 @@
 //////////////////////////////////////////////////
 CEventHandler::CEventHandler()
 {
-	m_pGameServer = 0;
+	m_pGameServer = nullptr;
 	Clear();
 }
 
@@ -22,12 +22,12 @@ void CEventHandler::SetGameServer(CGameContext *pGameServer)
 	m_pGameServer = pGameServer;
 }
 
-void *CEventHandler::Create(int Type, int Size, int64_t Mask)
+void *CEventHandler::Create(int Type, int Size, CClientMask Mask)
 {
 	if(m_NumEvents == MAX_EVENTS)
-		return 0;
+		return nullptr;
 	if(m_CurrentOffset + Size >= MAX_DATASIZE)
-		return 0;
+		return nullptr;
 
 	void *p = &m_aData[m_CurrentOffset];
 	m_aOffsets[m_NumEvents] = m_CurrentOffset;
@@ -49,7 +49,7 @@ void CEventHandler::Snap(int SnappingClient)
 {
 	for(int i = 0; i < m_NumEvents; i++)
 	{
-		if(SnappingClient == SERVER_DEMO_CLIENT || CmaskIsSet(m_aClientMasks[i], SnappingClient))
+		if(SnappingClient == SERVER_DEMO_CLIENT || m_aClientMasks[i].test(SnappingClient))
 		{
 			CNetEvent_Common *pEvent = (CNetEvent_Common *)&m_aData[m_aOffsets[i]];
 			if(!NetworkClipped(GameServer(), SnappingClient, vec2(pEvent->m_X, pEvent->m_Y)))
@@ -81,11 +81,17 @@ void CEventHandler::EventToSixup(int *pType, int *pSize, const char **ppData)
 		pEvent7->m_X = pEvent->m_X;
 		pEvent7->m_Y = pEvent->m_Y;
 
+		pEvent7->m_ClientId = 0;
+		pEvent7->m_Angle = 0;
+
 		// This will need some work, perhaps an event wrapper for damageind,
 		// a scan of the event array to merge multiple damageinds
 		// or a separate array of "damage ind" events that's added in while snapping
 		pEvent7->m_HealthAmount = 1;
 
+		pEvent7->m_ArmorAmount = 0;
+		pEvent7->m_Self = 0;
+
 		*ppData = s_aEventStore;
 	}
 	else if(*pType == NETEVENTTYPE_SOUNDGLOBAL) // No more global sounds for the server
@@ -96,7 +102,7 @@ void CEventHandler::EventToSixup(int *pType, int *pSize, const char **ppData)
 		*pType = -protocol7::NETEVENTTYPE_SOUNDWORLD;
 		*pSize = sizeof(*pEvent7);
 
-		pEvent7->m_SoundID = pEvent->m_SoundID;
+		pEvent7->m_SoundId = pEvent->m_SoundId;
 		pEvent7->m_X = pEvent->m_X;
 		pEvent7->m_Y = pEvent->m_Y;
 
