@@ -3,7 +3,9 @@
 #ifndef GAME_SERVER_EVENTHANDLER_H
 #define GAME_SERVER_EVENTHANDLER_H
 
-#include <stdint.h>
+#include <cstdint>
+
+#include <engine/shared/protocol.h>
 
 class CEventHandler
 {
@@ -16,7 +18,7 @@ class CEventHandler
 	int m_aTypes[MAX_EVENTS]; // TODO: remove some of these arrays
 	int m_aOffsets[MAX_EVENTS];
 	int m_aSizes[MAX_EVENTS];
-	int64_t m_aClientMasks[MAX_EVENTS];
+	CClientMask m_aClientMasks[MAX_EVENTS];
 	char m_aData[MAX_DATASIZE];
 
 	class CGameContext *m_pGameServer;
@@ -29,7 +31,14 @@ public:
 	void SetGameServer(CGameContext *pGameServer);
 
 	CEventHandler();
-	void *Create(int Type, int Size, int64_t Mask = -1LL);
+	void *Create(int Type, int Size, CClientMask Mask = CClientMask().set());
+
+	template<typename T>
+	T *Create(CClientMask Mask = CClientMask().set())
+	{
+		return static_cast<T *>(Create(T::ms_MsgId, sizeof(T), Mask));
+	}
+
 	void Clear();
 	void Snap(int SnappingClient);
 
