@@ -3,12 +3,7 @@
 #define GAME_SERVER_GAMEMODES_DDRACE_H
 
 #include <game/server/gamecontroller.h>
-#include <game/server/teams.h>
 
-#include <map>
-#include <vector>
-
-struct CScoreInitResult;
 class CGameControllerDDRace : public IGameController
 {
 public:
@@ -17,27 +12,16 @@ public:
 
 	CScore *Score();
 
-	void OnCharacterSpawn(class CCharacter *pChr) override;
 	void HandleCharacterTiles(class CCharacter *pChr, int MapIndex) override;
+	void SetArmorProgress(CCharacter *pCharacter, int Progress) override;
 
 	void OnPlayerConnect(class CPlayer *pPlayer) override;
 	void OnPlayerDisconnect(class CPlayer *pPlayer, const char *pReason) override;
 
+	void OnReset() override;
+
 	void Tick() override;
 
 	void DoTeamChange(class CPlayer *pPlayer, int Team, bool DoChatMsg = true) override;
-
-	int64_t GetMaskForPlayerWorldEvent(int Asker, int ExceptID = -1) override;
-
-	void InitTeleporter();
-
-	int GetPlayerTeam(int ClientID) const;
-
-	CGameTeams m_Teams;
-
-	std::map<int, std::vector<vec2>> m_TeleOuts;
-	std::map<int, std::vector<vec2>> m_TeleCheckOuts;
-
-	std::shared_ptr<CScoreInitResult> m_pInitResult;
 };
 #endif // GAME_SERVER_GAMEMODES_DDRACE_H
