@@ -4,13 +4,14 @@
 #define GAME_SERVER_PLAYER_H
 
 #include <base/vmath.h>
-
 #include <engine/shared/protocol.h>
+#include <game/alloc.h>
+#include <game/server/save.h>
 
-#include "alloc.h"
 #include "teeinfo.h"
 
 #include <memory>
+#include <optional>
 
 class CCharacter;
 class CGameContext;
@@ -18,20 +19,13 @@ class IServer;
 struct CNetObj_PlayerInput;
 struct CScorePlayerResult;
 
-enum
-{
-	WEAPON_GAME = -3, // team switching etc
-	WEAPON_SELF = -2, // console kill command
-	WEAPON_WORLD = -1, // death tiles etc
-};
-
 // player object
 class CPlayer
 {
 	MACRO_ALLOC_POOL_ID()
 
 public:
-	CPlayer(CGameContext *pGameServer, uint32_t UniqueClientID, int ClientID, int Team);
+	CPlayer(CGameContext *pGameServer, uint32_t UniqueClientId, int ClientId, int Team);
 	~CPlayer();
 
 	void Reset();
@@ -41,8 +35,8 @@ public:
 	CCharacter *ForceSpawn(vec2 Pos); // required for loading savegames
 	void SetTeam(int Team, bool DoChatMsg = true);
 	int GetTeam() const { return m_Team; }
-	int GetCID() const { return m_ClientID; }
-	uint32_t GetUniqueCID() const { return m_UniqueClientID; }
+	int GetCid() const { return m_ClientId; }
+	uint32_t GetUniqueCid() const { return m_UniqueClientId; }
 	int GetClientVersion() const;
 	bool SetTimerType(int TimerType);
 
@@ -59,8 +53,9 @@ public:
 	void OnPredictedEarlyInput(CNetObj_PlayerInput *pNewInput);
 	void OnDisconnect();
 
-	void KillCharacter(int Weapon = WEAPON_GAME);
+	void KillCharacter(int Weapon = WEAPON_GAME, bool SendKillMsg = true);
 	CCharacter *GetCharacter();
+	const CCharacter *GetCharacter() const;
 
 	void SpectatePlayerName(const char *pName);
 
@@ -76,8 +71,10 @@ public:
 	// used for snapping to just update latency if the scoreboard is active
 	int m_aCurLatency[MAX_CLIENTS];
 
+	int m_SentSnaps = 0;
+
 	// used for spectator mode
-	int m_SpectatorID;
+	int m_SpectatorId;
 
 	bool m_IsReady;
 
@@ -92,6 +89,7 @@ public:
 	int m_LastSetSpectatorMode;
 	int m_LastChangeInfo;
 	int m_LastEmote;
+	int m_LastEmoteGlobal;
 	int m_LastKill;
 	int m_aLastCommands[4];
 	int m_LastCommandPos;
@@ -104,12 +102,10 @@ public:
 
 	int m_DieTick;
 	int m_PreviousDieTick;
-	int m_Score;
+	std::optional<int> m_Score;
 	int m_JoinTick;
-	bool m_ForceBalanced;
 	int m_LastActionTick;
 	int m_TeamChangeTick;
-	bool m_SentSemicolonTip;
 
 	// network latency calculations
 	struct
@@ -123,7 +119,7 @@ public:
 	} m_Latency;
 
 private:
-	const uint32_t m_UniqueClientID;
+	const uint32_t m_UniqueClientId;
 	CCharacter *m_pCharacter;
 	int m_NumInputs;
 	CGameContext *m_pGameServer;
@@ -134,12 +130,13 @@ private:
 	//
 	bool m_Spawning;
 	bool m_WeakHookSpawn;
-	int m_ClientID;
+	int m_ClientId;
 	int m_Team;
 
 	int m_Paused;
 	int64_t m_ForcePauseTime;
 	int64_t m_LastPause;
+	bool m_Afk;
 
 	int m_DefEmote;
 	int m_OverrideEmote;
@@ -165,15 +162,17 @@ public:
 	};
 
 	bool m_DND;
+	bool m_Whispers;
 	int64_t m_FirstVoteTick;
 	char m_aTimeoutCode[64];
 
 	void ProcessPause();
 	int Pause(int State, bool Force);
 	int ForcePause(int Time);
-	int IsPaused();
+	int IsPaused() const;
+	bool CanSpec() const;
 
-	bool IsPlaying();
+	bool IsPlaying() const;
 	int64_t m_Last_KickVote;
 	int64_t m_Last_Team;
 	int m_ShowOthers;
@@ -181,8 +180,21 @@ public:
 	vec2 m_ShowDistance;
 	bool m_SpecTeam;
 	bool m_NinjaJetpack;
-	bool m_Afk;
-	bool m_HasFinishScore;
+
+	// camera info is used sparingly for converting aim target to absolute world coordinates
+	class CCameraInfo
+	{
+		friend class CPlayer;
+		bool m_HasCameraInfo;
+		float m_Zoom;
+		int m_Deadzone;
+		int m_FollowFactor;
+
+	public:
+		vec2 ConvertTargetToWorld(vec2 Position, vec2 Target) const;
+		void Write(const CNetMsg_Cl_CameraInfo *pMsg);
+		void Reset();
+	} m_CameraInfo;
 
 	int m_ChatScore;
 
@@ -190,6 +202,10 @@ public:
 
 	void UpdatePlaytime();
 	void AfkTimer();
+	void SetAfk(bool Afk);
+	void SetInitialAfk(bool Afk);
+	bool IsAfk() const { return m_Afk; }
+
 	int64_t m_LastPlaytime;
 	int64_t m_LastEyeEmote;
 	int64_t m_LastBroadcast;
@@ -206,15 +222,19 @@ public:
 	bool CanOverrideDefaultEmote() const;
 
 	bool m_FirstPacket;
-	int64_t m_LastSQLQuery;
+	int64_t m_LastSqlQuery;
 	void ProcessScoreResult(CScorePlayerResult &Result);
 	std::shared_ptr<CScorePlayerResult> m_ScoreQueryResult;
 	std::shared_ptr<CScorePlayerResult> m_ScoreFinishResult;
 	bool m_NotEligibleForFinish;
 	int64_t m_EligibleForFinishCheck;
 	bool m_VotedForPractice;
-	int m_SwapTargetsClientID; //Client ID of the swap target for the given player
+	int m_SwapTargetsClientId; //Client ID of the swap target for the given player
 	bool m_BirthdayAnnounced;
+
+	int m_RescueMode;
+
+	CSaveTee m_LastTeleTee;
 };
 
 #endif
