@@ -8,24 +8,43 @@
 
 class IGameController;
 class CGameContext;
+class CGameWorld;
 class CCharacter;
 class CSaveTeam;
 
+enum
+{
+	RESCUEMODE_AUTO = 0,
+	RESCUEMODE_MANUAL,
+	NUM_RESCUEMODES
+};
+
+enum class ESaveResult
+{
+	SUCCESS,
+	TEAM_FLOCK,
+	TEAM_NOT_FOUND,
+	CHAR_NOT_FOUND,
+	NOT_STARTED,
+	TEAM_0_MODE,
+	DRAGGER_ACTIVE
+};
+
 class CSaveTee
 {
 public:
 	CSaveTee();
 	~CSaveTee() = default;
-	void Save(CCharacter *pchr);
-	void Load(CCharacter *pchr, int Team, bool IsSwap = false);
+	void Save(CCharacter *pchr, bool AddPenalty = true);
+	bool Load(CCharacter *pchr, int Team, bool IsSwap = false);
 	char *GetString(const CSaveTeam *pTeam);
 	int FromString(const char *pString);
 	void LoadHookedPlayer(const CSaveTeam *pTeam);
 	bool IsHooking() const;
 	vec2 GetPos() const { return m_Pos; }
 	const char *GetName() const { return m_aName; }
-	int GetClientID() const { return m_ClientID; }
-	void SetClientID(int ClientID) { m_ClientID = ClientID; }
+	int GetClientId() const { return m_ClientId; }
+	void SetClientId(int ClientId) { m_ClientId = ClientId; }
 
 	enum
 	{
@@ -37,7 +56,7 @@ public:
 	};
 
 private:
-	int m_ClientID;
+	int m_ClientId;
 
 	char m_aString[2048];
 	char m_aName[16];
@@ -59,6 +78,15 @@ private:
 		int m_Got;
 	} m_aWeapons[NUM_WEAPONS];
 
+	// ninja
+	struct
+	{
+		vec2 m_ActivationDir;
+		int m_ActivationTick;
+		int m_CurrentMoveTime;
+		int m_OldVelAmount;
+	} m_Ninja;
+
 	int m_LastWeapon;
 	int m_QueuedWeapon;
 
@@ -120,28 +148,42 @@ private:
 	char m_aGameUuid[UUID_MAXSTRSIZE];
 };
 
+class CSaveHotReloadTee
+{
+public:
+	CSaveHotReloadTee() = default;
+	~CSaveHotReloadTee() = default;
+	void Save(CCharacter *pChr, bool AddPenalty = true);
+	bool Load(CCharacter *pChr, int Team, bool IsSwap = false);
+
+private:
+	CSaveTee m_SaveTee;
+	bool m_Super;
+	bool m_Invincible;
+	CSaveTee m_SavedTeleTee;
+};
+
 class CSaveTeam
 {
 public:
-	CSaveTeam(IGameController *pController);
+	CSaveTeam();
 	~CSaveTeam();
 	char *GetString();
 	int GetMembersCount() const { return m_MembersCount; }
 	// MatchPlayers has to be called afterwards
 	int FromString(const char *pString);
 	// returns true if a team can load, otherwise writes a nice error Message in pMessage
-	bool MatchPlayers(const char (*paNames)[MAX_NAME_LENGTH], const int *pClientID, int NumPlayer, char *pMessage, int MessageLen);
-	int Save(int Team);
-	void Load(int Team, bool KeepCurrentWeakStrong);
-	CSaveTee *m_pSavedTees;
+	bool MatchPlayers(const char (*paNames)[MAX_NAME_LENGTH], const int *pClientId, int NumPlayer, char *pMessage, int MessageLen) const;
+	ESaveResult Save(CGameContext *pGameServer, int Team, bool Dry = false, bool Force = false);
+	bool Load(CGameContext *pGameServer, int Team, bool KeepCurrentWeakStrong, bool IgnorePlayers = false);
 
-	// returns true if an error occured
-	static bool HandleSaveError(int Result, int ClientID, CGameContext *pGameContext);
+	CSaveTee *m_pSavedTees = nullptr;
+
+	// returns true if an error occurred
+	static bool HandleSaveError(ESaveResult Result, int ClientId, CGameContext *pGameContext);
 
 private:
-	CCharacter *MatchCharacter(int ClientID, int SaveID, bool KeepCurrentCharacter);
-
-	IGameController *m_pController;
+	CCharacter *MatchCharacter(CGameContext *pGameServer, int ClientId, int SaveId, bool KeepCurrentCharacter) const;
 
 	char m_aString[65536];
 
@@ -151,13 +193,13 @@ private:
 		int m_EndTime;
 		int m_Type;
 	};
-	SSimpleSwitchers *m_pSwitchers;
+	SSimpleSwitchers *m_pSwitchers = nullptr;
 
-	int m_TeamState;
-	int m_MembersCount;
-	int m_HighestSwitchNumber;
-	int m_TeamLocked;
-	int m_Practice;
+	int m_TeamState = 0;
+	int m_MembersCount = 0;
+	int m_HighestSwitchNumber = 0;
+	int m_TeamLocked = 0;
+	int m_Practice = 0;
 };
 
 #endif // GAME_SERVER_SAVE_H
