@@ -2,6 +2,7 @@
 #define GAME_SERVER_SCOREWORKER_H
 
 #include <memory>
+#include <optional>
 #include <string>
 #include <utility>
 #include <vector>
@@ -38,6 +39,7 @@ struct CScorePlayerResult : ISqlResult
 		BROADCAST,
 		MAP_VOTE,
 		PLAYER_INFO,
+		PLAYER_TIMECP,
 	} m_MessageKind;
 	union
 	{
@@ -45,35 +47,34 @@ struct CScorePlayerResult : ISqlResult
 		char m_aBroadcast[1024];
 		struct
 		{
-			float m_Time;
+			std::optional<float> m_Time;
 			float m_aTimeCp[NUM_CHECKPOINTS];
-			int m_Score;
-			int m_HasFinishScore;
 			int m_Birthday; // 0 indicates no birthday
-		} m_Info;
+			char m_aRequestedPlayer[MAX_NAME_LENGTH];
+		} m_Info = {};
 		struct
 		{
 			char m_aReason[VOTE_REASON_LENGTH];
 			char m_aServer[32 + 1];
 			char m_aMap[MAX_MAP_LENGTH + 1];
 		} m_MapVote;
-	} m_Data; // PLAYER_INFO
+	} m_Data = {}; // PLAYER_INFO
 
 	void SetVariant(Variant v);
 };
 
-struct CScoreInitResult : ISqlResult
+struct CScoreLoadBestTimeResult : ISqlResult
 {
-	CScoreInitResult() :
+	CScoreLoadBestTimeResult() :
 		m_CurrentRecord(0)
 	{
 	}
 	float m_CurrentRecord;
 };
 
-struct CSqlInitData : ISqlData
+struct CSqlLoadBestTimeRequest : ISqlData
 {
-	CSqlInitData(std::shared_ptr<CScoreInitResult> pResult) :
+	CSqlLoadBestTimeRequest(std::shared_ptr<CScoreLoadBestTimeResult> pResult) :
 		ISqlData(std::move(pResult))
 	{
 	}
@@ -101,13 +102,13 @@ struct CSqlPlayerRequest : ISqlData
 
 struct CScoreRandomMapResult : ISqlResult
 {
-	CScoreRandomMapResult(int ClientID) :
-		m_ClientID(ClientID)
+	CScoreRandomMapResult(int ClientId) :
+		m_ClientId(ClientId)
 	{
 		m_aMap[0] = '\0';
 		m_aMessage[0] = '\0';
 	}
-	int m_ClientID;
+	int m_ClientId;
 	char m_aMap[MAX_MAP_LENGTH];
 	char m_aMessage[512];
 };
@@ -138,7 +139,7 @@ struct CSqlScoreData : ISqlData
 	char m_aGameUuid[UUID_MAXSTRSIZE];
 	char m_aName[MAX_MAP_LENGTH];
 
-	int m_ClientID;
+	int m_ClientId;
 	float m_Time;
 	char m_aTimestamp[TIMESTAMP_STR_LENGTH];
 	float m_aCurrentTimeCp[NUM_CHECKPOINTS];
@@ -149,10 +150,9 @@ struct CSqlScoreData : ISqlData
 
 struct CScoreSaveResult : ISqlResult
 {
-	CScoreSaveResult(int PlayerID, IGameController *pController) :
+	CScoreSaveResult(int PlayerId) :
 		m_Status(SAVE_FAILED),
-		m_SavedTeam(CSaveTeam(pController)),
-		m_RequestingPlayer(PlayerID)
+		m_RequestingPlayer(PlayerId)
 	{
 		m_aMessage[0] = '\0';
 		m_aBroadcast[0] = '\0';
@@ -169,7 +169,7 @@ struct CScoreSaveResult : ISqlResult
 	char m_aBroadcast[512];
 	CSaveTeam m_SavedTeam;
 	int m_RequestingPlayer;
-	CUuid m_SaveID;
+	CUuid m_SaveId;
 };
 
 struct CSqlTeamScoreData : ISqlData
@@ -185,15 +185,16 @@ struct CSqlTeamScoreData : ISqlData
 	char m_aTimestamp[TIMESTAMP_STR_LENGTH];
 	unsigned int m_Size;
 	char m_aaNames[MAX_CLIENTS][MAX_NAME_LENGTH];
+	CUuid m_TeamrankUuid;
 };
 
-struct CSqlTeamSave : ISqlData
+struct CSqlTeamSaveData : ISqlData
 {
-	CSqlTeamSave(std::shared_ptr<CScoreSaveResult> pResult) :
+	CSqlTeamSaveData(std::shared_ptr<CScoreSaveResult> pResult) :
 		ISqlData(std::move(pResult))
 	{
 	}
-	virtual ~CSqlTeamSave(){};
+	virtual ~CSqlTeamSaveData(){};
 
 	char m_aClientName[MAX_NAME_LENGTH];
 	char m_aMap[MAX_MAP_LENGTH];
@@ -202,21 +203,20 @@ struct CSqlTeamSave : ISqlData
 	char m_aServer[5];
 };
 
-struct CSqlTeamLoad : ISqlData
+struct CSqlTeamLoadRequest : ISqlData
 {
-	CSqlTeamLoad(std::shared_ptr<CScoreSaveResult> pResult) :
+	CSqlTeamLoadRequest(std::shared_ptr<CScoreSaveResult> pResult) :
 		ISqlData(std::move(pResult))
 	{
 	}
-	virtual ~CSqlTeamLoad(){};
+	virtual ~CSqlTeamLoadRequest(){};
 
 	char m_aCode[128];
 	char m_aMap[MAX_MAP_LENGTH];
 	char m_aRequestingPlayer[MAX_NAME_LENGTH];
-	int m_ClientID;
 	// struct holding all player names in the team or an empty string
 	char m_aClientNames[MAX_CLIENTS][MAX_NAME_LENGTH];
-	int m_aClientID[MAX_CLIENTS];
+	int m_aClientId[MAX_CLIENTS];
 	int m_NumPlayer;
 };
 
@@ -234,31 +234,42 @@ public:
 		m_BestTime = 0;
 		for(float &BestTimeCp : m_aBestTimeCp)
 			BestTimeCp = 0;
+
+		m_RecordStopTick = -1;
 	}
 
-	void Set(float Time, float aTimeCp[NUM_CHECKPOINTS])
+	void Set(float Time, const float aTimeCp[NUM_CHECKPOINTS])
 	{
 		m_BestTime = Time;
 		for(int i = 0; i < NUM_CHECKPOINTS; i++)
 			m_aBestTimeCp[i] = aTimeCp[i];
 	}
 
+	void SetBestTimeCp(const float aTimeCp[NUM_CHECKPOINTS])
+	{
+		for(int i = 0; i < NUM_CHECKPOINTS; i++)
+			m_aBestTimeCp[i] = aTimeCp[i];
+	}
+
 	float m_BestTime;
 	float m_aBestTimeCp[NUM_CHECKPOINTS];
+
+	int m_RecordStopTick;
+	float m_RecordFinishTime;
 };
 
 struct CTeamrank
 {
-	CUuid m_TeamID;
+	CUuid m_TeamId;
 	char m_aaNames[MAX_CLIENTS][MAX_NAME_LENGTH];
 	unsigned int m_NumNames;
 	CTeamrank();
 
 	// Assumes that a database query equivalent to
 	//
-	//     SELECT TeamID, Name [, ...] -- the order is important
+	//     SELECT TeamId, Name [, ...] -- the order is important
 	//     FROM record_teamrace
-	//     ORDER BY TeamID, Name
+	//     ORDER BY TeamId, Name
 	//
 	// was executed and that the result line of the first team member is already selected.
 	// Afterwards the team member of the next team is selected.
@@ -269,17 +280,20 @@ struct CTeamrank
 	bool NextSqlResult(IDbConnection *pSqlServer, bool *pEnd, char *pError, int ErrorSize);
 
 	bool SamePlayers(const std::vector<std::string> *pvSortedNames);
+
+	static bool GetSqlTop5Team(IDbConnection *pSqlServer, bool *pEnd, char *pError, int ErrorSize, char (*paMessages)[512], int *StartLine, int Count);
 };
 
 struct CScoreWorker
 {
-	static bool Init(IDbConnection *pSqlServer, const ISqlData *pGameData, char *pError, int ErrorSize);
+	static bool LoadBestTime(IDbConnection *pSqlServer, const ISqlData *pGameData, char *pError, int ErrorSize);
 
 	static bool RandomMap(IDbConnection *pSqlServer, const ISqlData *pGameData, char *pError, int ErrorSize);
 	static bool RandomUnfinishedMap(IDbConnection *pSqlServer, const ISqlData *pGameData, char *pError, int ErrorSize);
 	static bool MapVote(IDbConnection *pSqlServer, const ISqlData *pGameData, char *pError, int ErrorSize);
 
 	static bool LoadPlayerData(IDbConnection *pSqlServer, const ISqlData *pGameData, char *pError, int ErrorSize);
+	static bool LoadPlayerTimeCp(IDbConnection *pSqlServer, const ISqlData *pGameData, char *pError, int ErrorSize);
 	static bool MapInfo(IDbConnection *pSqlServer, const ISqlData *pGameData, char *pError, int ErrorSize);
 	static bool ShowRank(IDbConnection *pSqlServer, const ISqlData *pGameData, char *pError, int ErrorSize);
 	static bool ShowTeamRank(IDbConnection *pSqlServer, const ISqlData *pGameData, char *pError, int ErrorSize);
@@ -291,11 +305,11 @@ struct CScoreWorker
 	static bool ShowTopPoints(IDbConnection *pSqlServer, const ISqlData *pGameData, char *pError, int ErrorSize);
 	static bool GetSaves(IDbConnection *pSqlServer, const ISqlData *pGameData, char *pError, int ErrorSize);
 
-	static bool SaveTeam(IDbConnection *pSqlServer, const ISqlData *pGameData, bool Failure, char *pError, int ErrorSize);
-	static bool LoadTeam(IDbConnection *pSqlServer, const ISqlData *pGameData, bool Failure, char *pError, int ErrorSize);
+	static bool SaveTeam(IDbConnection *pSqlServer, const ISqlData *pGameData, Write w, char *pError, int ErrorSize);
+	static bool LoadTeam(IDbConnection *pSqlServer, const ISqlData *pGameData, Write w, char *pError, int ErrorSize);
 
-	static bool SaveScore(IDbConnection *pSqlServer, const ISqlData *pGameData, bool Failure, char *pError, int ErrorSize);
-	static bool SaveTeamScore(IDbConnection *pSqlServer, const ISqlData *pGameData, bool Failure, char *pError, int ErrorSize);
+	static bool SaveScore(IDbConnection *pSqlServer, const ISqlData *pGameData, Write w, char *pError, int ErrorSize);
+	static bool SaveTeamScore(IDbConnection *pSqlServer, const ISqlData *pGameData, Write w, char *pError, int ErrorSize);
 };
 
 #endif // GAME_SERVER_SCOREWORKER_H
