@@ -23,31 +23,32 @@ class CGameTeams
 	bool m_aTeeFinished[MAX_CLIENTS];
 	int m_aLastChat[MAX_CLIENTS];
 
-	int m_aTeamState[NUM_TEAMS];
-	bool m_aTeamLocked[NUM_TEAMS];
-	uint64_t m_aInvited[NUM_TEAMS];
-	bool m_aPractice[NUM_TEAMS];
-	std::shared_ptr<CScoreSaveResult> m_apSaveTeamResult[NUM_TEAMS];
-	uint64_t m_aLastSwap[NUM_TEAMS];
-	bool m_aTeamSentStartWarning[NUM_TEAMS];
+	int m_aTeamState[NUM_DDRACE_TEAMS];
+	bool m_aTeamLocked[NUM_DDRACE_TEAMS];
+	bool m_aTeamFlock[NUM_DDRACE_TEAMS];
+	CClientMask m_aInvited[NUM_DDRACE_TEAMS];
+	bool m_aPractice[NUM_DDRACE_TEAMS];
+	std::shared_ptr<CScoreSaveResult> m_apSaveTeamResult[NUM_DDRACE_TEAMS];
+	uint64_t m_aLastSwap[MAX_CLIENTS]; // index is id of player who initiated swap
+	bool m_aTeamSentStartWarning[NUM_DDRACE_TEAMS];
 	// `m_aTeamUnfinishableKillTick` is -1 by default and gets set when a
 	// team becomes unfinishable. If the team hasn't entered practice mode
 	// by that time, it'll get killed to prevent people not understanding
 	// the message from playing for a long time in an unfinishable team.
-	int m_aTeamUnfinishableKillTick[NUM_TEAMS];
+	int m_aTeamUnfinishableKillTick[NUM_DDRACE_TEAMS];
 
 	class CGameContext *m_pGameContext;
 
 	/**
 	* Kill the whole team.
 	* @param Team The team id to kill
-	* @param NewStrongID The player with that id will get strong hook on everyone else, -1 will set the normal spawning order
-	* @param ExceptID The player that should not get killed
+	* @param NewStrongId The player with that id will get strong hook on everyone else, -1 will set the normal spawning order
+	* @param ExceptId The player that should not get killed
 	*/
-	void KillTeam(int Team, int NewStrongID, int ExceptID = -1);
+	void KillTeam(int Team, int NewStrongId, int ExceptId = -1);
 	bool TeamFinished(int Team);
-	void OnTeamFinish(CPlayer **Players, unsigned int Size, float Time, const char *pTimestamp);
-	void OnFinish(CPlayer *Player, float Time, const char *pTimestamp);
+	void OnTeamFinish(int Team, CPlayer **Players, unsigned int Size, int TimeTicks, const char *pTimestamp);
+	void OnFinish(CPlayer *Player, int TimeTicks, const char *pTimestamp);
 
 public:
 	enum
@@ -66,13 +67,13 @@ public:
 	CGameTeams(CGameContext *pGameContext);
 
 	// helper methods
-	CCharacter *Character(int ClientID)
+	CCharacter *Character(int ClientId)
 	{
-		return GameServer()->GetPlayerChar(ClientID);
+		return GameServer()->GetPlayerChar(ClientId);
 	}
-	CPlayer *GetPlayer(int ClientID)
+	CPlayer *GetPlayer(int ClientId)
 	{
-		return GameServer()->m_apPlayers[ClientID];
+		return GameServer()->m_apPlayers[ClientId];
 	}
 
 	class CGameContext *GameServer()
@@ -84,33 +85,34 @@ public:
 		return m_pGameContext->Server();
 	}
 
-	void OnCharacterStart(int ClientID);
-	void OnCharacterFinish(int ClientID);
-	void OnCharacterSpawn(int ClientID);
-	void OnCharacterDeath(int ClientID, int Weapon);
+	void OnCharacterStart(int ClientId);
+	void OnCharacterFinish(int ClientId);
+	void OnCharacterSpawn(int ClientId);
+	void OnCharacterDeath(int ClientId, int Weapon);
 	void Tick();
 
 	// returns nullptr if successful, error string if failed
-	const char *SetCharacterTeam(int ClientID, int Team);
+	const char *SetCharacterTeam(int ClientId, int Team);
 	void CheckTeamFinished(int Team);
 
 	void ChangeTeamState(int Team, int State);
 
-	int64_t TeamMask(int Team, int ExceptID = -1, int Asker = -1);
+	CClientMask TeamMask(int Team, int ExceptId = -1, int Asker = -1, int VersionFlags = CGameContext::FLAG_SIX | CGameContext::FLAG_SIXUP);
 
 	int Count(int Team) const;
 
 	// need to be very careful using this method. SERIOUSLY...
-	void SetForceCharacterTeam(int ClientID, int Team);
+	void SetForceCharacterTeam(int ClientId, int Team);
 
 	void Reset();
 	void ResetRoundState(int Team);
 	void ResetSwitchers(int Team);
 
-	void SendTeamsState(int ClientID);
+	void SendTeamsState(int ClientId);
 	void SetTeamLock(int Team, bool Lock);
+	void SetTeamFlock(int Team, bool Mode);
 	void ResetInvited(int Team);
-	void SetClientInvited(int Team, int ClientID, bool Invited);
+	void SetClientInvited(int Team, int ClientId, bool Invited);
 
 	int GetDDRaceState(CPlayer *Player);
 	int GetStartTime(CPlayer *Player);
@@ -118,22 +120,23 @@ public:
 	void SetDDRaceState(CPlayer *Player, int DDRaceState);
 	void SetStartTime(CPlayer *Player, int StartTime);
 	void SetLastTimeCp(CPlayer *Player, int LastTimeCp);
-	void KillSavedTeam(int ClientID, int Team);
-	void ResetSavedTeam(int ClientID, int Team);
+	void KillCharacterOrTeam(int ClientId, int Team);
+	void ResetSavedTeam(int ClientId, int Team);
 	void RequestTeamSwap(CPlayer *pPlayer, CPlayer *pTargetPlayer, int Team);
 	void SwapTeamCharacters(CPlayer *pPrimaryPlayer, CPlayer *pTargetPlayer, int Team);
+	void CancelTeamSwap(CPlayer *pPlayer, int Team);
 	void ProcessSaveTeam();
 
 	int GetFirstEmptyTeam() const;
 
-	bool TeeStarted(int ClientID)
+	bool TeeStarted(int ClientId)
 	{
-		return m_aTeeStarted[ClientID];
+		return m_aTeeStarted[ClientId];
 	}
 
-	bool TeeFinished(int ClientID)
+	bool TeeFinished(int ClientId)
 	{
-		return m_aTeeFinished[ClientID];
+		return m_aTeeFinished[ClientId];
 	}
 
 	int GetTeamState(int Team)
@@ -149,9 +152,17 @@ public:
 		return m_aTeamLocked[Team];
 	}
 
-	bool IsInvited(int Team, int ClientID)
+	bool TeamFlock(int Team)
 	{
-		return m_aInvited[Team] & 1LL << ClientID;
+		if(Team <= TEAM_FLOCK || Team >= TEAM_SUPER)
+			return false;
+
+		return m_aTeamFlock[Team];
+	}
+
+	bool IsInvited(int Team, int ClientId)
+	{
+		return m_aInvited[Team].test(ClientId);
 	}
 
 	bool IsStarted(int Team)
@@ -159,29 +170,29 @@ public:
 		return m_aTeamState[Team] == CGameTeams::TEAMSTATE_STARTED;
 	}
 
-	void SetStarted(int ClientID, bool Started)
+	void SetStarted(int ClientId, bool Started)
 	{
-		m_aTeeStarted[ClientID] = Started;
+		m_aTeeStarted[ClientId] = Started;
 	}
 
-	void SetFinished(int ClientID, bool Finished)
+	void SetFinished(int ClientId, bool Finished)
 	{
-		m_aTeeFinished[ClientID] = Finished;
+		m_aTeeFinished[ClientId] = Finished;
 	}
 
-	void SetSaving(int TeamID, std::shared_ptr<CScoreSaveResult> &SaveResult)
+	void SetSaving(int TeamId, std::shared_ptr<CScoreSaveResult> &SaveResult)
 	{
-		m_apSaveTeamResult[TeamID] = SaveResult;
+		m_apSaveTeamResult[TeamId] = SaveResult;
 	}
 
-	bool GetSaving(int TeamID)
+	bool GetSaving(int TeamId)
 	{
-		if(TeamID < TEAM_FLOCK || TeamID >= TEAM_SUPER)
+		if(TeamId < TEAM_FLOCK || TeamId >= TEAM_SUPER)
 			return false;
-		if(g_Config.m_SvTeam != SV_TEAM_FORCED_SOLO && TeamID == TEAM_FLOCK)
+		if(g_Config.m_SvTeam != SV_TEAM_FORCED_SOLO && TeamId == TEAM_FLOCK)
 			return false;
 
-		return m_apSaveTeamResult[TeamID] != nullptr;
+		return m_apSaveTeamResult[TeamId] != nullptr;
 	}
 
 	void SetPractice(int Team, bool Enabled)
@@ -199,7 +210,12 @@ public:
 		if(Team < TEAM_FLOCK || Team >= TEAM_SUPER)
 			return false;
 		if(g_Config.m_SvTeam != SV_TEAM_FORCED_SOLO && Team == TEAM_FLOCK)
+		{
+			if(m_pGameContext->PracticeByDefault())
+				return true;
+
 			return false;
+		}
 
 		return m_aPractice[Team];
 	}
