@@ -1,14 +1,30 @@
 #include "teehistorian.h"
 
+#include <base/system.h>
+
 #include <engine/shared/config.h>
 #include <engine/shared/json.h>
+#include <engine/shared/packer.h>
 #include <engine/shared/snapshot.h>
+
 #include <game/gamecore.h>
 
+class CTeehistorianPacker : public CAbstractPacker
+{
+public:
+	CTeehistorianPacker() :
+		CAbstractPacker(m_aBuffer, sizeof(m_aBuffer))
+	{
+	}
+
+private:
+	unsigned char m_aBuffer[1024 * 64];
+};
+
 static const char TEEHISTORIAN_NAME[] = "teehistorian@ddnet.tw";
 static const CUuid TEEHISTORIAN_UUID = CalculateUuid(TEEHISTORIAN_NAME);
 static const char TEEHISTORIAN_VERSION[] = "2";
-static const char TEEHISTORIAN_VERSION_MINOR[] = "4";
+static const char TEEHISTORIAN_VERSION_MINOR[] = "9";
 
 #define UUID(id, name) static const CUuid UUID_##id = CalculateUuid(name);
 #include <engine/shared/teehistorian_ex_chunks.h>
@@ -33,8 +49,8 @@ enum
 CTeeHistorian::CTeeHistorian()
 {
 	m_State = STATE_START;
-	m_pfnWriteCallback = 0;
-	m_pWriteCallbackUserdata = 0;
+	m_pfnWriteCallback = nullptr;
+	m_pWriteCallbackUserdata = nullptr;
 }
 
 void CTeeHistorian::Reset(const CGameInfo *pGameInfo, WRITE_CALLBACK pfnWriteCallback, void *pUser)
@@ -47,14 +63,14 @@ void CTeeHistorian::Reset(const CGameInfo *pGameInfo, WRITE_CALLBACK pfnWriteCal
 	m_LastWrittenTick = 0;
 	// Tick 0 is implicit at the start, game starts as tick 1.
 	m_TickWritten = true;
-	m_MaxClientID = MAX_CLIENTS;
+	m_MaxClientId = MAX_CLIENTS;
 
-	// `m_PrevMaxClientID` is initialized in `BeginPlayers`
+	// `m_PrevMaxClientId` is initialized in `BeginPlayers`
 	for(auto &PrevPlayer : m_aPrevPlayers)
 	{
 		PrevPlayer.m_Alive = false;
 		// zero means no id
-		PrevPlayer.m_UniqueClientID = 0;
+		PrevPlayer.m_UniqueClientId = 0;
 		PrevPlayer.m_Team = 0;
 	}
 	for(auto &PrevTeam : m_aPrevTeams)
@@ -81,6 +97,18 @@ void CTeeHistorian::WriteHeader(const CGameInfo *pGameInfo)
 	str_timestamp_ex(pGameInfo->m_StartTime, aStartTime, sizeof(aStartTime), "%Y-%m-%dT%H:%M:%S%z");
 	sha256_str(pGameInfo->m_MapSha256, aMapSha256, sizeof(aMapSha256));
 
+	char aPrevGameUuid[UUID_MAXSTRSIZE];
+	char aPrevGameUuidJson[64];
+	if(pGameInfo->m_HavePrevGameUuid)
+	{
+		FormatUuid(pGameInfo->m_PrevGameUuid, aPrevGameUuid, sizeof(aPrevGameUuid));
+		str_format(aPrevGameUuidJson, sizeof(aPrevGameUuidJson), "\"prev_game_uuid\":\"%s\",", aPrevGameUuid);
+	}
+	else
+	{
+		aPrevGameUuidJson[0] = 0;
+	}
+
 	char aCommentBuffer[128];
 	char aServerVersionBuffer[128];
 	char aStartTimeBuffer[128];
@@ -100,6 +128,7 @@ void CTeeHistorian::WriteHeader(const CGameInfo *pGameInfo)
 		"\"version\":\"%s\","
 		"\"version_minor\":\"%s\","
 		"\"game_uuid\":\"%s\","
+		"%s"
 		"\"server_version\":\"%s\","
 		"\"start_time\":\"%s\","
 		"\"server_name\":\"%s\","
@@ -115,6 +144,7 @@ void CTeeHistorian::WriteHeader(const CGameInfo *pGameInfo)
 		TEEHISTORIAN_VERSION,
 		TEEHISTORIAN_VERSION_MINOR,
 		aGameUuid,
+		aPrevGameUuidJson,
 		E(aServerVersionBuffer, pGameInfo->m_pServerVersion),
 		E(aStartTimeBuffer, aStartTime),
 		E(aServerNameBuffer, pGameInfo->m_pServerName),
@@ -142,7 +172,7 @@ void CTeeHistorian::WriteHeader(const CGameInfo *pGameInfo)
 		First = false; \
 	}
 
-#define MACRO_CONFIG_COL(Name, ScriptName, Def, Save, Desc) MACRO_CONFIG_INT(Name, ScriptName, Def, 0, 0, Save, Desc)
+#define MACRO_CONFIG_COL(Name, ScriptName, Def, Flags, Desc) MACRO_CONFIG_INT(Name, ScriptName, Def, 0, 0, Flags, Desc)
 
 #define MACRO_CONFIG_STR(Name, ScriptName, Len, Def, Flags, Desc) \
 	if((Flags)&CFGFLAG_SERVER && !((Flags)&CFGFLAG_NONTEEHISTORIC) && str_comp(pGameInfo->m_pConfig->m_##Name, (Def)) != 0) \
@@ -161,12 +191,11 @@ void CTeeHistorian::WriteHeader(const CGameInfo *pGameInfo)
 #undef MACRO_CONFIG_COL
 #undef MACRO_CONFIG_STR
 
-	str_format(aJson, sizeof(aJson), "},\"tuning\":{");
+	str_copy(aJson, "},\"tuning\":{");
 	Write(aJson, str_length(aJson));
 
 	First = true;
 
-	const float TicksPerSecond = 50.0f;
 #define MACRO_TUNING_PARAM(Name, ScriptName, Value, Description) \
 	if(pGameInfo->m_pTuning->m_##Name.Get() != (int)((Value)*100)) \
 	{ \
@@ -180,7 +209,7 @@ void CTeeHistorian::WriteHeader(const CGameInfo *pGameInfo)
 #include <game/tuning.h>
 #undef MACRO_TUNING_PARAM
 
-	str_format(aJson, sizeof(aJson), "},\"uuids\":[");
+	str_copy(aJson, "},\"uuids\":[");
 	Write(aJson, str_length(aJson));
 
 	for(int i = 0; i < pGameInfo->m_pUuids->NumUuids(); i++)
@@ -191,7 +220,7 @@ void CTeeHistorian::WriteHeader(const CGameInfo *pGameInfo)
 		Write(aJson, str_length(aJson));
 	}
 
-	str_format(aJson, sizeof(aJson), "]}");
+	str_copy(aJson, "]}");
 	Write(aJson, str_length(aJson));
 	Write("", 1); // Null termination.
 }
@@ -200,7 +229,7 @@ void CTeeHistorian::WriteExtra(CUuid Uuid, const void *pData, int DataSize)
 {
 	EnsureTickWritten();
 
-	CPacker Ex;
+	CTeehistorianPacker Ex;
 	Ex.Reset();
 	Ex.AddInt(-TEEHISTORIAN_EX);
 	Ex.AddRaw(&Uuid, sizeof(Uuid));
@@ -228,20 +257,20 @@ void CTeeHistorian::BeginPlayers()
 {
 	dbg_assert(m_State == STATE_BEFORE_PLAYERS, "invalid teehistorian state");
 
-	m_PrevMaxClientID = m_MaxClientID;
+	m_PrevMaxClientId = m_MaxClientId;
 	// ensure that PLAYER_{DIFF, NEW, OLD} don't cause an implicit tick after a TICK_SKIP
-	// by not overwriting m_MaxClientID during RecordPlayer
-	m_MaxClientID = -1;
+	// by not overwriting m_MaxClientId during RecordPlayer
+	m_MaxClientId = -1;
 
 	m_State = STATE_PLAYERS;
 }
 
-void CTeeHistorian::EnsureTickWrittenPlayerData(int ClientID)
+void CTeeHistorian::EnsureTickWrittenPlayerData(int ClientId)
 {
-	dbg_assert(ClientID > m_MaxClientID, "invalid player data order");
-	m_MaxClientID = ClientID;
+	dbg_assert(ClientId > m_MaxClientId, "invalid player data order");
+	m_MaxClientId = ClientId;
 
-	if(!m_TickWritten && (ClientID > m_PrevMaxClientID || m_LastWrittenTick + 1 != m_Tick))
+	if(!m_TickWritten && (ClientId > m_PrevMaxClientId || m_LastWrittenTick + 1 != m_Tick))
 	{
 		WriteTick();
 	}
@@ -253,27 +282,27 @@ void CTeeHistorian::EnsureTickWrittenPlayerData(int ClientID)
 	}
 }
 
-void CTeeHistorian::RecordPlayer(int ClientID, const CNetObj_CharacterCore *pChar)
+void CTeeHistorian::RecordPlayer(int ClientId, const CNetObj_CharacterCore *pChar)
 {
 	dbg_assert(m_State == STATE_PLAYERS, "invalid teehistorian state");
 
-	CTeehistorianPlayer *pPrev = &m_aPrevPlayers[ClientID];
+	CTeehistorianPlayer *pPrev = &m_aPrevPlayers[ClientId];
 	if(!pPrev->m_Alive || pPrev->m_X != pChar->m_X || pPrev->m_Y != pChar->m_Y)
 	{
-		EnsureTickWrittenPlayerData(ClientID);
+		EnsureTickWrittenPlayerData(ClientId);
 
-		CPacker Buffer;
+		CTeehistorianPacker Buffer;
 		Buffer.Reset();
 		if(pPrev->m_Alive)
 		{
 			int dx = pChar->m_X - pPrev->m_X;
 			int dy = pChar->m_Y - pPrev->m_Y;
-			Buffer.AddInt(ClientID);
+			Buffer.AddInt(ClientId);
 			Buffer.AddInt(dx);
 			Buffer.AddInt(dy);
 			if(m_Debug)
 			{
-				dbg_msg("teehistorian", "diff cid=%d dx=%d dy=%d", ClientID, dx, dy);
+				dbg_msg("teehistorian", "diff cid=%d dx=%d dy=%d", ClientId, dx, dy);
 			}
 		}
 		else
@@ -281,12 +310,12 @@ void CTeeHistorian::RecordPlayer(int ClientID, const CNetObj_CharacterCore *pCha
 			int x = pChar->m_X;
 			int y = pChar->m_Y;
 			Buffer.AddInt(-TEEHISTORIAN_PLAYER_NEW);
-			Buffer.AddInt(ClientID);
+			Buffer.AddInt(ClientId);
 			Buffer.AddInt(x);
 			Buffer.AddInt(y);
 			if(m_Debug)
 			{
-				dbg_msg("teehistorian", "new cid=%d x=%d y=%d", ClientID, x, y);
+				dbg_msg("teehistorian", "new cid=%d x=%d y=%d", ClientId, x, y);
 			}
 		}
 		Write(Buffer.Data(), Buffer.Size());
@@ -296,44 +325,44 @@ void CTeeHistorian::RecordPlayer(int ClientID, const CNetObj_CharacterCore *pCha
 	pPrev->m_Alive = true;
 }
 
-void CTeeHistorian::RecordDeadPlayer(int ClientID)
+void CTeeHistorian::RecordDeadPlayer(int ClientId)
 {
 	dbg_assert(m_State == STATE_PLAYERS, "invalid teehistorian state");
 
-	CTeehistorianPlayer *pPrev = &m_aPrevPlayers[ClientID];
+	CTeehistorianPlayer *pPrev = &m_aPrevPlayers[ClientId];
 	if(pPrev->m_Alive)
 	{
-		EnsureTickWrittenPlayerData(ClientID);
+		EnsureTickWrittenPlayerData(ClientId);
 
-		CPacker Buffer;
+		CTeehistorianPacker Buffer;
 		Buffer.Reset();
 		Buffer.AddInt(-TEEHISTORIAN_PLAYER_OLD);
-		Buffer.AddInt(ClientID);
+		Buffer.AddInt(ClientId);
 		if(m_Debug)
 		{
-			dbg_msg("teehistorian", "old cid=%d", ClientID);
+			dbg_msg("teehistorian", "old cid=%d", ClientId);
 		}
 		Write(Buffer.Data(), Buffer.Size());
 	}
 	pPrev->m_Alive = false;
 }
 
-void CTeeHistorian::RecordPlayerTeam(int ClientID, int Team)
+void CTeeHistorian::RecordPlayerTeam(int ClientId, int Team)
 {
-	if(m_aPrevPlayers[ClientID].m_Team != Team)
+	if(m_aPrevPlayers[ClientId].m_Team != Team)
 	{
-		m_aPrevPlayers[ClientID].m_Team = Team;
+		m_aPrevPlayers[ClientId].m_Team = Team;
 
 		EnsureTickWritten();
 
-		CPacker Buffer;
+		CTeehistorianPacker Buffer;
 		Buffer.Reset();
-		Buffer.AddInt(ClientID);
+		Buffer.AddInt(ClientId);
 		Buffer.AddInt(Team);
 
 		if(m_Debug)
 		{
-			dbg_msg("teehistorian", "player_team cid=%d team=%d", ClientID, Team);
+			dbg_msg("teehistorian", "player_team cid=%d team=%d", ClientId, Team);
 		}
 
 		WriteExtra(UUID_TEEHISTORIAN_PLAYER_TEAM, Buffer.Data(), Buffer.Size());
@@ -348,7 +377,7 @@ void CTeeHistorian::RecordTeamPractice(int Team, bool Practice)
 
 		EnsureTickWritten();
 
-		CPacker Buffer;
+		CTeehistorianPacker Buffer;
 		Buffer.Reset();
 		Buffer.AddInt(Team);
 		Buffer.AddInt(Practice);
@@ -377,7 +406,7 @@ void CTeeHistorian::EnsureTickWritten()
 
 void CTeeHistorian::WriteTick()
 {
-	CPacker TickPacker;
+	CTeehistorianPacker TickPacker;
 	TickPacker.Reset();
 
 	int dt = m_Tick - m_LastWrittenTick - 1;
@@ -407,13 +436,13 @@ void CTeeHistorian::BeginInputs()
 	m_State = STATE_INPUTS;
 }
 
-void CTeeHistorian::RecordPlayerInput(int ClientID, uint32_t UniqueClientID, const CNetObj_PlayerInput *pInput)
+void CTeeHistorian::RecordPlayerInput(int ClientId, uint32_t UniqueClientId, const CNetObj_PlayerInput *pInput)
 {
-	CPacker Buffer;
+	CTeehistorianPacker Buffer;
 
-	CTeehistorianPlayer *pPrev = &m_aPrevPlayers[ClientID];
+	CTeehistorianPlayer *pPrev = &m_aPrevPlayers[ClientId];
 	CNetObj_PlayerInput DiffInput;
-	if(pPrev->m_UniqueClientID == UniqueClientID)
+	if(pPrev->m_UniqueClientId == UniqueClientId)
 	{
 		if(mem_comp(&pPrev->m_Input, pInput, sizeof(pPrev->m_Input)) == 0)
 		{
@@ -423,11 +452,11 @@ void CTeeHistorian::RecordPlayerInput(int ClientID, uint32_t UniqueClientID, con
 		Buffer.Reset();
 
 		Buffer.AddInt(-TEEHISTORIAN_INPUT_DIFF);
-		CSnapshotDelta::DiffItem((int *)&pPrev->m_Input, (int *)pInput, (int *)&DiffInput, sizeof(DiffInput) / sizeof(int));
+		CSnapshotDelta::DiffItem((int *)&pPrev->m_Input, (int *)pInput, (int *)&DiffInput, sizeof(DiffInput) / sizeof(int32_t));
 		if(m_Debug)
 		{
 			const int *pData = (const int *)&DiffInput;
-			dbg_msg("teehistorian", "diff_input cid=%d %d %d %d %d %d %d %d %d %d %d", ClientID,
+			dbg_msg("teehistorian", "diff_input cid=%d %d %d %d %d %d %d %d %d %d %d", ClientId,
 				pData[0], pData[1], pData[2], pData[3], pData[4],
 				pData[5], pData[6], pData[7], pData[8], pData[9]);
 		}
@@ -440,28 +469,28 @@ void CTeeHistorian::RecordPlayerInput(int ClientID, uint32_t UniqueClientID, con
 		DiffInput = *pInput;
 		if(m_Debug)
 		{
-			dbg_msg("teehistorian", "new_input cid=%d", ClientID);
+			dbg_msg("teehistorian", "new_input cid=%d", ClientId);
 		}
 	}
-	Buffer.AddInt(ClientID);
-	for(int i = 0; i < (int)(sizeof(DiffInput) / sizeof(int)); i++)
+	Buffer.AddInt(ClientId);
+	for(size_t i = 0; i < sizeof(DiffInput) / sizeof(int32_t); i++)
 	{
 		Buffer.AddInt(((int *)&DiffInput)[i]);
 	}
-	pPrev->m_UniqueClientID = UniqueClientID;
+	pPrev->m_UniqueClientId = UniqueClientId;
 	pPrev->m_Input = *pInput;
 
 	Write(Buffer.Data(), Buffer.Size());
 }
 
-void CTeeHistorian::RecordPlayerMessage(int ClientID, const void *pMsg, int MsgSize)
+void CTeeHistorian::RecordPlayerMessage(int ClientId, const void *pMsg, int MsgSize)
 {
 	EnsureTickWritten();
 
-	CPacker Buffer;
+	CTeehistorianPacker Buffer;
 	Buffer.Reset();
 	Buffer.AddInt(-TEEHISTORIAN_MESSAGE);
-	Buffer.AddInt(ClientID);
+	Buffer.AddInt(ClientId);
 	Buffer.AddInt(MsgSize);
 	Buffer.AddRaw(pMsg, MsgSize);
 
@@ -469,87 +498,120 @@ void CTeeHistorian::RecordPlayerMessage(int ClientID, const void *pMsg, int MsgS
 	{
 		CUnpacker Unpacker;
 		Unpacker.Reset(pMsg, MsgSize);
-		int MsgID = Unpacker.GetInt();
-		int Sys = MsgID & 1;
-		MsgID >>= 1;
-		dbg_msg("teehistorian", "msg cid=%d sys=%d msgid=%d", ClientID, Sys, MsgID);
+		int MsgId = Unpacker.GetInt();
+		int Sys = MsgId & 1;
+		MsgId >>= 1;
+		dbg_msg("teehistorian", "msg cid=%d sys=%d msgid=%d", ClientId, Sys, MsgId);
 	}
 
 	Write(Buffer.Data(), Buffer.Size());
 }
 
-void CTeeHistorian::RecordPlayerJoin(int ClientID, int Protocol)
+void CTeeHistorian::RecordPlayerJoin(int ClientId, int Protocol)
 {
 	dbg_assert(Protocol == PROTOCOL_6 || Protocol == PROTOCOL_7, "invalid version");
 	EnsureTickWritten();
 
 	{
-		CPacker Buffer;
+		CTeehistorianPacker Buffer;
 		Buffer.Reset();
-		Buffer.AddInt(ClientID);
+		Buffer.AddInt(ClientId);
 		if(m_Debug)
 		{
-			dbg_msg("teehistorian", "joinver%d cid=%d", Protocol == PROTOCOL_6 ? 6 : 7, ClientID);
+			dbg_msg("teehistorian", "joinver%d cid=%d", Protocol == PROTOCOL_6 ? 6 : 7, ClientId);
 		}
 		CUuid Uuid = Protocol == PROTOCOL_6 ? UUID_TEEHISTORIAN_JOINVER6 : UUID_TEEHISTORIAN_JOINVER7;
 		WriteExtra(Uuid, Buffer.Data(), Buffer.Size());
 	}
 
-	CPacker Buffer;
+	CTeehistorianPacker Buffer;
 	Buffer.Reset();
 	Buffer.AddInt(-TEEHISTORIAN_JOIN);
-	Buffer.AddInt(ClientID);
+	Buffer.AddInt(ClientId);
 
 	if(m_Debug)
 	{
-		dbg_msg("teehistorian", "join cid=%d", ClientID);
+		dbg_msg("teehistorian", "join cid=%d", ClientId);
 	}
 
 	Write(Buffer.Data(), Buffer.Size());
 }
 
-void CTeeHistorian::RecordPlayerReady(int ClientID)
+void CTeeHistorian::RecordPlayerRejoin(int ClientId)
 {
 	EnsureTickWritten();
 
-	CPacker Buffer;
+	CTeehistorianPacker Buffer;
 	Buffer.Reset();
-	Buffer.AddInt(ClientID);
+	Buffer.AddInt(ClientId);
 
 	if(m_Debug)
 	{
-		dbg_msg("teehistorian", "player_ready cid=%d", ClientID);
+		dbg_msg("teehistorian", "player_rejoin cid=%d", ClientId);
+	}
+
+	WriteExtra(UUID_TEEHISTORIAN_PLAYER_REJOIN, Buffer.Data(), Buffer.Size());
+}
+
+void CTeeHistorian::RecordPlayerReady(int ClientId)
+{
+	EnsureTickWritten();
+
+	CTeehistorianPacker Buffer;
+	Buffer.Reset();
+	Buffer.AddInt(ClientId);
+
+	if(m_Debug)
+	{
+		dbg_msg("teehistorian", "player_ready cid=%d", ClientId);
 	}
 
 	WriteExtra(UUID_TEEHISTORIAN_PLAYER_READY, Buffer.Data(), Buffer.Size());
 }
 
-void CTeeHistorian::RecordPlayerDrop(int ClientID, const char *pReason)
+void CTeeHistorian::RecordPlayerDrop(int ClientId, const char *pReason)
 {
 	EnsureTickWritten();
 
-	CPacker Buffer;
+	CTeehistorianPacker Buffer;
 	Buffer.Reset();
 	Buffer.AddInt(-TEEHISTORIAN_DROP);
-	Buffer.AddInt(ClientID);
+	Buffer.AddInt(ClientId);
 	Buffer.AddString(pReason, 0);
 
 	if(m_Debug)
 	{
-		dbg_msg("teehistorian", "drop cid=%d reason='%s'", ClientID, pReason);
+		dbg_msg("teehistorian", "drop cid=%d reason='%s'", ClientId, pReason);
 	}
 
 	Write(Buffer.Data(), Buffer.Size());
 }
 
-void CTeeHistorian::RecordConsoleCommand(int ClientID, int FlagMask, const char *pCmd, IConsole::IResult *pResult)
+void CTeeHistorian::RecordPlayerName(int ClientId, const char *pName)
 {
 	EnsureTickWritten();
 
-	CPacker Buffer;
+	CTeehistorianPacker Buffer;
+	Buffer.Reset();
+	Buffer.AddInt(ClientId);
+	Buffer.AddString(pName, 0);
+
+	if(m_Debug)
+	{
+		dbg_msg("teehistorian", "player_name cid=%d name='%s'", ClientId, pName);
+	}
+
+	WriteExtra(UUID_TEEHISTORIAN_PLAYER_NAME, Buffer.Data(), Buffer.Size());
+}
+
+void CTeeHistorian::RecordConsoleCommand(int ClientId, int FlagMask, const char *pCmd, IConsole::IResult *pResult)
+{
+	EnsureTickWritten();
+
+	CTeehistorianPacker Buffer;
 	Buffer.Reset();
 	Buffer.AddInt(-TEEHISTORIAN_CONSOLE_COMMAND);
-	Buffer.AddInt(ClientID);
+	Buffer.AddInt(ClientId);
 	Buffer.AddInt(FlagMask);
 	Buffer.AddString(pCmd, 0);
 	Buffer.AddInt(pResult->NumArguments());
@@ -560,7 +622,7 @@ void CTeeHistorian::RecordConsoleCommand(int ClientID, int FlagMask, const char
 
 	if(m_Debug)
 	{
-		dbg_msg("teehistorian", "ccmd cid=%d cmd='%s'", ClientID, pCmd);
+		dbg_msg("teehistorian", "ccmd cid=%d cmd='%s'", ClientId, pCmd);
 	}
 
 	Write(Buffer.Data(), Buffer.Size());
@@ -576,33 +638,33 @@ void CTeeHistorian::RecordTestExtra()
 	WriteExtra(UUID_TEEHISTORIAN_TEST, "", 0);
 }
 
-void CTeeHistorian::RecordPlayerSwap(int ClientID1, int ClientID2)
+void CTeeHistorian::RecordPlayerSwap(int ClientId1, int ClientId2)
 {
 	EnsureTickWritten();
 
-	CPacker Buffer;
+	CTeehistorianPacker Buffer;
 	Buffer.Reset();
-	Buffer.AddInt(ClientID1);
-	Buffer.AddInt(ClientID2);
+	Buffer.AddInt(ClientId1);
+	Buffer.AddInt(ClientId2);
 
 	WriteExtra(UUID_TEEHISTORIAN_PLAYER_SWITCH, Buffer.Data(), Buffer.Size());
 }
 
-void CTeeHistorian::RecordTeamSaveSuccess(int Team, CUuid SaveID, const char *pTeamSave)
+void CTeeHistorian::RecordTeamSaveSuccess(int Team, CUuid SaveId, const char *pTeamSave)
 {
 	EnsureTickWritten();
 
-	CPacker Buffer;
+	CTeehistorianPacker Buffer;
 	Buffer.Reset();
 	Buffer.AddInt(Team);
-	Buffer.AddRaw(&SaveID, sizeof(SaveID));
+	Buffer.AddRaw(&SaveId, sizeof(SaveId));
 	Buffer.AddString(pTeamSave, 0);
 
 	if(m_Debug)
 	{
-		char aSaveID[UUID_MAXSTRSIZE];
-		FormatUuid(SaveID, aSaveID, sizeof(aSaveID));
-		dbg_msg("teehistorian", "save_success team=%d save_id=%s team_save='%s'", Team, aSaveID, pTeamSave);
+		char aSaveId[UUID_MAXSTRSIZE];
+		FormatUuid(SaveId, aSaveId, sizeof(aSaveId));
+		dbg_msg("teehistorian", "save_success team=%d save_id=%s team_save='%s'", Team, aSaveId, pTeamSave);
 	}
 
 	WriteExtra(UUID_TEEHISTORIAN_SAVE_SUCCESS, Buffer.Data(), Buffer.Size());
@@ -612,7 +674,7 @@ void CTeeHistorian::RecordTeamSaveFailure(int Team)
 {
 	EnsureTickWritten();
 
-	CPacker Buffer;
+	CTeehistorianPacker Buffer;
 	Buffer.Reset();
 	Buffer.AddInt(Team);
 
@@ -624,21 +686,21 @@ void CTeeHistorian::RecordTeamSaveFailure(int Team)
 	WriteExtra(UUID_TEEHISTORIAN_SAVE_FAILURE, Buffer.Data(), Buffer.Size());
 }
 
-void CTeeHistorian::RecordTeamLoadSuccess(int Team, CUuid SaveID, const char *pTeamSave)
+void CTeeHistorian::RecordTeamLoadSuccess(int Team, CUuid SaveId, const char *pTeamSave)
 {
 	EnsureTickWritten();
 
-	CPacker Buffer;
+	CTeehistorianPacker Buffer;
 	Buffer.Reset();
 	Buffer.AddInt(Team);
-	Buffer.AddRaw(&SaveID, sizeof(SaveID));
+	Buffer.AddRaw(&SaveId, sizeof(SaveId));
 	Buffer.AddString(pTeamSave, 0);
 
 	if(m_Debug)
 	{
-		char aSaveID[UUID_MAXSTRSIZE];
-		FormatUuid(SaveID, aSaveID, sizeof(aSaveID));
-		dbg_msg("teehistorian", "load_success team=%d save_id=%s team_save='%s'", Team, aSaveID, pTeamSave);
+		char aSaveId[UUID_MAXSTRSIZE];
+		FormatUuid(SaveId, aSaveId, sizeof(aSaveId));
+		dbg_msg("teehistorian", "load_success team=%d save_id=%s team_save='%s'", Team, aSaveId, pTeamSave);
 	}
 
 	WriteExtra(UUID_TEEHISTORIAN_LOAD_SUCCESS, Buffer.Data(), Buffer.Size());
@@ -648,7 +710,7 @@ void CTeeHistorian::RecordTeamLoadFailure(int Team)
 {
 	EnsureTickWritten();
 
-	CPacker Buffer;
+	CTeehistorianPacker Buffer;
 	Buffer.Reset();
 	Buffer.AddInt(Team);
 
@@ -673,86 +735,124 @@ void CTeeHistorian::EndTick()
 	m_State = STATE_BEFORE_TICK;
 }
 
-void CTeeHistorian::RecordDDNetVersionOld(int ClientID, int DDNetVersion)
+void CTeeHistorian::RecordDDNetVersionOld(int ClientId, int DDNetVersion)
 {
-	CPacker Buffer;
+	CTeehistorianPacker Buffer;
 	Buffer.Reset();
-	Buffer.AddInt(ClientID);
+	Buffer.AddInt(ClientId);
 	Buffer.AddInt(DDNetVersion);
 
 	if(m_Debug)
 	{
-		dbg_msg("teehistorian", "ddnetver_old cid=%d ddnet_version=%d", ClientID, DDNetVersion);
+		dbg_msg("teehistorian", "ddnetver_old cid=%d ddnet_version=%d", ClientId, DDNetVersion);
 	}
 
 	WriteExtra(UUID_TEEHISTORIAN_DDNETVER_OLD, Buffer.Data(), Buffer.Size());
 }
 
-void CTeeHistorian::RecordDDNetVersion(int ClientID, CUuid ConnectionID, int DDNetVersion, const char *pDDNetVersionStr)
+void CTeeHistorian::RecordDDNetVersion(int ClientId, CUuid ConnectionId, int DDNetVersion, const char *pDDNetVersionStr)
 {
-	CPacker Buffer;
+	CTeehistorianPacker Buffer;
 	Buffer.Reset();
-	Buffer.AddInt(ClientID);
-	Buffer.AddRaw(&ConnectionID, sizeof(ConnectionID));
+	Buffer.AddInt(ClientId);
+	Buffer.AddRaw(&ConnectionId, sizeof(ConnectionId));
 	Buffer.AddInt(DDNetVersion);
 	Buffer.AddString(pDDNetVersionStr, 0);
 
 	if(m_Debug)
 	{
-		char aConnnectionID[UUID_MAXSTRSIZE];
-		FormatUuid(ConnectionID, aConnnectionID, sizeof(aConnnectionID));
-		dbg_msg("teehistorian", "ddnetver cid=%d connection_id=%s ddnet_version=%d ddnet_version_str=%s", ClientID, aConnnectionID, DDNetVersion, pDDNetVersionStr);
+		char aConnnectionId[UUID_MAXSTRSIZE];
+		FormatUuid(ConnectionId, aConnnectionId, sizeof(aConnnectionId));
+		dbg_msg("teehistorian", "ddnetver cid=%d connection_id=%s ddnet_version=%d ddnet_version_str=%s", ClientId, aConnnectionId, DDNetVersion, pDDNetVersionStr);
 	}
 
 	WriteExtra(UUID_TEEHISTORIAN_DDNETVER, Buffer.Data(), Buffer.Size());
 }
 
-void CTeeHistorian::RecordAuthInitial(int ClientID, int Level, const char *pAuthName)
+void CTeeHistorian::RecordAuthInitial(int ClientId, int Level, const char *pAuthName)
 {
-	CPacker Buffer;
+	CTeehistorianPacker Buffer;
 	Buffer.Reset();
-	Buffer.AddInt(ClientID);
+	Buffer.AddInt(ClientId);
 	Buffer.AddInt(Level);
 	Buffer.AddString(pAuthName, 0);
 
 	if(m_Debug)
 	{
-		dbg_msg("teehistorian", "auth_init cid=%d level=%d auth_name=%s", ClientID, Level, pAuthName);
+		dbg_msg("teehistorian", "auth_init cid=%d level=%d auth_name=%s", ClientId, Level, pAuthName);
 	}
 
 	WriteExtra(UUID_TEEHISTORIAN_AUTH_INIT, Buffer.Data(), Buffer.Size());
 }
 
-void CTeeHistorian::RecordAuthLogin(int ClientID, int Level, const char *pAuthName)
+void CTeeHistorian::RecordAuthLogin(int ClientId, int Level, const char *pAuthName)
 {
-	CPacker Buffer;
+	CTeehistorianPacker Buffer;
 	Buffer.Reset();
-	Buffer.AddInt(ClientID);
+	Buffer.AddInt(ClientId);
 	Buffer.AddInt(Level);
 	Buffer.AddString(pAuthName, 0);
 
 	if(m_Debug)
 	{
-		dbg_msg("teehistorian", "auth_login cid=%d level=%d auth_name=%s", ClientID, Level, pAuthName);
+		dbg_msg("teehistorian", "auth_login cid=%d level=%d auth_name=%s", ClientId, Level, pAuthName);
 	}
 
 	WriteExtra(UUID_TEEHISTORIAN_AUTH_LOGIN, Buffer.Data(), Buffer.Size());
 }
 
-void CTeeHistorian::RecordAuthLogout(int ClientID)
+void CTeeHistorian::RecordAuthLogout(int ClientId)
 {
-	CPacker Buffer;
+	CTeehistorianPacker Buffer;
 	Buffer.Reset();
-	Buffer.AddInt(ClientID);
+	Buffer.AddInt(ClientId);
 
 	if(m_Debug)
 	{
-		dbg_msg("teehistorian", "auth_logout cid=%d", ClientID);
+		dbg_msg("teehistorian", "auth_logout cid=%d", ClientId);
 	}
 
 	WriteExtra(UUID_TEEHISTORIAN_AUTH_LOGOUT, Buffer.Data(), Buffer.Size());
 }
 
+void CTeeHistorian::RecordAntibot(const void *pData, int DataSize)
+{
+	if(m_Debug)
+	{
+		dbg_msg("teehistorian", "antibot data_size=%d", DataSize);
+	}
+
+	WriteExtra(UUID_TEEHISTORIAN_ANTIBOT, pData, DataSize);
+}
+
+void CTeeHistorian::RecordPlayerFinish(int ClientId, int TimeTicks)
+{
+	CTeehistorianPacker Buffer;
+	Buffer.Reset();
+	Buffer.AddInt(ClientId);
+	Buffer.AddInt(TimeTicks);
+	if(m_Debug)
+	{
+		dbg_msg("teehistorian", "player_finish cid=%d time=%d", ClientId, TimeTicks);
+	}
+
+	WriteExtra(UUID_TEEHISTORIAN_PLAYER_FINISH, Buffer.Data(), Buffer.Size());
+}
+
+void CTeeHistorian::RecordTeamFinish(int TeamId, int TimeTicks)
+{
+	CTeehistorianPacker Buffer;
+	Buffer.Reset();
+	Buffer.AddInt(TeamId);
+	Buffer.AddInt(TimeTicks);
+	if(m_Debug)
+	{
+		dbg_msg("teehistorian", "team_finish cid=%d time=%d", TeamId, TimeTicks);
+	}
+
+	WriteExtra(UUID_TEEHISTORIAN_TEAM_FINISH, Buffer.Data(), Buffer.Size());
+}
+
 void CTeeHistorian::Finish()
 {
 	dbg_assert(m_State == STATE_START || m_State == STATE_INPUTS || m_State == STATE_BEFORE_ENDTICK || m_State == STATE_BEFORE_TICK, "invalid teehistorian state");
@@ -766,7 +866,7 @@ void CTeeHistorian::Finish()
 		EndTick();
 	}
 
-	CPacker Buffer;
+	CTeehistorianPacker Buffer;
 	Buffer.Reset();
 	Buffer.AddInt(-TEEHISTORIAN_FINISH);
 
