@@ -7,8 +7,8 @@
 enum
 {
 	TEAM_FLOCK = 0,
-	TEAM_SUPER = MAX_CLIENTS,
-	NUM_TEAMS = TEAM_SUPER + 1,
+	TEAM_SUPER = 64,
+	NUM_DDRACE_TEAMS = TEAM_SUPER + 1,
 	VANILLA_TEAM_SUPER = VANILLA_MAX_CLIENTS
 };
 
@@ -31,27 +31,17 @@ public:
 
 	CTeamsCore();
 
-	bool SameTeam(int ClientID1, int ClientID2) const;
+	bool SameTeam(int ClientId1, int ClientId2) const;
 
-	bool CanKeepHook(int ClientID1, int ClientID2) const;
-	bool CanCollide(int ClientID1, int ClientID2) const;
+	bool CanKeepHook(int ClientId1, int ClientId2) const;
+	bool CanCollide(int ClientId1, int ClientId2) const;
 
-	int Team(int ClientID) const;
-	void Team(int ClientID, int Team);
+	int Team(int ClientId) const;
+	void Team(int ClientId, int Team);
 
 	void Reset();
-	void SetSolo(int ClientID, bool Value)
-	{
-		dbg_assert(ClientID >= 0 && ClientID < MAX_CLIENTS, "Invalid client id");
-		m_aIsSolo[ClientID] = Value;
-	}
-
-	bool GetSolo(int ClientID) const
-	{
-		if(ClientID < 0 || ClientID >= MAX_CLIENTS)
-			return false;
-		return m_aIsSolo[ClientID];
-	}
+	void SetSolo(int ClientId, bool Value);
+	bool GetSolo(int ClientId) const;
 };
 
 #endif
