@@ -1,6 +1,7 @@
 #include "test.h"
 #include <gtest/gtest.h>
 
+#include <base/system.h>
 #include <engine/shared/csv.h>
 
 static void Expect(int NumColumns, const char *const *ppColumns, const char *pExpected)
@@ -48,7 +49,7 @@ TEST(Csv, Simple)
 	Expect(3, apCols3, "я,,й");
 	const char *apCols4[] = {""};
 	Expect(1, apCols4, "");
-	const char *apCols5[] = {0};
+	const char *apCols5[] = {nullptr};
 	Expect(0, apCols5, "");
 }
 
