@@ -3,6 +3,7 @@
 
 #include <base/detect.h>
 #include <engine/server/databases/connection.h>
+#include <engine/server/databases/connection_pool.h>
 #include <engine/shared/config.h>
 #include <game/server/scoreworker.h>
 
@@ -12,24 +13,6 @@
 int DummyMysqlInit = (MysqlInit(), 1);
 #endif
 
-char *CSaveTeam::GetString()
-{
-	// Dummy implementation for testing
-	return nullptr;
-}
-
-int CSaveTeam::FromString(char const *)
-{
-	// Dummy implementation for testing
-	return 1;
-}
-
-bool CSaveTeam::MatchPlayers(const char (*paNames)[MAX_NAME_LENGTH], const int *pClientID, int NumPlayer, char *pMessage, int MessageLen)
-{
-	// Dummy implementation for testing
-	return false;
-}
-
 TEST(SQLite, Version)
 {
 	ASSERT_GE(sqlite3_libversion_number(), 3025000) << "SQLite >= 3.25.0 required for Window functions";
@@ -40,8 +23,8 @@ struct Score : public testing::TestWithParam<IDbConnection *>
 	Score()
 	{
 		Connect();
-		Init();
-		InsertMap();
+		LoadBestTime();
+		InsertMap("Kobra 3", "Zerodin", "Novice", 5, 5);
 	}
 
 	~Score()
@@ -67,22 +50,22 @@ struct Score : public testing::TestWithParam<IDbConnection *>
 		ASSERT_FALSE(m_pConn->ExecuteUpdate(&NumInserted, m_aError, sizeof(m_aError))) << m_aError;
 	}
 
-	void Init()
+	void LoadBestTime()
 	{
-		CSqlInitData initData(std::make_shared<CScoreInitResult>());
-		str_copy(initData.m_aMap, "Kobra 3", sizeof(initData.m_aMap));
-		ASSERT_FALSE(CScoreWorker::Init(m_pConn, &initData, m_aError, sizeof(m_aError))) << m_aError;
+		CSqlLoadBestTimeRequest loadBestTimeReq(std::make_shared<CScoreLoadBestTimeResult>());
+		str_copy(loadBestTimeReq.m_aMap, "Kobra 3", sizeof(loadBestTimeReq.m_aMap));
+		ASSERT_FALSE(CScoreWorker::LoadBestTime(m_pConn, &loadBestTimeReq, m_aError, sizeof(m_aError))) << m_aError;
 	}
 
-	void InsertMap()
+	void InsertMap(const char *pName, const char *pMapper, const char *pServer, int Points, int Stars)
 	{
 		char aTimestamp[32];
 		str_timestamp_format(aTimestamp, sizeof(aTimestamp), FORMAT_SPACE);
 		char aBuf[512];
 		str_format(aBuf, sizeof(aBuf),
 			"%s into %s_maps(Map, Server, Mapper, Points, Stars, Timestamp) "
-			"VALUES (\"Kobra 3\", \"Novice\", \"Zerodin\", 5, 5, %s)",
-			m_pConn->InsertIgnore(), m_pConn->GetPrefix(), m_pConn->InsertTimestampAsUtc());
+			"VALUES (\"%s\", \"%s\", \"%s\", %d, %d, %s)",
+			m_pConn->InsertIgnore(), m_pConn->GetPrefix(), pName, pServer, pMapper, Points, Stars, m_pConn->InsertTimestampAsUtc());
 		ASSERT_FALSE(m_pConn->PrepareStatement(aBuf, m_aError, sizeof(m_aError))) << m_aError;
 		m_pConn->BindString(1, aTimestamp);
 		int NumInserted = 0;
@@ -97,13 +80,13 @@ struct Score : public testing::TestWithParam<IDbConnection *>
 		str_copy(ScoreData.m_aMap, "Kobra 3", sizeof(ScoreData.m_aMap));
 		str_copy(ScoreData.m_aGameUuid, "8d300ecf-5873-4297-bee5-95668fdff320", sizeof(ScoreData.m_aGameUuid));
 		str_copy(ScoreData.m_aName, "nameless tee", sizeof(ScoreData.m_aName));
-		ScoreData.m_ClientID = 0;
+		ScoreData.m_ClientId = 0;
 		ScoreData.m_Time = Time;
 		str_copy(ScoreData.m_aTimestamp, "2021-11-24 19:24:08", sizeof(ScoreData.m_aTimestamp));
 		for(int i = 0; i < NUM_CHECKPOINTS; i++)
 			ScoreData.m_aCurrentTimeCp[i] = WithTimeCheckPoints ? i : 0;
 		str_copy(ScoreData.m_aRequestingPlayer, "deen", sizeof(ScoreData.m_aRequestingPlayer));
-		ASSERT_FALSE(CScoreWorker::SaveScore(m_pConn, &ScoreData, false, m_aError, sizeof(m_aError))) << m_aError;
+		ASSERT_FALSE(CScoreWorker::SaveScore(m_pConn, &ScoreData, Write::NORMAL, m_aError, sizeof(m_aError))) << m_aError;
 	}
 
 	void ExpectLines(const std::shared_ptr<CScorePlayerResult> &pPlayerResult, std::initializer_list<const char *> Lines, bool All = false)
@@ -142,8 +125,9 @@ struct SingleScore : public Score
 	}
 };
 
-TEST_P(SingleScore, Top)
+TEST_P(SingleScore, TopRegional)
 {
+	g_Config.m_SvRegionalRankings = true;
 	ASSERT_FALSE(CScoreWorker::ShowTop(m_pConn, &m_PlayerRequest, m_aError, sizeof(m_aError))) << m_aError;
 	ExpectLines(m_pPlayerResult,
 		{"------------ Global Top ------------",
@@ -151,14 +135,33 @@ TEST_P(SingleScore, Top)
 			"------------ GER Top ------------"});
 }
 
-TEST_P(SingleScore, Rank)
+TEST_P(SingleScore, Top)
 {
+	g_Config.m_SvRegionalRankings = false;
+	ASSERT_FALSE(CScoreWorker::ShowTop(m_pConn, &m_PlayerRequest, m_aError, sizeof(m_aError))) << m_aError;
+	ExpectLines(m_pPlayerResult,
+		{"------------ Global Top ------------",
+			"1. nameless tee Time: 01:40.00",
+			"-----------------------------------------"});
+}
+
+TEST_P(SingleScore, RankRegional)
+{
+	g_Config.m_SvRegionalRankings = true;
 	ASSERT_FALSE(CScoreWorker::ShowRank(m_pConn, &m_PlayerRequest, m_aError, sizeof(m_aError))) << m_aError;
 	ExpectLines(m_pPlayerResult, {"nameless tee - 01:40.00 - better than 100% - requested by brainless tee", "Global rank 1 - GER unranked"}, true);
 }
 
-TEST_P(SingleScore, TopServer)
+TEST_P(SingleScore, Rank)
 {
+	g_Config.m_SvRegionalRankings = false;
+	ASSERT_FALSE(CScoreWorker::ShowRank(m_pConn, &m_PlayerRequest, m_aError, sizeof(m_aError))) << m_aError;
+	ExpectLines(m_pPlayerResult, {"nameless tee - 01:40.00 - better than 100% - requested by brainless tee", "Global rank 1"}, true);
+}
+
+TEST_P(SingleScore, TopServerRegional)
+{
+	g_Config.m_SvRegionalRankings = true;
 	str_copy(m_PlayerRequest.m_aServer, "USA", sizeof(m_PlayerRequest.m_aServer));
 	ASSERT_FALSE(CScoreWorker::ShowTop(m_pConn, &m_PlayerRequest, m_aError, sizeof(m_aError))) << m_aError;
 	ExpectLines(m_pPlayerResult,
@@ -168,13 +171,33 @@ TEST_P(SingleScore, TopServer)
 			"1. nameless tee Time: 01:40.00"});
 }
 
-TEST_P(SingleScore, RankServer)
+TEST_P(SingleScore, TopServer)
 {
+	g_Config.m_SvRegionalRankings = false;
+	str_copy(m_PlayerRequest.m_aServer, "USA", sizeof(m_PlayerRequest.m_aServer));
+	ASSERT_FALSE(CScoreWorker::ShowTop(m_pConn, &m_PlayerRequest, m_aError, sizeof(m_aError))) << m_aError;
+	ExpectLines(m_pPlayerResult,
+		{"------------ Global Top ------------",
+			"1. nameless tee Time: 01:40.00",
+			"-----------------------------------------"});
+}
+
+TEST_P(SingleScore, RankServerRegional)
+{
+	g_Config.m_SvRegionalRankings = true;
 	str_copy(m_PlayerRequest.m_aServer, "USA", sizeof(m_PlayerRequest.m_aServer));
 	ASSERT_FALSE(CScoreWorker::ShowRank(m_pConn, &m_PlayerRequest, m_aError, sizeof(m_aError))) << m_aError;
 	ExpectLines(m_pPlayerResult, {"nameless tee - 01:40.00 - better than 100% - requested by brainless tee", "Global rank 1 - USA rank 1"}, true);
 }
 
+TEST_P(SingleScore, RankServer)
+{
+	g_Config.m_SvRegionalRankings = false;
+	str_copy(m_PlayerRequest.m_aServer, "USA", sizeof(m_PlayerRequest.m_aServer));
+	ASSERT_FALSE(CScoreWorker::ShowRank(m_pConn, &m_PlayerRequest, m_aError, sizeof(m_aError))) << m_aError;
+	ExpectLines(m_pPlayerResult, {"nameless tee - 01:40.00 - better than 100% - requested by brainless tee", "Global rank 1"}, true);
+}
+
 TEST_P(SingleScore, LoadPlayerData)
 {
 	InsertRank(120.0, true);
@@ -182,7 +205,7 @@ TEST_P(SingleScore, LoadPlayerData)
 	ASSERT_FALSE(CScoreWorker::LoadPlayerData(m_pConn, &m_PlayerRequest, m_aError, sizeof(m_aError))) << m_aError;
 
 	EXPECT_EQ(m_pPlayerResult->m_MessageKind, CScorePlayerResult::PLAYER_INFO);
-	ASSERT_EQ(m_pPlayerResult->m_Data.m_Info.m_Time, 0.0);
+	ASSERT_FALSE(m_pPlayerResult->m_Data.m_Info.m_Time.has_value());
 	for(auto &Time : m_pPlayerResult->m_Data.m_Info.m_aTimeCp)
 	{
 		ASSERT_EQ(Time, 0);
@@ -193,7 +216,8 @@ TEST_P(SingleScore, LoadPlayerData)
 	ASSERT_FALSE(CScoreWorker::LoadPlayerData(m_pConn, &m_PlayerRequest, m_aError, sizeof(m_aError))) << m_aError;
 
 	EXPECT_EQ(m_pPlayerResult->m_MessageKind, CScorePlayerResult::PLAYER_INFO);
-	ASSERT_EQ(m_pPlayerResult->m_Data.m_Info.m_Time, 100.0);
+	ASSERT_TRUE(m_pPlayerResult->m_Data.m_Info.m_Time.has_value());
+	ASSERT_EQ(*m_pPlayerResult->m_Data.m_Info.m_Time, 100.0);
 	for(int i = 0; i < NUM_CHECKPOINTS; i++)
 	{
 		ASSERT_EQ(m_pPlayerResult->m_Data.m_Info.m_aTimeCp[i], i);
@@ -204,7 +228,7 @@ TEST_P(SingleScore, LoadPlayerData)
 	ASSERT_FALSE(CScoreWorker::LoadPlayerData(m_pConn, &m_PlayerRequest, m_aError, sizeof(m_aError))) << m_aError;
 
 	EXPECT_EQ(m_pPlayerResult->m_MessageKind, CScorePlayerResult::PLAYER_INFO);
-	ASSERT_EQ(m_pPlayerResult->m_Data.m_Info.m_Time, 0.0);
+	ASSERT_FALSE(m_pPlayerResult->m_Data.m_Info.m_Time.has_value());
 	for(int i = 0; i < NUM_CHECKPOINTS; i++)
 	{
 		ASSERT_EQ(m_pPlayerResult->m_Data.m_Info.m_aTimeCp[i], i);
@@ -222,7 +246,7 @@ TEST_P(SingleScore, TimesExists)
 
 	str_copy(aBuf, m_pPlayerResult->m_Data.m_aaMessages[1] + str_length(m_pPlayerResult->m_Data.m_aaMessages[1]) - 10, 11);
 	EXPECT_STREQ(aBuf, ", 01:40.00");
-	EXPECT_STREQ(m_pPlayerResult->m_Data.m_aaMessages[2], "----------------------------------------------------");
+	EXPECT_STREQ(m_pPlayerResult->m_Data.m_aaMessages[2], "-------------------------------------------");
 	for(int i = 3; i < CScorePlayerResult::MAX_MESSAGES; i++)
 	{
 		EXPECT_STREQ(m_pPlayerResult->m_Data.m_aaMessages[i], "");
@@ -240,24 +264,46 @@ struct TeamScore : public Score
 {
 	void SetUp() override
 	{
+		InsertTeamRank(100.0);
+	}
+
+	void InsertTeamRank(float Time = 100.0)
+	{
+		str_copy(g_Config.m_SvSqlServerName, "USA", sizeof(g_Config.m_SvSqlServerName));
 		CSqlTeamScoreData teamScoreData;
+		CSqlScoreData ScoreData(std::make_shared<CScorePlayerResult>());
 		str_copy(teamScoreData.m_aMap, "Kobra 3", sizeof(teamScoreData.m_aMap));
+		str_copy(ScoreData.m_aMap, "Kobra 3", sizeof(ScoreData.m_aMap));
 		str_copy(teamScoreData.m_aGameUuid, "8d300ecf-5873-4297-bee5-95668fdff320", sizeof(teamScoreData.m_aGameUuid));
+		str_copy(ScoreData.m_aGameUuid, "8d300ecf-5873-4297-bee5-95668fdff320", sizeof(ScoreData.m_aGameUuid));
 		teamScoreData.m_Size = 2;
 		str_copy(teamScoreData.m_aaNames[0], "nameless tee", sizeof(teamScoreData.m_aaNames[0]));
 		str_copy(teamScoreData.m_aaNames[1], "brainless tee", sizeof(teamScoreData.m_aaNames[1]));
-		teamScoreData.m_Time = 100.0;
+		teamScoreData.m_Time = Time;
+		ScoreData.m_Time = Time;
 		str_copy(teamScoreData.m_aTimestamp, "2021-11-24 19:24:08", sizeof(teamScoreData.m_aTimestamp));
-		ASSERT_FALSE(CScoreWorker::SaveTeamScore(m_pConn, &teamScoreData, false, m_aError, sizeof(m_aError))) << m_aError;
+		str_copy(ScoreData.m_aTimestamp, "2021-11-24 19:24:08", sizeof(ScoreData.m_aTimestamp));
+		for(int i = 0; i < NUM_CHECKPOINTS; i++)
+			ScoreData.m_aCurrentTimeCp[i] = 0;
+		ASSERT_FALSE(CScoreWorker::SaveTeamScore(m_pConn, &teamScoreData, Write::NORMAL, m_aError, sizeof(m_aError))) << m_aError;
 
 		str_copy(m_PlayerRequest.m_aMap, "Kobra 3", sizeof(m_PlayerRequest.m_aMap));
 		str_copy(m_PlayerRequest.m_aRequestingPlayer, "brainless tee", sizeof(m_PlayerRequest.m_aRequestingPlayer));
+		str_copy(ScoreData.m_aRequestingPlayer, "brainless tee", sizeof(ScoreData.m_aRequestingPlayer));
+
+		str_copy(ScoreData.m_aName, "nameless tee", sizeof(ScoreData.m_aName));
+		ScoreData.m_ClientId = 0;
+		ASSERT_FALSE(CScoreWorker::SaveScore(m_pConn, &ScoreData, Write::NORMAL, m_aError, sizeof(m_aError))) << m_aError;
+		str_copy(ScoreData.m_aName, "brainless tee", sizeof(ScoreData.m_aName));
+		ScoreData.m_ClientId = 1;
+		ASSERT_FALSE(CScoreWorker::SaveScore(m_pConn, &ScoreData, Write::NORMAL, m_aError, sizeof(m_aError))) << m_aError;
 		m_PlayerRequest.m_Offset = 0;
 	}
 };
 
 TEST_P(TeamScore, All)
 {
+	g_Config.m_SvRegionalRankings = false;
 	ASSERT_FALSE(CScoreWorker::ShowTeamTop5(m_pConn, &m_PlayerRequest, m_aError, sizeof(m_aError))) << m_aError;
 	ExpectLines(m_pPlayerResult,
 		{"------- Team Top 5 -------",
@@ -265,23 +311,46 @@ TEST_P(TeamScore, All)
 			"-------------------------------"});
 }
 
+TEST_P(TeamScore, TeamTop5Regional)
+{
+	g_Config.m_SvRegionalRankings = true;
+	str_copy(m_PlayerRequest.m_aServer, "USA", sizeof(m_PlayerRequest.m_aServer));
+	ASSERT_FALSE(CScoreWorker::ShowTeamTop5(m_pConn, &m_PlayerRequest, m_aError, sizeof(m_aError))) << m_aError;
+	ExpectLines(m_pPlayerResult,
+		{"------- Team Top 5 -------",
+			"1. brainless tee & nameless tee Team Time: 01:40.00",
+			"----- USA Team Top -----",
+			"1. brainless tee & nameless tee Team Time: 01:40.00"});
+}
+
 TEST_P(TeamScore, PlayerExists)
 {
-	str_copy(m_PlayerRequest.m_aName, "brainless tee", sizeof(m_PlayerRequest.m_aMap));
+	str_copy(m_PlayerRequest.m_aName, "brainless tee", sizeof(m_PlayerRequest.m_aName));
 	ASSERT_FALSE(CScoreWorker::ShowPlayerTeamTop5(m_pConn, &m_PlayerRequest, m_aError, sizeof(m_aError))) << m_aError;
 	ExpectLines(m_pPlayerResult,
 		{"------- Team Top 5 -------",
 			"1. brainless tee & nameless tee Team Time: 01:40.00",
-			"-------------------------------"});
+			"---------------------------------"});
 }
 
 TEST_P(TeamScore, PlayerDoesntExist)
 {
-	str_copy(m_PlayerRequest.m_aName, "foo", sizeof(m_PlayerRequest.m_aMap));
+	str_copy(m_PlayerRequest.m_aName, "foo", sizeof(m_PlayerRequest.m_aName));
 	ASSERT_FALSE(CScoreWorker::ShowPlayerTeamTop5(m_pConn, &m_PlayerRequest, m_aError, sizeof(m_aError))) << m_aError;
 	ExpectLines(m_pPlayerResult, {"foo has no team ranks"});
 }
 
+TEST_P(TeamScore, RankUpdates)
+{
+	InsertTeamRank(98.0);
+	str_copy(m_PlayerRequest.m_aName, "brainless tee", sizeof(m_PlayerRequest.m_aName));
+	ASSERT_FALSE(CScoreWorker::ShowPlayerTeamTop5(m_pConn, &m_PlayerRequest, m_aError, sizeof(m_aError))) << m_aError;
+	ExpectLines(m_pPlayerResult,
+		{"------- Team Top 5 -------",
+			"1. brainless tee & nameless tee Team Time: 01:38.00",
+			"---------------------------------"});
+}
+
 struct MapInfo : public Score
 {
 	MapInfo()
@@ -331,6 +400,21 @@ TEST_P(MapInfo, Fuzzy)
 	}
 }
 
+TEST_P(MapInfo, FuzzyCase)
+{
+	InsertMap("Reflect", "DarkOort", "Dummy", 20, 3);
+	InsertMap("reflects", "Ninjed & Pipou", "Solo", 16, 4);
+	str_copy(m_PlayerRequest.m_aName, "reflect", sizeof(m_PlayerRequest.m_aName));
+	ASSERT_FALSE(CScoreWorker::MapInfo(m_pConn, &m_PlayerRequest, m_aError, sizeof(m_aError))) << m_aError;
+
+	EXPECT_EQ(m_pPlayerResult->m_MessageKind, CScorePlayerResult::DIRECT);
+	EXPECT_THAT(m_pPlayerResult->m_Data.m_aaMessages[0], testing::MatchesRegex("\"Reflect\" by DarkOort on Dummy, ★★★✰✰, 20 points, released .* ago, 0 finishes by 0 tees"));
+	for(int i = 1; i < CScorePlayerResult::MAX_MESSAGES; i++)
+	{
+		EXPECT_STREQ(m_pPlayerResult->m_Data.m_aaMessages[i], "");
+	}
+}
+
 TEST_P(MapInfo, DoesntExit)
 {
 	str_copy(m_PlayerRequest.m_aName, "f", sizeof(m_PlayerRequest.m_aName));
@@ -366,6 +450,18 @@ TEST_P(MapVote, Fuzzy)
 	EXPECT_STREQ(m_pPlayerResult->m_Data.m_MapVote.m_aServer, "novice");
 }
 
+TEST_P(MapVote, FuzzyCase)
+{
+	InsertMap("Reflect", "DarkOort", "Dummy", 20, 3);
+	InsertMap("reflects", "Ninjed & Pipou", "Solo", 16, 4);
+	str_copy(m_PlayerRequest.m_aName, "reflect", sizeof(m_PlayerRequest.m_aName));
+	ASSERT_FALSE(CScoreWorker::MapVote(m_pConn, &m_PlayerRequest, m_aError, sizeof(m_aError))) << m_aError;
+	EXPECT_EQ(m_pPlayerResult->m_MessageKind, CScorePlayerResult::MAP_VOTE);
+	EXPECT_STREQ(m_pPlayerResult->m_Data.m_MapVote.m_aMap, "Reflect");
+	EXPECT_STREQ(m_pPlayerResult->m_Data.m_MapVote.m_aReason, "/map");
+	EXPECT_STREQ(m_pPlayerResult->m_Data.m_MapVote.m_aServer, "dummy");
+}
+
 TEST_P(MapVote, DoesntExist)
 {
 	str_copy(m_PlayerRequest.m_aName, "f", sizeof(m_PlayerRequest.m_aName));
@@ -472,7 +568,7 @@ TEST_P(RandomMap, NoStars)
 {
 	m_RandomMapRequest.m_Stars = -1;
 	ASSERT_FALSE(CScoreWorker::RandomMap(m_pConn, &m_RandomMapRequest, m_aError, sizeof(m_aError))) << m_aError;
-	EXPECT_EQ(m_pRandomMapResult->m_ClientID, 0);
+	EXPECT_EQ(m_pRandomMapResult->m_ClientId, 0);
 	EXPECT_STREQ(m_pRandomMapResult->m_aMap, "Kobra 3");
 	EXPECT_STREQ(m_pRandomMapResult->m_aMessage, "");
 }
@@ -481,7 +577,7 @@ TEST_P(RandomMap, StarsExists)
 {
 	m_RandomMapRequest.m_Stars = 5;
 	ASSERT_FALSE(CScoreWorker::RandomMap(m_pConn, &m_RandomMapRequest, m_aError, sizeof(m_aError))) << m_aError;
-	EXPECT_EQ(m_pRandomMapResult->m_ClientID, 0);
+	EXPECT_EQ(m_pRandomMapResult->m_ClientId, 0);
 	EXPECT_STREQ(m_pRandomMapResult->m_aMap, "Kobra 3");
 	EXPECT_STREQ(m_pRandomMapResult->m_aMessage, "");
 }
@@ -490,7 +586,7 @@ TEST_P(RandomMap, StarsDoesntExist)
 {
 	m_RandomMapRequest.m_Stars = 3;
 	ASSERT_FALSE(CScoreWorker::RandomMap(m_pConn, &m_RandomMapRequest, m_aError, sizeof(m_aError))) << m_aError;
-	EXPECT_EQ(m_pRandomMapResult->m_ClientID, 0);
+	EXPECT_EQ(m_pRandomMapResult->m_ClientId, 0);
 	EXPECT_STREQ(m_pRandomMapResult->m_aMap, "");
 	EXPECT_STREQ(m_pRandomMapResult->m_aMessage, "No maps found on this server!");
 }
@@ -499,7 +595,7 @@ TEST_P(RandomMap, UnfinishedExists)
 {
 	m_RandomMapRequest.m_Stars = -1;
 	ASSERT_FALSE(CScoreWorker::RandomUnfinishedMap(m_pConn, &m_RandomMapRequest, m_aError, sizeof(m_aError))) << m_aError;
-	EXPECT_EQ(m_pRandomMapResult->m_ClientID, 0);
+	EXPECT_EQ(m_pRandomMapResult->m_ClientId, 0);
 	EXPECT_STREQ(m_pRandomMapResult->m_aMap, "Kobra 3");
 	EXPECT_STREQ(m_pRandomMapResult->m_aMessage, "");
 }
@@ -508,14 +604,24 @@ TEST_P(RandomMap, UnfinishedDoesntExist)
 {
 	InsertRank();
 	ASSERT_FALSE(CScoreWorker::RandomUnfinishedMap(m_pConn, &m_RandomMapRequest, m_aError, sizeof(m_aError))) << m_aError;
-	EXPECT_EQ(m_pRandomMapResult->m_ClientID, 0);
+	EXPECT_EQ(m_pRandomMapResult->m_ClientId, 0);
 	EXPECT_STREQ(m_pRandomMapResult->m_aMap, "");
-	EXPECT_STREQ(m_pRandomMapResult->m_aMessage, "You have no more unfinished maps on this server!");
+	EXPECT_STREQ(m_pRandomMapResult->m_aMessage, "nameless tee has no more unfinished maps on this server!");
 }
 
 auto g_pSqliteConn = CreateSqliteConnection(":memory:", true);
 #if defined(CONF_TEST_MYSQL)
-auto g_pMysqlConn = CreateMysqlConnection("ddnet", "record", "ddnet", "thebestpassword", "localhost", "", 3306, true);
+CMysqlConfig gMysqlConfig{
+	"ddnet", // database
+	"record", // prefix
+	"ddnet", // user
+	"thebestpassword", // password
+	"localhost", // ip
+	"", // bindaddr
+	3306, // port
+	true, // setup
+};
+auto g_pMysqlConn = CreateMysqlConnection(gMysqlConfig);
 #endif
 
 auto g_TestValues
