@@ -2,7 +2,6 @@
 #include <base/system.h>
 #include <engine/shared/datafile.h>
 #include <engine/storage.h>
-#include <game/gamecore.h>
 #include <game/mapitems.h>
 
 // global new layers data (set by ReplaceAreaTiles and ReplaceAreaQuads)
@@ -10,8 +9,9 @@ void *g_apNewData[1024];
 void *g_apNewItem[1024];
 int g_aNewDataSize[1024];
 
-struct MapObject // quad pivot or tile layer
+class CMapObject // quad pivot or tile layer
 {
+public:
 	static constexpr float ms_aStandardScreen[2] = {1430 / 2.f, 1050 / 2.f};
 
 	float m_aLayerOffset[2];
@@ -33,25 +33,25 @@ const CMapItemGroup *GetLayerGroup(CDataFileReader &, int);
 void ReplaceAreaTiles(CDataFileReader[2], const float[][2][2], const CMapItemGroup *[2], CMapItemLayer *[2]);
 void RemoveDestinationTiles(CMapItemLayerTilemap *, CTile *, float[2][2]);
 void ReplaceDestinationTiles(CMapItemLayerTilemap *[2], CTile *[2], float[2][2][2]);
-bool AdaptVisibleAreas(const float[2][2][2], const MapObject[2], float[2][2][2]);
-bool AdaptReplaceableAreas(const float[2][2][2], const float[2][2][2], const MapObject[2], float[2][2][2]);
+bool AdaptVisibleAreas(const float[2][2][2], const CMapObject[2], float[2][2][2]);
+bool AdaptReplaceableAreas(const float[2][2][2], const float[2][2][2], const CMapObject[2], float[2][2][2]);
 
 void ReplaceAreaQuads(CDataFileReader[2], const float[][2][2], const CMapItemGroup *[2], CMapItemLayer *[2], int);
 bool RemoveDestinationQuads(const float[2][2], const CQuad *, int, const CMapItemGroup *, CQuad *, int &);
 bool InsertDestinationQuads(const float[2][2][2], const CQuad *, int, const CMapItemGroup *[2], CQuad *, int &);
-bool AdaptVisiblePoint(const float[2][2][2], const float[2][2], const MapObject[2], float[2]);
+bool AdaptVisiblePoint(const float[2][2][2], const float[2][2], const CMapObject[2], float[2]);
 
-MapObject CreateMapObject(const CMapItemGroup *, int, int, int, int);
-void SetExtendedArea(MapObject &);
-bool GetVisibleArea(const float[2][2], MapObject, float[2][2] = 0x0);
-bool GetReplaceableArea(const float[2][2], MapObject, float[2][2]);
+CMapObject CreateMapObject(const CMapItemGroup *, int, int, int, int);
+void SetExtendedArea(CMapObject &);
+bool GetVisibleArea(const float[2][2], const CMapObject &, float[2][2] = nullptr);
+bool GetReplaceableArea(const float[2][2], const CMapObject &, float[2][2]);
 
-void GetGameAreaDistance(const float[2][2][2], const MapObject[2], const float[2][2][2], float[2]);
-void GetGameAreaDistance(const float[2][2][2], const MapObject[2], const float[2][2], float[2]);
-void GetSignificantScreenPos(MapObject, const float[2][2], const float[2][2], float[2]);
+void GetGameAreaDistance(const float[2][2][2], const CMapObject[2], const float[2][2][2], float[2]);
+void GetGameAreaDistance(const float[2][2][2], const CMapObject[2], const float[2][2], float[2]);
+void GetSignificantScreenPos(const CMapObject &, const float[2][2], const float[2][2], float[2]);
 void ConvertToTiles(const float[2][2], int[2][2]);
 
-bool GetLineIntersection(const float[2], const float[2], float[2] = 0x0);
+bool GetLineIntersection(const float[2], const float[2], float[2] = nullptr);
 bool GetLineIntersection(const float[2], float);
 void SetInexistent(float *, int);
 bool IsInexistent(const float *, int);
@@ -72,18 +72,18 @@ int main(int argc, const char *argv[])
 	}
 
 	char aaMapNames[3][64];
-	snprintf(aaMapNames[0], 64, "%s", argv[1]); //from_map
-	snprintf(aaMapNames[1], 64, "%s", argv[4]); //to_map
-	snprintf(aaMapNames[2], 64, "%s", argv[9]); //output_map
+	str_copy(aaMapNames[0], argv[1]); //from_map
+	str_copy(aaMapNames[1], argv[4]); //to_map
+	str_copy(aaMapNames[2], argv[9]); //output_map
 
 	float aaaGameAreas[2][2][2];
 
 	for(int i = 0; i < 2; i++)
 	{
-		aaaGameAreas[i][0][0] = atof(argv[2 + i * 3]) * 32; //x
-		aaaGameAreas[i][1][0] = atof(argv[3 + i * 3]) * 32; //y
-		aaaGameAreas[i][0][1] = aaaGameAreas[i][0][0] + atof(argv[7]) * 32; //x + width
-		aaaGameAreas[i][1][1] = aaaGameAreas[i][1][0] + atof(argv[8]) * 32; //y + height
+		aaaGameAreas[i][0][0] = str_tofloat(argv[2 + i * 3]) * 32; //x
+		aaaGameAreas[i][1][0] = str_tofloat(argv[3 + i * 3]) * 32; //y
+		aaaGameAreas[i][0][1] = aaaGameAreas[i][0][0] + str_tofloat(argv[7]) * 32; //x + width
+		aaaGameAreas[i][1][1] = aaaGameAreas[i][1][0] + str_tofloat(argv[8]) * 32; //y + height
 	}
 
 	cmdline_free(argc, argv);
@@ -92,14 +92,20 @@ int main(int argc, const char *argv[])
 		aaMapNames[0], aaMapNames[1], aaaGameAreas[0][0][0], aaaGameAreas[0][1][0], aaaGameAreas[1][0][0], aaaGameAreas[1][1][0],
 		aaaGameAreas[0][0][1] - aaaGameAreas[0][0][0], aaaGameAreas[0][1][1] - aaaGameAreas[0][1][0], aaMapNames[2]);
 
-	IStorage *pStorage = CreateLocalStorage();
+	std::unique_ptr<IStorage> pStorage = CreateLocalStorage();
+	if(!pStorage)
+	{
+		log_error("map_replace_area", "Error creating local storage");
+		return -1;
+	}
+
 	for(int i = 0; i < 1024; i++)
 	{
-		g_apNewData[i] = g_apNewItem[i] = 0;
+		g_apNewData[i] = g_apNewItem[i] = nullptr;
 		g_aNewDataSize[i] = 0;
 	}
 
-	return ReplaceArea(pStorage, aaMapNames, aaaGameAreas) ? 0 : 1;
+	return ReplaceArea(pStorage.get(), aaMapNames, aaaGameAreas) ? 0 : 1;
 }
 
 bool ReplaceArea(IStorage *pStorage, const char aaMapNames[3][64], const float aaaGameAreas[][2][2])
@@ -124,7 +130,7 @@ bool ReplaceArea(IStorage *pStorage, const char aaMapNames[3][64], const float a
 		for(int j = 0; j < 2; j++)
 		{
 			apLayerGroups[j] = GetLayerGroup(aInputMaps[j], i + 1);
-			apItem[j] = (CMapItemLayer *)aInputMaps[j].GetItem(aLayersStart[j] + i, 0, 0);
+			apItem[j] = (CMapItemLayer *)aInputMaps[j].GetItem(aLayersStart[j] + i);
 		}
 
 		if(!apLayerGroups[0] || !apLayerGroups[1])
@@ -165,16 +171,21 @@ void SaveOutputMap(CDataFileReader &InputMap, CDataFileWriter &OutputMap)
 {
 	for(int i = 0; i < InputMap.NumItems(); i++)
 	{
-		int ID, Type;
-		void *pItem = InputMap.GetItem(i, &Type, &ID);
+		int Id, Type;
+		CUuid Uuid;
+		void *pItem = InputMap.GetItem(i, &Type, &Id, &Uuid);
 
+		// Filter ITEMTYPE_EX items, they will be automatically added again.
 		if(Type == ITEMTYPE_EX)
+		{
 			continue;
+		}
+
 		if(g_apNewItem[i])
 			pItem = g_apNewItem[i];
 
 		int Size = InputMap.GetItemSize(i);
-		OutputMap.AddItem(Type, ID, Size, pItem);
+		OutputMap.AddItem(Type, Id, Size, pItem, &Uuid);
 	}
 
 	for(int i = 0; i < InputMap.NumData(); i++)
@@ -205,7 +216,7 @@ bool CompareLayers(const char aaMapNames[3][64], CDataFileReader aInputMaps[2])
 	{
 		CMapItemLayer *apItem[2];
 		for(int j = 0; j < 2; j++)
-			apItem[j] = (CMapItemLayer *)aInputMaps[j].GetItem(aStart[j] + i, 0, 0);
+			apItem[j] = (CMapItemLayer *)aInputMaps[j].GetItem(aStart[j] + i);
 
 		if(apItem[0]->m_Type != apItem[1]->m_Type)
 		{
@@ -229,11 +240,11 @@ void CompareGroups(const char aaMapNames[3][64], CDataFileReader aInputMaps[2])
 	{
 		CMapItemGroup *apItem[2];
 		for(int j = 0; j < 2; j++)
-			apItem[j] = (CMapItemGroup *)aInputMaps[j].GetItem(aStart[j] + i, 0, 0);
+			apItem[j] = (CMapItemGroup *)aInputMaps[j].GetItem(aStart[j] + i);
 
-		bool bSameConfig = apItem[0]->m_ParallaxX == apItem[1]->m_ParallaxX && apItem[0]->m_ParallaxY == apItem[1]->m_ParallaxY && apItem[0]->m_OffsetX == apItem[1]->m_OffsetX && apItem[0]->m_OffsetY == apItem[1]->m_OffsetY && apItem[0]->m_UseClipping == apItem[1]->m_UseClipping && apItem[0]->m_ClipX == apItem[1]->m_ClipX && apItem[0]->m_ClipY == apItem[1]->m_ClipY && apItem[0]->m_ClipW == apItem[1]->m_ClipW && apItem[0]->m_ClipH == apItem[1]->m_ClipH;
+		bool SameConfig = apItem[0]->m_ParallaxX == apItem[1]->m_ParallaxX && apItem[0]->m_ParallaxY == apItem[1]->m_ParallaxY && apItem[0]->m_OffsetX == apItem[1]->m_OffsetX && apItem[0]->m_OffsetY == apItem[1]->m_OffsetY && apItem[0]->m_UseClipping == apItem[1]->m_UseClipping && apItem[0]->m_ClipX == apItem[1]->m_ClipX && apItem[0]->m_ClipY == apItem[1]->m_ClipY && apItem[0]->m_ClipW == apItem[1]->m_ClipW && apItem[0]->m_ClipH == apItem[1]->m_ClipH;
 
-		if(!bSameConfig)
+		if(!SameConfig)
 			dbg_msg("map_replace_area", "WARNING: different configuration on layergroup #%d, this might lead to unexpected results", i);
 	}
 }
@@ -245,12 +256,12 @@ const CMapItemGroup *GetLayerGroup(CDataFileReader &InputMap, const int LayerNum
 
 	for(int i = 0; i < Num; i++)
 	{
-		CMapItemGroup *pItem = (CMapItemGroup *)InputMap.GetItem(Start + i, 0, 0);
+		CMapItemGroup *pItem = (CMapItemGroup *)InputMap.GetItem(Start + i);
 		if(LayerNumber >= pItem->m_StartLayer && LayerNumber <= pItem->m_StartLayer + pItem->m_NumLayers)
 			return pItem;
 	}
 
-	return 0x0;
+	return nullptr;
 }
 
 void ReplaceAreaTiles(CDataFileReader aInputMaps[2], const float aaaGameAreas[][2][2], const CMapItemGroup *apLayerGroups[2], CMapItemLayer *apItem[2])
@@ -258,7 +269,7 @@ void ReplaceAreaTiles(CDataFileReader aInputMaps[2], const float aaaGameAreas[][
 	CMapItemLayerTilemap *apTilemap[2];
 	CTile *apTile[2];
 	float aaaVisibleAreas[2][2][2], aaaReplaceableAreas[2][2][2];
-	MapObject aObs[2];
+	CMapObject aObs[2];
 
 	for(int i = 0; i < 2; i++)
 	{
@@ -309,7 +320,7 @@ void ReplaceDestinationTiles(CMapItemLayerTilemap *apTilemap[2], CTile *apTile[2
 			apTile[1][x1 + (y1 * apTilemap[1]->m_Width)] = apTile[0][x0 + (y0 * apTilemap[0]->m_Width)];
 }
 
-bool AdaptVisibleAreas(const float aaaGameAreas[2][2][2], const MapObject aObs[2], float aaaVisibleAreas[2][2][2])
+bool AdaptVisibleAreas(const float aaaGameAreas[2][2][2], const CMapObject aObs[2], float aaaVisibleAreas[2][2][2])
 {
 	float aDistance[2];
 	GetGameAreaDistance(aaaGameAreas, aObs, aaaVisibleAreas, aDistance);
@@ -332,7 +343,7 @@ bool AdaptVisibleAreas(const float aaaGameAreas[2][2][2], const MapObject aObs[2
 	return true;
 }
 
-bool AdaptReplaceableAreas(const float aaaGameAreas[2][2][2], const float aaaVisibleAreas[2][2][2], const MapObject aObs[2], float aaaReplaceableAreas[2][2][2])
+bool AdaptReplaceableAreas(const float aaaGameAreas[2][2][2], const float aaaVisibleAreas[2][2][2], const CMapObject aObs[2], float aaaReplaceableAreas[2][2][2])
 {
 	float aDistance[2], aScreenPos[2];
 	GetGameAreaDistance(aaaGameAreas, aObs, aaaVisibleAreas, aDistance);
@@ -374,10 +385,10 @@ void ReplaceAreaQuads(CDataFileReader aInputMaps[2], const float aaaGameAreas[][
 	apQuads[2] = new CQuad[apQuadLayer[0]->m_NumQuads + apQuadLayer[1]->m_NumQuads];
 	int QuadsCounter = 0;
 
-	bool bDataChanged = RemoveDestinationQuads(aaaGameAreas[1], apQuads[1], apQuadLayer[1]->m_NumQuads, apLayerGroups[1], apQuads[2], QuadsCounter);
-	bDataChanged |= InsertDestinationQuads(aaaGameAreas, apQuads[0], apQuadLayer[0]->m_NumQuads, apLayerGroups, apQuads[2], QuadsCounter);
+	bool DataChanged = RemoveDestinationQuads(aaaGameAreas[1], apQuads[1], apQuadLayer[1]->m_NumQuads, apLayerGroups[1], apQuads[2], QuadsCounter);
+	DataChanged |= InsertDestinationQuads(aaaGameAreas, apQuads[0], apQuadLayer[0]->m_NumQuads, apLayerGroups, apQuads[2], QuadsCounter);
 
-	if(bDataChanged)
+	if(DataChanged)
 	{
 		g_apNewData[apQuadLayer[1]->m_Data] = apQuads[2];
 		g_aNewDataSize[apQuadLayer[1]->m_Data] = ((int)sizeof(CQuad)) * QuadsCounter;
@@ -390,15 +401,15 @@ void ReplaceAreaQuads(CDataFileReader aInputMaps[2], const float aaaGameAreas[][
 
 bool RemoveDestinationQuads(const float aaGameArea[2][2], const CQuad *pQuads, const int NumQuads, const CMapItemGroup *pLayerGroup, CQuad *pDestQuads, int &QuadsCounter)
 {
-	bool bDataChanged = false;
+	bool DataChanged = false;
 
 	for(int i = 0; i < NumQuads; i++)
 	{
-		MapObject Ob = CreateMapObject(pLayerGroup, fx2f(pQuads[i].m_aPoints[4].x), fx2f(pQuads[i].m_aPoints[4].y), 0, 0);
+		CMapObject Ob = CreateMapObject(pLayerGroup, fx2f(pQuads[i].m_aPoints[4].x), fx2f(pQuads[i].m_aPoints[4].y), 0, 0);
 
 		if(GetVisibleArea(aaGameArea, Ob))
 		{
-			bDataChanged = true;
+			DataChanged = true;
 			continue;
 		}
 
@@ -406,16 +417,16 @@ bool RemoveDestinationQuads(const float aaGameArea[2][2], const CQuad *pQuads, c
 		QuadsCounter++;
 	}
 
-	return bDataChanged;
+	return DataChanged;
 }
 
 bool InsertDestinationQuads(const float aaaGameAreas[2][2][2], const CQuad *pQuads, const int NumQuads, const CMapItemGroup *apLayerGroups[2], CQuad *pDestQuads, int &QuadsCounter)
 {
-	bool bDataChanged = false;
+	bool DataChanged = false;
 
 	for(int i = 0; i < NumQuads; i++)
 	{
-		MapObject aObs[2];
+		CMapObject aObs[2];
 		aObs[0] = CreateMapObject(apLayerGroups[0], fx2f(pQuads[i].m_aPoints[4].x), fx2f(pQuads[i].m_aPoints[4].y), 0, 0);
 		float aaVisibleArea[2][2];
 
@@ -435,23 +446,23 @@ bool InsertDestinationQuads(const float aaaGameAreas[2][2][2], const CQuad *pQua
 			}
 
 			QuadsCounter++;
-			bDataChanged = true;
+			DataChanged = true;
 		}
 	}
 
-	return bDataChanged;
+	return DataChanged;
 }
 
-bool AdaptVisiblePoint(const float aaaGameAreas[2][2][2], const float aaVisibleArea[2][2], const MapObject aObs[2], float aPos[2])
+bool AdaptVisiblePoint(const float aaaGameAreas[2][2][2], const float aaVisibleArea[2][2], const CMapObject aObs[2], float aPos[2])
 {
 	float aDistance[2], aScreenPos[2];
 	GetGameAreaDistance(aaaGameAreas, aObs, aaVisibleArea, aDistance);
-	GetSignificantScreenPos(aObs[0], aaVisibleArea, 0x0, aScreenPos);
+	GetSignificantScreenPos(aObs[0], aaVisibleArea, nullptr, aScreenPos);
 
 	for(int i = 0; i < 2; i++)
 		aPos[i] = aaVisibleArea[i][0] + aDistance[i] + aObs[1].m_aLayerOffset[i] - (aScreenPos[i] + aDistance[i]) * aObs[1].m_aSpeed[i];
 
-	MapObject FinalOb = aObs[1];
+	CMapObject FinalOb = aObs[1];
 	for(int i = 0; i < 2; i++)
 		FinalOb.m_aaBaseArea[i][0] = FinalOb.m_aaBaseArea[i][1] += aPos[i];
 	SetExtendedArea(FinalOb);
@@ -459,9 +470,9 @@ bool AdaptVisiblePoint(const float aaaGameAreas[2][2][2], const float aaVisibleA
 	return GetVisibleArea(aaaGameAreas[1], FinalOb);
 }
 
-MapObject CreateMapObject(const CMapItemGroup *pLayerGroup, const int PosX, const int PosY, const int Width, const int Height)
+CMapObject CreateMapObject(const CMapItemGroup *pLayerGroup, const int PosX, const int PosY, const int Width, const int Height)
 {
-	MapObject Ob;
+	CMapObject Ob;
 
 	Ob.m_aaBaseArea[0][0] = PosX - pLayerGroup->m_OffsetX;
 	Ob.m_aaBaseArea[1][0] = PosY - pLayerGroup->m_OffsetY;
@@ -479,8 +490,8 @@ MapObject CreateMapObject(const CMapItemGroup *pLayerGroup, const int PosX, cons
 
 	for(int i = 0; i < 2; i++)
 	{
-		Ob.m_aaScreenOffset[i][0] = -Ob.ms_aStandardScreen[i];
-		Ob.m_aaScreenOffset[i][1] = Ob.ms_aStandardScreen[i];
+		Ob.m_aaScreenOffset[i][0] = -CMapObject::ms_aStandardScreen[i];
+		Ob.m_aaScreenOffset[i][1] = CMapObject::ms_aStandardScreen[i];
 		if(Ob.m_aSpeed[i] < 0)
 			std::swap(Ob.m_aaScreenOffset[i][0], Ob.m_aaScreenOffset[i][1]);
 	}
@@ -489,7 +500,7 @@ MapObject CreateMapObject(const CMapItemGroup *pLayerGroup, const int PosX, cons
 	return Ob;
 }
 
-void SetExtendedArea(MapObject &Ob)
+void SetExtendedArea(CMapObject &Ob)
 {
 	SetInexistent((float *)Ob.m_aaExtendedArea, 4);
 
@@ -499,7 +510,7 @@ void SetExtendedArea(MapObject &Ob)
 		{
 			float aInspectedArea[2];
 			if(GetLineIntersection(Ob.m_aaBaseArea[i], Ob.m_aaScreenOffset[i], aInspectedArea))
-				memcpy(Ob.m_aaExtendedArea[i], aInspectedArea, sizeof(float[2]));
+				mem_copy(Ob.m_aaExtendedArea[i], aInspectedArea, sizeof(float[2]));
 			continue;
 		}
 
@@ -511,7 +522,7 @@ void SetExtendedArea(MapObject &Ob)
 	}
 }
 
-bool GetVisibleArea(const float aaGameArea[2][2], const MapObject Ob, float aaVisibleArea[2][2])
+bool GetVisibleArea(const float aaGameArea[2][2], const CMapObject &Ob, float aaVisibleArea[2][2])
 {
 	if(IsInexistent((float *)Ob.m_aaExtendedArea, 4))
 		return false;
@@ -520,13 +531,13 @@ bool GetVisibleArea(const float aaGameArea[2][2], const MapObject Ob, float aaVi
 		SetInexistent((float *)aaVisibleArea, 4);
 
 	float aaInspectedArea[2][2];
-	memcpy(aaInspectedArea, aaGameArea, sizeof(float[2][2]));
+	mem_copy(aaInspectedArea, aaGameArea, sizeof(float[2][2]));
 
 	for(int i = 0; i < 2; i++)
 	{
 		if(Ob.m_aSpeed[i] == 1)
 		{
-			memcpy(aaInspectedArea[i], Ob.m_aaExtendedArea[i], sizeof(float[2]));
+			mem_copy(aaInspectedArea[i], Ob.m_aaExtendedArea[i], sizeof(float[2]));
 			continue;
 		}
 
@@ -538,12 +549,12 @@ bool GetVisibleArea(const float aaGameArea[2][2], const MapObject Ob, float aaVi
 	}
 
 	if(aaVisibleArea)
-		memcpy(aaVisibleArea, aaInspectedArea, sizeof(float[2][2]));
+		mem_copy(aaVisibleArea, aaInspectedArea, sizeof(float[2][2]));
 
 	return true;
 }
 
-bool GetReplaceableArea(const float aaVisibleArea[2][2], const MapObject Ob, float aaReplaceableArea[2][2])
+bool GetReplaceableArea(const float aaVisibleArea[2][2], const CMapObject &Ob, float aaReplaceableArea[2][2])
 {
 	SetInexistent((float *)aaReplaceableArea, 4);
 	if(IsInexistent((float *)aaVisibleArea, 4))
@@ -576,7 +587,7 @@ bool GetReplaceableArea(const float aaVisibleArea[2][2], const MapObject Ob, flo
 	return true;
 }
 
-void GetGameAreaDistance(const float aaaGameAreas[2][2][2], const MapObject aObs[2], const float aaaVisibleAreas[2][2][2], float aDistance[2])
+void GetGameAreaDistance(const float aaaGameAreas[2][2][2], const CMapObject aObs[2], const float aaaVisibleAreas[2][2][2], float aDistance[2])
 {
 	for(int i = 0; i < 2; i++)
 	{
@@ -591,15 +602,15 @@ void GetGameAreaDistance(const float aaaGameAreas[2][2][2], const MapObject aObs
 	}
 }
 
-void GetGameAreaDistance(const float aaaGameAreas[2][2][2], const MapObject aObs[2], const float aaVisibleArea[2][2], float aDistance[2])
+void GetGameAreaDistance(const float aaaGameAreas[2][2][2], const CMapObject aObs[2], const float aaVisibleArea[2][2], float aDistance[2])
 {
 	float aaaVisibleAreas[2][2][2];
-	memcpy(aaaVisibleAreas[0], aaVisibleArea[0], sizeof(float[2][2]));
-	memcpy(aaaVisibleAreas[1], aaVisibleArea[0], sizeof(float[2][2]));
+	mem_copy(aaaVisibleAreas[0], aaVisibleArea[0], sizeof(float[2][2]));
+	mem_copy(aaaVisibleAreas[1], aaVisibleArea[0], sizeof(float[2][2]));
 	GetGameAreaDistance(aaaGameAreas, aObs, aaaVisibleAreas, aDistance);
 }
 
-void GetSignificantScreenPos(const MapObject Ob, const float aaVisibleArea[2][2], const float aaReplaceableArea[2][2], float aScreen[2])
+void GetSignificantScreenPos(const CMapObject &Ob, const float aaVisibleArea[2][2], const float aaReplaceableArea[2][2], float aScreen[2])
 {
 	for(int i = 0; i < 2; i++)
 	{
@@ -618,8 +629,8 @@ void ConvertToTiles(const float aaArea[2][2], int aaTiles[2][2])
 {
 	for(int i = 0; i < 2; i++)
 	{
-		aaTiles[i][0] = floor((floor(aaArea[i][0] * 100.0f) / 100.0f) / 32.0f);
-		aaTiles[i][1] = ceil((floor(aaArea[i][1] * 100.0f) / 100.0f) / 32.0f);
+		aaTiles[i][0] = std::floor((std::floor(aaArea[i][0] * 100.0f) / 100.0f) / 32.0f);
+		aaTiles[i][1] = std::ceil((std::floor(aaArea[i][1] * 100.0f) / 100.0f) / 32.0f);
 	}
 }
 
@@ -630,13 +641,13 @@ bool GetLineIntersection(const float aLine1[2], const float aLine2[2], float aIn
 		std::min(aLine1[1], aLine2[1])};
 
 	if(aIntersection)
-		SetInexistent((float *)aIntersection, 2);
+		SetInexistent(aIntersection, 2);
 
 	if(aBorders[0] - aBorders[1] > 0.01f)
 		return false;
 
 	if(aIntersection)
-		memcpy(aIntersection, aBorders, sizeof(float[2]));
+		mem_copy(aIntersection, aBorders, sizeof(float[2]));
 
 	return true;
 }
